#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: run dvips inside latex2html or fake it.

# initialize the error status to failure
set stat=1

# Usage
goto start
use:
echo 'Usage: dvipsl2h.bat DVIPS_COMMAND_LINE_PARAMETERS'
echo '       The parameters *must* be of the form'
echo '          -S1 -i ... -o OUTPUTROOT DVIFILE'
echo '       where .dvi *must* be omitted from DVIFILE.'
echo '       (Dvipsl2h will run pdftk and pdftops instead of dvips if'
echo '       a current DVIFILE.pdf file is found.  This accommodates'
echo '       use of pdflatex, xelatex, or lualatex.)'
goto err2
start:

# see whether we are called for inquiry
if ("$1" == '?') goto use
if ("$1" == '/?') goto use
if ("$1" == '-h') goto use
if ("$1" == '--help') goto use
if ("$1" == '') goto use

# respond to version inquiries during rebuilds
if ("$1" != '--version') goto ver2
dvips --version
if ($status) goto err2
goto end
ver2:
if ("$1" != '-?') goto ver3
dvips -?
if ($status) goto err2
goto end
ver3:
if ("$1" != '-v') goto verend
dvips -v
if ($status) goto err2
goto end
verend:

# notify that we are running
echo Running dvipsl2h.bat

# open a log file
if !(-f dvipsl2h.log) then
    echo 'dvipsl2h log:' > dvipsl2h.log
endif
if ($status) goto errsys
if !(-f dvipsl2h.log) goto errsys

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# write what we are doing to the log
echo "dvipsl2h.bat $1 $2 $3 $4 $5 $6 $7 $8 $9 ..." >> dvipsl2h.log

# get rid of the first two parameters
if ("$1" != '-S1') goto erripars
if ("$2" != '-i') goto erripars
shift
shift
echo "dvipsl2h.bat -S1 -i $1 $2 $3 $4 $5 $6 $7 $8 $9" >> dvipsl2h.log

# figure out whether to run dvips or pdftk and pdftops
if ("$9" == '') goto not9
echo "Images file is $9" >> dvipsl2h.log
if ("$7" != '-o') goto errpars
if !(-f "$9.tex") goto errnotex
"$L2HTOP/bin/l2hnewer.exe" "$9".tex "$9".pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto dodvips
goto dopdftk
not9:
if ("$8" == '') goto not8
echo "Images file is $8" >> dvipsl2h.log
if ("$6" != '-o') goto errpars
if !(-f "$8.tex") goto errnotex
"$L2HTOP/bin/l2hnewer.exe" "$8".tex "$8".pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto dodvips
goto dopdftk
not8:
if ("$7" == '') goto not7
echo "Images file is $7" >> dvipsl2h.log
if ("$5" != '-o') goto errpars
if !(-f "$7.tex") goto errnotex
"$L2HTOP/bin/l2hnewer.exe" "$7".tex "$7".pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto dodvips
goto dopdftk
not7:
if ("$6" == '') goto not6
echo "Images file is $6" >> dvipsl2h.log
if ("$4" != '-o') goto errpars
if !(-f "$6.tex") goto errnotex
"$L2HTOP/bin/l2hnewer.exe" "$6".tex "$6".pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto dodvips
goto dopdftk
not6:
if ("$5" == '') goto not5
echo "Images file is $5" >> dvipsl2h.log
if ("$3" != '-o') goto errpars
if !(-f "$5.tex") goto errnotex
"$L2HTOP/bin/l2hnewer.exe" "$5".tex "$5".pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto dodvips
goto dopdftk
not5:
if ("$4" == '') goto not4
echo "Images file is $4" >> dvipsl2h.log
if ("$2" != '-o') goto errpars
if !(-f "$4.tex") goto errnotex
"$L2HTOP/bin/l2hnewer.exe" "$4".tex "$4".pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto dodvips
goto dopdftk
not4:
if ("$3" == '') goto not3
echo "Images file is $3" >> dvipsl2h.log
if ("$1" != '-o') goto errpars
if !(-f "$3.tex") goto errnotex
"$L2HTOP/bin/l2hnewer.exe" "$3".tex "$3".pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto dodvips
goto dopdftk
not3:
goto errpars

# Run dvips to create postscript images from images.dvi
dodvips:
echo 'Using dvips ' >> dvipsl2h.log

# see whether dvips is in the path
which dvips
if ($status) goto errnodp

# run it
echo "Running dvips -S1 -i $1 $2 $3 $4 $5 $6 $7 $8 $9" >> dvipsl2h.log
if ("$9" != "") goto dodvips9
if ("$8" != "") goto dodvips8
if ("$7" != "") goto dodvips7
if ("$6" != "") goto dodvips6
if ("$5" != "") goto dodvips5
if ("$4" != "") goto dodvips4
dvips -S1 -i "$1" "$2" "$3"
set stat="$status"
if ($stat) goto errdvips
echo 'error status 0 ' >> dvipsl2h.log
goto done
dodvips4:
dvips -S1 -i "$1" "$2" "$3" "$4"
set stat="$status"
if ($stat) goto errdvips
echo 'error status 0 ' >> dvipsl2h.log
goto done
dodvips5:
dvips -S1 -i "$1" "$2" "$3" "$4" "$5"
set stat="$status"
if ($stat) goto errdvips
echo 'error status 0 ' >> dvipsl2h.log
goto done
dodvips6:
dvips -S1 -i "$1" "$2" "$3" "$4" "$5" "$6"
set stat="$status"
if ($stat) goto errdvips
echo 'error status 0 ' >> dvipsl2h.log
goto done
dodvips7:
dvips -S1 -i "$1" "$2" "$3" "$4" "$5" "$6" "$7"
set stat="$status"
if ($stat) goto errdvips
echo 'error status 0 ' >> dvipsl2h.log
goto done
dodvips8:
dvips -S1 -i "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8"
set stat="$status"
if ($stat) goto errdvips
echo 'error status 0 ' >> dvipsl2h.log
goto done
dodvips9:
dvips -S1 -i "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9"
set stat="$status"
if ($stat) goto errdvips
echo 'error status 0 ' >> dvipsl2h.log
goto done

# use pdftk and pdftops to create the images from images.pdf
dopdftk:
echo 'Really running pdftk and pdftops on the images pdf file.' >> dvipsl2h.log

# find the -o argument
loop:
if ("$1" == '-o') goto endloop
if ("$1" == '') goto errpars
shift
goto loop
endloop:
shift
if ("$1" == '') goto errpars
if !(-f "$2.pdf") goto errpars

# see whether pdftk is in the path
which pdftk
if ($status) goto errnotk

# run it
echo "Running pdftk $2.pdf burst output $1%03d.pdf" >> dvipsl2h.log
pdftk "$2".pdf burst output "$1%03d.pdf"
set stat="$status"
if ($stat) goto errpdftk
echo 'error status 0 ' >> dvipsl2h.log

# create a list of files to process
echo 'Making a list of the created single-image pdf files.' >> dvipsl2h.log
ls -1 "$1"*.pdf > @tmpd.lst
if ($status) goto errsys
cat @tmpd.lst | fgrep "${1}001.pdf"
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto no001pdf
goto havlst
no001pdf:
echo " ** no ${1}001.pdf image found???"
echo " ** no ${1}001.pdf image found???" >> dvipsl2h.log
havlst:

# create a batch file to process the created single-image pdf files
echo 'Making a script to convert these pdf files to eps.' >> dvipsl2h.log
"$L2HTOP/bin/subbin.exe" "$L2HTOP/system-files/data/mkprocfils.sub" @tmpd.lst @tmpd.bat
if ($status) goto errmkbat
if !(-f @tmpd.bat) goto errmkbat

# see whether pdftops is in the path
which pdftops
if ($status) goto errnopp

# now convert the single-image pdf files to eps using pdftops
echo "Running pdftops on  $1*.pdf" >> dvipsl2h.log
set errlev=0
source "@tmpd.bat" "$L2HTOP/system-files/bin/file_pdfcnv.bat"
if ($status) goto errdobat
if ("$errlev" == 2) goto errdocnv
if ("$errlev" == 1) goto errbadcnv

# clean up
if (-f ../@system/keeptmps.yes) goto endclean
if (-f @tmpd.bat) rm @tmpd.bat
if (-f @tmpd.lst) rm @tmpd.lst
endclean:

# done
done:
goto end

# errors
errl2h:
echo "*** dvipsl2h.bat: cannot find file $L2HTOP/install_l2h.desktop."
echo ' ** You may need to rerun install_l2h in the l2h folder.'
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop." >> dvipsl2h.log
echo ' ** You may need to rerun install_l2h in the l2h folder.' >> dvipsl2h.log
set stat=9
goto err
erripars:
echo '*** dvipsl2h.bat: the first two parameters must be: -S1 -i'
echo '*** Fatal error: the first two parameters must be: -S1 -i' >> dvipsl2h.log
set stat=9
goto err
errpars:
echo '*** dvipsl2h.bat: the parameters MUST be of the form'
echo ' **    -S1 -i ... -o OUTPUTROOT DVIFILE'
echo ' ** where the .dvi MUST be omitted from DVIFILE'
echo '*** Fatal error: the parameters MUST be of the form' >> dvipsl2h.log
echo ' **    -S1 -i ... -o OUTPUTROOT DVIFILE' >> dvipsl2h.log
echo ' ** where the .dvi MUST be omitted from DVIFILE' >> dvipsl2h.log
set stat=9
goto err
errnotex:
echo '*** dvipsl2h.bat: the images tex file does not exist.'
echo ' ** (Or the .dvi has not been omitted from the parameters?)'
echo '*** Fatal error: the images tex file does not exist.' >> dvipsl2h.log
echo ' ** (Or the .dvi has not been omitted from the parameters?)' >> dvipsl2h.log
set stat=9
goto err
errnodp:
echo '*** dvipsl2h.bat: LaTeX program dvips is missing.  Install it first.'
echo '*** Fatal error: LaTeX program dvips is missing.  Install it first.' >> dvipsl2h.log
set stat=9
goto err
errdvips:
echo "*** dvipsl2h.bat: status ${stat}: dvips -S1 -i $1 $2 $3 $4 $5 $6 $7 $8 $9"
echo "*** failed: error status $stat" >> dvipsl2h.log
goto err
errnotk:
echo '*** dvipsl2h.bat: program pdftk is missing.  Install it first.'
echo '*** Fatal error: Program pdftk is missing.  Install it first.' >> dvipsl2h.log
set stat=9
goto err
errpdftk:
echo "*** dvipsl2h.bat: status ${stat}: pdftk $2.pdf burst output "'"'"$1%03d.pdf"'"'
echo "*** failed: error status $stat" >> dvipsl2h.log
goto err
errmkbat:
echo '*** dvipsl2h.bat: unable to make a batch file to convert the pdf'
echo ' ** images to eps.'
echo '*** Fatal error: unable to make a batch file to convert the pdf' >> dvipsl2h.log
echo ' ** images to eps.' >> dvipsl2h.log
set stat=9
goto err
errnopp:
echo '*** dvipsl2h.bat: Program pdftops is missing.  Install it first.'
echo '*** Fatal error: Program pdftops is missing.  Install it first.' >> dvipsl2h.log
set stat=9
goto err
errdobat:
echo '*** dvipsl2h.bat: system error converting the pdf images to eps.'
echo '*** Fatal error: system error converting the pdf images to eps.' >> dvipsl2h.log
set stat=9
goto err
errdocnv:
echo '*** dvipsl2h.bat: the pdftops program failed to convert at least'
echo ' ** one image.'
echo '*** Fatal error: the pdftops program failed to convert at least' >> dvipsl2h.log
echo ' ** one image.' >> dvipsl2h.log
set stat=9
goto err
errbadcnv:
echo '*** dvipsl2h.bat: an image to convert to eps was not found or'
echo ' ** already converted.'
echo '*** Fatal error: an image to convert to eps was not found or' >> dvipsl2h.log
echo ' ** already converted.' >> dvipsl2h.log
set stat=9
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo ' ** If not, try rerunning install_l2h in the l2h_folder.'
if (-f dvipsl2h.log) then
    echo '*** Operating system error.  Check disk space and file protections.' >> dvipsl2h.log
endif
if (-f dvipsl2h.log) then
    echo ' ** If not, try rerunning install_l2h in the l2h_folder.' >> dvipsl2h.log
endif
set stat=9
err:
sleep 1
sleep 2
err2:
exit $stat

end:
