#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: run latex within latex2html.

#                              IMPORTANT
# -------------------------------------------------------------------
# This file may have to be edited to run LaTeX correctly inside
# latex2html on your system.
#
# In particular, this file sets a variable TEXINPUTS to include the
# next higher folder in file searches.  However, some very old
# systems use variable TEXINPUT instead of TEXINPUTS.  In that case,
# you may have to change that here, and in init.bat in the l2h
# folder.  Alternatively, some buggy systems ignore TEXINPUTS.
#
# Note further that this batch file uses plain latex as the engine
# unless you define \webengine differently in your document and then
# process your document through l2h_menu or at least through
# makewebpages.bat.  If you mess around with that in this file, you
# will probably also need to change dvipsl2h.bat correspondingly.

# initialize the error status to failure
set stat=1

# Usage
goto start
use:
echo 'Usage: latexl2h.bat LATEX_COMMAND_PARAMETERS'
goto err2
start:

# see whether we are called for inquiry
if ("$1" == '?') goto use
if ("$1" == '/?') goto use
if ("$1" == '-h') goto use
if ("$1" == '--help') goto use
if ("$1" == '') goto use

# notify that we are running
echo Running latexl2h.bat

# set an empty TEXINPUTS
if !($?TEXINPUTS) setenv TEXINPUTS ''

# open a log file
if !(-f latexl2h.log) then
    echo 'latexl2h log:' > latexl2h.log
endif
if ($status) goto errsys
if !(-f latexl2h.log) goto errsys

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/init.bat") goto errl2h

# write what we are doing to the log
echo "latexl2h.bat $1 $2 $3 $4 $5 $6 $7 $8 $9" >> latexl2h.log

# delete any existing @resetti.bat
if (-f @resetti.bat) rm @resetti.bat
if ($status) goto errsys
if (-e @resetti.bat) goto errsys


# ***************************************************************
# START OF POSSIBLE MODIFICATION
# ***************************************************************

# Set TEXINPUT or TEXINPUTS for your system. It must include the
# next higher directory '..' also.   The following lines should
# work on standard TeX systems.
#
# However, some older systems use TEXINPUT instead of TEXINPUTS.
# In that case, you must change that name both here and in file
# init.bat in the l2h folder.  Do *not* change the lower case
# texinputs folder name!
#
# Some buggy MS Windows TeX systems ignore TEXINPUTS.  In that
# case, install a newer version or contact the author for advice.
#
# check whether TEXINPUTS needs to be initialized by init.bat
echo "TEXINPUTS was: $TEXINPUTS" >> latexl2h.log
echo "':${TEXINPUTS}:'" | fgrep ":$L2HTOP/texinputs:"
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto run_init
echo "':${TEXINPUTS}:'" | fgrep "':.:"
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto run_init
goto check_ti
run_init:
source "$L2HTOP/init.bat"
if ($status) goto errini
echo "':${TEXINPUTS}:'" | fgrep ":$L2HTOP/texinputs:"
if ($status) goto errsys
echo "':${TEXINPUTS}:'" | fgrep "':.:"
if ($status) goto errsys
check_ti:
echo "':${TEXINPUTS}:'" | fgrep ':..:'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto change_ti
goto have_ti
change_ti:
echo "setenv TEXINPUTS '$TEXINPUTS'" > @resetti.bat
echo 'if ("$TEXINPUTS" == "") unsetenv TEXINPUTS' >> @resetti.bat
setenv TEXINPUTS ".:.$TEXINPUTS"
echo "':${TEXINPUTS}:'" | fgrep "':.:..:"
if ($status) goto errsys
have_ti:
echo "TEXINPUTS now: $TEXINPUTS" >> latexl2h.log

# ***************************************************************
# END OF POSSIBLE MODIFICATION
# ***************************************************************

# The remaining lines should not be modified

# replace \usepackage[...]{inputenc} in images.tex with whatever l2h
# found in the internationalization section, if anything

# skip if we are called without l2h assist
if !(-f ../@system/texfiles.lst) goto endrep0

# create a substitutions file
echo '\\!\M\J\\usepackage[\:]{inputenc}\\!\M\J' > @tmpl.sub
if ($status) goto errsys
cat ../@system/texfiles.lst | fgrep ':TOIMGSTEX:' >> @tmpl.sub
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto endrep0

# do the replacement
if ("$9" == '') goto not9
echo "Images file is $9" >> latexl2h.log
if !(-f "$9") goto errnotex
"$L2HTOP/system-files/bin/subbin_big.exe" @tmpl.sub "$9" @tmpl.tmp
if ($status) goto errchng
if (-f @tmpl.tmp) then
    echo 'replacing the internationalization header.' >> latexl2h.log
endif
if (-f @tmpl.tmp) mv @tmpl.tmp "$9"
if ($status) goto errsys
goto endrep
not9:
if ("$8" == '') goto not8
echo "Images file is $8" >> latexl2h.log
if !(-f "$8") goto errnotex
"$L2HTOP/system-files/bin/subbin_big.exe" @tmpl.sub "$8" @tmpl.tmp
if ($status) goto errchng
if (-f @tmpl.tmp) then
    echo 'replacing the internationalization header.' >> latexl2h.log
endif
if (-f @tmpl.tmp) mv @tmpl.tmp "$8"
if ($status) goto errsys
goto endrep
not8:
if ("$7" == '') goto not7
echo "Images file is $7" >> latexl2h.log
if !(-f "$7") goto errnotex
"$L2HTOP/system-files/bin/subbin_big.exe" @tmpl.sub "$7" @tmpl.tmp
if ($status) goto errchng
if (-f @tmpl.tmp) then
    echo 'replacing the internationalization header.' >> latexl2h.log
endif
if (-f @tmpl.tmp) mv @tmpl.tmp "$7"
if ($status) goto errsys
goto endrep
not7:
if ("$6" == '') goto not6
echo "Images file is $6" >> latexl2h.log
if !(-f "$6") goto errnotex
"$L2HTOP/system-files/bin/subbin_big.exe" @tmpl.sub "$6" @tmpl.tmp
if ($status) goto errchng
if (-f @tmpl.tmp) then
    echo 'replacing the internationalization header.' >> latexl2h.log
endif
if (-f @tmpl.tmp) mv @tmpl.tmp "$6"
if ($status) goto errsys
goto endrep
not6:
if ("$5" == '') goto not5
echo "Images file is $5" >> latexl2h.log
if !(-f "$5") goto errnotex
"$L2HTOP/system-files/bin/subbin_big.exe" @tmpl.sub "$5" @tmpl.tmp
if ($status) goto errchng
if (-f @tmpl.tmp) then
    echo 'replacing the internationalization header.' >> latexl2h.log
endif
if (-f @tmpl.tmp) mv @tmpl.tmp "$5"
if ($status) goto errsys
goto endrep
not5:
if ("$4" == '') goto not4
echo "Images file is $4" >> latexl2h.log
if !(-f "$4") goto errnotex
"$L2HTOP/system-files/bin/subbin_big.exe" @tmpl.sub "$4" @tmpl.tmp
if ($status) goto errchng
if (-f @tmpl.tmp) then
    echo 'replacing the internationalization header.' >> latexl2h.log
endif
if (-f @tmpl.tmp) mv @tmpl.tmp "$4"
if ($status) goto errsys
goto endrep
not4:
if ("$3" == '') goto not3
echo "Images file is $3" >> latexl2h.log
if !(-f "$3") goto errnotex
"$L2HTOP/system-files/bin/subbin_big.exe" @tmpl.sub "$3" @tmpl.tmp
if ($status) goto errchng
if (-f @tmpl.tmp) then
    echo 'replacing the internationalization header.' >> latexl2h.log
endif
if (-f @tmpl.tmp) mv @tmpl.tmp "$3"
if ($status) goto errsys
goto endrep
not3:
if ("$2" == '') goto not2
echo "Images file is $2" >> latexl2h.log
if !(-f "$2") goto errnotex
"$L2HTOP/system-files/bin/subbin_big.exe" @tmpl.sub "$2" @tmpl.tmp
if ($status) goto errchng
if (-f @tmpl.tmp) then
    echo 'replacing the internationalization header.' >> latexl2h.log
endif
if (-f @tmpl.tmp) mv @tmpl.tmp "$2"
if ($status) goto errsys
goto endrep
not2:
if ("$1" == '') goto not1
echo "Images file is $1" >> latexl2h.log
if !(-f "$1") goto errnotex
"$L2HTOP/system-files/bin/subbin_big.exe" @tmpl.sub "$1" @tmpl.tmp
if ($status) goto errchng
if (-f @tmpl.tmp) then
    echo 'replacing the internationalization header.' >> latexl2h.log
endif
if (-f @tmpl.tmp) mv @tmpl.tmp "$1"
if ($status) goto errsys
goto endrep
not1:
goto errpar

# done changing images.tex
endrep0:
echo 'No updated internationalization header given.'
endrep:

# Figure out the latex engine to use

# set the default latex engine to plain latex
setenv we 'latex'
# Some old systems run latex as "tex &latex". In that case, change
# the line above correspondingly.

# use the default if we are called without l2h assist
if !(-f ../@system/setwe.bat) goto endeng

# change the engine to whatever is set in \webengine in index.tex
source ../@system/setwe.bat

# have engine
endeng:
echo "Using the engine $we" >> latexl2h.log

# check that the engine is in the path
set we1=`echo "$we" | cut -d' ' -f1`
which "$we1"
if ($status) goto errnolat

# now run latex, or whatever engine variable we is set to
echo "Running $we $1 $2 $3 $4 $5 $6 $7 $8 $9" >> latexl2h.log
$we $1 $2 $3 $4 $5 $6 $7 $8 $9
set stat="$status"
if ($stat) goto errlat
echo 'error status 0' >> latexl2h.log

# reset TEXINPUTS
if (-f @resetti.bat) source "@resetti.bat"

# clean up
if (-f ../@system/keeptmps.yes) goto endclean
if (-f @resetti.bat) rm @resetti.bat
if (-f @tmpl.sub) rm @tmpl.sub
endclean:
unsetenv we

# done
done:
goto end

# errors
errl2h:
echo "*** latexl2h.bat: cannot find file $L2HTOP/init.bat."
echo ' ** You may need to rerun install_l2h in the l2h folder.'
echo "*** Fatal error: missing file $L2HTOP/init.bat." >> latexl2h.log
echo ' ** You may need to rerun install_l2h in the l2h folder.' >> latexl2h.log
set stat=9
goto err
errpar:
echo '*** latexl2h.bat: missing latex command parameters.'
echo '*** Fatal error: missing latex command parameters.' >> latexl2h.log
set stat=9
goto err
errini:
echo "*** latexl2h.bat: error executing $L2HTOP/init.bat."
echo ' ** You may need to rerun install_l2h in the l2h folder.'
echo "*** Fatal error: error executing $L2HTOP/init.bat." >> latexl2h.log
echo ' ** You may need to rerun install_l2h in the l2h folder.' >> latexl2h.log
set stat=9
goto err
errnotex:
echo '*** latexl2h.bat: cannot find the images tex file.'
echo '*** Fatal error: cannot find the images tex file.' >> latexl2h.log
set stat=9
goto err
errchng:
echo '*** latexl2h.bat: could not update the internationalization header.'
echo '*** Fatal error: could not update the internationalization header.' >> latexl2h.log
set stat=9
goto err
errnolat:
echo "*** latexl2h.bat: Webengine "`echo "$we" | cut -d' ' -f1`" not found."
echo ' ** Check for typos, or install it if correct.'
echo "*** Fatal error: Webengine "`echo "$we" | cut -d' ' -f1`" not found." >> latexl2h.log
echo ' ** Check for typos, or install it if correct.' >> latexl2h.log
set stat=9
goto err
errlat:
echo "*** latexl2h.bat: status ${stat}: $we $1 $2 $3 $4 $5 $6 $7 $8 $9"
echo "*** failed: error status $stat" >> latexl2h.log
if (-f ../ll2htest.tex) goto err2
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo ' ** If not, try rerunning install_l2h in the l2h_folder.'
if (-f latexl2h.log) then
    echo '*** Operating system error.  Check disk space and file protections.' >> latexl2h.log
endif
if (-f latexl2h.log) then
    echo ' ** If not, try rerunning install_l2h in the l2h_folder.' >> latexl2h.log
endif
set stat=9
err:
sleep 1
sleep 2
err2:
if (-f @resetti.bat) source "@resetti.bat"
exit $stat

end:
