#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Process index.tex in the current folder into HTML
#          web pages, pushed into a subfolder named web-pages.

# Usage
goto start
use:
echo 'Usage: webpages.bat [options]'
echo '       Here [options] are any desired special latex2html options.'
echo '       Before use, latex index.tex to create a current aux file.'
echo
echo 'After execution, check error logs'
echo '   l2h.log and texexpand.log in the folder with index.tex;'
echo '   images.log and .tex, latexl2h.log, dvipsl2h.log, ppmquant.log,'
echo '   and pstoimg.log in the web pages subfolder;'
echo 'for problems.  Load index.html (in the current directory, with your'
echo 'index.tex file) in your modern browser and browse down.'
echo
goto err
start:

# usage
if ("$1" == '?') goto use
if ("$1" == '/?') goto use
if ("$1" == '-h') goto use
if ("$1" == '--help') goto use

# turn on the bell
which xset
if !($status) xset b 100

# check for the input file
if !(-f index.tex) goto errtex

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# check for the presence of essential files
if !(-fr "$L2HTOP/init.bat") goto errfils
if !(-fr "$L2HTOP/l2hconf.pm") goto errfils
if !(-fr "$L2HTOP/L2hos.pm") goto errfils
# bin
if !(-frx "$L2HTOP/bin/filechk.bat") goto errfils
"$L2HTOP/bin/filechk.bat"
if ($status) goto errfils
# L2hos
if !(-fr "$L2HTOP/L2hos/Unix.pm") goto errfils
# styles (sample)
if !(-fr "$L2HTOP/styles/html.perl") goto errfils
if !(-fr "$L2HTOP/styles/makeidx.perl") goto errfils
if !(-fr "$L2HTOP/styles/natbib.perl") goto errfils
# system-files
if !(-frx "$L2HTOP/system-files/bin/filechk.bat") goto errfils
"$L2HTOP/system-files/bin/filechk.bat"
if ($status) goto errfils
if !(-frx "$L2HTOP/system-files/data/filechk.bat") goto errfils
"$L2HTOP/system-files/data/filechk.bat"
if ($status) goto errfils
# versions (sample)
if !(-fr "$L2HTOP/versions/html4_0.pl") goto errfils
if !(-fr "$L2HTOP/versions/math.pl") goto errfils
if !(-fr "$L2HTOP/versions/i18n.pl") goto errfils
if !(-fr "$L2HTOP/versions/latin1.pl") goto errfils
if !(-fr "$L2HTOP/versions/unicode.pl") goto errfils
if !(-fr "$L2HTOP/versions/utf8.pl") goto errfils

# check that index.aux is newer
"$L2HTOP/bin/l2hnewer.exe" index.tex index.aux
set stat="$status"
if ($stat >= 2) goto errdat
if ($stat >= 1) goto errretex

# check that $L2HTOP/temp exists and is writable
if !(-e "$L2HTOP/temp") mkdir "$L2HTOP/temp"
if ($status) goto errtemp
if !(-dw "$L2HTOP/temp/") goto errtemp
if (-f "$L2HTOP/temp/tmp.txt") rm "$L2HTOP/temp/tmp.txt"
if (-e "$L2HTOP/temp/tmp.txt") goto errtemp
echo 'test' > "$L2HTOP/temp/tmp.txt"
if ($status) goto errtemp
if !(-f "$L2HTOP/temp/tmp.txt") goto errtemp
rm "$L2HTOP/temp/tmp.txt"

# initialize
source "$L2HTOP/system-files/bin/doinit.bat" quiet
if ($status) goto err

# check that $L2HTOP/temp is empty
if (-f @system/asktemp.no) goto tempok
ls -1 "$L2HTOP/temp" | grep '.'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto tempok
echo 'There is stuff in the "temp" subfolder of l2h.  Unless currently other'
echo 'web pages are being made, this must be garbage left over from previous'
echo 'processing.  If so, delete it now.   (But leave the empty temp folder.)'
echo
"$L2HTOP/bin/choose.exe" cCq "After noting the above, hit c to continue, C to CONTINUE, or q to quit: c"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto errq
if ($stat >= 2) goto tempaok
if ($stat >= 1) goto tempok
goto errq
tempaok:
echo 'Accept a nonempty temp folder.' > @system/asktemp.no
tempok:

# check whether we are already active
if !(-f @system/in-mkwp.txt) goto nouse
echo '*** It appears that makewebpages is already in use for this document.'
echo '    Only one copy of makewebpages can run at any time.'
echo '    (If you are sure only one copy *is* running, hit the 1 key now.)'
choose.exe q1 "Hit q to quit: q"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto rmuse
goto err
rmuse:
rm @system/in-mkwp.txt
if ($status) goto errsys
if (-e @system/in-mkwp.txt) goto errsys
nouse:
echo 'makewebpages is in use' > @system/in-mkwp.txt
if !(-f @system/in-mkwp.txt) goto errsys

# check perl
which perl
if ($status) goto errperl
fgrep '#\! '"`which perl`" "$L2HTOP/bin/latex2html"
if ($status) goto errperl2
fgrep '#\! '"`which perl`" "$L2HTOP/bin/pstoimg"
if ($status) goto errperl2
fgrep '#\! '"`which perl`" "$L2HTOP/bin/texexpand"
if ($status) goto errperl2
fgrep "'`which perl`'" "$L2HTOP/l2hconf.pm"
if ($status) goto errperl3

# really see whether we need to reapply latex
set errlev='0'
source "$L2HTOP/system-files/bin/checknewer.bat" aux
if ($status) goto err
if ("$errlev" != 0) goto errretex

# check the source for bad constructs
set errlev='0'
source "@system/texfiles.bat" "$L2HTOP/system-files/bin/checktex.bat"
if ($status) goto badchk
goto endchk
badchk:
if ("$errlev" == 2) goto err
if ("$errlev" == 1) goto errchk
goto errbat
endchk:

# Check for essential user files
if !(-f index.css) goto misfils
if !(-f settings.pl) goto misfils
goto endfils
misfils:
echo
echo 'The current folder misses one or both of the web page "theme" files'
echo 'index.css and settings.pl. (They come as matched pairs.)  Therefore,'
echo 'to make web pages, you will first need to select a theme for them.'
echo
choose.exe cq "Hit Enter to do so or q to quit making web pages: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto errq
if ($stat >= 1) goto getthm
goto errq
getthm:
"$L2HTOP/system-files/bin/gettheme.bat" settings.pl
if ($status) goto err
if !(-f index.css) goto errsys
if !(-f settings.pl) goto errsys
endfils:

# run any file makewp_start.bat
if !(-f makewp_start.bat) goto wpsend
source makewp_start.bat
if ($status) goto errwps
wpsend:

# put the web page background color in @transcol.txt
clearold.bat y @transcol.txt
if ($status) goto err
"$L2HTOP/system-files/bin/getpgch.bat"
if ($status) goto errtcol
if !(-f @transcol.txt) goto errtcol

# check against any in web-pages
if !(-f web-pages/@transcol.txt) goto colchke
cmp -s @transcol.txt web-pages/@transcol.txt
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto colchkb
goto colchke
colchkb:
echo '** The page color used to generate the existing web-page images is'
echo '   different from the new page color (or unknown).  To avoid "ghosting"'
echo '   around math symbols and lines in certain figures, you should clean'
echo '   up folder web-pages to remake everything from scratch.'
echo '      c: Continue remaking web pages as is despite the problem.'
echo '      d: Delete folder web-pages to remake the web pages from scratch.'
echo '         WARNING: anything you might have inside folder web-pages and'
echo '         not inside its subfolder keep will likely get deleted.'
echo '      q: Quit the web page generation.'
choose.exe qdc "Make your choice from the menu above, then hit its key"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto colchke
if ($stat >= 2) goto colchkd
goto errq
colchkd:
"$L2HTOP/system-files/bin/delwp.bat"
if ($status) goto err
colchke:

# put the web page font size in @fontsize.txt
clearold.bat y @fontsize.txt
if ($status) goto err
subbin.exe "$L2HTOP/system-files/data/getfsiz.sub" index.css @fontsize.txt
if ($status) goto errfsiz
if !(-f @fontsize.txt) goto errfsiz

# check against any in web-pages
if !(-f web-pages/@fontsize.txt) goto fntchke
cmp -s @fontsize.txt web-pages/@fontsize.txt
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto fntchkb
goto fntchke
fntchkb:
echo '** The font size used to generate the existing web-page images is'
echo '   different from the new font size (or unknown).  To avoid improperly'
echo '   sized math symbols and figures, you should delete folder web-pages'
echo '   to remake it from scratch.'
echo '      c: Continue remaking web pages as is despite the problem.'
echo '      d: Delete folder web-pages to remake the web pages from scratch.'
echo '         WARNING: anything you might have inside folder web-pages and'
echo '         not inside its subfolder keep will likely get deleted.'
echo '      q: Quit the web page generation.'
choose.exe qdc "Make your choice from the menu above, then hit its key"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto fntchke
if ($stat >= 2) goto fntchkd
goto errq
fntchkd:
"$L2HTOP/system-files/bin/delwp.bat"
if ($status) goto err
fntchke:

# initialize .latex2html-init
clearold.bat y .latex2html-init
if ($status) goto err
echo '# Do not edit this file: processing web pages with l2h will overwrite it.' > .latex2html-init
if ($status) goto errsys
if !(-f .latex2html-init) goto errsys
echo '# Instead edit settings.pl; processing web pages with l2h will put that here.' >> .latex2html-init
if ($status) goto errsys
cat settings.pl >> .latex2html-init
if ($status) goto errsys

# get the \webpages strings out of index.tex and convert them to perl
clearold.bat y @system/webpages.pl
if ($status) goto err
echo '# \webpages strings from the tex source(s):' > @system/webpages.pl
if ($status) goto errsys
if !(-f @system/webpages.pl) goto errsys
source "@system/texfiles.bat" "$L2HTOP/system-files/bin/getwebpages.bat" @system/webpages.pl
if ($status) goto err
cat @system/webpages.pl >> .latex2html-init
if ($status) goto errsys
echo '$RUNNING_TITLE = "$TITLE";' >> .latex2html-init
if ($status) goto errsys

# add the web page color in @transcol.txt to .latex2html-init as pagecolor
clearold.bat y @getpgc.pl
if ($status) goto err
"$L2HTOP/system-files/bin/getpgc.exe"
if ($status) goto errpcol
if !(-f @getpgc.pl) goto errpcol
cat @getpgc.pl >> .latex2html-init
if ($status) goto errsys
movenew.bat @getpgc.pl @system/@getpgc.pl y
if ($status) goto err

# implement the web page names
if !(-f index.pl) goto end_nams
l2hnewer.exe index.tex index.pl
set stat="$status"
if ($stat >= 2) goto errdat
if ($stat >= 1) goto del_nams
cat "$L2HTOP/system-files/data/htmlnames1.pl" >> .latex2html-init
if ($status) goto errsys
cat index.pl >> .latex2html-init
if ($status) goto errsys
cat "$L2HTOP/system-files/data/htmlnames2.pl" >> .latex2html-init
if ($status) goto errsys
goto end_nams
del_nams:
echo 'Note: deleting expired web page names file index.pl'
clearold.bat y index.pl
if ($status) goto err
end_nams:

# close .latex2html-init
echo '1; # This must be the last line' >> .latex2html-init
if ($status) goto errsys

# create the web-pages folder if it does not yet exist
if !(-e web-pages) mkdir web-pages
if ($status) goto errsys
if !(-d web-pages/) goto errsys

# copy index.css and settings.pl into it
copynew.bat index.css web-pages/index.css c y
if ($status) goto err
chmod a+r web-pages/index.css
if ($status) goto errsys
copynew.bat settings.pl web-pages/settings.pl c y
if ($status) goto err

# move the transparant color and the font size into web-pages
movenew.bat @fontsize.txt web-pages/@fontsize.txt y
if ($status) goto err
movenew.bat @transcol.txt web-pages/@transcol.txt y
if ($status) goto err

# change index.html using the \webpages definitions in @system/webpages.pl
if !(-f index.html) goto endhidx
if !(-f @system/webpages.pl) goto endhidx
clearold.bat y @system/indexhtml.sub
if ($status) goto err
subbin.exe "$L2HTOP/system-files/data/htmlidx.sub" @system/webpages.pl @system/indexhtml.sub
if ($status) goto errhidx
if !(-f @system/indexhtml.sub) goto endhidx
subbin.exe @system/indexhtml.sub index.html index@new.html
if ($status) goto errhidx
if !(-f index@new.html) goto endhidx
movenew.bat index@new.html index.html y
if ($status) goto err
endhidx:

# remove any existing buttons
source "$L2HTOP/system-files/bin/buttonsall.bat" "$L2HTOP/system-files/bin/remicon.bat"

# remove backups, which become useless
cd web-pages
"$L2HTOP/system-files/bin/delbcksets.bat" "@htmllist" '*'
cd ..

# set the browser to style_a (used for image alignments in l2hcrop.exe)
setenv L2H_BROWSER 'a'

# if L2H_JPGQ is set, or there are jpgs, create savjpegs, put the jpgs in it
"$L2HTOP/system-files/bin/savjpegs.bat" web-pages
if ($status) goto errsjp

# Finally, process index.tex with LaTeX2HTML
clearold.bat y @l2h.err web-pages/latexl2h.log web-pages/dvipsl2h.log web-pages/ppmquant.log web-pages/images.log
if ($status) goto err
echo 'This file is an error marker to be removed by latex2html.' > @l2h.err
if !(-f @l2h.err) goto errsys
echo 'Now running:'
echo "$L2HTOP/bin/latex2html $1 $2 $3 $4 $5 $6 $7 $8 $9 index"
"$L2HTOP/bin/latex2html" $1 $2 $3 $4 $5 $6 $7 $8 $9 index
if ($status) goto badl2h
set errlev='0'
goto endl2h
badl2h:
echo '*** latex2html returned an error status\!'
choose.exe anykey "Hit any key to acknowledge the above"
set errlev='1'
endl2h:
if !(-f @l2h.err) goto l2hfin
echo '*** latex2html did not finish successfully\!'
choose.exe anykey "Hit any key to acknowledge the above"
set errlev='1'
l2hfin:

# swap in the desired jpg files
"$L2HTOP/system-files/bin/projpegs.bat" web-pages
if ($status) goto errpjp

# set protection
chmod u=rwx,go=rx web-pages
chmod a+r,go-w web-pages/*
if (-d web-pages/keep/) chmod a+x,go-w web-pages/keep
chmod a+x,go-w .
if (-f index.html) chmod u=rw,go=r index.html

# warn about problems
if !(-f web-pages/latexl2h.log) choose.exe anykey "No latexl2h log file produced??  Hit any key"
if !(-f web-pages/latexl2h.log) goto wrnrun1a
echo
cat web-pages/latexl2h.log | fgrep '**'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto wrnrun1
echo
choose.exe anykey "After noting the errors above in latexl2h.log, hit any key"
set errlev='1'
wrnrun1:
if !(-f web-pages/dvipsl2h.log) choose.exe anykey "No dvipsl2h log file produced??  Hit any key"
wrnrun1a:
if !(-f web-pages/dvipsl2h.log) goto wrnrun1b
echo
cat web-pages/dvipsl2h.log | fgrep '**'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto wrnrun1b
echo
choose.exe anykey "After noting the errors above in dvipsl2h.log, hit any key"
set errlev='1'
wrnrun1b:
if !(-f web-pages/ppmquant.log) goto wrnrun2
echo
cat web-pages/ppmquant.log | fgrep '**'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto wrnrun2
echo
choose.exe anykey "After noting the errors above in ppmquant.log, hit any key"
set errlev='1'
wrnrun2:
if !(-f web-pages/images.log) goto wrnrun3
echo
# the space between / and the quote is needed in DOS for some reason
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " web-pages/images.log
if ($status) goto errrl
if !(-f web-pages/images.@rl) goto errrl
cat web-pages/images.@rl | fgrep 'Errors: 0' > @system/@tmp.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto badlog
cat web-pages/images.@rl | fgrep 'Major warnings: 0' > @system/@tmp.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto badlog
goto wrnrun3
badlog:
echo
echo 'Note: You may want to check out errors in images.log more carefully.'
echo 'Both images.log and the corresponding images.tex can be found in the'
echo 'web-pages folder.  Together they can tell you what image(s) might'
echo 'have problems.'
echo
choose.exe anykey "After noting the above problems in web-pages/images.log, hit any key"
set errlev='1'
wrnrun3:
if ("$errlev" != 0) goto done

echo '========================================================================'
echo '                       Web pages created'
echo '========================================================================'

# done
done:
if (-f @system/in-mkwp.txt) rm @system/in-mkwp.txt
goto end

# errors
errtex:
pwd
echo '*** There is no file index.tex in the current folder, as listed above.'
echo '    So no web pages can be made.'
goto err
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errfils:
echo '*** Fatal error: essential files are missing from the l2h folder.'
echo '    (Or you may not have enough privileges to access them.)'
echo '    You will need to restore them from the original zip file.'
goto err
errdat:
echo '*** Unknown error comparing the dates of files.'
goto err
errretex:
echo '*** Please first run latex on index.tex.  (Index.aux is out of date.)'
echo '    Then try to make web pages again.'
goto err
errtemp:
echo '*** Temporary folder "temp" inside l2h is not usable.  This folder'
echo '    must exist to make web pages, and you must have permission to'
echo '    create and remove files in it.  Please fix, or run l2h as an'
echo '    administrator, or re-install l2h from scratch as yourself.'
goto err
errperl:
echo '^G*** Perl is not in your path.  Please install it first.'
goto err
errperl2:
echo '^G*** There is a problem with perl in the latex2html files.'
echo '    The files latex2html, pstoimg, and texexpand in the "bin"'
echo '    subfolder of l2h do not begin as'
echo '        #\! '"`which perl`"
echo '    Please fix this and then try again.  (Rerun install_l2h).'
goto err
errperl3:
echo '^G*** There is a problem with perl in file l2hconf.pm in the l2h folder.'
echo '    This file is supposed to define'
echo '       $PERL = '"'`which perl`';"
echo '    Please fix this and then try again.  (Rerun install_l2h).'
goto err
errbat:
echo '*** Problem processing the list of files that index.tex depends on.'
echo '      Check for unusual \input, \include, or \bibliography commands.'
goto err
errchk:
echo '*** Please fix the latex2html problem spots shown above.'
echo '    Then try running latex and making web pages again.'
goto err
errwps:
echo '*** Error running script makewp_start.bat.'
goto err
errtcol:
echo '*** Unable to extract the BODY background color out of index.css.'
goto err
errfsiz:
echo '*** Unable to extract the BODY font size out of index.css.'
goto err
errpcol:
echo '*** Unable to convert the BODY background-color of index.css.'
goto err
errhidx:
echo '*** Unable to change the index.html headers and footers.'
goto err
errsjp:
echo '*** Error saving the old jpg files.'
goto err
errpjp:
echo '*** Error processing the jpg files.'
goto err
errrl:
echo '*** Error reading images.log in the web-pages subfolder.'
goto err
errq:
echo '*** Aborted by user.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
if (-f @system/in-mkwp.txt) rm @system/in-mkwp.txt
exit 1

end:
