#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Run the l2h menu.

# Usage
goto start
use:
echo 'Usage: menu.bat'
goto err
start:

# usage
if ("$1" == '?') goto use
if ("$1" == '/?') goto use
if ("$1" == '-h') goto use
if ("$1" == '--help') goto use

# print a message
echo 'Starting the l2h menu...'

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# check the basic files needed for execution
if !(-frx "$L2HTOP/bin/filechk.bat") goto errfils
"$L2HTOP/bin/filechk.bat"
if ($status) goto errfils
if !(-frx "$L2HTOP/system-files/bin/filechk.bat") goto errfils
"$L2HTOP/system-files/bin/filechk.bat"
if ($status) goto errfils
if !(-frx "$L2HTOP/system-files/data/filechk.bat") goto errfils
"$L2HTOP/system-files/data/filechk.bat"
if ($status) goto errfils
if !(-frx "$L2HTOP/system-files/wiz/filechk.bat") goto errfils
"$L2HTOP/system-files/wiz/filechk.bat"
if ($status) goto errfils
if !(-fr "$L2HTOP/texinputs/frames.sty") goto errfils
if !(-fr "$L2HTOP/texinputs/html.sty") goto errfils
if !(-frx "$L2HTOP/themes/white-blue/filechk.bat") goto errfils
"$L2HTOP/themes/white-blue/filechk.bat"
if ($status) goto errfils

# check whether we are already active
if !(-f @system/in-use.txt) goto nouse
echo '*** It appears that the l2h menu is already in use for this document.'
echo '    Only one copy of the menu can run at any time.'
echo '    (If you are sure only one copy *is* running, hit the 1 key now.)'
"$L2HTOP/bin/choose.exe" q1 "Hit q to quit: q"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto rmuse
goto err
rmuse:
rm @system/in-use.txt
if ($status) goto errsys
if (-e @system/in-use.txt) goto errsys
nouse:

# run menu_start if it exists
set p2popt=''
if !(-f menu_start.bat) goto nomstart
source menu_start.bat
if ($status) goto errmstart
nomstart:

# check that index.tex exists
if (-f index.tex) goto havidx
askexamp:
"$L2HTOP/system-files/bin/getexample.bat"
if ($status) goto err
if !(-f index.tex) goto errsys
havidx:

# initialize
source "$L2HTOP/system-files/bin/doinit.bat"
if ($status) goto err

# mark as active
echo 'l2h is in use' > @system/in-use.txt
if !(-f @system/in-use.txt) goto errsys

# initialize the texfiles list
source "$L2HTOP/system-files/bin/checknewer.bat" log
if ($status) goto err
if ("$errlev" != 0) goto iniend
# sanity check for *equal* ages
l2hnewer.exe index.log index.tex
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto san2
goto inidel
# sanity check for index.aux
san2:
l2hnewer.exe index.tex index.aux
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto inidel
l2hnewer.exe index.aux index.tex
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto iniend
inidel:
if (-f index.log) clearold.bat y index.log
if ($status) goto err
iniend:

# offer to open index.tex in a text editor
if (-f @system/open_index.no) goto editend
"$L2HTOP/system-files/bin/doedit.bat" N
editend:

# set the initial user menu answer to 0
set ans='0'

# initialize the ps and pdf file status variables
set ss='o'
set sp='o'

# write the appropiate menu
showmenu:
if !(-f index.tex) goto errgone

# see whether we need to recreate the file list
l2hnewer.exe index.tex @system/texfiles.lst
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto dochfl
l2hnewer.exe @system/texfiles.lst @system/texfiles.bat
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto dochfl

# if not, look whether the log is current
if !(-f index.log) goto nolog
set errlev='0'
source "@system/texfiles.bat" "$L2HTOP/system-files/bin/newer.bat" index.log
if ($status) goto errdat
if ("$errlev" != 0) goto nolog
goto rl

# recreate the file list and see whether the log is current
dochfl:
set errlev='0'
source "$L2HTOP/system-files/bin/checknewer.bat" log
if ($status) goto err
if ("$errlev" != 0) goto nolog
goto rl

# there is no log file to look at
nolog:
set log='4'
clear
echo ' ==================================== MENU ===================================='
echo
echo
echo
echo
echo '                     There is no up-to-date index.log file'
echo '                             Press l to create one'
echo
echo
echo
echo ' ------------------------------------------------------------------------------'
goto showkeys

# run readlog.exe
rl:
clear
echo ' ==================================== MENU ===================================='
if ("$ans" == f) goto dorlf
if ("$ans" == b) goto dorlb
if ("$ans" == n) goto dorln
if ("$ans" == d) goto dorld
if ("$ans" == t) goto dorlt
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log same -10
if ($status) goto errrl
goto didrl
dorlf:
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log ini -10
if ($status) goto errrl
goto didrl
dorlb:
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log prev -10
if ($status) goto errrl
goto didrl
dorln:
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log next -10
if ($status) goto errrl
goto didrl
dorld:
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log nextdiff -10
if ($status) goto errrl
goto didrl
dorlt:
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log nexttype -10
if ($status) goto errrl
goto didrl
didrl:
if !(-f index.@rl) goto errrl

# find the error status
set log='3'
cat index.@rl | fgrep 'Errors: 0' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto havlev
set log='2'
cat index.@rl | fgrep 'Major warnings: 0' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto havlev
set log='1'
cat index.@rl | fgrep 'Minor warnings: 0' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto havlev
set log='0'
havlev:

# show the menu keys
showkeys:
set keys='q 2aPtpdSnsblf\Hh'
# h and H
if ("$log" == 0) goto show_hn
if ("$log" == 4) goto show_hn
goto show_hy
show_hn:
echo ' -- Load help on the above problem     H: Load general help in the web browser.'
set keys='Hq'
goto show_he
show_hy:
echo ' h: Load help on the above problem     H: Load general help in the web browser.'
set keys='Hh'
show_he:
# nil and \
echo '    in the default web browser.        \: Load index.tex in an editor.'
set keys='\'"$keys"
# f
if ("$log" == 0) goto show_fn
if ("$log" == 4) goto show_fn
cat index.@rl | fgrep 'Selected problem: 1 ' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto show_fy
show_fn:
echon.exe " -- Show first problem.               *"
set keys="q$keys"
goto show_fe
show_fy:
echon.exe " f: Show first problem.               *"
set keys="f$keys"
show_fe:
# l
if ("$log" == 4) goto show_ly
echo ' l: Run latex (after changing index.tex).'
goto show_le
show_ly:
echo ' l: Run latex (or /pdfengine if set).'
show_le:
set keys="l$keys"
# b
if ("$log" == 0) goto show_bn
if ("$log" == 4) goto show_bn
cat index.@rl | fgrep 'Selected problem: 1 ' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto show_by
show_bn:
echon.exe " -- Show one problem back.            *"
set keys="q$keys"
goto show_be
show_by:
echon.exe " b: Show one problem back.            *"
set keys="b$keys"
show_be:
# s
if !(-f index.ps) goto show_sy
set errlev='0'
source "@system/texfiles.bat" "$L2HTOP/system-files/bin/newer.bat" index.ps q
if ($status) goto errdat
if ("$errlev" != 0) goto show_sy
echo ' s: (after changing index.tex and saving)'
set ss='n'
goto show_se
show_sy:
echo ' s: Try to create index.ps.'
set ss='o'
show_se:
set keys="s$keys"
# n
if ("$log" == 0) goto show_nn
if ("$log" == 4) goto show_nn
cat index.@rl | fgrep 'Problems left: 0' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto show_ny
show_nn:
echon.exe " -- Show next problem.                *"
set keys="q$keys"
goto show_ne
show_ny:
echon.exe " n: Show next problem.                *"
set keys="n$keys"
show_ne:
# S
if (-f index.ps) goto show_csy
echo ' -- Load index.ps in the default viewer.'
set keys="q$keys"
goto show_cse
show_csy:
if ("$ss" == o) echo ' S: Load OLD index.ps in default viewer.'
if ("$ss" != o) echo ' S: Load index.ps in the default viewer.'
set keys="S$keys"
show_cse:
# d
if ("$log" == 0) goto show_dn
if ("$log" == 4) goto show_dn
cat index.@rl | fgrep 'Problems left: 0' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto show_dy
show_dn:
echon.exe " -- Show next different problem.      *"
set keys="q$keys"
goto show_de
show_dy:
echon.exe " d: Show next different problem.      *"
set keys="d$keys"
show_de:
# p
if !(-f index.pdf) goto show_py
set errlev='0'
source "@system/texfiles.bat" "$L2HTOP/system-files/bin/newer.bat" index.pdf q
if ($status) goto errdat
if ("$errlev" != 0) goto show_py
echo ' p: (after changing index.tex and saving)'
set sp='n'
goto show_pe
show_py:
echo ' p: Try to create index.pdf.'
set sp='o'
show_pe:
set keys="p$keys"
# t
if ("$log" == 0) goto show_tn
if ("$log" == 4) goto show_tn
cat index.@rl | fgrep 'Problems left: 0' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto show_ty
show_tn:
echon.exe " -- Show next type of problem.        *"
set keys="q$keys"
goto show_te
show_ty:
echon.exe " t: Show next type of problem.        *"
set keys="t$keys"
show_te:
# P
if (-f index.pdf) goto show_cpy
echo ' -- Load index.pdf in the default viewer.'
set keys="q$keys"
goto show_cpe
show_cpy:
if ("$sp" == o) echo ' P: Load OLD index.pdf in default viewer.'
if ("$sp" != o) echo ' P: Load index.pdf in the default viewer.'
set keys="P$keys"
show_cpe:
# a and 2
if ("$log" == 0) goto show_an
if ("$log" == 4) goto show_an
goto show_ay
show_an:
echo ' -- Show all problems concisely.       2: Menu 2 (make web pages, etc.)'
set keys="2q$keys"
goto show_ae
show_ay:
echo ' a: Show all problems concisely.       2: Menu 2 (make web pages, etc.)'
set keys="2a$keys"
show_ae:
# Space and q
if ("$log" != 4) goto show__y
echo ' -- Full log using keys Space and q.   q: Quit l2h.            -: refresh menu.'
set keys="qq$keys"
goto show__e
show__y:
echo '  : Full log using keys Space and q.   q: Quit l2h.            -: refresh menu.'
set keys="q $keys"
show__e:

# show a recommendation
if ("$log" == 4) goto rec4
if ("$log" == 3) goto rec23
if ("$log" == 2) goto rec23
if ("$log" == 1) goto rec1
goto rec0
rec4:
echo ' RECOMMENDATION: There is no up-to-date log file.  You need to hit l to'
echo ' create one.  Without a log, l2h cannot say anything else sensibly.'
echo
set ans='l'
goto askkey
rec23:
echo ' RECOMMENDATION: File index.tex has problems.  Load it in a text editor if you'
echo ' did not already.  Try to fix the first problem and save index.tex. Then rerun'
echo ' latex with key l.  (Or, to look at the results, try keys p, P, s, and/or S.)'
set ans='l'
goto askkey
rec1:
echo ' RECOMMENDATION: The file index.tex (as saved) seems to have only minor issues.'
if ("$sp" == n) goto rec1a
echo ' You could create a pdf using key p and see whether they are worth fixing.'
set ans='p'
goto rec1b
rec1a:
echo ' Check the pdf whether they are worth fixing.  Or press 2 to make webpages.'
set ans='2'
rec1b:
echo ' (To fix a problem, edit index.tex, fix and save, and rerun latex with l.)'
goto askkey
rec0:
echo ' RECOMMENDATION: The file index.tex (as saved) seems to have no problems.'
if ("$sp" == n) goto rec0a
echo ' You could create a pdf using key p and see whether everything is as you want.'
set ans='p'
goto rec0b
rec0a:
echo ' Check the pdf, maybe using key P, and see whether everything is as you want.'
set ans='2'
rec0b:
echo ' Or press 2 to go to the next menu, the one that allows you to make web pages.'
goto askkey

# ask for the menu choice
askkey:
choose.exe "$keys-\!" "Make a choice from the menu above and then press its key: $ans"
set stat="$status"
if ($stat >= 20) goto errsys
if ($stat >= 19) goto docmd
if ($stat >= 18) goto showmenu
if ($stat >= 17) goto mnu_h
if ($stat >= 16) goto mnu_ch
if ($stat >= 15) goto mnu_bs
if ($stat >= 14) goto mnu_f
if ($stat >= 13) goto mnu_l
if ($stat >= 12) goto mnu_b
if ($stat >= 11) goto mnu_s
if ($stat >= 10) goto mnu_n
if ($stat >= 9) goto mnu_cs
if ($stat >= 8) goto mnu_d
if ($stat >= 7) goto mnu_p
if ($stat >= 6) goto mnu_t
if ($stat >= 5) goto mnu_cp
if ($stat >= 4) goto mnu_a
if ($stat >= 3) goto mn2
if ($stat >= 2) goto mnu__
if ($stat >= 1) goto done
goto errq

# execute a command
docmd:
echo -n 'Shell command: '
set ans="$<"
if ("$ans" == "") goto docmde
if ("$ans" == "q") goto docmde
$ans
choose.exe anykey "Press any key"
docmde:
set ans='-'
goto showmenu

# h
mnu_h:
set ans='h'
if !(-f index.@rl) goto mnu_hen
clearold.bat y @system/tmp.tmp @system/tmp.html
if ($status) goto err
echo "$L2HTOP/help-files/" > @system/tmp.tmp
if ($status) goto errsys
if !(-f @system/tmp.tmp) goto errsys
cat index.@rl | fgrep '.html#' >> @system/tmp.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_hen
subbin.exe "$L2HTOP/system-files/data/mkreloc.sub" @system/tmp.tmp @system/tmp.html
if ($status) goto mnu_hep
if !(-f @system/tmp.html) goto errsys
xdg-open "@system/tmp.html"
if ($status) goto mnu_hel
goto showmenu
mnu_hen:
echo '** Cannot find a help page for the current problem in index.@rl.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_hep:
echo '** An error occurred while trying to create a pointer to the help page.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_hel:
echo '** An error occurred while trying to load the web page.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu

# H
mnu_ch:
set ans='H'
xdg-open "$L2HTOP/help-files/intro_to_latex.html"
if ($status) choose.exe anykey "** An error occurred trying to load the web page. Hit any key to acknowledge"
goto showmenu

# \
mnu_bs:
set ans='/'
if !(-f index.tex) goto errgone
"$L2HTOP/system-files/bin/doedit.bat"
if ($status) choose.exe anykey "** Hit any key to return to the l2h menu"
goto showmenu

# f
mnu_f:
set ans='f'
goto showmenu

# l
mnu_l:
set ans='l'
if !(-f index.tex) goto errgone
# update pdfengine and see whether a current log already exists
source "$L2HTOP/system-files/bin/checknewer.bat" log
if ($status) goto err
if ("$errlev" != 0) goto mnu_l0
echo
echo '** An up-to-date log file index.log already exists.  (Did you forget to'
echo '   save index.tex?  If so, save now before hitting Enter.)'
choose.exe cq "   Hit c to continue or q to abort running latex: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto showmenu
if ($stat >= 1) goto mnu_l0
goto errq
# jump-to point for having latex run
mnu_l0:
# find the pdf engine
echo
echo 'Finding the pdf engine...'
setenv pe 'latex'
source "@system/setpe.bat"
if ("$pe" == '') goto mnu_leb
set pe1=`echo "$pe" | cut -d' ' -f1`
which "$pe1"
if ($status) goto mnu_lep
# create a temporary file with the current date
clearold.bat y @system/tmp.tmp
if ($status) goto err
echo 'This is a marker for checking file dates.' > @system/tmp.tmp
if !(-f @system/tmp.tmp) goto errsys
# run the pdf engine
clearold.bat y index.log
if ($status) goto err
echo
echo "RUNNING: $pe --interaction=nonstopmode index..."
$pe --interaction=nonstopmode index
if ($status) echo '** An error occurred.'
if !(-f index.log) goto mnu_lel
# check that an aux file was created
l2hnewer.exe @system/tmp.tmp index.aux
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_lea
# set a rerun status
set ss='0'
# apply makeindex if an idx file was created
if !(-f index.idx) goto mnu_l1e
l2hnewer.exe @system/tmp.tmp index.idx
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_l1e
clearold.bat y @system/index.ind
if ($status) goto err
if !(-f index.ind) goto mnu_l1a
movenew.bat index.ind @system/index.ind y
if ($status) goto err
mnu_l1a:
which makeindex
if ($status) goto errmki
echo
echo 'RUNNING: makeindex index...'
makeindex index
if ($status) choose.exe anykey "** Error running makeindex.  Hit any key to acknowledge"
if !(-f index.ind) goto mnu_l1e
if !(-f @system/index.ind) goto mnu_l1b
cmp -s @system/index.ind index.ind
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_l1b
goto mnu_l1e
mnu_l1b:
set ss='1'
mnu_l1e:
# run bibtex
cat @system/texfiles.lst | fgrep ' run bibtex ' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_l2e
clearold.bat y @system/index.bbl
if ($status) goto err
if !(-f index.bbl) goto mnu_l2a
movenew.bat index.bbl @system/index.bbl y
if ($status) goto err
mnu_l2a:
echo
echo 'RUNNING: bibtex index...'
which bibtex
if ($status) goto errbib
bibtex index
if ($status) choose.exe anykey "** Error running bibtex.  Hit any key to acknowledge"
if ("$ss" == 1) goto mnu_l2e
if !(-f index.bbl) goto mnu_l2e
if !(-f @system/index.bbl) goto mnu_l2b
cmp -s @system/index.bbl index.bbl
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_l2b
goto mnu_l2e
mnu_l2b:
set ss='1'
mnu_l2e:
# rerun latex up to three times
if ("$ss" != 0) goto mnu_l3a
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log all
if ($status) goto errrl
if !(-f index.@rl) goto errrl
cat index.@rl | fgrep 'Rerun latex' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_l3e
mnu_l3a:
clearold.bat y index.log
if ($status) goto err
echo
echo "RUNNING: $pe --interaction=nonstopmode index..."
$pe --interaction=nonstopmode index
if ($status) echo '** An error occurred.'
if !(-f index.log) goto mnu_lel
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log all
if ($status) goto errrl
if !(-f index.@rl) goto errrl
cat index.@rl | fgrep 'Rerun latex' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_l3e
clearold.bat y index.log
if ($status) goto err
echo
echo "RUNNING: $pe --interaction=nonstopmode index..."
$pe --interaction=nonstopmode index
if ($status) echo '** An error occurred.'
if !(-f index.log) goto mnu_lel
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log all
if ($status) goto errrl
if !(-f index.@rl) goto errrl
cat index.@rl | fgrep 'Rerun latex' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_l3e
clearold.bat y index.log
if ($status) goto err
echo
echo "RUNNING: $pe --interaction=nonstopmode index..."
$pe --interaction=nonstopmode index
if ($status) echo '** An error occurred.'
if !(-f index.log) goto mnu_lel
mnu_l3e:
# done running latex
if !(-f index.pdf) goto mnu_l3ee
chmod u=rw,go=r index.pdf
if ($status) goto errsys
mnu_l3ee:
if ("$ans" == s) goto mnu_srt
if ("$ans" == p) goto mnu_prt
goto showmenu
mnu_leb:
echo '** The pdfengine is blank.  Please fix this in index.tex and save.'
echo '   Then try again.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_lep:
echo '** The pdfengine'
echo "      $pe"
echo '   (or the word up to the first space if any) is not in your path, so'
echo '   should not work.  Fix this in index.tex and save.  Then try again.'
echo '   If necessary, quit and try rerunning install_l2h.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_lel:
echo '** Running latex did not produce a log file index.log.  Without it,'
echo '   there is nothing l2h can do.  Check that /pdfengine is correctly'
echo '   set and in your path, or if not set, that latex is in your path.'
echo '   If necessary, quit and try rerunning install_l2h.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_lea:
echo '** Running latex did not produce a file index.aux.  That may mean that'
echo '   you are missing a needed package, or that something else is wrong'
echo '   right at the start of document index.tex.  In any case, further'
echo '   processing of index.tex has to be aborted until the problem is fixed.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu

# b
mnu_b:
set ans='b'
goto showmenu

# s
mnu_s:
set ans='s'
if !(-f index.tex) goto errgone
# see whether a current ps file already exists
source "$L2HTOP/system-files/bin/checknewer.bat" ps
if ($status) goto err
if ("$errlev" != 0) goto mnu_s0
echo
echo '** An up-to-date postscript file index.ps already exists.  (Did you'
echo '   forget to save index.tex?  If so, save now before hitting Enter.)'
choose.exe cq "   Hit c to continue or q to abort recreating index.ps: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto showmenu
if ($stat >= 1) goto mnu_s0
goto errq
# jump-to-point for having a postscript file made
mnu_s0:
# see whether we already have a dvi or pdf file
if !(-f index.dvi) goto mnu_s1a
set errlev='0'
source "@system/texfiles.bat" "$L2HTOP/system-files/bin/newer.bat" index.dvi
if ($status) goto errdat
if ("$errlev" == 0) goto mnu_sd
mnu_s1a:
if !(-f index.pdf) goto mnu_s1b
set errlev='0'
source "@system/texfiles.bat" "$L2HTOP/system-files/bin/newer.bat" index.pdf
if ($status) goto errdat
if ("$errlev" == 0) goto mnu_sp
mnu_s1b:
# go run latex to create a dvi or pdf file
goto mnu_l0
mnu_srt:
# see whether we have a dvi or pdf file now
if !(-f index.dvi) goto mnu_s2a
l2hnewer.exe @system/tmp.tmp index.dvi
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_s2a
goto mnu_sd
mnu_s2a:
if !(-f index.pdf) goto mnu_s2b
l2hnewer.exe @system/tmp.tmp index.pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_s2b
goto mnu_sp
mnu_s2b:
goto mnu_ses
# convert the dvi file to ps using dvips
mnu_sd:
which dvips
if ($status) goto errdvi
echo 'RUNNING: dvips -o index.ps index'
dvips -o index.ps index
if ($status) choose.exe anykey "** An error occurred running dvips.  Hit any key to acknowledge"
l2hnewer.exe index.dvi index.ps
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) choose.exe anykey "** Creation of index.ps by dvips failed.  Hit any key to acknowledge"
goto showmenu
# convert the pdf file to ps using pdftops
mnu_sp:
which pdftops
if ($status) goto errpp
echo 'RUNNING: pdftops index.pdf'
pdftops index.pdf
if ($status) choose.exe anykey "** An error occurred running pdftops.  Hit any key to acknowledge"
l2hnewer.exe index.pdf index.ps
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) choose.exe anykey "** Creation of index.ps by pdftops failed.  Hit any key to acknowledge"
goto showmenu
mnu_ses:
echo '** Pdfengine'
echo "      $pe"
echo '   produced no output that can be used to create postscript file'
echo '   index.ps.  Fix any problems in file index.tex and then try again.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu

# n
mnu_n:
set ans='n'
goto showmenu

# S
mnu_cs:
set ans='S'
if !(-f index.ps) goto mnu_csee
xdg-open index.ps
if ($status) goto mnu_csel
goto showmenu
mnu_csee:
echo '** Postscript file index.ps is gone.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_csel:
echo '** An error occurred trying to load Postscript file index.ps.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu

# d
mnu_d:
set ans='d'
goto showmenu

# p
mnu_p:
set ans='p'
if !(-f index.tex) goto errgone
# see whether a current pdf file already exists
source "$L2HTOP/system-files/bin/checknewer.bat" pdf
if ($status) goto err
if ("$errlev" != 0) goto mnu_p0
echo
echo '** An up-to-date pdf file index.pdf already exists.  (Did you forget to'
echo '   save index.tex?  If so, save now before hitting Enter.)'
echo ' r: Remake it.'
echo ' o: Do not remake index.pdf, but optimize it for the web with pdfopt'
echo ' q: Quit'
choose.exe roq "Make a choice from the menu above and press its key: r"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto showmenu
if ($stat >= 2) goto mnu_po
if ($stat >= 1) goto mnu_p0
goto errq
# optimize the existing pdf
mnu_po:
which pdfopt
if ($status) goto mnu_peom
clearold.bat y @tmp.pdf
if ($status) goto err
pdfopt index.pdf @tmp.pdf
if ($status) goto mnu_peof
if !(-f @tmp.pdf) goto mnu_peof
movenew.bat @tmp.pdf index.pdf y
if ($status) goto err
chmod u=rw,go=r index.pdf
if ($status) goto errsys
choose.exe anykey "Seems to have optimized OK.  Hit any key"
goto showmenu
# jump-to-point for having a pdf file made
mnu_p0:
# see whether we already have a dvi file
if !(-f index.dvi) goto mnu_p1
set errlev='0'
source "@system/texfiles.bat" "$L2HTOP/system-files/bin/newer.bat" index.dvi
if ($status) goto errdat
if ("$errlev" == 0) goto mnu_pd
mnu_p1:
# go run latex to create a dvi or pdf file
goto mnu_l0
mnu_prt:
# see whether we have a dvi or pdf file now
if !(-f index.dvi) goto mnu_p2a
l2hnewer.exe @system/tmp.tmp index.dvi
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_p2a
goto mnu_pd
mnu_p2a:
if !(-f index.pdf) goto mnu_p2b
l2hnewer.exe @system/tmp.tmp index.pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_p2b
goto showmenu
mnu_p2b:
goto mnu_pes
# convert the dvi file to pdf using dvips and ps2pdf
mnu_pd:
which dvips
if ($status) goto errdvi
echo 'RUNNING: dvips -Ppdf -o index.ps index'
dvips -Ppdf -o index.ps index
if ($status) choose.exe anykey "** An error occurred running dvips.  Hit any key to acknowledge"
l2hnewer.exe index.dvi index.ps
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_ped
which ps2pdf
if ($status) goto errp2p
echo "RUNNING: ps2pdf $p2popt index.ps"
ps2pdf $p2popt index.ps
if ($status) choose.exe anykey "** An error occurred running ps2pdf.  Hit any key to acknowledge"
l2hnewer.exe index.ps index.pdf
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mnu_pepp
chmod u=rw,go=r index.pdf
if ($status) goto errsys
goto showmenu
mnu_peom:
echo '** Cannot optimize; the ghostscript script pdfopt is not in your path.'
echo '   (Note: if your /pdfengine is plain latex, and your ghostscript is'
echo '   GPL dating sep 2012 or later, you can try to turn on optimization'
echo '   by editing file menu_start.bat with a text editor.)'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_peof:
echo '** Error optimizing with ghostscript script pdfopt.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_pes:
echo '** Pdfengine'
echo "      $pe"
echo '   produced no output that can be used to create pdf file index.pdf.'
echo '   Fix any problems in file index.tex and then try again.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_ped:
echo '** In the DVI-PS-PDF chain, dvips failed.  So no pdf could be produced.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_pepp:
echo '** Creation of index.pdf by ps2pdf failed.  No pdf was produced.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu

# t
mnu_t:
set ans='t'
goto showmenu

# P
mnu_cp:
set ans='P'
if !(-f index.pdf) goto mnu_cpee
xdg-open index.pdf
if ($status) goto mnu_cpel
goto showmenu
mnu_cpee:
echo '** Pdf file index.pdf is gone.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu
mnu_cpel:
echo '** An error occurred trying to load pdf file index.pdf.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu

# a
mnu_a:
set ans='a'
if !(-f index.log) goto mnu_ael
clear
echo
"$L2HTOP/system-files/bin/readlog.exe" - "$L2HTOP/system-files/data/ " index.log all 24
choose.exe anykey "Hit any key to return to the l2h menu"
goto showmenu
mnu_ael:
echo '** Log file index.log is gone.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu

# Space
mnu__:
set ans='_'
if !(-f index.log) goto mnu__el
clear
more index.log
choose.exe anykey "Hit any key to return to the l2h menu"
goto showmenu
mnu__el:
echo '** Log file index.log is gone.'
choose.exe anykey "   Hit any key to acknowledge"
goto showmenu


# ***************************** Web pages menu ******************************
mn2:
if !(-f index.tex) goto errgone

# write the top line
clear
echo ' =================================== MENU 2 ==================================='

# show the menu keys
set keys='q1UTSsIFCBufHhjWwxRr'
# r
echo ' r: Remove latex from index.tex, producing index@hide.html and index@lost.html'
set keys='r'
# R
if !(-f index@hide.html) goto shw2_crn
if !(-f index@save.sav) goto shw2_crn
if !(-f index@save.enc) goto shw2_crn
goto shw2_cry
shw2_crn:
echo ' -- Restore latex to index@hide.html, thus creating an updated index.tex file.'
set keys="q$keys"
goto shw2_cre
shw2_cry:
set keys="R$keys"
l2hnewer.exe index.tex index@hide.html
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto shw2_cro
l2hnewer.exe index@hide.html index@save.enc
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto shw2_crc
shw2_cro:
echo '(R) Restore latex to index@hide.html, thus creating an updated index.tex file.'
goto shw2_cre
shw2_crc:
echo ' R: Restore latex to index@hide.html, thus creating an updated index.tex file.'
shw2_cre:
# x
echo ' x: Undo the last R (or maybe x) command if any.'
set keys="x$keys"
#
echo
# echo w and W
if (-f web-pages/index.html) goto shw2_wy
echo ' w: Make web pages.'
echo ' -- Show the web pages in the web browser.'
set keys="qw$keys"
goto shw2_we
shw2_wy:
set keys="Ww$keys"
l2hnewer.exe index.tex web-pages/index.html
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto shw2_wy2
echo '(w) Remake web pages.  But the web pages seem to be newer than index.tex.'
echo ' W: Show the web pages in the web browser.'
goto shw2_we
shw2_wy2:
echo ' w: Remake web pages.  The web pages are older than index.tex.'
echo '(W) Show the web pages in the web browser.'
shw2_we:
#
echo
# j
if ($?L2H_JPGQ) goto shw2_jy
echo ' j: Change web page jpg generation setting.  (Currently: off)'
goto shw2_je
shw2_jy:
echo " j: Change web page jpg generation setting.  (Currently: on at quality $L2H_JPGQ)"
shw2_je:
set keys="j$keys"
# h and H
if (-f web-pages/index.html) goto shw2_hn2
echo ' -- Hyphenate web pages.  (No web pages.)'
set keys="qq$keys"
goto shw2_he
shw2_hn2:
cat web-pages/index.html | fgrep ' (Already l2h-hyphenated as: ' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto shw2_hn3
echon.exe " -- Hyphenate web pages. *"
cat web-pages/index.html | fgrep ' (Already l2h-hyphenated as: '
set keys="qq$keys"
goto shw2_he
shw2_hn3:
cat web-pages/index.html | fgrep ' (Already l2h-finalized.)' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto shw2_hn4
echo ' -- Hyphenate web pages.  (Already l2h-finalized.)'
set keys="qq$keys"
goto shw2_he
shw2_hn4:
if (-f @system/language.bat) goto shw2_hy
echo ' -- Hyphenate web pages.               H: Set the language to hyphenate first.'
set keys="Hq$keys"
goto shw2_he
shw2_hy:
echon.exe " h: Hyphenate web pages assuming they are in *"
source "@system/language.bat" e
echo '.  H: change'
set keys="Hh$keys"
shw2_he:
# f
if (-f web-pages/index.html) goto shw2_fn2
echo ' -- Finalize the web pages.  (No web pages.)'
set keys="q$keys"
goto shw2_fe
shw2_fn2:
cat web-pages/index.html | fgrep ' (Already l2h-finalized.)' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto shw2_fy
echo ' -- Finalize the web pages.  (Already l2h-finalized.)'
set keys="q$keys"
goto shw2_fe
shw2_fy:
echo ' f: Finalize the web pages, preventing bad line breaks and such.'
set keys="f$keys"
shw2_fe:
# u
echo ' u: Undo the last h or f (or maybe u) command if any.'
set keys="u$keys"
#
echo
# B C F I s S T
if !(-f index.css) goto shw2_cbn
if !(-f settings.pl) goto shw2_cbn
goto shw2_cby
shw2_cbn:
echo ' -- Change the web page buttons only.  (Not the entire theme.)'
echo ' -- Change the web page colors only.  (Not the entire theme.)'
echo ' -- Change the web page font sizes and/or figure sizes.'
echo ' -- Change individual style-sheet properties.  (Requires knowledge of html.)'
echo ' -- Change to serif web page fonts.    -- Change to sans-serif web page fonts.'
echo ' T: Select the web page theme.'
set keys="Tqqqqqq$keys"
goto shw2_cbe
shw2_cby:
echo ' B: Change the web page buttons only.  (Not the entire theme.)'
echo ' C: Change the web page colors only.  (Not the entire theme.)'
echo ' F: Change the web page font sizes and/or figure sizes.'
echo ' I: Change individual style-sheet properties.  (Requires knowledge of html.)'
echo ' s: Change to serif web page fonts.    S: Change to sans-serif web page fonts.'
echo ' T: Change the web page theme.'
set keys="TSsIFCB$keys"
shw2_cbe:
# U
echo ' U: Undo the last B, C, F, I, or T (or maybe U) command if any.'
set keys="U$keys"
#
echo
# 1
echo ' 1: Return to menu 1    c: Clean up    q: Quit l2h.            -: refresh menu.'
set keys="q1$keys"

# ask for the menu choice
echo
choose.exe "$keys-c\!" "Make a choice from the menu above and then press its key: 1"
set stat="$status"
if ($stat >= 24) goto errsys
if ($stat >= 23) goto docmd2
if ($stat >= 22) goto mn2_c
if ($stat >= 21) goto mn2
if ($stat >= 20) goto mn2_r
if ($stat >= 19) goto mn2_cr
if ($stat >= 18) goto mn2_x
if ($stat >= 17) goto mn2_w
if ($stat >= 16) goto mn2_cw
if ($stat >= 15) goto mn2_j
if ($stat >= 14) goto mn2_h
if ($stat >= 13) goto mn2_ch
if ($stat >= 12) goto mn2_f
if ($stat >= 11) goto mn2_u
if ($stat >= 10) goto mn2_cb
if ($stat >= 9) goto mn2_cc
if ($stat >= 8) goto mn2_cf
if ($stat >= 7) goto mn2_ci
if ($stat >= 6) goto mn2_s
if ($stat >= 5) goto mn2_cs
if ($stat >= 4) goto mn2_ct
if ($stat >= 3) goto mn2_cu
if ($stat >= 2) goto showmenu
if ($stat >= 1) goto done
goto errq

docmd2:
echo -n 'Shell command: '
set ans="$<"
if ("$ans" == "") goto docmd2e
if ("$ans" == "q") goto docmd2e
$ans
choose.exe anykey "Press any key"
docmd2e:
set ans='-'
goto mn2

mn2_c:
set ans='c'
clear
echo '                           CLEAN UP'
echo
echo 'Short on disk space?  Or do all these files in the document folder'
echo 'look too messy?  Clean up can help by removing unneeded files\!'
echo
echo 'Select from the following menu:'
echo
echo ' c: Cleanup deletes:'
echo '       all backups except of those of latex source index.tex;'
echo '       index.ps and index.dvi;'
echo '       no longer needed processing and orphaned files.'
echo
echo ' s: Supercleanup deletes also:'
echo '       backups of index.tex;'
echo '       files with information about the processing of the document.'
echo
echo ' q: Quit, no cleanup.'
echo
choose.exe csq "Make a choice from the menu above and press its key: q"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto mn2
if ($stat >= 2) goto mn2_c2
if ($stat >= 1) goto mn2_c1
goto mn2
mn2_c1:
l2hclean.bat
goto mn2_pause
mn2_c2:
l2hclean.bat s
goto mn2_pause

mn2_r:
set ans='r'
if !(-f index.tex) goto errgone
if !(-f lremcnv.sub) goto mn2_r01
if !(-f lremcnvbck.sub) goto mn2_rerc
goto mn2_r02
mn2_r01:
if (-f lremcnvbck.sub) goto mn2_rerc
mn2_r02:
clear
echo
echo 'This will create web pages called index@hide.html and index@lost.html'
echo 'containing just the text parts of your latex file index.tex.  That'
echo 'can be useful for running an external spell or grammar checker.'
echo 'WARNING: Do not try to do this without first reading the web page'
echo '         remove_latex in the l2h help-files folder.'
echo
echo 'Please select from the following menu'
echo
echo '   h: Try to load the above help page in the default browser.'
echo '   M: Create MS Word-compatible web pages.'
if (-f @system/codepage.bat) goto mn2_r0y
echo '   w: Select the character encoding for WordPerfect-compatible files.'
echo '   -- Create WordPerfect compatible web pages. (Set encoding first.)'
goto mn2_r0e
mn2_r0y:
echo '   w: Change the character encoding for WordPerfect-compatible files.'
echon.exe "   W: Create WordPerfect compatible web pages using *"
source "@system/codepage.bat" e
if ($status) goto mn2_red
echo '.'
mn2_r0e:
echo '   d: Simply delete the web pages index@hide.html and index@lost.html.'
echo '   q: Quit'
echo
mn2_r0a:
choose.exe hMwWdq "Make a choice from the menu above and then press its key"
set stat="$status"
if ($stat >= 7) goto errsys
if ($stat >= 6) goto mn2
if ($stat >= 5) goto mn2_rd
if ($stat >= 4) goto mn2_rcw
if ($stat >= 3) goto mn2_rw
if ($stat >= 2) goto mn2_rcm
if ($stat >= 1) goto mn2_rh
goto errq
mn2_rh:
xdg-open "$L2HTOP/help-files/remove_latex.html"
if ($status) echo '*** Error loading the web page'
goto mn2_r0a
mn2_rcm:
echo
if !(-f index.tex) goto errgone
clearold.bat n index@hide.html index@lost.html
if ($status) goto mn2_ack
clearold.bat y index@save.sav index@save.enc @tmp.enc @tmp.tex
if ($status) goto err
copynew.bat "$L2HTOP/system-files/data/uc_latex.sub" @tmp.sub c n
if ($status) goto mn2_ack
copynew.bat index.tex @tmp.tex c y
if ($status) goto err
if !(-f lremcnv.sub) goto mn2_rcm1
clearold.bat y @tmp.tex
if ($status) goto err
echo 'Converting index.tex using lremcnv.sub'
"$L2HTOP/system-files/bin/subbin_big.exe" lremcnv.sub index.tex @tmp.tex
if ($status) goto mn2_ack
if !(-f @tmp.tex) goto mn2_rem
mn2_rcm1:
echo 'Creating index@save.sav...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/tex_enc.sub" @tmp.tex @tmp.enc
if ($status) goto mn2_ack
if !(-f @tmp.enc) goto mn2_rem
if !(-f index@save.sav) goto mn2_rem
echo 'Creating index@hide.html...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/enc_mshtml.sub" @tmp.enc index@hide.html
if ($status) goto mn2_ack
if !(-f index@hide.html) goto mn2_rem
echo 'Creating index@lost.html...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/html_lost.sub" index@hide.html index@lost.html
if ($status) goto mn2_ack
if !(-f index@lost.html) goto mn2_rem
echo 'Creating index@save.enc...'
copynew.bat @tmp.enc index@save.enc r y
if ($status) goto err
clearold.bat y @tmp.enc @tmp.sub @tmp.tex
goto mn2_pause
mn2_rw:
echo
clearold.bat y @system/tmp.lst @system/tmp.tmp @system/tmp.bat
if ($status) goto err
ls -1 "$L2HTOP/convert/UTF-8_"*/convertb.sub > @system/tmp.lst
if !(-f @system/tmp.lst) goto errsys
cat @system/tmp.lst | fgrep '/convert/UTF-8_WINDOWS-1252/convertb.sub'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_rec
sellist.exe @system/tmp "encoding conversion" UTF-8_WINDOWS-1252 20 2 /
if ($status) goto mn2_ack
if !(-f @system/tmp.tmp) goto mn2_ack
cat "$L2HTOP/system-files/data/codepage.bat" >> @system/tmp.tmp
if ($status) goto errsys
subbin.exe "$L2HTOP/system-files/data/mksubfil.sub" @system/tmp.tmp @system/tmp.bat
if ($status) goto mn2_ack
if !(-f @system/tmp.bat) goto mn2_reu
movenew.bat @system/tmp.bat @system/codepage.bat y
if ($status) goto err
goto mn2_r
mn2_rcw:
echo
if !(-f index.tex) goto errgone
if !(-f @system/codepage.bat) goto mn2_res
source "@system/codepage.bat"
if ($status) goto mn2_ack
if !(-f convert.sub) goto mn2_red
clearold.bat n index@hide.html index@lost.html
if ($status) goto mn2_ack
clearold.bat y index@save.sav index@save.enc @tmp.enc @tmp.html
if ($status) goto err
copynew.bat "$L2HTOP/system-files/data/uc_latex.sub" @tmp.sub c n
if ($status) goto mn2_ack
copynew.bat index.tex @tmp.tex c y
if ($status) goto err
if !(-f lremcnv.sub) goto mn2_rcw1
clearold.bat y @tmp.tex
if ($status) goto err
echo 'Converting index.tex using lremcnv.sub'
"$L2HTOP/system-files/bin/subbin_big.exe" lremcnv.sub index.tex @tmp.tex
if ($status) goto mn2_ack
if !(-f @tmp.tex) goto mn2_rem
mn2_rcw1:
echo 'Creating index@save.sav...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/tex_enc.sub" @tmp.tex @tmp.enc
if ($status) goto mn2_ack
if !(-f @tmp.enc) goto mn2_rem
if !(-f index@save.sav) goto mn2_rem
echo 'Creating UTF-8 web page @tmp.html...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/enc_wphtml.sub" @tmp.enc @tmp.html
if ($status) goto mn2_ack
if !(-f @tmp.html) goto mn2_rem
echon.exe "Creating index@hide.html using conversion *"
source "@system/codepage.bat" e
if ($status) goto mn2_red
echo '...'
"$L2HTOP/system-files/bin/subbin_big.exe" convert.sub @tmp.html index@hide.html
if ($status) goto mn2_ack
if !(-f index@hide.html) goto mn2_rem
echo 'Creating index@lost.html...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/html_lost.sub" index@hide.html index@lost.html
if ($status) goto mn2_ack
if !(-f index@lost.html) goto mn2_rem
echo 'Creating index@save.enc...'
copynew.bat @tmp.enc index@save.enc r y
if ($status) goto err
clearold.bat y @tmp.enc @tmp.html @tmp.sub @tmp.tex convert.sub convertb.sub 8bit_to_uc.sub num_str-map.sub
if ($status) goto err
goto mn2_pause
mn2_rd:
echo
echo 'This will move index@save.enc, index@save.sav, index@hide.html and'
echo 'index@lost.html into the @system folder for subsequent deletion.'
choose.exe cq "Hit c to do so or q to quit: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto mn2
if ($stat >= 1) goto mn2_rd1
goto errq
mn2_rd1:
movenew.bat index@hide.html @system/index@hide.html y
if ($status) goto err
movenew.bat index@lost.html @system/index@lost.html y
if ($status) goto err
movenew.bat index@save.enc @system/index@save.enc y
if ($status) goto err
movenew.bat index@save.sav @system/index@save.sav y
if ($status) goto err
goto mn2
mn2_red:
echo '** The saved encoding file is corrupt.  Try setting the encoding'
echo '   again.  Otherwise, you may need to rerun install_l2h, with the'
echo '   current folder somewhere inside the l2h folder.'
goto mn2_ack
mn2_rerc:
echo '** Only one of the files lremcnv.sub and lremcnvbck.sub exists.'
echo '   They must come in pairs.'
goto mn2_ack
mn2_rem:
echo '** There was no error in processing.  However, the expected file was'
echo '   not produced??  Is index.tex empty or trivial?'
goto mn2_ack
mn2_rec:
echo '** Error listing the encoding conversions in folder'
echo "      $L2HTOP/convert"
echo '   The UTF-8_WINDOWS-1252 default encoding is missing or corrupt.'
echo '   Check folder and file protections or reinstall l2h.'
goto mn2_ack
mn2_reu:
echo '** The encoding selection failed for unknown reasons.'
goto mn2_ack
mn2_res:
echo '** You need to select an encoding first.  See the help web page.'
choose.exe anykey "   Hit any key to try again"
goto mn2_r

mn2_cr:
set ans='R'
if !(-f index.tex) goto errgone
if !(-f lremcnv.sub) goto mn2_cr01
if !(-f lremcnvbck.sub) goto mn2_crer
goto mn2_cr02
mn2_cr01:
if (-f lremcnvbck.sub) goto mn2_crer
mn2_cr02:
if !(-f index@save.enc) goto mn2_cref
if !(-f index@save.sav) goto mn2_cref
if !(-f index@hide.html) goto mn2_cref
cat index@hide.html | fgrep 'MSHTMLENDMARKER321999999.' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_cr0w
goto mn2_cr0e
mn2_cr0w:
cat index@hide.html | fgrep 'WPHTMLENDMARKER321999999.' > @system/@null.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_creb
set ans='-'
mn2_cr0e:
clear
echo
echo 'This will use the current files:'
ls -l index@hide.html
ls -l index@save.*
echo 'to replace your current LaTeX source file index.tex:'
ls -l index.tex
echo
l2hnewer.exe index@hide.html index@save.enc
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_cr1
echo 'WARNING: File index@hide.html seems to be UNCHANGED from when it was created'
mn2_cr1:
l2hnewer.exe index.tex index@hide.html
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) echo 'WARNING: File index.tex to be replaced is NEWER than index@hide.html\!'
echo 'Do you want to proceed with this?'
choose.exe cq "Hit key c to continue or q to abort: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto mn2
if ($stat >= 1) goto mn2_cr2
goto errq
mn2_cr2:
clearold.bat y @tmp1.enc @tmp2.enc @tmp.html @tmp.tex
if ($status) goto err
copynew.bat "$L2HTOP/system-files/data/uc_latex.sub" @tmp.sub c n
if ($status) goto mn2_ack
goto mk_idxlst
mn2_cr2r:
if ("$ans" == '-') goto mn2_cr3w
echo 'Converting the MS Word web page back to encoded form...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/mshtml_enc.sub" index@hide.html @tmp1.enc
if ($status) goto mn2_cres
if !(-f @tmp1.enc) goto mn2_crem
goto mn2_cr3e
mn2_cr3w:
echo 'Trying to restore UTF-8 encoding to the  WordPerfect web page, hoping'
echo 'that the current encoding is still the same as the one earlier...'
if !(-f @system/codepage.bat) goto mn2_crec
source "@system/codepage.bat"
if ($status) goto mn2_ack
if !(-f convertb.sub) goto mn2_cred
"$L2HTOP/system-files/bin/subbin_big.exe" convertb.sub index@hide.html @tmp.html
if ($status) goto mn2_creu
if !(-f @tmp.html) copynew.bat index@hide.html @tmp.html c y
if ($status) goto err
clearold.bat y convert.sub convertb.sub 8bit_to_uc.sub num_str-map.sub
if ($status) goto err
echo 'Converting the web page back to encoded form...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/wphtml_enc.sub" @tmp.html @tmp1.enc
if ($status) goto mn2_cres
if !(-f @tmp1.enc) goto mn2_crem
mn2_cr3e:
echo 'Reconciling tags and whitespace with the original encoded file...'
"$L2HTOP/system-files/bin/reconcile.exe" index@save.enc @tmp1.enc @tmp2.enc
if ($status) goto mn2_cres
if !(-f @tmp2.enc) goto mn2_crem
echo 'Preparing to create the new index.tex...'
"$L2HTOP/system-files/bin/dofiles.bat" "@system/idx" - '*' -
if ($status) goto mn2_ack
if (-e index@new.tex) goto errsys
if !(-f lremcnvbck.sub) goto mn2_cr31
echo 'Creating index@new.tex...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/enc_tex.sub" @tmp2.enc @tmp.tex
if ($status) goto mn2_cres
if !(-f @tmp.tex) goto mn2_crem
echo 'Converting index@new.tex using lremcnvbck.sub...'
"$L2HTOP/system-files/bin/subbin_big.exe" lremcnvbck.sub @tmp.tex index@new.tex
if ($status) goto mn2_cres
if !(-f index@new.tex) goto mn2_crem
goto mn2_cr32
mn2_cr31:
echo 'Creating index@new.tex...'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/enc_tex.sub" @tmp2.enc index@new.tex
if ($status) goto mn2_cres
if !(-f index@new.tex) goto mn2_crem
mn2_cr32:
clearold.bat y @tmp1.enc @tmp2.enc @tmp.html @tmp.sub @tmp.tex
mn2_cr4:
echo
echo '   v: View differences between the current index.tex and index@new.tex'
echo '      (using Space to scroll and q to quit)'
echo '   r: Replace index.tex by index@new.tex (please view differences first\!)'
echo '   q: Quit, leave index.tex as is'
echo
choose.exe vrq "Make a choice from the menu above and press its key: v"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto mn2
if ($stat >= 2) goto mn2_cr4r
if ($stat >= 1) goto mn2_cr4v
goto errq
mn2_cr4v:
diff -t index.tex index@new.tex | more
choose.exe anykey "End of differences. Hit any key"
goto mn2_cr4
mn2_cr4r:
echo 'Replacing index.tex...'
"$L2HTOP/system-files/bin/dofiles.bat" "@system/idx" - '*' +
if ($status) goto mn2_ack
echo
echo
echo
echo 'The old version of index.tex is still available if you find you need to'
echo 'fix small details.  It is called index@old.tex.  If you want to restore'
echo 'the old index.tex completely, use the x key.'
echo
echo 'ALSO PLEASE EXIT YOUR EDITOR WITH INDEX.TEX AND THEN LOAD THE NEW ONE.'
echo
echo 'After reading the above, hit c to move the index@hide/lost/save.* web page'
choose.exe cq "files into @system for subsequent deletion.  Or hit q to leave them: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto mn2
if ($stat >= 1) goto mn2_cr5
goto errq
mn2_cr5:
movenew.bat index@hide.html @system/index@hide.html y
if ($status) goto err
movenew.bat index@lost.html @system/index@lost.html y
if ($status) goto err
movenew.bat index@save.enc @system/index@save.enc y
if ($status) goto err
movenew.bat index@save.sav @system/index@save.sav y
if ($status) goto err
goto mn2
mn2_cref:
echo '** At least one of the needed files index@hide.html, index@save.enc and'
echo '   index@save.sav does not exist.  No revised index.tex can be created.'
goto mn2_ack
mn2_creb:
echo '** File index@hide.html is corrupt: it misses the correct end marker.'
goto mn2_ack
mn2_crec:
echo '** The character encoding used to create index@hide.html has been lost'
echo '   for unknown reasons.  To fix this, you could try to reselect the'
echo '   same encoding using the r key.'
goto mn2_ack
mn2_cred:
echo '** The saved encoding file is corrupt.  Try setting the encoding'
echo '   again using the r key.  Otherwise, you may need to rerun install_l2h,'
echo '   with the current folder somewhere inside the l2h folder.'
goto mn2_ack
mn2_cres:
echo '** There was an error trying to create the revised index.tex, see above'
echo '   for clues.'
goto mn2_ack
mn2_creu:
echo '** Unable to convert index@hide.html back to UTF-8.  Try resaving it'
echo '   from WordPerfect, making sure you do not use "Save", but "File"'
echo '   "Publish" "HTML", and that in the "Tools" "Settings" "Environment"'
echo '   "General" "Code Page", the correct character encoding is selected.'
goto mn2_ack
mn2_crer:
echo '** Only one of the files lremcnv.sub and lremcnvbck.sub exists.'
echo '   They must come in pairs.'
goto mn2_ack
mn2_crem:
echo '** There was no error in processing.  However, the expected file was'
echo '   not produced for unknown reasons.'
goto mn2_ack

mn2_x:
set ans='x'
if !(-f index.tex) goto errgone
set keys='qxR'
clear
echo
if (-f index@old.tex) goto mn2_x1y
if (-f index@older.tex) goto mn2_x1y
if (-f index@oldest.tex) goto mn2_x1y
echo ' -- Cannot undo the last R command.  (No more backups.)'
set keys='q'
goto mn2_x1e
mn2_x1y:
echo ' R: Try to UNDO the last R command, restoring the previous index.tex file.'
set keys='R'
mn2_x1e:
if (-f index@new.tex) goto mn2_x2y
echo ' -- Cannot redo the last R command.  (No suitable temporary).'
set keys="q$keys"
goto mn2_x2e
mn2_x2y:
echo ' x: Try to REDO the last R command instead, (undo the last x+R command).'
set keys="x$keys"
mn2_x2e:
echo ' q: Return to menu 2.'
set keys="q$keys"
echo
echo 'WARNING: Redo is very limited.  It will only work immediately after the'
echo '         corresponding undo command.'
echo
choose.exe "$keys" "Make a choice from the menu above and then press its key: q"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto mn2_xcr
if ($stat >= 2) goto mn2_xx
if ($stat >= 1) goto mn2
goto errq
mn2_xcr:
goto mk_idxlst
mn2_xcrr:
echo
"$L2HTOP/system-files/bin/undo.bat" "@system/idx" '*'
if ($status) goto mn2_ack
echo
echo 'ALSO PLEASE EXIT YOUR EDITOR WITH INDEX.TEX AND THEN LOAD THE NEW ONE.'
echo
goto mn2_pause
mn2_xx:
echo
"$L2HTOP/system-files/bin/dofiles.bat" "@system/idx" - '*' +
if ($status) goto mn2_ack
echo
echo 'ALSO PLEASE EXIT YOUR EDITOR WITH INDEX.TEX AND THEN LOAD THE NEW ONE.'
echo
goto mn2_pause

mn2_w:
set ans='w'
if !(-f index.tex) goto errgone
clear
if (-d web-pages/) goto mn2_w1
echo
echo 'This will create a web pages version of index.tex.'
echo
echo ' c: Continue creating the web pages.'
set keys='qqc'
goto mn2_w1e
mn2_w1:
echo
echo 'Since creating images is normally slow, there is an option to remake'
echo 'just the web pages.  But this option may not work correctly if you made'
echo 'nontrivial changes to your web page creation settings.  Signs that the'
echo 'web pages must be remade from scratch are: (1) "ghosting" around math'
echo 'characters and/or lines in figures; (2) improperly sized math characters'
echo 'and/or figures.'
echo
echo ' c: Remake the web pages, but not any presumably still usable images.'
echo ' s: Remake the web pages from scratch.'
echo '    WARNING: anything you might have inside folder web-pages, and not'
echo '    inside its subfolder keep, may get deleted.'
set keys='qsc'
mn2_w1e:
echo ' q: return to menu 2.'
echo
choose.exe "$keys" "Make a choice from the menu above and then press its key: c"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto mn2_wc
if ($stat >= 2) goto mn2_ws
if ($stat >= 1) goto mn2
goto errq
mn2_ws:
"$L2HTOP/system-files/bin/delwp.bat"
if ($status) goto err
mn2_wc:
makewebpages.bat
if ($status) goto mn2_ack
echo
echo 'Additional note: latex2html is a complainer.  If you were not explicitly'
echo 'notified of problems, check the web pages themselves if things are OK.'
echo
if ("$keys" != qsc) goto mn2_wp
echo 'Do not forget to hit "Reload" or "Refresh" on your browser when viewing'
echo 'the remade web pages.'
echo
mn2_wp:
choose.exe anykey "After reading the above messages, hit any key to return to menu 2"
goto mn2

mn2_cw:
set ans='W'
if !(-f web-pages/index.html) goto mn2_cwee
xdg-open web-pages/index.html
if ($status) goto mn2_cwel
goto mn2
mn2_cwee:
echo '** Web page web-pages/index.html is gone.'
goto mn2_ack
mn2_cwel:
echo '** An error occurred trying to load web page web-pages/index.html.'
goto mn2_ack

mn2_j:
set ans='j'
clear
mn2_jt:
echo
echo ' w: Explain why some jpg images could be desirable.  Or not.'
echo ' h: Explain how to get jpg images made.'
echo ' 0: Turn jpg generation off.'
echo ' 1: Set jpg quality to  75.'
echo ' 2: Set jpg quality to  80.'
echo ' 3: Set jpg quality to  85.'
echo ' 4: Set jpg quality to  87.'
echo ' 5: Set jpg quality to  90.'
echo ' 6: Set jpg quality to  95.'
echo ' 7: Set jpg quality to 100.'
echo ' q: Return to menu 2.'
echo
if ($?L2H_JPGQ) goto mn2_jty
echo 'Currently jpg generation is OFF.'
goto mn2_jte
mn2_jty:
echo "Currently jpg generation is ON at quality $L2H_JPGQ."
mn2_jte:
echo
choose.exe  wh01234567q "Make a choice from the menu above and then press its key"
set stat="$status"
if ($stat >= 12) goto errsys
if ($stat >= 11) goto mn2
if ($stat >= 10) goto mn2_j7
if ($stat >= 9) goto mn2_j6
if ($stat >= 8) goto mn2_j5
if ($stat >= 7) goto mn2_j4
if ($stat >= 6) goto mn2_j3
if ($stat >= 5) goto mn2_j2
if ($stat >= 4) goto mn2_j1
if ($stat >= 3) goto mn2_j0
if ($stat >= 2) goto mn2_jh
if ($stat >= 1) goto mn2_jw
goto errq
mn2_jw:
cat "$L2HTOP/system-files/data/jpgwhy.txt"
choose.exe anykey "After reading the above, hit any key to return"
goto mn2_jt
mn2_jh:
cat "$L2HTOP/system-files/data/jpghow.txt"
choose.exe anykey "After reading the above, hit any key to return"
goto mn2_jt
mn2_j0:
unsetenv L2H_JPGQ
goto mn2
mn2_j1:
setenv L2H_JPGQ '75'
goto mn2
mn2_j2:
setenv L2H_JPGQ '80'
goto mn2
mn2_j3:
setenv L2H_JPGQ '85'
goto mn2
mn2_j4:
setenv L2H_JPGQ '87'
goto mn2
mn2_j5:
setenv L2H_JPGQ '90'
goto mn2
mn2_j6:
setenv L2H_JPGQ '95'
goto mn2
mn2_j7:
setenv L2H_JPGQ '100'
goto mn2

mn2_h:
set ans='h'
if !(-f web-pages/index.html) goto mn2_hef
if !(-f @system/language.bat) goto mn2_hel
cat web-pages/index.html | fgrep ' (Already l2h-finalized.)'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_h2
goto mn2_heb
mn2_h2:
cat web-pages/index.html | fgrep ' (Already l2h-hyphenated as: '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_h3
goto mn2_heb
mn2_h3:
echo
echon.exe "This will hyphenate the web pages assuming they are in *"
source "@system/language.bat" e
if ($status) goto mn2_hee
echo '.'
echo
echo 'Note that any web page remake will clobber hyphenation.   So there is'
echo 'not much reason to hyphenate them until they seem to be in final form.'
echo
choose.exe cq "Hit the c key to continue with hyphenation or q to abort: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto mn2
if ($stat >= 1) goto mn2_h4
goto errq
mn2_h4:
cd web-pages
source ../@system/language.bat
if ($status) goto mn2_ack0
if !(-f hyphenate.txt) goto errsys
if !(-f hyphenate.sub) goto errsys
"$L2HTOP/system-files/bin/dofiles.bat" "@htmllist" '*'.html '*' file_subbn.bat hyphenate.sub i
if ($status) goto mn2_ack0
clearold.bat y hyphenate.txt hyphenate.sub
cd ..
goto mn2_pause
mn2_hef:
echo '** At least some web pages are gone.'
goto mn2_ack
mn2_hel:
echo '** Cannot hyphenate the web pages: no language is set with H.'
goto mn2_ack
mn2_heb:
echo '** File index.html is already hyphenated or finalized.  Use u to'
echo '   undo or remake the web pages with w.'
goto mn2_ack
mn2_hee:
echo
echo '** The language definition file is corrupt.  Try setting the language'
echo '   again.  Otherwise, you may need to rerun install_l2h, with the'
echo '   current folder somewhere inside the l2h folder.'
goto mn2_ack

mn2_ch:
set ans='H'
clearold.bat y @system/tmp.lst @system/tmp.tmp @system/tmp.bat
if ($status) goto err
ls -1 "$L2HTOP/hyphenate/"*/hyphenate.sub > @system/tmp.lst
if !(-f @system/tmp.lst) goto errsys
cat @system/tmp.lst | fgrep '/hyphenate/English_US/hyphenate.sub'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_chel
sellist.exe @system/tmp language English_US 20 2 /
if ($status) goto mn2_ack
if !(-f @system/tmp.tmp) goto mn2_ack
cat "$L2HTOP/system-files/data/language.bat" >> @system/tmp.tmp
if ($status) goto errsys
subbin.exe "$L2HTOP/system-files/data/mksubfil.sub" @system/tmp.tmp @system/tmp.bat
if ($status) goto mn2_ack
if !(-f @system/tmp.bat) goto mn2_cheu
movenew.bat @system/tmp.bat @system/language.bat y
if ($status) goto err
goto mn2
mn2_chel:
echo '** Error listing the languages, including English_US, in folder'
echo "      $L2HTOP/hyphenate"
echo '   Check folder and file protections.'
goto mn2_ack
mn2_cheu:
echo '** The language selection failed for unknown reasons.'
goto mn2_ack

mn2_f:
set ans='f'
if !(-f web-pages/index.html) goto mn2_fef
cat web-pages/index.html | fgrep ' (Already l2h-finalized.)'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_f1
goto mn2_feb
mn2_f1:
echo
echo 'This will finalize the web pages.  This is intended to prevent bad line'
echo 'breaks at inline mathematics.  It also cleans up some ugly spots left'
echo 'behind by the web page generation.'
echo
echo 'Note that any web page remake will clobber finalization.   So there is'
echo 'not much reason to finalize them until they seem to be in final form.'
echo 'Further note that if you also want to hyphenate the web pages,'
echo 'hyphenation should be done *before* finalizing.'
echo
choose.exe cq "Hit the c key to continue finalizing or q to abort: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto mn2
if ($stat >= 1) goto mn2_f2
goto errq
mn2_f2:
cd web-pages
"$L2HTOP/system-files/bin/dofiles.bat" "@htmllist" '*'.html '*' file_subbn.bat "$L2HTOP/system-files/data/clean.sub" i
if ($status) goto mn2_ack0
cd ..
goto mn2_pause
mn2_fef:
echo '** At least some web pages are gone.'
goto mn2_ack
mn2_feb:
echo '** File index.html is already finalized.'
goto mn2_ack

mn2_u:
set ans='u'
set keys='qrc'
clear
echo
if (-f web-pages/@htmllist@old.bat) goto mn2_u1y
if (-f web-pages/@htmllist@older.bat) goto mn2_u1y
if (-f web-pages/@htmllist@oldest.bat) goto mn2_u1y
echo ' -- Cannot undo the last h or f command.  (No more backups.)'
set keys='q'
goto mn2_u1e
mn2_u1y:
echo ' c: Continue trying to UNDO the last h or f command.  This will restore'
echo '    the web pages to the state they were in before the h or f command.'
set keys='c'
mn2_u1e:
echo
if !(-f web-pages/@htmllist.bat) goto mn2_u2n
ls -1 web-pages/*.html | fgrep '@new' > @system/@null.tmp
if ($status) goto mn2_u2n
goto mn2_u2y
mn2_u2n:
echo ' -- Cannot redo the last h or f command.  (No  suitable temporaries.)'
set keys="q$keys"
goto mn2_u2e
mn2_u2y:
echo ' r: Try to REDO the last h or f command instead, (undo the last u command).'
set keys="r$keys"
mn2_u2e:
echo '    Redo only works immediately after the corresponding u command.'
echo
echo ' q: Return to menu 2.'
set keys="q$keys"
echo
echo 'Note: Undo does not work after any web page rebuild as these mess up the'
echo '      image numbers.  In that case web page rebuild, rehyphenation, and'
echo '      refinalizing must be used.  (The only reason that the u command is'
echo '      provided is that it is much quicker than rebuilding.)'
echo
choose.exe "$keys" "Make a choice from the menu above and then press its key: q"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto mn2_uc
if ($stat >= 2) goto mn2_ur
if ($stat >= 1) goto mn2
goto errq
mn2_uc:
cd web-pages
"$L2HTOP/system-files/bin/undo.bat" "@htmllist" '*'
if ($status) goto mn2_ack0
cd ..
goto mn2_pause
mn2_ur:
cd web-pages
"$L2HTOP/system-files/bin/dofiles.bat" "@htmllist" - '*' +
if ($status) goto mn2_ack0
cd ..
goto mn2_pause

mn2_cb:
set ans='B'
if !(-f index.css) goto mn2_cbei
if !(-f settings.pl) goto mn2_cbei
clear
mn2_cbt:
echo
echo ' c: Change the button colors of a selected web page theme and use these'
echo '    changed buttons for the current document.'
echo
echo ' h: Explain how I could create my own web page buttons.'
echo ' i: Initialize a folder mybuttons with the buttons of a selected theme'
echo '    so that I can customize them.'
echo ' s: Select my own buttons, as found in folder mybuttons, for the current'
echo '    document.'
echo
echo ' q: Return to menu 2.'
echo
choose.exe  chisq "Make a choice from the menu above and then press its key"
set stat="$status"
if ($stat >= 6) goto errsys
if ($stat >= 5) goto mn2
if ($stat >= 4) goto mn2_cbs
if ($stat >= 3) goto mn2_cbi
if ($stat >= 2) goto mn2_cbh
if ($stat >= 1) goto mn2_cbc
goto errq
mn2_cbc:
echo
echo 'Select a theme to take the buttons from.'
echo
"$L2HTOP/system-files/bin/gettheme.bat" colors.lst -
if ($status) goto mn2_ack
if !(-f @system/copthm.bat) goto errsys
source "@system/copthm.bat" copbut
if ($status) goto mn2_ack
if !(-f buttons@new/next.gif) goto errsys
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " chicons
if ($status) goto mn2_ack
if !(-f buttons@new/@colors.sub) goto errsys
if !(-f buttons@new/@colors_g.sub) goto errsys
cd buttons@new
source "$L2HTOP/system-files/bin/buttonscol.bat" "$L2HTOP/system-files/bin/cnvicon.bat" @colors.sub @colors_g.sub
if ($status) goto mn2_ack0
cd ..
source "$L2HTOP/system-files/bin/buttonsall.bat" "$L2HTOP/system-files/bin/file_exist.bat" "buttons@new/"
if ($status) goto mn2_cbeg
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' +
if ($status) goto mn2_ack
if !(-f buttons/next.gif) goto errsys
if (-d web-pages/) goto updwp
goto mn2_pause
mn2_cbh:
cat "$L2HTOP/system-files/data/buthow.txt"
choose.exe anykey "After reading the above, hit any key to return to the buttons menu"
goto mn2_cbt
mn2_cbi:
if !(-e mybuttons) goto mn2_cbi2
echo
echo '** A folder mybuttons already exists.  You must move it elsewhere,'
echo '   rename it, or delete it first.  Then try the i key again.'
echo
choose.exe anykey "After reading the above, hit any key to return to the buttons menu"
goto mn2_cbt
mn2_cbi2:
"$L2HTOP/system-files/bin/gettheme.bat" next.gif -
if ($status) goto mn2_ack
if !(-f @system/copthm.bat) goto errsys
source "@system/copthm.bat" copbut
if ($status) goto mn2_ack
if !(-d buttons@new/) goto errsys
mv buttons@new mybuttons
if ($status) goto errsys
if !(-f mybuttons/next.gif) goto errsys
echo 'Folder mybuttons with buttons to modify has been created.'
goto mn2_pause
mn2_cbs:
if (-d mybuttons/) goto mn2_cbs2
echo '** There is no subfolder mybuttons with buttons in the current folder.'
echo '   You must create such a folder first.'
goto mn2_ack
mn2_cbs2:
source "$L2HTOP/system-files/bin/buttonsall.bat" "$L2HTOP/system-files/bin/file_exist.bat" "mybuttons/"
if ($status) goto mn2_cbeb
goto mk_thmlst
mn2_cbsr:
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' -
if ($status) goto mn2_ack
if (-e buttons@new) goto errsys
if (-e settings@new.pl) goto errsys
mkdir buttons@new
if !(-d buttons@new/) goto errsys
cp -p mybuttons/*.gif buttons@new
if ($status) goto errsys
source "$L2HTOP/system-files/bin/buttonsall.bat" "$L2HTOP/system-files/bin/file_exist.bat" "buttons@new/"
if ($status) goto errsys
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " myicons
if ($status) goto mn2_ack
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' +
if ($status) goto mn2_ack
if !(-f buttons/next.gif) goto errsys
if (-d web-pages/) goto updwp
goto mn2_pause
mn2_cbei:
echo '** Before you can change the buttons, you need to select a theme first.'
goto mn2_ack
mn2_cbeg:
echo '** Required buttons have disappeared from buttons@new.'
goto mn2_ack
mn2_cbeb:
echo '** Required buttons are missing from the mybuttons folder. Add the missing'
echo '   buttons, (maybe by copying them over from a theme), then try again.'
goto mn2_ack

mn2_cc:
set ans='C'
if !(-f settings.pl) goto mn2_ccei
if !(-f index.css) goto mn2_ccei
cat settings.pl | fgrep '$MAIN_COLOR'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_cc1
clear
echo
echo '** Sorry.  You are using a theme with frames.  Guided changing of the'
echo '   colors in themes with frames is currently not implemented.'
echo
echo '   (However, an experienced, careful user might be able to go into file'
echo '   settings.pl with a text editor and change the colors manually.'
echo '   They are of the form #XXXXXX and there are plenty of web pages that'
echo '   will tell you what values for XXXXXX give what colors.  Some I saw:'
echo '      http://visibone.com/colorlab/'
echo '      http://the-light.com/colclick.html'
echo '      http://www.webmonkey.com/2010/02/color_charts/'
echo '   I am sure there are more.  Be sure to remake the web pages from'
echo '   scratch if you change the main background color.  Otherwise there'
echo '   will be "ghosting" around math symbols and lines in figures.)'
echo
goto mn2_ack
mn2_cc1:
clear
echo
echo 'This will change the colors used for the web pages.  It will prompt you'
echo 'for the colors.  (At the prompt, you can enter ? to get some help with'
echo 'how to specify the new color.  Enter a q to abort.)'
echo
if !(-d web-pages/) goto mn2_cc1b
echo 'Note that changing the web page background color will require a complete'
echo 'remake of your web pages.  Without such a complete remake, there will be'
echo 'ugly "ghosting" around math symbols and lines in figures.'
echo
mn2_cc1b:
choose.exe cq "Hit c to continue changing colors or q to quit: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto mn2
if ($stat >= 1) goto mn2_cc2
goto errq
mn2_cc2:
goto mk_thmlst
mn2_cc2r:
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' -
if ($status) goto mn2_ack
if (-e index@new.css) goto errsys
echo
echo '                   Enter ? for help or q to quit.'
echo
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " colors
if ($status) goto mn2_ack
if !(-f index@new.css) goto mn2_ccen
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' +
if ($status) goto mn2_ack
if (-d web-pages/) goto updwp
goto mn2_pause
mn2_ccei:
echo '** Before you can change the colors, you need to select a theme first.'
goto mn2_ack
mn2_ccen:
echo '** No color changes seem to have been made.'
goto mn2_ack

mn2_cf:
set ans='F'
if !(-f settings.pl) goto mn2_cfei
if !(-f index.css) goto mn2_cfei
clear
echo
echo 'This will change font and figure sizes of the web pages.  It will prompt'
echo 'you for the sizes.  (At the prompt, you can enter ? to get some help with'
echo 'that.  Enter a q to abort.)'
echo
if !(-d web-pages/) goto mn2_cf1
echo 'Note that size changes require a complete remake of your web pages'
echo 'from scratch.  If you do not remake, figures and mathematical symbols'
echo 'will still have their old sizes.'
echo
mn2_cf1:
choose.exe cq "Hit c to continue changing sizes or q to quit: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto mn2
if ($stat >= 1) goto mn2_cf2
goto errq
mn2_cf2:
goto mk_thmlst
mn2_cf2r:
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' -
if ($status) goto mn2_ack
if (-e index@new.css) goto errsys
if (-e settings@new.pl) goto errsys
echo
echo '                   Enter ? for help or q to quit.'
echo
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " fntsiz
if ($status) goto mn2_ack
if (-f index@new.css) goto mn2_cf3
if !(-f settings@new.pl) goto mn2_cfen
mn2_cf3:
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' +
if ($status) goto mn2_ack
if (-d web-pages/) goto updwp
goto mn2_pause
mn2_cfei:
echo '** Before you can change the font sizes, you need to select a theme first.'
goto mn2_ack
mn2_cfen:
echo '** No nontrivial changes seem to have been made.'
goto mn2_ack

mn2_ci:
set ans='I'
if !(-f index.css) goto mn2_ciei
if !(-f settings.pl) goto mn2_ciei
clear
echo
echo 'This will change the font size or colors of individual style sheet'
echo 'entries.   Probably most useful if you know some html.'
echo
echo 'If you want to change the main font size this way, take into account'
echo 'that the math symbols will keep their original size.  Use the F key'
echo 'in the second l2h menu to change all fontsizes consistently.'
echo
if !(-d web-pages/) goto mn2_ci1
echo 'Also, if you change the web page background color, the web pages'
echo 'will need to be remade from scratch.  Otherwise "ghosting" will result'
echo 'around math symbols and lines in figures.'
echo
mn2_ci1:
choose.exe cq "Hit c to continue changing style items or q to quit: c"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto mn2
if ($stat >= 1) goto mn2_ci2
goto errq
mn2_ci2:
goto mk_thmlst
mn2_ci2r:
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' -
if ($status) goto mn2_ack
if (-e index@new.css) goto errsys
echo
echo '                   Enter ? for help or q to quit.'
echo
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " setatt
if ($status) goto mn2_ack
if !(-f index@new.css) goto mn2_cien
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' +
if ($status) goto mn2_ack
if (-d web-pages/) goto updwp
goto mn2_pause
mn2_ciei:
echo '** Before you can change the style sheet, you need to select a theme first.'
goto mn2_ack
mn2_cien:
echo '** No nontrivial changes seem to have been made.'
goto mn2_ack

mn2_s:
set ans='s'
if !(-f index.css) goto mn2_sei
if !(-f settings.pl) goto mn2_sei
clearold.bat y @system/tmp.tmp
if ($status) goto err
subbin.exe "$L2HTOP/system-files/data/serif.sub" index.css @system/tmp.tmp
if ($status) goto mn2_ses
if !(-f @system/tmp.tmp) goto mn2_sen
cmp -s @system/tmp.tmp index.css
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_sd
goto mn2_sen
mn2_sd:
movenew.bat @system/tmp.tmp index.css y
if ($status) goto err
if (-d web-pages/) goto mn2_s2
echo
echo 'At least some web page fonts have been changed to serif.'
echo '(Use key S in the second l2h menu to undo the changes.)'
echo
goto mn2_pause
mn2_s2:
copynew.bat index.css web-pages/index.css c y
if ($status) goto err
chmod u=rw,go=r web-pages/index.css
if ($status) goto errsys
echo
echo 'At least some web page fonts have been changed to serif.  Do not forget'
echo 'to use the "Reload" or "Refresh" button on your web browser to see the'
echo 'results.  (Use key S in the second l2h menu to undo the changes.)'
echo
goto mn2_pause
mn2_sei:
echo '** Before you can change the fonts, you need to select a theme first.'
goto mn2_ack
mn2_ses:
echo '** Failed to change the fonts in the style file.'
goto mn2_ack
mn2_sen:
echo '** No changes were made.  Either the web pages already used serif'
echo '   fonts or the fonts in the style file index.css are different from'
echo '   the l2h assumptions.'
goto mn2_ack

mn2_cs:
set ans='S'
if !(-f index.css) goto mn2_csei
if !(-f settings.pl) goto mn2_csei
clearold.bat y @system/tmp.tmp
if ($status) goto err
subbin.exe "$L2HTOP/system-files/data/sans.sub" index.css @system/tmp.tmp
if ($status) goto mn2_cses
if !(-f @system/tmp.tmp) goto mn2_csen
cmp -s @system/tmp.tmp index.css
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto mn2_csd
goto mn2_csen
mn2_csd:
movenew.bat @system/tmp.tmp index.css y
if ($status) goto err
if (-d web-pages/) goto mn2_cs2
echo
echo 'At least some web page fonts have been changed to sans-serif.'
echo '(Use key s in the second l2h menu to undo the changes.)'
echo
goto mn2_pause
mn2_cs2:
copynew.bat index.css web-pages/index.css c y
if ($status) goto err
chmod u=rw,go=r web-pages/index.css
if ($status) goto errsys
echo
echo 'At least some web page fonts have been changed to sans-serif.  Do not'
echo 'forget to use the "Reload" or "Refresh" button on your web browser to'
echo 'see the results.  (Use key s in the second l2h menu to undo the changes.)'
echo
goto mn2_pause
mn2_csei:
echo '** Before you can change the fonts, you need to select a theme first.'
goto mn2_ack
mn2_cses:
echo '** Failed to change the fonts in the style file.'
goto mn2_ack
mn2_csen:
echo '** No changes were made.  Either the web pages already used sans-serif'
echo '   fonts or the fonts in the style file index.css are different from'
echo '   the l2h assumptions.'
goto mn2_ack

mn2_ct:
set ans='T'
"$L2HTOP/system-files/bin/gettheme.bat" settings.pl
if ($status) goto mn2_ack
if !(-f settings.pl) goto errsys
if !(-f index.css) goto errsys
if (-d web-pages/) goto updwp
goto mn2_pause

mn2_cu:
set ans='U'
set keys='qUc'
clear
echo
if (-f @system/theme@old.bat) goto mn2_cu1y
if (-f @system/theme@older.bat) goto mn2_cu1y
if (-f @system/theme@oldest.bat) goto mn2_cu1y
echo ' -- Cannot undo the last B, C, F, I, or T command. (No more backups.)'
set keys='q'
goto mn2_cu1e
mn2_cu1y:
echo ' c: Continue with trying to UNDO the last B, C, F, I, or T command.'
set keys='c'
mn2_cu1e:
echo
if (-f index@new.css) goto mn2_cu2y
if (-f settings@new.pl) goto mn2_cu2y
if (-d buttons@new/) goto mn2_cu2y
echo ' -- Cannot redo the last B, C, F, I, or T command.  (No  suitable temporaries.)'
set keys="q$keys"
goto mn2_cu2e
mn2_cu2y:
echo ' U: Try to REDO the last B, C, F, I, or T command instead, (undo U).'
set keys="U$keys"
mn2_cu2e:
echo
echo ' q: Return to menu 2.'
set keys="q$keys"
echo
echo 'Note: Remake of the web pages will typically be needed after an undo or'
echo '      redo.  (If the original command required a remake, then so will'
echo '      its undo.)'
echo
choose.exe "$keys" "Make a choice from the menu above and then press its key: q"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto mn2_cuc
if ($stat >= 2) goto mn2_cucu
if ($stat >= 1) goto mn2
goto errq
mn2_cuc:
goto mk_thmlst
mn2_cucr:
"$L2HTOP/system-files/bin/undo.bat" "@system/theme" '*'
if ($status) goto mn2_ack
goto updwp
mn2_cucu:
"$L2HTOP/system-files/bin/dofiles.bat" "@system/theme" - '*' +
if ($status) goto mn2_ack
goto updwp

# subroutines

# create @system/idx.lst
mk_idxlst:
clearold.bat y @system/idx.lst
if ($status) goto err
echo 'index.tex' > @system/idx.lst
if ($status) goto errsys
if !(-f @system/idx.lst) goto errsys
cat @system/idx.lst | fgrep 'index.tex' > @system/@null.tmp
if ($status) goto errsys
if ("$ans" == R) goto mn2_cr2r
if ("$ans" == '-') goto mn2_cr2r
if ("$ans" == x) goto mn2_xcrr
goto errsys

# create @system/theme.lst
mk_thmlst:
clearold.bat y @system/theme.lst
if ($status) goto err
echo 'index.css' > @system/theme.lst
if ($status) goto errsys
if !(-f @system/theme.lst) goto errsys
cat @system/theme.lst | fgrep 'index.css' > @system/@null.tmp
if ($status) goto errsys
echo 'settings.pl' >> @system/theme.lst
if ($status) goto errsys
cat @system/theme.lst | fgrep 'settings.pl' > @system/@null.tmp
if ($status) goto errsys
echo 'index.html' >> @system/theme.lst
if ($status) goto errsys
cat @system/theme.lst | fgrep 'index.html' > @system/@null.tmp
if ($status) goto errsys
echo 'buttons.F0L' >> @system/theme.lst
if ($status) goto errsys
cat @system/theme.lst | fgrep 'buttons.F0L' > @system/@null.tmp
if ($status) goto errsys
if ("$ans" == B) goto mn2_cbsr
if ("$ans" == C) goto mn2_cc2r
if ("$ans" == F) goto mn2_cf2r
if ("$ans" == I) goto mn2_ci2r
if ("$ans" == U) goto mn2_cucr
goto errsys

# update the web pages for changes in settings
updwp:
source "$L2HTOP/system-files/bin/updwp.bat" + + + + + +
if ($status) goto updwpep
echo
echo '   r: Remake the web pages.'
echo '   s: Remake the web pages from scratch.'
echo '   q: Quit, do not remake the web pages.'
echo
echo 'Warning: When examining the current state of the web pages, do not forget'
echo '         to press the "Reload" or "Refresh" button on your browser.'
echo
choose.exe rsq "Select from the menu above and then press its letter: $ans"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto mn2
if ($stat >= 2) goto mn2_ws
if ($stat >= 1) goto mn2_wc
goto errq
updwpep:
choose.exe anykey "** Failed to update the web pages.  Hit any key to return to menu 2"
goto mn2

# pause and return to menu 2
mn2_pause:
choose.exe anykey "No errors reported.  Hit any key to return to menu 2"
goto mn2
mn2_ack0:
cd ..
mn2_ack:
choose.exe anykey "** Failed.  Hit any key to acknowledge and return to menu 2"
goto mn2

# done
done:
if !(-f menu_end.bat) goto nomend
source menu_end.bat
if ($status) goto errmend
nomend:
if (-f @system/in-use.txt) rm @system/in-use.txt
echo 'Exiting menu.bat.'
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errfils:
echo '*** Essential system files are missing from the l2h folder or you do'
echo '    not have access to them.  Please re-install l2h from scratch.'
goto err
errmstart:
echo '*** Error running menu_start.bat.  Must abort.'
goto err
errdat:
echo '*** Problem examining the dates of index.tex and associated files.'
echo '    See above or check index.tex for unusual \input or \include commands.'
goto err
errrl:
echo '*** Error reading the log file index.log.'
goto err
errmki:
echo '*** Program makeindex is not in your path.  Install it first.'
goto err
errbib:
echo '*** Program bibtex is not in your path.  Install it first.'
goto err
errdvi:
echo '*** Program dvips is not in your path.  Install it first.'
goto err
errpp:
echo '*** Program pdftops is not in your path.  Install it first.'
goto err
errp2p:
echo '*** Program ps2pdf is not in your path.  Install it first.'
goto err
errmbat:
echo '*** Error making a script for batch file processing.'
goto err
errmend:
echo '*** Error running menu_end.bat.'
goto err
errgone:
echo '*** File index.tex is gone\!'
choose.exe cq "    Hit c to select a new one or q to quit"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto errq
if ($stat >= 1) goto askexamp
goto errq
errq:
echo '*** Terminated by user.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
if (-f @system/in-use.txt) rm @system/in-use.txt
exit 1

end:
