#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Fake bad ppmquant using good pnmremap and pnmcolormap.

# Usage
goto start
use:
echo 'Usage: ppmquant.bat [-quiet] [-floyd] number_of_colors infile'
echo '       Results are send to STDOUT.'
echo '       Parameters are order and case sensitive\!'
echo '       Parameters -quiet and -floyd are ignored and can be omitted.'
echo '       Leaves a log file ppmquant.log behind if it tried to run.'
goto err
start:

# see whether we are called for inquiry
if ("$1" == '?') goto use
if ("$1" == '/?') goto use
if ("$1" == '-h') goto use
if ("$1" == '--help') goto use
if ("$1" == '') goto use

# open a log file
if !(-f ppmquant.log) then
    echo 'ppmquant log:' > ppmquant.log
endif
if !(-f ppmquant.log) goto err

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h
if !(-x "$L2HTOP/bin/pnmcolormap") goto errfils
if !(-x "$L2HTOP/bin/pnmremap") goto errfils

# write what we are doing to the log
echo "ppmquant.bat $1 $2 $3 $4 $5 $6 $7 $8 $9" >> ppmquant.log

# check whether we have valid arguments
if ("$1" == '-quiet') shift
if ("$1" == '-floyd') shift
if ("$1" == '') goto errpar
if ("$2" == '') goto errpar
if !(-f "$2") goto errfil

# create a map file
echo "Running $L2HTOP/bin/pnmcolormap $1 $2" >> ppmquant.log
"$L2HTOP/bin/pnmcolormap" "$1" "$2" > @tmpp.map
if ($status) goto errmap
echo 'error status 0' >> ppmquant.log

# create a remapped file on standard output
echo "Running $L2HTOP/bin/pnmremap -map=@tmpp.map $2" >> ppmquant.log
"$L2HTOP/bin/pnmremap" -map=@tmpp.map "$2"
if ($status) goto errremap
echo 'error status 0' >> ppmquant.log

# clean up
rm @tmpp.map

# done
done:
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop." >> ppmquant.log
echo ' ** You may need to rerun install_l2h.' >> ppmquant.log
goto err
errfils:
echo "*** Folder $L2HTOP/bin does not contain the required" >> ppmquant.log
echo ' ** *executable* programs pnmcolormap and pnmremap.' >> ppmquant.log
goto err
errpar:
echo '*** Usage: ppmquant.bat [-quiet] [-floyd] number_of_colors infile' >> ppmquant.log
goto err
errfil:
echo "*** File $2 not found." >> ppmquant.log
goto err
errmap:
echo "*** Error running pnmcolormap $1 $2" >> ppmquant.log
goto err
errremap:
echo "*** Error running pnmremap -map=@tmpp.map $2" >> ppmquant.log
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.' >> ppmquant.log
echo ' ** If not, try rerunning install_l2h in the l2h_folder.' >> ppmquant.log
err:
exit 1

end:
