#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: this is called from .desktop files to run programs

# Usage
goto start
use:
echo 'Usage: From a /desktop file:'
echo "          Exec='.../runscript.bat' %k% COMMAND [PARAMETERS]"
echo '       There can be up to 8 parameters.  Note that any trailing empty'
echo '       parameters will be explicitly removed.'
echo '       File COMMAND is looked for in:'
echo '          1) the current folder'
echo "          2) $L2HTOP/bin"
echo "          3) $L2HTOP/system-files/bin"
echo '          4) the PATH, after sourcing $L2HTOP/init.bat'
echo '       in that order.  COMMAND should not be quoted.'
echo '       Before the command is executed, a cd is done to the location of'
echo "       the desktop file."
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h
if !(-f "$L2HTOP/init.bat") goto errinif

# if the first parameter is still %k, stay in the current folder
if ("$1" == "%k") goto endcd

# get the location of the clicked desktop file.
set loc="$1:h"
if ("$loc" == "") goto errloc
if !(-d "$loc") goto errloc

# cd to that location
cd "$loc"
if ($status) goto errsys
endcd:

# remove the %k parameter
shift
if ($status) goto errsys

# Look for the script
set prog="./$1"
if (-e "$prog") goto check
set prog="$L2HTOP/bin/$1"
if (-e "$prog") goto check
set prog="$L2HTOP/system-files/bin/$1"
if (-e "$prog") goto check
source "$L2HTOP/init.bat"
if ($status) goto errini
set prog="$1"
which "$prog"
if ($status) goto errprog

# if the command is tcsh, consider creating a ~/.tcshrc file
if ("$prog" != "tcsh") goto endcheck
if (-f ~/.tcshrc) goto endcheck
if (-f ~/.cshrc) goto endcheck
if !(-f "$L2HTOP/system-files/data/tcshrc.tcs") goto endcheck
echo "You do not have a .tcshrc file.  Should I copy a basic one into your"
echo "HOME folder, for some protection against accidental file deletion?"
"$L2HTOP/bin/choose.exe" ynN "Press y for yes, n for no, or N for NO: y"
set stat=$status
if ($stat >= 4) goto errsys
if ($stat >= 3) goto rentrc
if ($stat >= 2) goto endcheck
if ($stat >= 1) goto coptrc
goto errab
coptrc:
cp "$L2HTOP/system-files/data/tcshrc.tcs" ~/.tcshrc
if ($status) goto errsys
goto endcheck
rentrc:
mv "$L2HTOP/system-files/data/tcshrc.tcs" "$L2HTOP/system-files/data/tcshrc.not"
if ($status) goto errsys
goto endcheck

# check whether the script is executable
check:
if !(-x "$prog") chmod a+x "$prog"
if ($status) goto errprot
if !(-x "$prog") goto errprot
endcheck:

# remove the program parameter
shift
if ($status) goto errsys

# run the command
run:
if ("$8" != "") goto par8
if ("$7" != "") goto par7
if ("$6" != "") goto par6
if ("$5" != "") goto par5
if ("$4" != "") goto par4
if ("$3" != "") goto par3
if ("$2" != "") goto par2
if ("$1" != "") goto par1
"$prog"
if ($status) goto errrun
goto paus
par1:
"$prog" "$1"
if ($status) goto errrun
goto paus
par2:
"$prog" "$1" "$2"
if ($status) goto errrun
goto paus
par3:
"$prog" "$1" "$2" "$3"
if ($status) goto errrun
goto paus
par4:
"$prog" "$1" "$2" "$3" "$4"
if ($status) goto errrun
goto paus
par5:
"$prog" "$1" "$2" "$3" "$4" "$5"
if ($status) goto errrun
goto paus
par6:
"$prog" "$1" "$2" "$3" "$4" "$5" "$6"
if ($status) goto errrun
goto paus
par7:
"$prog" "$1" "$2" "$3" "$4" "$5" "$6" "$7"
if ($status) goto errrun
goto paus
par8:
"$prog" "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8"
if ($status) goto errrun
goto paus

# pause
paus:
if ("$prog" == "tcsh") goto done
"$L2HTOP/bin/choose.exe" anykey "Done.  Hit any key to exit"

# done
done:
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errloc:
echo '*** Cannot find the location of the clicked file.'
goto err
errprog:
echo "*** Program $1 not found."
goto err
errab:
echo '*** User aborted.'
goto err
errprot:
echo "*** Insufficient privileges to run $1."
goto err
errinif:
echo "*** Cannot find file $L2HTOP/init.bat.  You must restore folder l2h."
goto err
errini:
echo "*** Error running $L2HTOP/init.bat.  Please rerun the installation"
echo '    procedure.  If this is a fresh installation and you ran the'
echo '    installation procedure, contact the author.'
goto err
errrun:
echo "Program failed: $prog $1 $2 $3 $4 $5 $6 $7"
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
echo -n "Read the error messages above and then hit Enter to exit: "
set ans="$<"
exit 1

end:
