#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: This is a substitute for the linux xdg-open command.
#          Use if your operating system does not support xdg-open
#          or if you want to control opening files yourself.
#          To activate, make a *copy* of this file:
#             cp -p xdg-open.bat xdg-open.
#             chmod u=rwx,go=rx xdg-open.
#          Then edit it to your preferences.

# Usage
goto start
use:
echo 'Usage: xdg-open FILE'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# checks
if !(-f "$1") goto errfil

# show a header
echo
echo '======================================================================'
echo 'The xdg-open script, a substitute for the linux xdg-open command, has'
echo 'been asked to open the file'
echo "   $1"
echo "from location `pwd`."
echo

# direct to the appropriate file extension handler
echo "$1*" | fgrep -i '.txt*' > /dev/null
set stat=$status
if ($stat >= 2) goto errsys
if ($stat >= 1) goto nottxt
goto dotxt
nottxt:
echo "$1*" | fgrep -i '.html*' > /dev/null
set stat=$status
if ($stat >= 2) goto errsys
if ($stat >= 1) goto nothtml
goto dohtml
nothtml:
echo "$1*" | fgrep -i '.htm*' > /dev/null
set stat=$status
if ($stat >= 2) goto errsys
if ($stat >= 1) goto nothtm
goto dohtm
nothtm:
echo "$1*" | fgrep -i '.pdf*' > /dev/null
set stat=$status
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notpdf
goto dopdf
notpdf:
echo "$1*" | fgrep -i '.ps*' > /dev/null
set stat=$status
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notps
goto dops
notps:

# no more options
echo 'But I do not know what to do with this kind of file.'
"$L2HTOP/bin/choose.exe" anykey "Press any key to quit"
goto done

# handle text files
dotxt:
echo
"$L2HTOP/bin/choose.exe" ' q' "Press Space to scroll the file or q to quit"
set stat=$status
if ($stat >= 2) goto done
if ($stat >= 1) goto dotxt2
goto done
dotxt2:
more "$1"
"$L2HTOP/bin/choose.exe" anykey "Press any key to quit"
goto done

# handle html files
dohtml:
dohtm:
set keys=''
which firefox > /dev/null
if !($status) goto dohtmlfy
set keys="q$keys"
goto dohtmlfe
dohtmlfy:
echo ' f: show the web page using firefox'
set keys="f$keys"
dohtmlfe:
which lynx > /dev/null
if !($status) goto dohtmlly
set keys="q$keys"
goto dohtmlle
dohtmlly:
echo ' l: show the web page using lynx'
set keys="l$keys"
dohtmlle:
which netscape > /dev/null
if !($status) goto dohtmlny
set keys="q$keys"
goto dohtmlne
dohtmlny:
echo ' n: show the web page using netscape'
set keys="n$keys"
dohtmlne:
echo ' t: type the raw web page file'
set keys="t$keys"
echo ' q: quit'
set keys="q$keys"
echo
"$L2HTOP/bin/choose.exe" "$keys" "Make a choice from the menu above and press its key"
set stat=$status
if ($stat >= 6) goto done
if ($stat >= 5) goto dohtmlf
if ($stat >= 4) goto dohtmll
if ($stat >= 3) goto dohtmln
if ($stat >= 2) goto dotxt
goto done
dohtmlf:
nohup firefox "$1" >& /dev/null &
goto done
dohtmll:
lynx "$1"
goto done
dohtmln:
nohup netscape "$1" >& /dev/null &
goto done

# handle pdf files
dopdf:
set keys=''
which acroread > /dev/null
if !($status) goto dopdfay
set keys="q$keys"
goto dopdfae
dopdfay:
echo ' a: show the pdf using acroread'
set keys="a$keys"
dopdfae:
which firefox > /dev/null
if !($status) goto dopdffy
set keys="q$keys"
goto dopdffe
dopdffy:
echo ' f: show the pdf using firefox'
set keys="f$keys"
dopdffe:
which gv > /dev/null
if !($status) goto dopdfgy
set keys="q$keys"
goto dopdfge
dopdfgy:
echo ' g: show pdf using gv'
set keys="g$keys"
dopdfge:
which xpdf > /dev/null
if !($status) goto dopdfxy
set keys="q$keys"
goto dopdfxe
dopdfxy:
echo ' x: show pdf using xpdf'
set keys="x$keys"
dopdfxe:
echo ' q: quit'
set keys="q$keys"
echo
"$L2HTOP/bin/choose.exe" "$keys" "Make a choice from the menu above and press its key"
set stat=$status
if ($stat >= 6) goto done
if ($stat >= 5) goto dopdfa
if ($stat >= 4) goto dopdff
if ($stat >= 3) goto dopdfg
if ($stat >= 2) goto dopdfx
goto done
dopdfa:
nohup acroread "$1" >& /dev/null &
goto done
dopdff:
nohup firefox "$1" >& /dev/null &
goto done
dopdfg:
nohup gv "$1" >& /dev/null &
goto done
dopdfx:
nohup xpdf "$1" >& /dev/null &
goto done

# handle ps files
dops:
set keys=''
if !(-f "$1:r.dvi") goto dopsdn
which xdvi > /dev/null
if !($status) goto dopsdy
dopsdn:
set keys="q$keys"
goto dopsde
dopsdy:
echo ' d: show the dvi using xdvi'
set keys="d$keys"
dopsde:
which gv > /dev/null
if !($status) goto dopsgy
set keys="q$keys"
goto dopsge
dopsgy:
echo ' g: show ps using gv'
set keys="g$keys"
dopsge:
echo ' q: quit'
set keys="q$keys"
echo
"$L2HTOP/bin/choose.exe" "$keys" "Make a choice from the menu above and press its key"
set stat=$status
if ($stat >= 4) goto done
if ($stat >= 3) goto dopsd
if ($stat >= 2) goto dopsg
goto done
dopsd:
nohup xdvi "$1:r.dvi" >& /dev/null &
goto done
dopsg:
nohup gv "$1" >& /dev/null &
goto done

# done
done:
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errfil:
echo "*** File $1 not found."
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
