#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Initialize the various l2h settings.
#          1) Set L2HTOP.
#          2) Add . and L2HTOP/bin to the path.
#          3) Set TEXINPUTS as needed for l2h.

# Usage
goto start
use:
echo 'Usage: source init.bat'
goto err
start:

# show a message
echo
echo 'Running init.bat...'
echo

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# Show the old path
echo "PATH was: '$PATH'"

# Set TEXINPUTS if we did not do it yet, and show
echo
if !($?TEXINPUTS) setenv TEXINPUTS ''
if ("$TEXINPUTS" == '') goto noiti2
echo "TEXINPUTS now: '$TEXINPUTS'" | fgrep "'.:$L2HTOP/texinputs:"
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto noiti
goto endti
noiti:
echo "TEXINPUTS was: '$TEXINPUTS'"
noiti2:
echo "':${TEXINPUTS}:'" | fgrep "'::" > /dev/null
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto noinvti
echo "':${TEXINPUTS}:'" | fgrep ":$L2HTOP/texinputs:" > /dev/null
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) setenv TEXINPUTS ":$L2HTOP/texinputs$TEXINPUTS"
setenv TEXINPUTS ".$TEXINPUTS"
noinvti:
echo "':${TEXINPUTS}:'" | fgrep ":$L2HTOP/texinputs:" > /dev/null
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) setenv TEXINPUTS "$L2HTOP/texinputs:$TEXINPUTS"
echo "':${TEXINPUTS}:'" | fgrep "':.:" > /dev/null
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) setenv TEXINPUTS ".:$TEXINPUTS"
echo "':${TEXINPUTS}:'" | fgrep "::'" > /dev/null
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) setenv TEXINPUTS "${TEXINPUTS}:"
echo
echo "TEXINPUTS now: '$TEXINPUTS'"
endti:

# show L2HTOP
echo "L2HTOP now: '$L2HTOP'"

# add . and L2HTOP/bin to the path and show
echo ":${PATH}:" | fgrep ":$L2HTOP/bin:" > /dev/null
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto l2hset
goto l2hend
l2hset:
setenv PATH "$L2HTOP/bin:$PATH"
l2hend:
echo ":${PATH}:" | fgrep ":.:" > /dev/null
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto pntset
goto pntend
pntset:
setenv PATH ".:$PATH"
pntend:
rehash
echo "PATH now: '$PATH'"

# create an xdg-open command by hook or by crook
unalias xdg-open
rehash
which xdg-open
if !($status) goto havxdgo
alias xdg-open 'xdg-open.bat'
goto endxdgo
havxdgo:
alias xdg-open 'xdg-open \!* >& /dev/null'
endxdgo:

# done
done:
echo
echo 'Script init.bat done.'
echo
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
echo '    Initialization failed.'
exit 1

end:
