#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Initialize the l2h menu as needed and check whether
#          index.%1 is newer than the tex sources.

# Usage
goto start
use:
echo 'Usage: source doinit.bat [quiet]'
echo '       source checknewer.bat [FILETYPE]'
echo '       Updates texfiles.lst, texfiles.bat, *pe.bat, *we.bat.'
echo '       Sets errlev=1 if a [re]make of index.FILETYPE is needed.'
goto err
start:

# check that index.tex exists
if !(-f index.tex) goto errfil

# see whether we need to update the relevant files in @system
l2hnewer.exe index.tex @system/texfiles.lst
set stat="$status"
if ($stat >= 2) goto errdat
if ($stat >= 1) goto mklst
l2hnewer.exe @system/texfiles.lst @system/setpe.bat
set stat="$status"
if ($stat >= 2) goto errdat
if ($stat >= 1) goto mklst
l2hnewer.exe @system/texfiles.lst @system/checkpe.bat
set stat="$status"
if ($stat >= 2) goto errdat
if ($stat >= 1) goto mklst
l2hnewer.exe @system/texfiles.lst @system/setwe.bat
set stat="$status"
if ($stat >= 2) goto errdat
if ($stat >= 1) goto mklst
l2hnewer.exe @system/texfiles.lst @system/checkwe.bat
set stat="$status"
if ($stat >= 2) goto errdat
if ($stat >= 1) goto mklst
l2hnewer.exe @system/texfiles.lst @system/texfiles.bat
set stat="$status"
if ($stat >= 2) goto errdat
if ($stat >= 1) goto mklst

# check that nobody has replaced index.tex with one with another engine
if ("$1" == 'log') goto chkpe
if ("$1" == 'dvi') goto chkpe
if ("$1" == 'pdf') goto chkpe
if ("$1" == 'ps') goto chkpe
if ("$1" == 'aux') goto chkwe
goto mklstend
chkpe:
source @system/checkpe.bat
if ($status) goto dodel
goto mklstend
chkwe:
source @system/checkwe.bat
if ($status) goto dodel
goto mklstend

# index.tex is inconsistent with other files; do an emergency delete
dodel:
echo
echo '** An engine in index.tex has unexpectedly changed.'
echo '   (Or you have a bad character, like a tab instead of a space,'
echo '   inside your \pdfengine or \webengine definition.)'
echo '   In any case, some latex-created files are suspect and should be'
echo '   deleted.'
echo
choose.exe pi "Hit Enter to proceed with clean up or i to ignore the issue: p"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto enddel
if ($stat >= 1) goto dodel2
goto errq
dodel2:
if (-f index.log) rm index.log
if (-f index.dvi) rm index.dvi
if (-f index.aux) rm index.aux
enddel:

# update the files in @system
mklst:

# create a list of files involved with index.tex
if (-f @system/texfiles.lst) rm @system/texfiles.lst
if (-f @system/texfiles.lst) goto errsys
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/texfiles.sub" index.tex @system/texfiles.lst
if ($status) goto errlst
if !(-f @system/texfiles.lst) goto errlst
l2hnewer.exe index.tex @system/texfiles.lst
if ($status) goto errfdat

# remove any UTF-8 BOM
cat @system/texfiles.lst | fgrep ' UTF-8 BOM '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto endbom
echo '** Removing a UTF-8 BOM from index.tex'
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/rmbom.sub" index.tex index.@nb
if ($status) goto errbom
if !(-f index.@nb) goto errbom
mv index.tex index@bom.tex
if ($status) goto errsys
if (-f index.tex) goto errsys
mv index.@nb index.tex
if ($status) goto errsys
if !(-f index.tex) goto errsys
endbom:

# create setpe.bat
if (-f @system/setpe.bat) rm @system/setpe.bat
if (-f @system/setpe.bat) goto errsys
echo '#\!/usr/bin/tcsh -f' > @system/setpe.bat
if ($status) goto errsys
if !(-f @system/setpe.bat) goto errsys
cat @system/texfiles.lst | fgrep ' setenv pe ' >> @system/setpe.bat
set stat="$status"
if ($stat >= 2) goto errsys

# create checkpe.bat
if (-f @system/checkpe.bat) rm @system/checkpe.bat
if (-f @system/checkpe.bat) goto errsys
echo '#\!/usr/bin/tcsh -f' > @system/checkpe.bat
if ($status) goto errsys
if !(-f @system/checkpe.bat) goto errsys
cat @system/texfiles.lst | fgrep 'newcommand{\pdfengine}' >> @system/checkpe.bat
set stat="$status"
if ($stat >= 2) goto errsys

# create setwe.bat
if (-f @system/setwe.bat) rm @system/setwe.bat
if (-f @system/setwe.bat) goto errsys
echo '#\!/usr/bin/tcsh -f' > @system/setwe.bat
if ($status) goto errsys
if !(-f @system/setwe.bat) goto errsys
cat @system/texfiles.lst | fgrep ' setenv we ' >> @system/setwe.bat
set stat="$status"
if ($stat >= 2) goto errsys

# create checkwe.bat
if (-f @system/checkwe.bat) rm @system/checkwe.bat
if (-f @system/checkwe.bat) goto errsys
echo '#\!/usr/bin/tcsh -f' > @system/checkwe.bat
if ($status) goto errsys
if !(-f @system/checkwe.bat) goto errsys
cat @system/texfiles.lst | fgrep 'newcommand{\webengine}' >> @system/checkwe.bat
set stat="$status"
if ($stat >= 2) goto errsys

# create texfiles.bat
if (-f @system/texfiles.bat) rm @system/texfiles.bat
if (-f @system/texfiles.bat) goto errsys
subbin.exe "$L2HTOP/system-files/data/mkprocfils.sub" @system/texfiles.lst @system/texfiles.bat
if ($status) goto errmbat
if !(-f @system/texfiles.bat) goto errmbat

# end of creating the files in @system
mklstend:

# done if no extension
if ("$1" == '') goto done

# see whether index.%1 is newer than the sources.
set errlev=1
if !(-f "index.$1") goto done
set errlev=0
source @system/texfiles.bat "$L2HTOP/system-files/bin/newer.bat" index."$1"
if ($status) goto errbat
if ("$errlev" == 2) goto errbat

# done
done:
goto end

# errors
errfil:
pwd
echo '*** In the current folder, shown above, there is no LaTeX source file'
echo '    index.tex that can be turned into a pdf or web pages.'
goto err
errdat:
echo '*** Unknown error comparing the dates of files.'
goto err
errq:
echo '*** Aborted by user.'
goto err
errlst:
echo '*** Unable to find the list of files that index.tex depends on.'
echo '    See above or check for unusual \input or \include commands.'
goto err
errfdat:
if (-f @system/texfiles.lst) rm @system/texfiles.lst
echo '*** File index.tex seems to exist in the future only?  That will'
echo '    cause problems.  To give index.tex a sensible date, try making a'
echo '    trivial change to the file with your text editor, like adding and'
echo '    a space and deleting it again.  Then save index.tex.  After that,'
echo '    try whatever you were doing again.'
goto err
errbom:
echo '*** Unable to remove the UTF-8 BOM from index.tex.'
goto err
errmbat:
echo '*** Problem converting the list of files that index.tex depends on.'
echo '    See above or check for unusual \input or \include commands.'
goto err
errbat:
echo '*** Problem examining the dates of index.tex and associated files.'
echo '    See above or check index.tex for unusual \input or \include commands.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
