#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Check the tex in ROOT.EXT for latex2html problems.

# Usage
goto start
use:
echo 'Usage: set errlev=0'
echo '       source checktex.bat "ROOT" EXT'
echo '       Returns status 1 and errlev=1 if the file has errors.'
echo '       Returns status 1 and errlev=2 if there were processing errors.'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/system-files/data/checktex.sub") goto errl2h
if !(-f "$L2HTOP/system-files/bin/subbin_big.exe") goto errl2h

# skip if not a tex file or not found
echo "$1.$2?" | fgrep -i '.tex?'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto done
if !(-f "$1.$2") goto done

# run subbin.exe
if (-f "$1.@err") rm "$1.@err"
if (-e "$1.@err") goto errsys
echo "Checking $1.$2 for latex2html problems."
"$L2HTOP/system-files/bin/subbin_big.exe" "$L2HTOP/system-files/data/checktex.sub" "$1"."$2" "$1".@err
if ($status) goto errsub
if !(-f "$1.@err") goto done
cat "$1".@err | fgrep '?:'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto clean
echo "*** Please fix the above problems in file $1.$2 first."
set errlev=1
goto err2

# delete the error log
clean:
rm "$1.@err"

# done
done:
goto end

# errors
errl2h:
echo '*** Fatal error: cannot find essential l2h files.'
echo '    You may need to rerun install_l2h in the l2h folder.  Or even'
echo '    re-install l2h from scratch.'
goto err
errsub:
echo "*** Error checking file $1.$2 for latex2html problems."
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
set errlev=2
err2:
exit 1

end:
