#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Delete all backups of a given file type in the current folder.
#          Also deletes the list files.
#          Does not delete any files in subfolders.

# Usage
goto start
use:
echo 'Usage: delall.bat EXT EXT EXT ...'
echo '       Deletes *.EXT and @EXTlist*.bat after asking permission.'
echo '       EXT should not be quoted in DOS, but should be in unix.'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# check the basic files needed for execution
if !(-f "$L2HTOP/bin/choose.exe") goto errfils

# clear screen
clear

# loop on the extensions provided
loop:
if ("$1" == '') goto done

# see whether files will be deleted
if (-f "@$1list.lst") goto havfils
if (-f "@$1list.bat") goto havfils
if (-f "@$1list@old.bat") goto havfils
if (-f "@$1list@older.bat") goto havfils
if (-f "@$1list@oldest.bat") goto havfils
goto doloop
havfils:

# ask permission to delete the files
echo "Do you really want to permanently delete all $1 backups?"
"$L2HTOP/bin/choose.exe" ynq "Press y to delete them, n to leave them, or q to leave them and quit"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto done
if ($stat >= 2) goto doloop
if ($stat >= 1) goto dodel
goto errq

# delete them
dodel:
"$L2HTOP/system-files/bin/delbcksets.bat" "@$1list" '*'
if (-f "@$1list.lst") rm "@$1list.lst"
if (-f "@$1list.bat") rm "@$1list.bat"

# loop back
doloop:
shift
goto loop

# done
done:
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errfils:
echo '*** Fatal error: essential files are missing from the l2h folder.'
echo '    You may need to reinstall l2h from scratch.'
goto err
errq:
echo '*** User aborted.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
