#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Load index.tex in an editor.

# Usage
goto start
use:
echo 'Usage: source doinit.bat [quiet]'
echo '       doedit.bat [N]'
echo '       Specifying N will create and obey @system/open_index.no'
goto err
start:

# check that @system/open_index.no does not exist
if ("$1" == '') goto always
if (-f @system/open_index.no) goto done
always:

# check that index.tex exists
if !(-f index.tex) goto errfil

# start of the search for editors
clear
echo 'Searching for typical suitable editors on your system...'
echo

# initialize a "found one" status
set stat=1

# initialize the keys
set keys='NwsmEe'

# search for gedit
which gedit > @tmp.tmp
if ($status) goto show_e2
echo '  e  gedit (The standard GNU linux text editor.  This is a good choice'
echo '     if you are just starting to learn what LaTeX is.)'
echo
set stat=0
set keys='e'
goto show_ee
show_e2:
which nano > @tmp.tmp
if ($status) goto show_e3
echo '  e  nano (A standard unix text editor.  This is a good choice'
echo '     if you are just starting to learn what LaTeX is.  But it may'
echo '     not show multinational characters correctly.)'
echo
set stat=0
set keys='e'
goto show_ee
show_e3:
which pico > @tmp.tmp
if ($status) goto show_en
echo '  e  pico (A standard unix text editor.  This is a good choice'
echo '     if you are just starting to learn what LaTeX is.  But it may'
echo '     not show multinational characters correctly.)'
echo
set stat=0
set keys='e'
goto show_ee
show_en:
set keys='n'
show_ee:

# search for emacs
which emacs > @tmp.tmp
if ($status) goto show_cen
echo '  E  Emacs (This is a good choice if you know how to use it.)'
echo
set stat=0
set keys="E$keys"
goto show_cee
show_cen:
set keys="n$keys"
show_cee:

# search for texmaker
which texmaker > @tmp.tmp
if ($status) goto show_mn
echo '  m  TeXmaker (An advanced LaTeX editor.  This is a good choice if'
echo '     you already have a bit of experience with LaTeX.)'
echo
set stat=0
set keys="m$keys"
goto show_me
show_mn:
set keys="n$keys"
show_me:

# search for texstudio
which texstudio > @tmp.tmp
if ($status) goto show_sn
echo '  s  TeXstudio (A powerful LaTeX editor.  This is an excellent choice'
echo '     if you already have a bit of experience with LaTeX.)'
echo
set stat=0
set keys="s$keys"
goto show_se
show_sn:
set keys="n$keys"
show_se:

# search for texworks
which texworks > @tmp.tmp
if ($status) goto show_wn
echo '  w  TeXworks (A LaTeX editor that comes with TeXlive.  This is a good'
echo '     choice if you already have a bit of experience with LaTeX.)'
echo
set stat=0
set keys="w$keys"
goto show_we
show_wn:
set keys="n$keys"
show_we:

# notify if none were found
if ("$stat" != 0) goto wrnnone

# allow cop out
if ("$1" == '') goto cop_n
echo '  N  Never show this editor menu uninvited again.'
echo
set keys="N$keys"
goto cop_e
cop_n:
set keys="n$keys"
cop_e:

# show warning
echo 'WARNING: If you select an editor from the list above, it will be opened'
echo 'in another window.  But you still need the current, l2h, window too.'
echo 'Use the editor to change the document, but this window to process the'
echo 'changes.'
echo

# ask the user's choice
echo 'To open LaTeX document index.tex with an editor, press its letter from'
choose.exe n$keys "the list above.  Or press n to open index.tex yourself: n"
set stat="$status"
if ($stat >= 8) goto errsys
if ($stat >= 7) goto menu_e
if ($stat >= 6) goto menu_ce
if ($stat >= 5) goto menu_m
if ($stat >= 4) goto menu_s
if ($stat >= 3) goto menu_w
if ($stat >= 2) goto dontask
if ($stat >= 1) goto done
goto wrnq

# start basic editor
menu_e:
which gedit
if ($status) goto menu_e2
nohup gedit -b -s index.tex >& /dev/null &
if ($status) goto wrnfail
goto done
menu_e2:
which nano
if ($status) goto menu_e3
xterm -title 'index.tex' -e nano index.tex >& /dev/null &
if ($status) goto wrnfail
goto done
menu_e3:
xterm -title 'index.tex' -e pico index.tex >& /dev/null &
if ($status) goto wrnfail
goto done

# start emacs
menu_ce:
nohup emacs index.tex >& /dev/null &
if ($status) goto wrnfail
goto done

# start texmaker
menu_m:
nohup texmaker index.tex >& /dev/null &
if ($status) goto wrnfail
goto done

# start texstudio
menu_s:
nohup texstudio index.tex >& /dev/null &
if ($status) goto wrnfail
goto done

# start texworks
menu_w:
nohup texworks index.tex >& /dev/null &
if ($status) goto wrnfail
goto done

# don't ask this again
dontask:
echo 'Do not ask to open index.tex with a text editor' > @system/open_index.no
if !(-f @system/open_index.no) goto errsys
goto done

# done
done:
if (-f @tmp.tmp) rm @tmp.tmp
goto end

# warnings
wrnnone:
echo
echo 'No editors were found, not even pico.  Right-click index.tex or use'
echo 'the menus to load index.tex into a suitable editor, like pico or gedit.'
echo 'Stay away from word processors like Open Office, LyX, MS Word, ...'
echo 'These are not compatible with LaTeX files.  Instead consider something'
echo 'like the very nice TeXstudio after you gain some experience with LaTeX.'
echo
choose.exe anykey "After reading the above, hit any key to exit"
goto done
wrnq:
echo '*** Aborted by user.'
goto done
wrnfail:
echo
echo '*** Error opening index.tex with the selected editor.  Please use'
echo '    the Windows menus to do so.'
echo
choose.exe anykey "After reading the above, hit any key to exit"
goto done

# errors
errfil:
pwd
echo '*** In the current folder, shown above, there is no LaTeX source file'
echo '    index.tex that can be turned into a pdf or web pages.'
goto err
errwhich:
echo '*** doinit.bat has not been run correctly: @system/which.bat is missing.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
