#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Process a set of files to create new ones.
#          Does not maintain backups.
#          Trashes any file @tmp0.bat.

# Usage
goto start
use:
echo 'Usage: dofiles0.bat LISTROOT FILES EXT2 SCRIPT PARS'
echo '       LISTROOT is the root file name for the list of files to convert.'
echo '          If LISTROOT starts with @, it must be quoted in DOS.'
echo '          LISTROOT.bat is created from FILES.'
echo '       FILES is the file specification of the files to convert.'
echo '          Applies to the current folder and its subfolders.'
echo '          Should not be quoted in DOS.'
echo '          If FILES is - a file list LISTROOT.lst must already exist'
echo '             In such a file list, a specification NAME.F0L is'
echo '             assumed to indicate a extensionless folder named NAME.'
echo '             Otherwise folders are not supported at this time.'
echo '       EXT2 is the extension of the created files.'
echo '          If EXT2 is * the source file extension is used.'
echo '          EXT2 should be * if the source file extension contains wildcards.'
echo '       SCRIPT is the single-file script to use to create a new file.'
echo '          May not be quoted in DOS.'
echo '          Normally SCRIPT will be called for each file NAME.EXT as'
echo '             CALL SCRIPT NAME EXT EXT2 PARS'
echo '          SCRIPT may receive additional nil parameters in unix.'
echo '          SCRIPT must be in system-files/bin.'
echo '          If SCRIPT returns a nonzero error status for any single file,'
echo '          all processing aborts.'
echo '       PARS are the parameters used by the script following NAME EXT EXT2.'
echo '          If dofiles0 is used in a runscript file, only three (four) such'
echo '          parameters can be accommodated at this time.  Otherwise there'
echo '          can be five.'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# check the basic files needed for execution
if !(-rx "$L2HTOP/bin/filechk.bat") goto errfils
"$L2HTOP/bin/filechk.bat"
if ($status) goto errfils
if !(-rx "$L2HTOP/system-files/bin/filechk.bat") goto errfils
"$L2HTOP/system-files/bin/filechk.bat"
if ($status) goto errfils
if !(-rx "$L2HTOP/system-files/data/filechk.bat") goto errfils
"$L2HTOP/system-files/data/filechk.bat"
if ($status) goto errfils

# check the validity of SCRIPT
if !(-r "$L2HTOP/system-files/bin/$4") goto errscript

# initialize errlev
set errlev='0'

# see whether an existing list of files is to be used
if ("$2" != '-') goto mklst
if !(-f "$1.lst") goto errlst
goto chklst

# create the file list
mklst:
echo "$2" | fgrep '.'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto errspec
if (-f "$1.lst") rm "$1.lst"
if (-e "$1.lst") goto errsys
find . -name "$2" -type f > "$1".lst
if ($status) goto errmklst
if !(-f "$1.lst") goto errsys

# check the list for file types
chklst:
cat "$1".lst | fgrep -v '.'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto havpts
goto errpts
havpts:
cat "$1".lst | fgrep '.' > "$1".tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto errnon
if (-f "$1.tmp") rm "$1.tmp"

# create the batch file
if (-f "$1.bat") rm "$1.bat"
if (-e "$1.bat") goto errsys
"$L2HTOP/bin/subbin.exe" "$L2HTOP/system-files/data/mkprocfils.sub" "$1".lst "$1".bat
if ($status) goto errmkbat
if !(-f "$1.bat") goto errmkbat

# run the script with its parameters
"$L2HTOP/bin/clearold.bat" y @tmp0.bat
if ($status) goto err
set errlev='0'
source "$1".bat "$L2HTOP/system-files/bin/$4" "$3" "$5" "$6" "$7" "$8" "$9"
if ($status) goto errcnv
"$L2HTOP/bin/clearold.bat" y @tmp0.bat
if ($status) goto err

# done
done:
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errfils:
echo '*** Fatal error: essential files are missing from the l2h folder'
echo '    or you do not have enough access to them.'
echo '    You may need to reinstall l2h from scratch.'
goto err
errscript:
echo "*** Program error: There is no $4 in $L2HTOP/system-files/bin"
echo '    or you do not have enough access to it.'
goto err
errlst:
echo "*** Program error: The file list $1.lst is missing"
goto err
errspec:
echo "*** Program error: file specification $2 contains no file type"
goto err
errmklst:
echo "*** Error searching for files matching $2"
goto err
errpts:
echo '*** Fatal error: There are files without filetypes in the list of files'
echo '      to convert.'
goto err
errnon:
echo '*** Fatal error: The list of files to convert is empty'
goto err
errmkbat:
echo '*** Unable to create a script to process the selected files.'
goto err
errnodel:
echo '*** There are no files to delete.'
goto err
errcnv:
echo '*** Error processing the files.'
goto err
errq:
echo '*** User aborted.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
