#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: convert an image NAME.EXT into an image NAME@new.EXT2
#          In unix the created file will have the permissions of NAME.EXT.

# Usage
goto start
use:
echo 'Usage: source file_anyany.bat NAME EXT EXT2 PAR PARS'
echo '       NAME.EXT is the image to convert.'
echo '          EXT may be bmp, gif, jpg, jpeg, or png (case-insensitive).'
echo '       NAME@new.EXT2 is the converted file.'
echo '          EXT2 may be one of the EXT list, eps, or *.'
echo '          Aborts if NAME@new.EXT2 already exists.'
echo '       PAR is a parameter to file_anyany.bat, use _ plus:'
echo '          crop_ to crop the picture'
echo '          smaller_ to reduce the picture size'
echo '       PARS are the parameters to p?mtoEXT2, normally no more than 3.'
echo '          They should *not* be quoted in DOS.'
echo '       Uses @tmp*.tmp and @tmp.bat as temporaries.'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# list file
echo
echo "Processing $1.$2"

# check that the file to convert exists
if !(-f "$1.$2") goto wrnfil

# error if the target exists
if ("$3" == '*') goto out2
if (-e "$1@new.$3") goto errnew
goto outok
out2:
if (-e "$1@new.$2") goto errnew2
outok:

# clear temporary files
"$L2HTOP/bin/clearold.bat" y @tmp0.tmp @tmp.tmp @tmp1.tmp @tmp1.log @tmp2.tmp @tmp3.tmp @tmp.bat
if ($status) goto err

# select the initial conversion to pnm
echo " $2 " | fgrep -i ' bmp '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notbmp1
goto bmp
notbmp1:
echo " $2 " | fgrep -i ' gif '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notgif1
goto gif
notgif1:
echo " $2 " | fgrep -i ' jpg '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notjpg1
goto jpg
notjpg1:
echo " $2 " | fgrep -i ' jpeg '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notjpeg1
goto jpg
notjpeg1:
echo " $2 " | fgrep -i ' png '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notpng1
goto png
notpng1:
goto errtyp

# convert bmp
bmp:
"$L2HTOP/bin/bmptoppm" "$1"."$2" > @tmp.tmp
if ($status) goto errcnv
goto havpnm

# convert gif
gif:
"$L2HTOP/bin/giftopnm" "$1"."$2" > @tmp.tmp
if ($status) goto errcnv
goto havpnm

# convert jpg
jpg:
"$L2HTOP/bin/djpeg" "$1"."$2" > @tmp.tmp
if ($status) goto jpg2
goto havpnm
jpg2:
if !(-f @tmp.tmp) goto errcnv
cat @tmp.tmp | fgrep -c 'P'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto errcnv
echo "** Jpeg image $1.$2 seems to be corrupt."
echo '   However, some part may have been read OK.  Do you want to try'
echo '   to process that part to see whether it is usable?'
"$L2HTOP/bin/choose.exe" ny "Press y for yes or n for no: n"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto havpnm0
if ($stat >= 1) goto errcnv
goto errq

# convert png
png:
which pngtopnm > /dev/null
if ($status) goto errpng
pngtopnm "$1"."$2" > @tmp.tmp
if ($status) goto errcnv
goto havpnm

# pnm @tmp.tmp should have been created
havpnm0:
havpnm:
if !(-f @tmp.tmp) goto errsys

# apply any requested cropping
if ("$4" == '') goto crope
if ("$4" == _) goto crope
echo "$4" | fgrep -i '_crop_'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto cropr
if !(-e @tmp0.bat) then
    echo '#\!/usr/bin/tcsh -f' > @tmp0.bat
    if ($status) goto errsys
endif
if !(-f @tmp0.bat) goto errsys
cat @tmp0.bat | fgrep 'if ("$1" == crop) set errlev='
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto cropa
goto cropx
cropa:
set errlev='n'
if (-f cropdef.bat) source cropdef.bat
if ("$errlev" == '') goto errsys
echo
echo 'PLEASE SPECIFY WHAT CROPS TO DO ON THE IMAGE(S):'
echo
echo 'Any single crop removes a band of a single color (not pattern).'
echo 'For example, if an image has a bottom band of a single color going all'
echo 'the way from left to right, then a bottom (b) cut will remove it.'
cropa0:
echo
if ("$errlev" == n) goto cropa2
echo ' n: no crops, ignore the current list of crops'
echo ' 0: reset the list of crops to none'
echo ' 4: add left, right, top, and bottom crops to the list of crops'
echo ' l: add a left crop to the list of crops'
echo ' r: add a right crop to the list of crops'
echo ' t: add a top crop to the list of crops'
echo ' b: add a bottom crop to the list of crops'
echo ' c: crop according to the current list'
echo ' q: Abort processing.'
echo
cropa1:
echo "Pressing c will crop according to the list '$errlev'"
"$L2HTOP/bin/choose.exe" n04lrtbcq "Please make a choice from the menu above and press its character: c"
set stat="$status"
if ($stat >= 10) goto errsys
if ($stat >= 9) goto errq
if ($stat >= 8) goto cropae
if ($stat >= 7) goto cropab
if ($stat >= 6) goto cropat
if ($stat >= 5) goto cropar
if ($stat >= 4) goto cropal
if ($stat >= 3) goto cropaf
if ($stat >= 2) goto cropaz
if ($stat >= 1) goto cropan
goto errq
cropa2:
echo ' n: no crops'
echo ' 4: initialize the list of crops to left, right, top, and bottom crops'
echo ' l: initialize the list of crops to a left crop'
echo ' r: initialize the list of crops to a right crop'
echo ' t: initialize the list of crops to a top crop'
echo ' b: initialize the list of crops to a bottom crop'
echo ' q: Abort processing.'
echo
"$L2HTOP/bin/choose.exe" n4lrtbq "Please make a choice from the menu and press its character: n"
set stat="$status"
if ($stat >= 8) goto errsys
if ($stat >= 7) goto errq
if ($stat >= 6) goto cropab
if ($stat >= 5) goto cropat
if ($stat >= 4) goto cropar
if ($stat >= 3) goto cropal
if ($stat >= 2) goto cropaf
if ($stat >= 1) goto cropae
goto errq
cropaz:
set errlev='n'
goto cropa0
cropaf:
if ("$errlev" == n) goto cropafn
set errlev="${errlev}lrtb"
goto cropa1
cropafn:
set errlev='lrtb'
goto cropa0
cropab:
if ("$errlev" == n) goto cropabn
set errlev="${errlev}b"
goto cropa1
cropabn:
set errlev='b'
goto cropa0
cropat:
if ("$errlev" == n) goto cropatn
set errlev="${errlev}t"
goto cropa1
cropatn:
set errlev='t'
goto cropa0
cropar:
if ("$errlev" == n) goto croparn
set errlev="${errlev}r"
goto cropa1
croparn:
set errlev='r'
goto cropa0
cropal:
if ("$errlev" == n) goto cropaln
set errlev="${errlev}l"
goto cropa1
cropaln:
set errlev='l'
goto cropa0
cropan:
set errlev='n'
cropae:
echo 'if ("$1" == crop) set errlev='"'$errlev'" >> @tmp0.bat
if ($status) goto errsys
cropx:
set errlev=''
source "@tmp0.bat" crop
if ("$errlev" == '') goto errsys
if ("$errlev" == n) goto cropr
"$L2HTOP/system-files/bin/l2hcrop.exe" -n @tmp.tmp @tmp1.tmp -$errlev - - -
set stat="$status"
if ($stat >= 100) goto cropd
goto errcrop
cropd:
if !(-f @tmp1.tmp) goto errcrop
cmp -s @tmp.tmp @tmp1.tmp
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto cropm
"$L2HTOP/bin/clearold.bat" y @tmp1.tmp @tmp1.log
if ($status) goto err
goto cropr
cropm:
"$L2HTOP/bin/movenew.bat" @tmp.tmp @tmp0.tmp y
if ($status) goto err
"$L2HTOP/bin/movenew.bat" @tmp1.tmp @tmp.tmp y
if ($status) goto err
"$L2HTOP/bin/clearold.bat" y @tmp1.log
if ($status) goto err
cropr:
set errlev='0'
crope:

# apply any requested compression
if ("$4" == '') goto smallere
if ("$4" == _) goto smallere
echo "$4" | fgrep -i '_smaller_'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto smallerr
if !(-e @tmp0.bat) then
    echo '#\!/usr/bin/tcsh -f' > @tmp0.bat
    if ($status) goto errsys
endif
if !(-f @tmp0.bat) goto errsys
cat @tmp0.bat | fgrep 'if ("$1" == smaller) set errlev='
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto smallera
goto smallerx
smallera:
"$L2HTOP/bin/clearold.bat" y @tmp.bat
if ($status) goto err
set errlev='n'
if (-f sizedef.bat) source sizedef.bat
if ("$errlev" == '') goto errsys
if ("$errlev" == n) goto smallerd
if ("$errlev" == f) goto smallerd
if ("$errlev" == h) goto smallerd
if ("$errlev" == v) goto smallerd
if ("$errlev" == b) goto smallerd
set errlev='n'
smallerd:
echo
echo 'PLEASE SPECIFY THE SIZE REDUCTION METHOD FOR THE IMAGE(S):'
echo
echo ' n: no size reduction'
echo ' f: Specify a factor to compress the horizontal and vertical sizes by.'
echo ' h: Specify the maximum pixels horizontally.'
echo ' v: Specify the maximum pixels vertically.'
echo ' b: Specify the maximum pixels both horizontally and vertically.'
echo ' q: Abort processing.'
"$L2HTOP/bin/choose.exe" nfhvbq "Please make a choice from the menu and press its character: $errlev"
set stat="$status"
if ($stat >= 7) goto errsys
if ($stat >= 6) goto errq
if ($stat >= 5) goto smaller4
if ($stat >= 4) goto smaller3
if ($stat >= 3) goto smaller2
if ($stat >= 2) goto smaller1
if ($stat >= 1) goto smallern
goto errq
smaller4:
set errlev='1024'
if (-f horpixdef.bat) source horpixdef.bat
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " askint "Please enter the maximum number of pixels horizontally" errlev 2 65535 $errlev
if ($status) goto err
if !(-f @tmp.bat) goto errq
set errlev=''
source "@tmp.bat"
if ("$errlev" == '') goto errsys
"$L2HTOP/bin/echon.exe" 'if ("$1" == smaller) set errlev='"'H$errlev" >> @tmp0.bat
if ($status) goto errsys
"$L2HTOP/bin/clearold.bat" y @tmp.bat
if ($status) goto err
set errlev='1024'
if (-f verpixdef.bat) source verpixdef.bat
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " askint "Please enter the maximum number of pixels vertically" errlev 2 65535 $errlev
if ($status) goto err
if !(-f @tmp.bat) goto errq
set errlev=''
source "@tmp.bat"
if ("$errlev" == '') goto errsys
echo "V$errlev'" >> @tmp0.bat
if ($status) goto errsys
goto smallerx
smaller3:
set errlev='1024'
if (-f verpixdef.bat) source verpixdef.bat
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " askint "Please enter the maximum number of pixels vertically" errlev 2 65535 $errlev
if ($status) goto err
if !(-f @tmp.bat) goto errq
set errlev=''
source "@tmp.bat"
if ("$errlev" == '') goto errsys
echo 'if ("$1" == smaller) set errlev='"'V$errlev'" >> @tmp0.bat
if ($status) goto errsys
goto smallerx
smaller2:
set errlev='1024'
if (-f horpixdef.bat) source horpixdef.bat
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " askint "Please enter the maximum number of pixels horizontally" errlev 2 65535 $errlev
if ($status) goto err
if !(-f @tmp.bat) goto errq
set errlev=''
source "@tmp.bat"
if ("$errlev" == '') goto errsys
echo 'if ("$1" == smaller) set errlev='"'H$errlev'" >> @tmp0.bat
if ($status) goto errsys
goto smallerx
smaller1:
set errlev='1'
if (-f factordef.bat) source factordef.bat
echo 'Please enter the desired image compression factor now.'
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " asknum "Please enter a compression factor greater than 1" errlev 1 100 $errlev
if ($status) goto err
if !(-f @tmp.bat) goto errq
set errlev=''
source "@tmp.bat"
if ("$errlev" == '') goto errsys
echo 'if ("$1" == smaller) set errlev='"'$errlev'" >> @tmp0.bat
if ($status) goto errsys
goto smallerx
smallern:
echo 'if ("$1" == smaller) set errlev=n' >> @tmp0.bat
if ($status) goto errsys
smallerx:
set errlev=''
source "@tmp0.bat" smaller
if ("$errlev" == '') goto errsys
if ("$errlev" == n) goto smallerr
"$L2HTOP/system-files/bin/l2hcomp.exe" -n @tmp.tmp @tmp1.tmp -$errlev
set stat="$status"
if ($stat >= 100) goto smallerm
if ($stat >= 20) goto smallerr
goto errsiz
smallerm:
if !(-f @tmp1.tmp) goto errsiz
"$L2HTOP/bin/movenew.bat" @tmp.tmp @tmp0.tmp y
if ($status) goto err
"$L2HTOP/bin/movenew.bat" @tmp1.tmp @tmp.tmp y
if ($status) goto err
smallerr:
set errlev='0'
smallere:

# decide whether to create a new copy of an unchanged file
if ("$3" != '*') goto newe
if ("$4" == _) goto newe
if (-f @tmp0.tmp) goto newe
if !(-e @tmp0.bat) then
    echo '#\!/usr/bin/tcsh -f' > @tmp0.bat
    if ($status) goto errsys
endif
if !(-f @tmp0.bat) goto errsys
cat @tmp0.bat | fgrep 'if ("$1" == new) set errlev='
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto newa
goto newx
newa:
set errlev='n'
if (-f newdef.bat) source newdef.bat
if ("$errlev" == y) goto newa1
set errlev='n'
newa1:
echo
echo "** Image $1.$2 is unchanged."
echo 'Make new copies of unchanged images like this?'
"$L2HTOP/bin/choose.exe" ynq "Press y for yes, n for no, or q to abort processing: $errlev"
set stat="$status"
if ($stat >= 4) goto errsys
if ($stat >= 3) goto errq
if ($stat >= 2) goto newan
if ($stat >= 1) goto neway
goto errq
neway:
set errlev='y'
goto newae
newan:
set errlev='n'
goto newae
newae:
echo 'if ("$1" == new) set errlev='"'$errlev'" >> @tmp0.bat
if ($status) goto errsys
newx:
set errlev=''
source "@tmp0.bat" new
if ("$errlev" == y) goto newr
if ("$errlev" == n) goto wrnsame
goto errsys
newr:
set errlev='0'
newe:

# clean up the temporaries
"$L2HTOP/bin/clearold.bat" y @tmp0.tmp @tmp1.tmp
if ($status) goto err

# select the conversion to the final format
if ("$3" == '*') goto sel2
echo " $3 " | fgrep -i ' bmp '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notbmp2
goto tobmp
notbmp2:
echo " $3 " | fgrep -i ' eps '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto noteps2
goto toeps
noteps2:
echo " $3 " | fgrep -i ' gif '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notgif2
goto togif
notgif2:
echo " $3 " | fgrep -i ' jpg '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notjpg2
goto tojpg
notjpg2:
echo " $3 " | fgrep -i ' jpeg '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notjpeg2
goto tojpg
notjpeg2:
echo " $3 " | fgrep -i ' png '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notpng2
goto topng
notpng2:
goto errtyp2
sel2:
echo " $2 " | fgrep -i ' bmp '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notbmp3
goto tobmp
notbmp3:
echo " $2 " | fgrep -i ' gif '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notgif3
goto togif
notgif3:
echo " $2 " | fgrep -i ' jpg '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notjpg3
goto tojpg
notjpg3:
echo " $2 " | fgrep -i ' jpeg '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notjpeg3
goto tojpg
notjpeg3:
echo " $2 " | fgrep -i ' png '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto notpng3
goto topng
notpng3:
goto errprog

# convert to bmp (might need -bpp=24)
tobmp:
"$L2HTOP/bin/ppmtobmp" $5 $6 $7 $8 $9 @tmp.tmp > @tmp1.tmp
if ($status) goto errcnv2
goto ok

# convert to eps (might need -scale=0.25 -noturn)
toeps:
if ("$5" != '-scale=') goto toeps1
if !(-e @tmp0.bat) then
    echo '#\!/usr/bin/tcsh -f' > @tmp0.bat
    if ($status) goto errsys
endif
if !(-f @tmp0.bat) goto errsys
cat @tmp0.bat | fgrep 'if ("$1" == scale) set errlev='
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto toepsa
goto toeps0
toepsa:
"$L2HTOP/bin/clearold.bat" y @tmp.bat
if ($status) goto err
echo
echo 'PLEASE SPECIFY THE SCALE OF THE PICTURE(S):'
echo
echo 'This determines how big the eps pictures will show up in your document.'
echo 'Warning: if you make this value too big, graininess may show up in the'
echo 'printed picture.  To get a normal 300 dpi quality, this should be 0.25'
set errlev='0.25'
if (-f epsscaledef.bat) source epsscaledef.bat
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " asknum "Please enter the picture scale" errlev 0.0625 2 $errlev
if ($status) goto err
if !(-f @tmp.bat) goto errq
set errlev=''
source "@tmp.bat"
if ("$errlev" == '') goto errsys
echo 'if ("$1" == scale) set errlev='"'$errlev'" >> @tmp0.bat
if ($status) goto errsys
toeps0:
set errlev=''
source "@tmp0.bat" scale
if ("$errlev" == '') goto errsys
"$L2HTOP/bin/pnmtops" $5$errlev $6 $7 $8 $9 @tmp.tmp > @tmp1.tmp
if ($status) goto errcnv2
goto ok
toeps1:
"$L2HTOP/bin/pnmtops" $5 $6 $7 $8 $9 @tmp.tmp > @tmp1.tmp
if ($status) goto errcnv2
goto ok

# convert to gif (might need '-transparent=#FFFFFF', needs quotes)
togif:
"$L2HTOP/bin/pnmcolormap" 256 @tmp.tmp > @tmp2.tmp
if ($status) goto errcnv2
if !(-f @tmp2.tmp) goto errsys
"$L2HTOP/bin/pnmremap" -map=@tmp2.tmp @tmp.tmp > @tmp3.tmp
if ($status) goto errcnv2
if !(-f @tmp3.tmp) goto errsys
if ("$5" != '-transparent=#') goto togif1
if !(-e @tmp0.bat) then
    echo '#\!/usr/bin/tcsh -f' > @tmp0.bat
    if ($status) goto errsys
endif
if !(-f @tmp0.bat) goto errsys
cat @tmp0.bat | fgrep 'if ("$1" == transcol) set errlev='
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto togifa
goto togif0
togifa:
"$L2HTOP/bin/clearold.bat" y @tmp.bat
if ($status) goto err
echo
echo 'PLEASE SPECIFY THE COLOR TO MAKE TRANSPARENT:'
echo
set errlev='FFFFFF'
if (-f giftransdef.bat) source giftransdef.bat
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " askcol "Please enter the color to make transparent or ? for help" errlev $errlev
if ($status) goto err
if !(-f @tmp.bat) goto errq
set errlev=''
source "@tmp.bat"
if ("$errlev" == '') goto errsys
echo 'if ("$1" == transcol) set errlev='"'$errlev'" >> @tmp0.bat
if ($status) goto errsys
togif0:
set errlev=''
source "@tmp0.bat" transcol
if ("$errlev" == '') goto errsys
"$L2HTOP/bin/ppmtogif" $5$errlev $6 $7 $8 $9 @tmp3.tmp > @tmp1.tmp
if ($status) goto errcnv2
goto ok
togif1:
"$L2HTOP/bin/ppmtogif" $5 $6 $7 $8 $9 @tmp3.tmp > @tmp1.tmp
if ($status) goto errcnv2
goto ok

# convert to jpg (might need --quality=90)
tojpg:
if ("$5" != '--quality=') goto tojpg1
if !(-e @tmp0.bat) then
    echo '#\!/usr/bin/tcsh -f' > @tmp0.bat
    if ($status) goto errsys
endif
if !(-f @tmp0.bat) goto errsys
cat @tmp0.bat | fgrep 'if ("$1" == qual) set errlev='
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto tojpga
goto tojpg0
tojpga:
"$L2HTOP/bin/clearold.bat" y @tmp.bat
if ($status) goto err
echo
echo 'PLEASE SPECIFY THE JPEG QUALITY LEVEL:'
echo
echo 'Sensible values are from 50 to 95.  The standard value is 75, but'
echo 'artifacts might still be perceivable at that level.   With modern'
echo 'computer capabilities, 85 may be a better choice.'
set errlev='85'
if (-f jpgqualdef.bat) source jpgqualdef.bat
"$L2HTOP/system-files/bin/wizhlp.exe" "$L2HTOP/ " askint "Please enter the JPEG quality" errlev 30 100 $errlev
if ($status) goto err
if !(-f @tmp.bat) goto errq
set errlev=''
source "@tmp.bat"
if ("$errlev" == '') goto errsys
echo 'if ("$1" == qual) set errlev='"'$errlev'" >> @tmp0.bat
if ($status) goto errsys
tojpg0:
set errlev=''
source "@tmp0.bat" qual
if ("$errlev" == '') goto errsys
"$L2HTOP/bin/pnmtojpeg" $5$errlev $6 $7 $8 $9 @tmp.tmp > @tmp1.tmp
if ($status) goto errcnv2
goto ok
tojpg1:
"$L2HTOP/bin/pnmtojpeg" $5 $6 $7 $8 $9 @tmp.tmp > @tmp1.tmp
if ($status) goto errcnv2
goto ok

# convert to png (might need -transparent '=#FFFFFF', needs quotes)
topng:
which pnmtopng > /dev/null
if ($status) goto errpng2
pnmtopng $5 $6 $7 $8 $9 @tmp.tmp > @tmp1.tmp
if ($status) goto errcnv2
goto ok

# finish up
ok:
if !(-f @tmp1.tmp) goto errsys
set prot=`stat -c %a "$1.$2"`
if ("$3" == '*') goto ok1
"$L2HTOP/bin/movenew.bat" @tmp1.tmp "$1"@new."$3" y
if ($status) goto err
chmod $prot "$1"@new."$3"
if ($status) goto errsys
echo "       Created: $1@new.$3"
goto ok2
ok1:
"$L2HTOP/bin/movenew.bat" @tmp1.tmp "$1"@new."$2" y
if ($status) goto err
chmod $prot "$1"@new."$2"
if ($status) goto errsys
echo "       Created: $1@new.$2"
ok2:

# done
done:
"$L2HTOP/bin/clearold.bat" y @tmp.tmp @tmp1.tmp @tmp2.tmp @tmp3.tmp @tmp.bat
if ($status) goto err
goto end

# warnings
wrnfil:
echo "**   Not found: $1.$2"
goto done
wrnsame:
echo "**   No change: $1.$2"
goto done

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.'
goto err
errtyp:
echo "*** The file type $2 of file $1.$2 is not supported."
goto err
errnew:
echo "*** File $1@new.$3 already exists, though it should not."
goto err
errnew2:
echo "*** File $1@new.$2 already exists, though it should not."
goto err
errcnv:
echo "*** Error while trying to read image $1.$2."
goto err
errpng:
echo '*** Cannot convert: pngtopnm is not installed.'
goto err
errpng2:
echo '*** Cannot convert: pnmtopng is not installed.'
goto err
errcrop:
echo "*** Error while trying to crop $1.$2."
goto err
errsiz:
echo "*** Error while trying to reduce the size of $1.$2."
goto err
errtyp2:
echo "*** The file type $3 of file $1.$3 is not supported."
goto err
errcnv2:
echo '*** Error while trying to create the final image.'
goto err
errprog:
echo '*** Fatal program error in file_anyany.bat.'
goto err
errq:
echo '*** User aborted.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
