#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Move any ROOT.EXT to ROOT@LEVEL2.EXT if ROOT@LEVEL1.EXT exists.

# This is identical to file_bcklvmv.bat except that @%3 is only used in
# deciding whether to move. ROOT.EXT is the one actually moved.

# Usage
goto start
use:
echo 'Usage: set L2HTOP, check choose.exe and seterr.exe'
echo '       set errlev=0'
echo '       source file_bcklvmv0.bat ROOT EXT LEVEL1 LEVEL2'
echo '       The LEVEL2 file must not currently exist.'
echo '       Sets errlev to 1 if at least one file is moved.'
echo '       Returns error status 1 on error.'
goto err
start:

# This must work perfectly or things become a mess.  So the user is
# consulted when there is a problem.  Then he/she can be blamed...

# handle folders separately
if ("$2" != F0L) goto nofol

# check that the level2 folder does not exist
if (-e "$1@$4") goto errnewf

# done if there is no folder to move
if !(-e "$1") goto done
if !(-e "$1@$3") goto done

# move the folder
set errlev='1'
mv "$1" "$1@$4"
if (-d "$1@$4/") goto diditf
echo
echo "Unable to rename folder $1 to $1@$4"
echo 'It is essential that this is done correctly.'
echo 'Please do it manually.  Rename the subfolder'
echo "$1"
echo 'within the folder'
pwd
echo 'to'
echo "$1@$4"
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-d "$1@$4/") goto errmvf
diditf:

# check that the old folder is indeed gone
if !(-e "$1") goto done
echo
echo "While $1@$4 has been created, $1 is still there."
echo "Assuming that $1 is the same as $1@$4, remove subfolder"
echo "$1"
echo 'from the folder'
pwd
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if (-e "$1") goto erroldf
goto done

# not a folder
nofol:

# check that the level2 file does not exist
if (-e "$1@$4.$2") goto errnew

# done if there is no file to move
if !(-e "$1.$2") goto done
if !(-e "$1@$3.$2") goto done

# move the file
set errlev='1'
mv "$1.$2" "$1@$4.$2"
if (-f "$1@$4.$2") goto didit
echo
echo "Unable to rename file $1.$2 to $1@$4.$2"
echo 'It is essential that this is done correctly.'
echo 'Please do it manually.  Rename the file'
echo "$1.$2"
echo 'within the folder'
pwd
echo 'to'
echo "$1@$4.$2"
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-f "$1@$4.$2") goto errmv
didit:

# check that the old file is indeed gone
if !(-e "$1.$2") goto done
echo
echo "While $1@$4.$2 has been created, $1.$2 is still there."
echo "Assuming that $1.$2 is the same as $1@$4.$2, remove"
echo "$1.$2"
echo 'from the folder'
pwd
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if (-e "$1.$2") goto errold

# done
done:
goto end

# errors
errnewf:
echo "*** Folder $1@$4 exists, and it should not."
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errmvf:
echo "*** Unable to rename folder $1 to $1@$4"
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
erroldf:
echo "*** Folder $1 still exists, and it should not."
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errnew:
echo "*** File $1@$4.$2 exists, and it should not."
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errmv:
echo "*** Unable to rename file $1.$2 to $1@$4.$2"
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errold:
echo "*** File $1.$2 still exists, and it should not."
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
