#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Move any ROOT@LEVEL.EXT to ROOT.EXT.

# This is identical to file_bcklvmv.bat except that @%4 has been
# removed to allow moving to the current level and the target could
# exist if the source does not.

# Usage
goto start
use:
echo 'Usage: set L2HTOP, check choose.exe'
echo '       set errlev=0'
echo '       Ensure zero error level'
echo '       call file_bcklvmv1.bat ROOT EXT LEVEL'
echo '       The ROOT.EXT file must not currently exist.'
echo '       Sets errlev to 1 if at least one file is moved.'
echo '       Returns error status 1 on error.'
goto err
start:

# This must work perfectly or things become a mess.  So the user is
# consulted when there is a problem.  Then he/she can be blamed...

# handle folders separately
if ("$2" != F0L) goto nofol

# done if there is no folder to move
if !(-e "$1@$3") goto done

# check that the level2 folder does not exist
if (-e "$1") goto errnewf

# move the folder
set errlev='1'
mv "$1@$3" "$1"
if (-e "$1") goto diditf
echo
echo "Unable to rename folder $1@$3 to $1"
echo 'It is essential that this is done correctly.'
echo 'Please do it manually.  Rename the subfolder'
echo "$1@$3"
echo 'within the folder'
pwd
echo 'to'
echo "$1"
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-e "$1") goto errmvf
diditf:

# check that the old folder is indeed gone
if !(-e "$1@$3") goto done
echo
echo "While $1 has been created, $1@$3 is still there."
echo "Assuming that $1@$3 is the same as $1, remove subfolder"
echo "$1@$3"
echo 'from the folder'
pwd
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if (-e "$1@$3") goto erroldf
goto done

# not a folder
nofol:

# done if there is no file to move
if !(-e "$1@$3.$2") goto done

# check that the level2 file does not exist
if (-e "$1.$2") goto errnew

# move the file
set errlev='1'
mv "$1@$3.$2" "$1.$2"
if (-e "$1.$2") goto didit
echo
echo "Unable to rename file $1@$3.$2 to $1.$2"
echo 'It is essential that this is done correctly.'
echo 'Please do it manually.  Rename the file'
echo "$1@$3.$2"
echo 'within the folder'
pwd
echo 'to'
echo "$1.$2"
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-e "$1.$2") goto errmv
didit:

# check that the old file is indeed gone
if !(-e "$1@$3.$2") goto done
echo
echo "While $1.$2 has been created, $1@$3.$2 is still there."
echo "Assuming that $1@$3.$2 is the same as $1.$2, remove"
echo "$1@$3.$2"
echo 'from the folder'
pwd
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if (-e "$1@$3.$2") goto errold

# done
done:
goto end

# errors
errnewf:
echo "*** Folder $1 exists, and it should not."
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errmvf:
echo "*** Unable to rename folder $1@$3 to $1"
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
erroldf:
echo "*** Folder $1@$3 still exists, and it should not."
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errnew:
echo "*** File $1.$2 exists, and it should not."
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errmv:
echo "*** Unable to rename file $1@$3.$2 to $1.$2"
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errold:
echo "*** File $1@$3.$2 still exists, and it should not."
echo '    This shows corruption of the backups.  Undo will not work correctly.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
