#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: convert a file NAME.EXT into another using subbin.exe
#          Uses subbin_big.exe.
#          Creates an @new file which must not yet exist.
#          In unix the created file will have the permissions of NAME.EXT.

# Usage
goto start
use:
echo 'Usage: Ensure zero error level'
echo '       set errlev=0'
echo '       file_subbn.bat NAME EXT EXT2 SUBFILE FAIL [VERBOSITY]'
echo '       NAME.EXT is the file to convert'
echo '       NAME@new.EXT2 (NAME@new.EXT if EXT2 is *) is the converted file'
echo '       SUBFILE is the substitutions file'
echo '       FAIL:'
echo '          f: fail on a subbin.exe error (return error status 1)'
echo '          i: ignore it'
echo '       VERBOSITY: verbosity of subbin.exe'
echo '          -q: fairly quiet (default)'
echo '          -n: normal'
echo '          -v: verbose'
echo '          -V: more verbose'
echo
echo '       Watch for this file not being changed while in use.'
echo '       Returns error status 1 and sets errlev=2 on error.'
echo '       Sets errlev=1 on subbin.exe error.'
echo '       Uses @tmp.tmp as a temporary.'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# check that the file to convert exists
if !(-f "$1.$2") goto wrnfil

# check that the @new file does not yet exist
if ("$3" == '*') goto chknw2
if (-e "$1@new.$3") goto errnew
goto chknwe
chknw2:
if (-e "$1@new.$2") goto errnew2
chknwe:

# remove any existing file @tmp.tmp
if (-f @tmp.tmp) rm @tmp.tmp
if (-e @tmp.tmp) goto errsys

# run subbin.exe
if ("$6" == '') goto subdef
"$L2HTOP/system-files/bin/subbin_big.exe" "$4" "$1"."$2" @tmp.tmp "$6"
if ($status) goto wrnsub
goto subend
subdef:
"$L2HTOP/system-files/bin/subbin_big.exe" "$4" "$1"."$2" @tmp.tmp -q
if ($status) goto wrnsub
subend:

# warn if no file is produced
if !(-f @tmp.tmp) goto wrnnone

# remember the file permissions
set prot=`stat -c %a "$1.$2"`

# rename @tmp.tmp to the output file
if ("$3" == '*') goto domv2
mv @tmp.tmp "$1@new.$3"
if ($status) goto errmv
if (-e @tmp.tmp) goto errmv
if !(-f "$1@new.$3") goto errmv
chmod $prot "$1@new.$3"
if ($status) goto errmv
goto endmv
domv2:
mv @tmp.tmp "$1@new.$2"
if ($status) goto errmv2
if (-e @tmp.tmp) goto errmv2
if !(-f "$1@new.$2") goto errmv2
chmod $prot "$1@new.$2"
if ($status) goto errmv2
endmv:

# print a message
if ("$3" == '*') echo "       Created: $1@new.$2"
if ("$3" != '*') echo "       Created: $1@new.$3"

# done
done:
goto end

# warnings
wrnfil:
echo "**   Not found: $1.$2"
goto done
wrnsub:
echo "*** Conversion of file $1.$2 failed."
if ("$5" != i) goto err
set errlev='1'
goto done
wrnnone:
if ("$3" == '*') echo "** Not created: $1@new.$2"
if ("$3" != '*') echo "** Not created: $1@new.$3"
goto done

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errnew:
echo "*** Program error: file $1@new.$3 already exists."
goto err
errnew2:
echo "*** Program error: file $1@new.$2 already exists."
goto err
errmv:
echo "*** Unable to create file $1@new.$3???"
goto err
errmv2:
echo "*** Unable to create file $1@new.$2???"
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
set errlev='2'
exit 1

end:
