#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: convert a file NAME.EXT into another using subbin.exe
#          The subbin.exe executable used is a parameter.
#          Overwrites the target file if it already exists.
#          In unix the created file will have the permissions of NAME.EXT.

# Usage
goto start
use:
echo 'Usage: Ensure zero error level'
echo '       set errlev=0'
echo '       file_subpo.bat NAME EXT EXT2 EXESPEC SUBFILE FAIL [VERBOSITY]'
echo '       NAME.EXT is the file to convert'
echo '       NAME.EXT2 (NAME.EXT if EXT2 is *) is the converted file'
echo '       EXESPEC is the subbin.exe executable to use'
echo '       SUBFILE is the substitutions file'
echo '       FAIL:'
echo '          f: fail on a subbin.exe error (return error status 1)'
echo '          i: ignore it'
echo '       VERBOSITY: verbosity of subbin.exe'
echo '          -q: fairly quiet (default)'
echo '          -n: normal'
echo '          -v: verbose'
echo '          -V: more verbose'
echo
echo '       Watch for this file not being changed while in use.'
echo '       Returns error status 1 and sets errlev=2 on error.'
echo '       Sets errlev=1 on subbin.exe error.'
echo '       Uses @tmp.tmp as a temporary.'
goto err
start:

# check that the file to convert exists
if !(-f "$1.$2") goto wrnfil

# remove any existing file @tmp.tmp
if (-f @tmp.tmp) rm @tmp.tmp
if (-e @tmp.tmp) goto errsys

# run subbin.exe
if ("$7" == '') goto subdef
"$4" "$5" "$1"."$2" @tmp.tmp "$7"
if ($status) goto wrnsub
goto subend
subdef:
"$4" "$5" "$1"."$2" @tmp.tmp -q
if ($status) goto wrnsub
subend:

# warn if no file is produced
if !(-f @tmp.tmp) goto wrnnone

# remember the file permissions
set prot=`stat -c %a "$1.$2"`

# remove any preexisting file
if ("$3" == '*') goto dodel2
if !(-f "$1.$3") goto enddel
rm "$1.$3"
if ($status) goto errdel
if (-e "$1.$3") goto errdel
goto enddel
dodel2:
rm "$1.$2"
if ($status) goto errdel2
if (-e "$1.$2") goto errdel2
enddel:

# rename @tmp.tmp to the output file
if ("$3" == '*') goto domv2
mv @tmp.tmp "$1.$3"
if ($status) goto errmv
if (-e @tmp.tmp) goto errmv
if !(-f "$1.$3") goto errmv
chmod $prot "$1.$3"
if ($status) goto errmv
goto endmv
domv2:
mv @tmp.tmp "$1.$2"
if ($status) goto errmv2
if (-e @tmp.tmp) goto errmv2
if !(-f "$1.$2") goto errmv2
chmod $prot "$1.$2"
if ($status) goto errmv2
endmv:

# print a message
if ("$3" == '*') echo "       Changed: $1.$2"
if ("$3" != '*') echo "       Created: $1.$3"

# done
done:
goto end

# warnings
wrnfil:
echo "**   Not found: $1.$2"
goto done
wrnsub:
echo "*** Conversion of file $1.$2 failed."
if ("$6" != i) goto err
set errlev='1'
goto done
wrnnone:
if ("$3" == '*') echo "     Unchanged: $1.$2"
if ("$3" != '*') echo "** Not created: $1.$3"
goto done

# errors
errdel:
echo "*** Unable to delete existing file $1.$3???"
goto err
errdel2:
echo "*** Unable to delete existing file $1.$2???"
goto err
errmv:
echo "*** Unable to create file $1.$3???"
goto err
errmv2:
echo "*** Unable to create file $1.$2???"
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
set errlev='2'
exit 1

end:
