#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Get an example index.tex file (with the current date).

# Usage
goto start
use:
echo 'Usage: source doinit.bat (make sure that the @system subfolder exists)'
echo '       Remove any existing index.tex'
echo '       getexample.bat'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# show the menu and get the user's choice
askexa:
clear
echo '===================== Get an Example LaTeX File ======================'
echo 'There is no LaTeX source file index.tex in the current folder'
pwd
echo 'Please select an example from the next list:'
echo
echo '1: Standard.  This is the format the author of l2h uses himself'
echo '   to write his 1,600 page book on quantum mechanics, so this is the'
echo '   most well tested.  Do not use this if you write in Greek or a'
echo '   nonWestern European language.'
echo '2: Like 1, but set up for a scientific article instead of a book.'
echo '3: Like 1, but set up to use pdflatex or xelatex.  You may want'
echo '   to select this if you plan to use something like TeXstudio or'
echo '   TeXworks as your editor.  (Although you *can* configure them for'
echo '   the DVI-PS-PDF route and then use 1 or 2 above.)'
echo '4: Like 3, but set up for a scientific article instead of a book.'
echo '5: Set up for writing in Greek or nonWestern European languages.'
echo '   You must have XeLaTeX, pdftk, and pdftops installed to use this.'
echo '   (You will have to twiddle with this; the l2h author writes only'
echo '   Dutch and English and has no clue.)'
echo '6: Like 5, but set up for a scientific article instead of a book.'
echo '7: Special-purpose examples, including Thesis_FSU.'
echo '8: Tell me how to use my own existing LaTeX file.'
echo
"$L2HTOP/bin/choose.exe" 12345678q "Press the number of your choice from above or q to quit: q"
set stat="$status"
if ($stat >= 10) goto errsys
if ($stat >= 9) goto errq
if ($stat >= 8) goto myown
if ($stat >= 7) goto seltem
if ($stat >= 6) goto idx6
if ($stat >= 5) goto idx5
if ($stat >= 4) goto idx4
if ($stat >= 3) goto idx3
if ($stat >= 2) goto idx2
if ($stat >= 1) goto idx1
goto errq

# get the selected example
idx1:
"$L2HTOP/bin/copynew.bat" "$L2HTOP/help-files/index1.txt" index.tex r n
if ($status) goto err
goto asksty
idx2:
"$L2HTOP/bin/copynew.bat" "$L2HTOP/help-files/index2.txt" index.tex r n
if ($status) goto err
goto asksty
idx3:
"$L2HTOP/bin/copynew.bat" "$L2HTOP/help-files/index3.txt" index.tex r n
if ($status) goto err
goto asksty
idx4:
"$L2HTOP/bin/copynew.bat" "$L2HTOP/help-files/index4.txt" index.tex r n
if ($status) goto err
goto asksty
idx5:
"$L2HTOP/bin/copynew.bat" "$L2HTOP/help-files/index5.txt" index.tex r n
if ($status) goto err
goto asksty
idx6:
"$L2HTOP/bin/copynew.bat" "$L2HTOP/help-files/index6.txt" index.tex r n
if ($status) goto err
goto asksty

# ask the user for a template
seltem:

# clean up old template getting stuff
if !(-e @system) mkdir @system
if !(-d @system/) goto errsys
"$L2HTOP/bin/clearold.bat" y @system/coptem.lst @system/coptem.tmp @system/coptem.bat

# make a list of templates
ls -1 "$L2HTOP/templates/"*/index.tex > @system/coptem.lst
if ($status) goto errlst
if !(-f @system/coptem.lst) goto errsys
cat @system/coptem.lst | fgrep '/templates/Thesis_FSU/index.tex'
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto errdef

# ask the user's choice
"$L2HTOP/bin/sellist.exe" @system/coptem example Thesis_FSU 0 2 /
if ($status) goto errsys
if !(-f @system/coptem.tmp) goto askexa

# create a batch file to copy over the files
cat "$L2HTOP/system-files/data/coptem.bat" >> @system/coptem.tmp
if ($status) goto errsys
"$L2HTOP/bin/subbin.exe" "$L2HTOP/system-files/data/mksubfil.sub" @system/coptem.tmp @system/coptem.bat
if ($status) goto errsys
if !(-f @system/coptem.bat) goto errsys

# copy over the files in the folder
source "@system/coptem.bat"
if ($status) goto err
if !(-f index.tex) goto errsys
goto asksty

# explain how to use your own latex file
myown:
if !(-f "$L2HTOP/help-files/use_your_own.html") goto errown
echo
echo '   h: Try to load the help page use_your_own.html in the default browser.'
echo '   t: Just type the file instead.  (Use Space to scroll and q to quit.)'
echo '   r: Return to the example selection menu.'
echo '   p: Proceed, I have created a suitable index.tex.'
echo '   q: Quit l2h.'
echo
myowna:
"$L2HTOP/bin/choose.exe" htrpq "Make a choice from the menu above and press its key"
set stat="$status"
if ($stat >= 6) goto errsys
if ($stat >= 5) goto errq
if ($stat >= 4) goto myownp
if ($stat >= 3) goto askexa
if ($stat >= 2) goto myownt
if ($stat >= 1) goto myownh
goto errq
myownh:
xdg-open "$L2HTOP/help-files/use_your_own.html"
if ($status) "$L2HTOP/bin/choose.exe" anykey "** Error loading the web page.  Hit any key to acknowledge"
goto myown
myownt:
more "$L2HTOP/help-files/use_your_own.html"
if ($status) "$L2HTOP/bin/choose.exe" anykey "** No privileges accessing the web page.  Hit any key to acknowledge"
goto myown
myownp:
if (-f index.tex) goto asksty
echo '** index.tex not found.'
goto myowna

# ask whether to copy in various style files
asksty:
echo
echo 'If you plan to use a LaTeX editor like TeXstudio or TeXmaker to'
echo 'process your documents, various files from the texinputs subfolder of'
echo 'l2h need to be copied in with your document.'
echo
echo 'Do you want to do that?'
"$L2HTOP/bin/choose.exe" ny "Hit y for yes or n for no: n"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto docp
if ($stat >= 1) goto endcp
goto errq
docp:
"$L2HTOP/bin/copynew.bat" "$L2HTOP/texinputs/html.sty" html.sty c y
if ($status) goto err
"$L2HTOP/bin/copynew.bat" "$L2HTOP/texinputs/frames.sty" frames.sty c y
if ($status) goto err
"$L2HTOP/bin/copynew.bat" "$L2HTOP/texinputs/heqn.sty" heqn.sty c y
if ($status) goto err
"$L2HTOP/bin/copynew.bat" "$L2HTOP/texinputs/hthtml.sty" hthtml.sty c y
if ($status) goto err
"$L2HTOP/bin/copynew.bat" "$L2HTOP/texinputs/justify.sty" justify.sty c y
if ($status) goto err
endcp:

# done
done:
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
erridx:
echo '*** The selected example is missing from the l2h help files folder\!'
echo '    Please re-install l2h from scratch.'
goto err
errlst:
echo "*** Unable to list the examples in $L2HTOP/templates."
goto err
errdef:
echo '*** Cannot find the default Thesis_FSU example in'
echo "   $L2HTOP/templates"
goto err
errown:
echo '*** Help web page use_your_own.html is missing from the l2h help-files'
echo '    folder.  Please reinstall l2h from scratch.'
goto err
errq:
echo '*** Aborted by user.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
