#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: clean up the l2h directory after configure, make, make
#          check, make test, make install.

# Usage
goto start
use:
echo 'Usage: See rebuild_start.bat.'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# cd to the l2h folder
cd "$L2HTOP"
if ($status) goto errsys

# make sure that the perl files are there
if !(-f l2hconf.pm) goto errfils
if !(-f bin/latex2html) goto errfils
if !(-f bin/pstoimg) goto errfils
if !(-f bin/texexpand) goto errfils
# and that they are executable
chmod a+rx bin/latex2html
chmod a+rx bin/pstoimg
chmod a+rx bin/texexpand

# Describe the purpose
clear
echo '============================== Clean Up =============================='
echo
echo 'This final script will make the appearance of the l2h folder a lot'
echo 'less messy.  It will remove many files that are found in unchanged'
echo 'form in latex2html.zip and not used for anything.  It will also remove'
echo 'some stuff for which l2h uses equivalent replacements.'
echo
echo 'Do note that if you want an installation fully compatible with the'
echo 'original latex2html, you should not delete these files.  That is a'
echo 'potential concern if you have old latex2html documents.  There is no'
echo 'problem for l2h if the files are not removed.'
echo
echo 'Also, you might actually want to try to install some of the unused'
echo 'stuff, like IndicTeX-HTML, XyMTeX-HTML, cweb2html, foilhtml, makemap,'
echo 'or makeseg.  Or look at the latex2html docs folder.  In that case, do'
echo 'not delete the files.'
echo
echo 'Considering the above, do you want to clean up l2h?'
bin/choose.exe yn "Press y for yes or n for no:y"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto final
if ($stat >= 1) goto doit
goto errab
doit:

# IndicTeX-HTML is not changed from inside latex2html99.2.zip
if (-d IndicTeX-HTML/) rm -r IndicTeX-HTML
if ($status) goto errsys

# XyMTeX is not changed from inside latex2html99.2.zip
if (-d XyMTeX-HTML/) rm -r XyMTeX-HTML
if ($status) goto errsys

# keep cfgcache.pm for possible later reference
if (-f cfgcache.pm) mv cfgcache.pm system-files
if ($status) goto errsys

# cweb2html is not changed from inside latex2html99.2.zip
if (-d cweb2html/) rm -r cweb2html
if ($status) goto errsys

# docs is not changed from inside latex2html99.2.zip
if (-d docs/) rm -r docs
if ($status) goto errsys

# we use our own version, thanks
if (-f dot.latex2html-init) rm dot.latex2html-init
if ($status) goto errsys

# we have our own example, thanks
if (-f example/Makefile) rm example/Makefile
if ($status) goto errsys
if (-f example/report.tex) rm example/report.tex
if ($status) goto errsys
if (-f example/sec1.tex) rm example/sec1.tex
if ($status) goto errsys
if (-f example/sec2.tex) rm example/sec2.tex
if ($status) goto errsys

# foilhtml is not changed from inside latex2html99.2.zip
if (-d foilhtml/) rm -r foilhtml
if ($status) goto errsys

# makemap is not changed from inside latex2html99.2.zip
if (-f makemap) rm makemap
if ($status) goto errsys

# makeseg is not changed from inside latex2html99.2.zip
if (-d makeseg/) rm -r makeseg
if ($status) goto errsys

# we already have the document in various-docs
if (-f readme.hthtml) rm readme.hthtml
if ($status) goto errsys

# final messages
final:
echo
echo 'Some additional notes:'
echo '1) If you want to use the ldump option of latex2html, you may want to'
echo '   edit l2hconf.pm to manually correct the entry.'
echo '   I do not know whether ldump works, it seems pretty useless to me.'
echo '2) You may want to delete the l2hins installation folder too.'

# done
done:
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errfils:
echo '*** Files latex2html, pstoimg, and/or texexpand are missing from'
echo '    the l2h bin subfolder, and/or l2hconf.pm from l2h.'
goto err
errab:
echo '*** Aborted by the user.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
echo '    Execution cannot continue.'
exit 1

end:
