#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: start a rebuild of latex2html.

# Usage
goto start
use:
echo 'Usage: Install tcsh, LaTeX, ghostscript, and Perl.'
echo '       Unzip l2h, maintaining the folder structure.'
echo '       Do NOT unzip to a location were there are spaces in the path.'
echo '       Create the pnm binaries if needed.'
echo '       Compile the fortran programs in system-files/jpegs,'
echo '       system-files/subbin, system-files/util, and system-files/wiz'
echo '       if needed.  Use g77 or gfortran; see the make files.'
echo '       Go to the l2h folder and double-click install_l2h.'
echo '       In the terminal, issue the commands:'
echo '          cd ./l2h'
echo '       (where ./ is the path to the l2h folder, e.g. Desktop/), and'
echo '          source install'
echo '       Follow the directions.  On success:'
echo '          cd system-files/bin'
echo '          source rebuild_start.bat'
echo '       Follow the directions.'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# go to it
cd "$L2HTOP"
if ($status) goto errsys

# check that this file is sourced
if ("$0" != "tcsh") goto errsrc

# source init.bat
if !(-f init.bat) goto errfil
source init.bat
if ($status) goto errini
if !(-x bin/choose.exe) goto errfil

# check that L2HTOP/bin is in the path
echo ":${PATH}:" | fgrep ":$L2HTOP/bin:" > /dev/null
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto errpath

# check that there are no spaces in the path
pwd | fgrep ' '
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto pathok
goto errspc
pathok:

# go to the bin folder
cd bin
if ($status) goto errsys

# check for the essential pnm files
clear
echo 'Checking the core utilities needed by latex2html...'
if !(-f anytopnm) goto miscore
if !(-f pamflip) goto miscore
if !(-f pbmmake) goto miscore
if !(-f pnmcat) goto miscore
if !(-f pnmcolormap) goto miscore
if !(-f pnmcrop) goto miscore
if !(-f pnmfile) goto miscore
if !(-f pnmflip) goto miscore
if !(-f pnmremap) goto miscore
if !(-f ppmtogif) goto miscore
if !(-f ppmtojpeg) goto miscore
goto coreok
miscore:
echo "*** The following essential files are not in $L2HTOP/bin:"
if !(-f anytopnm) echo '       anytopnm'
if !(-f pamflip) echo '       pamflip'
if !(-f pbmmake) echo '       pbmmake'
if !(-f pnmcat) echo '       pnmcat'
if !(-f pnmcolormap) echo '       pnmcolormap'
if !(-f pnmcrop) echo '       pnmcrop'
if !(-f pnmfile) echo '       pnmfile'
if !(-f pnmflip) echo '       pnmflip'
if !(-f pnmremap) echo '       pnmremap'
if !(-f ppmtogif) echo '       ppmtogif'
if !(-f ppmtojpeg) echo '       ppmtojpeg'
echo '    They are vital to the core operation of LaTeX2HTML, and they should'
echo '    be in a safe location, or you will most likely find that LaTeX2HTML'
echo '    stops working at the next netpbm upgrade or any other system change.'
echo "    You should quit now and copy those files to $L2HTOP/bin."
echo
choose.exe qi "Hit Enter to quit or press i to proceed anyway: q"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto coreok
goto errab
coreok:

# check for missing files without which l2h can live
clear
echo 'Checking the converters...'
echo
if !(-f bmptoppm) goto misconv
if !(-f giftopnm) goto misconv
if !(-f jpegtopnm) goto misconv
if !(-f pnmtops) goto misconv
if !(-f ppmtobmp) goto misconv
goto convok
misconv:
echo "*** The following converters are not in $L2HTOP/bin:"
if !(-f bmptoppm) echo '       bmptoppm'
if !(-f giftopnm) echo '       giftopnm'
if !(-f jpegtopnm) echo '       jpegtopnm'
if !(-f pnmtops) echo '       pnmtops'
if !(-f ppmtobmp) echo '       ppmtobmp'
echo '    This will cause some utilities not to work as described, such as'
echo '    bmptoeps, important for inclusion of graphics, and bmptogif and'
echo '    giftobmp. important for editing icons.'
echo '    Move in the missing files or edit the utilities to set them up'
echo '    differently.'
echo
choose.exe akey "After reading the above message, hit Enter to continue"
if ($status) goto errab
convok:

# check for missing files that are not that important (or bug-ridden)
clear
echo 'Checking for other graphics converters...'
echo
if !(-f pcxtoppm) goto misother
if !(-f picttoppm) goto misother
if !(-f pngtopnm) goto misother
if !(-f pnmcut) goto misother
if !(-f pnmpad) goto misother
if !(-f pnmrotate) goto misother
if !(-f pnmscale) goto misother
if !(-f pnmtopng) goto misother
if !(-f sgitopnm) goto misother
if !(-f tifftopnm) goto misother
if !(-f xbmtopbm) goto misother
if !(-f xwdtopnm) goto misother
goto otherok
misother:
echo "Note: The following NONessential utilities are not in $L2HTOP/bin:"
if !(-f pcxtoppm) echo '       pcxtoppm'
if !(-f picttoppm) echo '       picttoppm'
if !(-f pngtopnm) echo '       pngtopnm'
if !(-f pnmcut) echo '       pnmcut'
if !(-f pnmpad) echo '       pnmpad'
if !(-f pnmrotate) echo '       pnmrotate'
if !(-f pnmscale) echo '       pnmscale'
if !(-f pnmtopng) echo '       pnmtopng'
if !(-f sgitopnm) echo '       sgitopnm'
if !(-f tifftopnm) echo '       tifftopnm'
if !(-f xbmtopbm) echo '       xbmtopbm'
if !(-f xwdtopnm) echo '       xwdtopnm'
echo
choose.exe akey "After reading the above message, hit Enter to continue"
if ($status) goto errab
otherok:

# go to the l2h folder
cd ..
if ($status) goto errsys

# check for unzip
which unzip
if ($status) goto erriz

# check that latex2html99.2.zip is there
if !(-f external-programs/latex2html99.2.zip) goto errnol

# unzip latex2html
echo 'The first thing is to unzip the original source code of latex2html.'
bin/choose.exe yn "Do you want to unzip latex2html now:y"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto errab
if ($stat >= 1) goto unzipit
goto errab
unzipit:
echo 'Unzipping latex2html 99.2 into l2hins'
unzip -o -qq external-programs/latex2html99.2.zip
if ($status) goto errunz

# random checks on the unzip
if !(-f l2hins/prefs.pm) goto errins
if !(-f l2hins/versions/html4_0.pl) goto errins

# copy in the modified files
echo
echo 'At this stage various fixes are to be applied to the latex2html code.'
echo 'These fixes are essential for l2h to run correctly.  Therefore you'
echo 'should answer y (yes) on the next question.'
echo
bin/choose.exe yn "Do you want to apply the fixes to latex2html:y"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto endfix
if ($stat >= 1) goto dofix
goto errab
dofix:
echo 'Moving the fixed-up files into position'
cd system-files/l2hmods
if ($status) goto errsys
'cp' *.* ../../l2hins
if ($status) goto errxc
cd versions
if ($status) goto errsys
'cp' *.* ../../../l2hins/versions
if ($status) goto errxc
cd ../styles
if ($status) goto errsys
'cp' *.* ../../../l2hins/styles
if ($status) goto errxc
cd ../config
if ($status) goto errsys
'cp' *.* ../../../l2hins/config
if ($status) goto errxc
cd ../L2hos
if ($status) goto errsys
'cp' *.* ../../../l2hins/L2hos
if ($status) goto errxc
cd ../../..
if ($status) goto errsys
endfix:

# clean up l2hins a bit
cd l2hins
if ($status) goto errsys
if (-f config.log) rm config.log
if ($status) goto errsys
if (-f cfgcache.pm) rm cfgcache.pm
if ($status) goto errsys
chmod u+x configure
if ($status) goto errsys

# list subsequent steps
echo
echo
echo 'You should now be in the latex2html installation folder l2hins.'
echo 'Edit prefs.pm to make the desired change(s).  Do not change what is not'
echo 'broke.  Then in the terminal issue the commands:'
echo '   ./configure'
echo '   make'
echo '   make check'
echo '   make test'
echo '   make install'
echo '   cd ../system-files/bin'
echo '   rebuild_end.bat'
echo 'Follow the directions.'
echo
echo 'Write the above down.  Or better, load the rebuild_latex2html.html'
echo 'web page in the help-files folder into your browser.'

# done
done:
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errsrc:
echo '*** rebuild_start.bat must be sourced from tcsh.'
goto err
errfil:
echo '*** Critical files are missing from the l2h folder.'
echo '    Please re-install from scratch and try again.'
goto err
errini:
echo '*** Error running init.bat.  Rerun install_l2h.'
goto err
errpath:
echo '*** The PATH is not correctly set by init.bat.  Rerun install_l2h.'
goto err
errspc:
echo '*** There are spaces in the current location, as shown above.'
echo '    The latex2html rebuild procedure does not work in such a location.'
echo '    So please move the l2h folder to a different location and rerun'
echo '    install_l2h.  After rebuild, you can move the l2h folder back to'
echo '    where it is now, provided that you rerun install_l2h afterwards.'
goto err
errab:
echo '*** Aborted by the user.'
goto err
erriz:
echo '*** The Info-Zip unzip command is not in the path.'
echo '    Please install unzip and try again.'
goto err
errnol:
echo '*** File latex2html99.2.zip is not in the 2h folder.'
echo '    Please restore it from the l2h zip file and try again.'
goto err
errunz:
echo '*** Error unzipping latex2html99.2.zip.'
goto err
errins:
echo '*** Essential files are missing from latex2html99.2.zip.'
goto err
errxc:
echo '*** Unable to move the fixed-up files to their destinations.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
echo '    The rebuild_start failed.  Fix the problem and rerun.'
exit 1

end:
