#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Restore a set of files from the backup set @old.

# Usage
goto start
use:
echo 'Usage: undo.bat LISTROOT EXT2'
echo '       If EXT2 is *, LISTROOT.bat may have to be rebuild afterwards.'
goto err
start:

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# check the basic files needed for execution
if !(-rx "$L2HTOP/bin/filechk.bat") goto errfils
"$L2HTOP/bin/filechk.bat"
if ($status) goto errfils
if !(-rx "$L2HTOP/system-files/bin/filechk.bat") goto errfils
"$L2HTOP/system-files/bin/filechk.bat"
if ($status) goto errfils
if !(-rx "$L2HTOP/system-files/data/filechk.bat") goto errfils
"$L2HTOP/system-files/data/filechk.bat"
if ($status) goto errfils

# no go if LISTROOT.bat does not exist
if !(-f "$1.bat") goto errfil

# see whether the @old level is empty
if !(-f "$1@old.bat") goto look20
set errlev='0'
echo
source "$1"@old.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvshw.bat" old
if ($status) goto errsys
if ("$errlev" != 0) goto dorst
rm "$1@old.bat"
if !(-f "$1@old.bat") goto look20
echo "** Unable to remove expired backup file list $1@old.bat."
echo '   Please do that manually now or undo will be corrupted.'
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-f "$1@old.bat") goto look20
goto errrm1

# restore the files from the @old level
dorst:
echo
echo 'WARNING: Restoring backups cannot be reversed.  (Except maybe once'
echo '         *immediately* after the fact.)  So you may want to have a look'
echo '         at the file dates of the backups above to check that they are'
echo '         not older than what you would want to restore.  (You may even'
echo '         want to look at the files themselves.)'
echo
"$L2HTOP/bin/choose.exe" cq "Hit c to continue with backup restoration or q to quit"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto errq
if ($stat >= 1) goto doit
goto errq
doit:

# get rid of orphaned @new files
set errlev='0'
echo
source "$1"@old.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvshw.bat" new
if ($status) goto errsys
if ("$errlev" == 0) goto enddelnw
echo
echo 'There are orphaned "@new" files, as listed above.  They are presumably'
echo 'left over from a failed (or undone) previous processing.  In any case,'
echo 'they will have to be removed to proceed.  (If you may want to keep them,'
echo 'move them out of the way to somewhere else *now*.)'
echo
"$L2HTOP/bin/choose.exe" qd "Press key d to delete the above files or q to quit undo"
set stat="$status"
if ($stat >= 3) goto errsys
if ($stat >= 2) goto dodelnw
goto errq
dodelnw:
source "$1"@old.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvrm.bat" new
if ($status) goto errrmn
enddelnw:

# move the current files to be replaced out of the way
source "$1"@old.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvmv0.bat" old new
if ($status) goto errmvn
rm "$1.bat"
if !(-f "$1.bat") goto mvold
echo "** Unable to remove expired file list $1.bat."
echo '   Please do that manually now or undo will be corrupted.'
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-f "$1.bat") goto mvold
goto errrm0

# move the backups into place
mvold:
cp -p "$1@old.bat" "$1.bat"
if ($status) goto errsys
if !(-f "$1.bat") goto errsys
source "$1"@old.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvmv1.bat" old
if ($status) goto errmv0
rm "$1@old.bat"
if !(-f "$1@old.bat") goto look2
echo "** Unable to remove expired backup file list $1@old.bat."
echo '   Please do that manually now or undo will be corrupted.'
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-f "$1@old.bat") goto look2
goto errrm1

# see whether the @older level is empty
look20:
echo '** No files could be restored: the latest backup level was empty'
echo "   or its $1@old.bat file corrupted."
look2:
echo 'Will now look for, but not yet restore, nonempty older backup levels.'
if !(-f "$1@older.bat") goto look3
source "$1"@older.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvchk.bat" older
if ($status) goto mv21
rm "$1@older.bat"
if !(-f "$1@older.bat") goto look3
echo "** Unable to remove expired backup file list $1@older.bat."
echo '   Please do that manually now or undo will be corrupted.'
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-f "$1@older.bat") goto look3
goto errrm2

# move the @older level into @old
mv21:
echo 'Found an older backup level.  Using undo a second time will restore it.'
cp -p "$1@older.bat" "$1@old.bat"
if ($status) goto errsys
if !(-f "$1@old.bat") goto errsys
source "$1"@older.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvrm.bat" old
if ($status) goto errstray
source "$1"@older.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvmv.bat" older old
if ($status) goto errmv
rm "$1@older.bat"
if !(-f "$1@older.bat") goto mv32
echo "** Unable to remove expired backup file list $1@older.bat."
echo '   Please do that manually now or undo will be corrupted.'
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-f "$1@older.bat") goto mv32
goto errrm2

# see whether the @oldest level is empty
look3:
if !(-f "$1@oldest.bat") goto look3e
source "$1"@oldest.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvchk.bat" oldest
if ($status) goto mv320
rm "$1@oldest.bat"
if !(-f "$1@oldest.bat") goto look3e
echo "** Unable to remove expired backup file list $1@oldest.bat."
echo '   Please do that manually now or undo will be corrupted.'
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-f "$1@oldest.bat") goto look3e
goto errrm3
look3e:
echo 'There are no older backups.'
goto done

# move the @oldest level into @older
mv320:
echo 'Found an older backup level.  Using undo two more times will restore it.'
mv32:
if !(-f "$1@oldest.bat") goto done
cp -p "$1@oldest.bat" "$1@older.bat"
if ($status) goto errsys
if !(-f "$1@older.bat") goto errsys
source "$1"@oldest.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvrm.bat" older
if ($status) goto errstray
source "$1"@oldest.bat "$L2HTOP/system-files/bin/file_extsel.bat" "$2" "$L2HTOP/system-files/bin/file_bcklvmv.bat" oldest older
if ($status) goto errmv
rm "$1@oldest.bat"
if !(-f "$1@oldest.bat") goto done
echo "** Unable to remove expired backup file list $1@oldest.bat."
echo '   Please do that manually now or undo will be corrupted.'
"$L2HTOP/bin/choose.exe" anykey "Hit any key when this is done"
if !(-f "$1@oldest.bat") goto done
goto errrm3

# done
done:
"$L2HTOP/bin/choose.exe" anykey "After reading the above, hit any key to finish"
goto end

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errfils:
echo '*** Fatal error: essential files are missing from the l2h folder.'
echo '    You may need to reinstall l2h from scratch.'
goto err
errfil:
echo "*** Cannot undo: there is no list of files $1.bat."
goto err
errrmn:
echo '*** Unable to remove the existing orphaned @new files.'
goto err
errmvn:
echo '*** Unable to move the existing files to be replaced out of the way.'
goto err
errrm0:
echo "*** File list $1.bat cannot be deleted.  Why?"
echo '    Check your permission to delete it.  Or a disk going bad?\!\!'
goto err
errrm1:
echo "*** Backup file list $1@old.bat cannot be deleted.  Why?"
echo '    Check your permission to delete it.  Or a disk going bad?\!\!'
goto err
errrm2:
echo "*** Backup file list $1@older.bat cannot be deleted.  Why?"
echo '    Check your permission to delete it.  Or a disk going bad?\!\!'
goto err
errrm3:
echo "*** Backup file list $1@oldest.bat cannot be deleted.  Why?"
echo '    Check your permission to delete it.  Or a disk going bad?\!\!'
goto err
errmv0:
echo '*** Unable to replace the current files by the backups.'
goto err
errmv:
echo '*** Unable to move older backups into more current levels.'
goto err
errstray:
echo '*** Unable to get rid of stray backups.'
goto err
errq:
echo '*** Aborted by the user.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
