#!/usr/bin/tcsh -f
set echo_style='bsd' ; setenv LC_ALL C #; set echo verbose

# Copyright 2012 Leon van Dommelen.  This software is made available
# under the GNU Public License version 3.  See folder copyrights.

# Purpose: Update any existing web pages if the settings have changed.
#          Returns ans=q if no remake is needed, r if remake is needed,
#          and s if remake from scratch is needed.

# Usage
goto start
use:
echo 'Usage: Ensure that folder @system exists.'
echo '       (This will use tmp1.txt, tmp2.txt and and tmp.bat in that folder.)'
echo '       source updwp.bat UPCSS UPBUT PL BUTSZ IMGSZ PGCOL [QUIET]'
echo '       UPCSS if - disables updating index.css'
echo '       UPBUT if - disables updating the buttons.'
echo '       PL if - disables the test on perl differences in general.'
echo '       BUTSZ if - disables the test on differences in button sizes.'
echo '       IMGSZ if - disables the test on image sizes.'
echo '       PGCOL if - disables the test on page color.'
echo '       [QUIET] if q suppresses messages about the need for remake.'
goto err
start:

# initialize to no remake
set ans='q'

# done if no web pages
if !(-e web-pages/) goto done

# set L2HTOP (do not change the format of the next two lines)
setenv L2HTOP '/home/user/l2hdir'
if !(-f "$L2HTOP/install_l2h.desktop") goto errl2h

# check for the configuration files
if !(-f index.css) goto errfils
if !(-f settings.pl) goto errfils

# copy index.css into the web-pages folder
if ("$1" == '-') goto upcssend
if (-f web-pages/index.css) rm web-pages/index.css
if (-e web-pages/index.css) goto errsys
cp -p index.css web-pages
if ($status) goto errsys
if !(-f web-pages/index.css) goto errsys
chmod a+r web-pages/index.css
if ($status) goto errsys
upcssend:

# update the buttons
if ("$2" == '-') goto upbutend
cat settings.pl | fgrep '..${dd}buttons' > @system/tmp.bat
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto upbut2
if !(-d buttons/) goto errbuts
source "$L2HTOP/system-files/bin/buttonsall.bat" "$L2HTOP/system-files/bin/updicon.bat" buttons/
if ($status) goto err
goto upbutend
upbut2:
if (-f @system/tmp1.txt) rm @system/tmp1.txt
if (-e @system/tmp1.txt) goto errsys
"$L2HTOP/bin/subbin.exe" "$L2HTOP/system-files/data/getthm.sub" settings.pl @system/tmp1.txt
if ($status) goto errsys
if !(-f @system/tmp1.txt) goto errpcur
cat "$L2HTOP/system-files/data/copthm.bat" >> @system/tmp1.txt
if ($status) goto errsys
if (-f @system/tmp.bat) rm @system/tmp.bat
if (-e @system/tmp.bat) goto errsys
"$L2HTOP/bin/subbin.exe" "$L2HTOP/system-files/data/mksubfil.sub" @system/tmp1.txt @system/tmp.bat
if ($status) goto errsys
if !(-f @system/tmp.bat) goto errsys
source "@system/tmp.bat" updicon
if ($status) goto err
upbutend:

# check that we have settings.pl in the web-pages folder
if !(-f web-pages/settings.pl) goto wrnfil

# test for differences in settings.pl in general
if ("$3" == '-') goto plend
if (-f @system/tmp1.txt) rm @system/tmp1.txt
if (-e @system/tmp1.txt) goto errsys
if (-f @system/tmp2.txt) rm @system/tmp2.txt
if (-e @system/tmp2.txt) goto errsys
cat settings.pl | fgrep -v '$ALTERNATIVE_ICONS' > @system/tmp1.txt
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto errpcur
if !(-f @system/tmp1.txt) goto errsys
cat web-pages/settings.pl | fgrep -v '$ALTERNATIVE_ICONS' > @system/tmp2.txt
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto wrnpref
if !(-f @system/tmp2.txt) goto errsys
cmp @system/tmp1.txt @system/tmp2.txt > @system/tmp.bat
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto plupd
goto pgcol
plupd:
set ans='r'
if ("$7" == q) goto plend
echo '** The web pages must be remade because the web page settings have changed.'
plend:

# test for differences in button sizes
if ("$4" == '-') goto butszend
if (-f @system/tmp1.txt) rm @system/tmp1.txt
if (-e @system/tmp1.txt) goto errsys
if (-f @system/tmp2.txt) rm @system/tmp2.txt
if (-e @system/tmp2.txt) goto errsys
cat settings.pl | fgrep "','WIDTH=" > @system/tmp1.txt
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto errpcur
if !(-f @system/tmp1.txt) goto errsys
cat web-pages/settings.pl | fgrep "','WIDTH=" > @system/tmp2.txt
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto wrnpref
if !(-f @system/tmp2.txt) goto errsys
cmp @system/tmp1.txt @system/tmp2.txt > @system/tmp.bat
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto butszupd
goto butszend
butszupd:
set ans='r'
if ("$7" == q) goto butszend
echo '** The web pages must be remade or the buttons will not be properly sized.'
butszend:

# test for differences in image sizes
if ("$5" == '-') goto imgszend
if (-f @system/tmp1.txt) rm @system/tmp1.txt
if (-e @system/tmp1.txt) goto errsys
if (-f @system/tmp2.txt) rm @system/tmp2.txt
if (-e @system/tmp2.txt) goto errsys
cat settings.pl | fgrep '_SCALE_FACTOR' > @system/tmp1.txt
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto errpcur
if !(-f @system/tmp1.txt) goto errsys
cat web-pages/settings.pl | fgrep '_SCALE_FACTOR' > @system/tmp2.txt
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto wrnpref
if !(-f @system/tmp2.txt) goto errsys
cmp @system/tmp1.txt @system/tmp2.txt > @system/tmp.bat
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto imgszupd
goto imgszend
imgszupd:
set ans='s'
if ("$7" == q) goto imgszend
echo '*** The web pages must not just be remade, but remade from scratch,'
echo '    because the needed sizes of mathematical and/or figure images have'
echo '    changed.'
imgszend:

# check page color differences
pgcol:
if ("$6" == '-') goto pgcolend
if !(-f web-pages/@transcol.txt) goto wrnpgc
if (-f @transcol.txt) rm @transcol.txt
if (-e @transcol.txt) goto errsys
"$L2HTOP/system-files/bin/getpgch.bat"
if ($status) goto err
if !(-f @transcol.txt) goto errsys
cmp @transcol.txt web-pages/@transcol.txt > @system/tmp.bat
set stat="$status"
if ($stat >= 2) goto errsys
if ($stat >= 1) goto pgcolupd
rm @transcol.txt
goto pgcolend
pgcolupd:
rm @transcol.txt
set ans='s'
if ("$7" == q) goto pgcolend
echo '*** The web pages must not just be remade, but remade from scratch,'
echo '    because the background color has changed.  If not, there will be'
echo '    ugly "ghosting" around mathematical symbols and lines in figures.'
pgcolend:

# done
done:
goto end

# warnings
wrnfil:
set ans='s'
echo '*** The used file settings.pl is gone from the web-pages folder.'
echo '    So it is recommended that you remake the web pages from scratch.'
goto done
wrnpref:
set ans='s'
echo '*** The previous file settings.pl seems to have been corrupt.'
echo '    So it is recommended that you remake the web pages from scratch.'
goto done
wrnpgc:
set ans='s'
echo '*** The previously used backgound color is gone from folder web-pages.'
echo '    So it is recommended that you remake the web pages from scratch.'
goto done

# errors
errl2h:
echo "*** Fatal error: cannot find file $L2HTOP/install_l2h.desktop."
echo '    You may need to rerun install_l2h in the l2h folder.  Make sure'
echo '    that the current folder is somewhere inside the l2h folder while'
echo '    doing that.'
goto err
errfils:
echo '*** File index.css and/or settings.pl not found.'
goto err
errbuts:
echo '*** The web page buttons are supposed to be taken from local subfolder'
echo '    buttons, but there is no such folder.'
goto err
errpcur:
echo '*** File settings.pl seems to be corrupt.  Try using a different theme.'
goto err
errsys:
echo '*** Operating system error.  Check disk space and file protections.'
echo '    If not, try rerunning install_l2h in the l2h_folder.'
err:
exit 1

end:
