      program dojpgs

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     Swap in jpegs for the gifs.

c     Requires input file @tmp.ind containing:
c        1) single-character filename prefix or * for none
c        2) folder separator (\ for DOS or / for unix)
c        3) new jpg files as found in the web pages folder
c        4) blank line
c        5) old jpg files saved in the savjpegs subfolder
c        6) blank line
c        7) the html files
c
c     For each new jpeg, prompts the user whether to use this jpg
c     instead of the gif.  Then changes the html files to use the
c     desired jpegs.
c
c     Creates DOS or tcsh script @dojpegs.bat, depending on separator,
c     with instructions to remove undesired or obsolete jpegs, move
c     desired ones into place, and move updated html files into place.
c     This script will not get rid of folder savjpegs.

c     avoid typos
      implicit none

c     declared file name length (change format 1 if changing this)
      integer fildim
      parameter (fildim=15)

c     read in record from [prefix]images.pl

c     declared size of the record
      integer recdim
      parameter (recdim=4096)
c     actual size of the record
      integer recmax
c     position in the record
      integer rec
c     record
      character*(recdim) recstr
c     file name
      character*(fildim) recfil
      integer recfll
      integer recprl
c     file number
      integer recfno
c     record counter
      integer reccnt

c     data base of the jpg files

c     declared number of jpg files
      integer jpgdim
      parameter (jpgdim=5000)
c     actual number of jpg files
      integer jpgmax
c     jpeg index
      integer jpg
c     whether this is an old or newly generated jpeg
      integer jpgage(jpgdim),new,old
      parameter (new=1,old=2)
c     jpg file number
      integer jpgfno(jpgdim)
c     the gif number it replaces
      integer jpgrep(jpgdim)
c     total substitution count in the html files
      integer jpgsub(jpgdim)
c     jpg file name
      character*(fildim) jpgfil(jpgdim)
      integer jpgfll(jpgdim)
c     jpeg file name prefix length
      integer jpgprl(jpgdim)
c     where the jpeg's record ends in the buffer
      integer jpgend(0:jpgdim)
      data jpgend(0)/0/
c     record buffer
      integer jpbdim
      parameter (jpbdim=400*jpgdim)
      character*1 jpbstr(jpbdim)

c     declared primary and secondary file buffer sizes
      integer bufdim
      parameter (bufdim=500000)
c     actual file buffer sizes
      integer bufmax,bufmx2
c     file buffers
      character*1 bufstr(bufdim),bufst2(bufdim)
c     whether we use buffer 2
      integer use2

c     html file name
      character*(fildim) htmfil
      integer htmfll

c     current age
      integer curage

c     default action on new jpegs
      character*1 defact

c     prefix
      character*1 prefix

c     directory delimiter
      character*1 dd

c     counter of matches found
      integer fndct
c     counter of nontrivial substitutions done
      integer subct
c     total nontrivial substitutions done
      integer totsct

c     position of Timg in the file name
      integer timg

c     number as character string
      character*20 cnum
      integer cnuml

c     file name length
      integer fll

c     line
      character*80 line

c     answer
      character*1 ans

c     character
      character*1 chr

c     generic indices
      integer i,i2

c executable

c     format to read in a jpg file
 1    format(a15)

c     generic character format
 10   format(a,a,a)

c     set the I/O routines
      call libini(10)

c     open the input file
      open(1,file='@tmp.ind',status='old',err=9900)
 110  format(a1)
c     read the prefix from it or * for none
      read(1,110,err=9910)prefix
c     read the directory delimiter from it
      read(1,110,err=9910)dd
      if(index('/\\',dd).le.0)call out('*** dojpegs: '//
     & 'invalid directory delimiter in input file @tmp.ind !')

c     open the script files
      call opennw(8,'@dojpegs.bat')
      if (dd.eq.'\\')then
         write(8,10,err=9920)'@echo off'
      else
         write(8,10,err=9920)'#!/usr/bin/tcsh -f'
      endif

c     read the jpg files from the input file
      curage=new
      defact='?'
      jpgmax=0
      do 2000 jpg=1,jpgdim
c        read name
         read(1,1,err=9910)jpgfil(jpg)
c        blank lines mean the end of the new and old jpeg sequences
         if(jpgfil(jpg).eq.' ')then
            if(curage.eq.old)goto 2010
            curage=old
            read(1,1,err=9910)jpgfil(jpg)
            if(jpgfil(jpg).eq.' ')goto 2010
         endif
c        next jpg
         jpgmax=jpg
c        initialize the jpeg data
         jpgage(jpg)=curage
         jpgrep(jpg)=curage-2
         jpgsub(jpg)=0
         jpgend(jpg)=jpgend(jpg-1)
c        find name length
         if(jpgfil(jpg)(fildim:fildim).ne.' ')call out('*** dojpegs: '//
     &    'jpeg file name "'//jpgfil(jpg)//'" too long in @tmp.ind!')
         if(jpgfil(jpg)(1:1).le.' ')call out('*** dojpegs: invalid '//
     &    'jpeg file name "'//jpgfil(jpg)(1:fildim-1)//
     &    '" in @tmp.ind!')
         jpgfll(jpg)=index(jpgfil(jpg),' ')-1
         if(jpgfil(jpg)(max(1,jpgfll(jpg)-3):jpgfll(jpg)).ne.'.jpg')
     &    call out('*** dojpegs: invalid jpeg file name "'//jpgfil(jpg)
     &    //'" in @tmp.ind!')
c        find file number
         do 1210 i=1,jpgfll(jpg)-4
            jpgprl(jpg)=i-1
            chr=jpgfil(jpg)(i:i)
            if(chr.ge.'1' .and. chr.le.'9')goto 1211
 1210    continue
         call out('*** dojpegs: unnumbered jpeg file name "'//
     &    jpgfil(jpg)(1:jpgfll(jpg))//'" in @tmp.ind!')
 1211    if(jpgprl(jpg).le.0)call out('*** dojpegs: jpeg file name "'//
     &    jpgfil(jpg)(1:jpgfll(jpg))//'" may not start with a digit!')
         jpgfno(jpg)=0
         do 1220 i=jpgprl(jpg)+1,jpgfll(jpg)-4
            chr=jpgfil(jpg)(i:i)
            if(chr.lt.'0' .or. chr.gt.'9')call out('*** dojpegs: '//
     &       'misnumbered jpeg file name "'//
     &       jpgfil(jpg)(1:jpgfll(jpg))//'" in @tmp.ind!')
            jpgfno(jpg)=jpgfno(jpg)*10+ichar(chr)-48
 1220    continue
c        ask whether to use this jpeg if new
         if(curage.eq.new)then
            if(defact.eq.'?')then
               call out2('Press "y" to use the jpeg version or "n" '//
     &          'to keep the gif version.  Press "a"')
               call out2('to use all further jpeg versions, "q" to '//
     &          'keep all further gif versions.  You')
               call out2('may want to load the jpegs and gifs in an '//
     &          'image viewer to decide.')
               line='Do you want to use image '//
     &          jpgfil(jpg)(1:jpgfll(jpg))//
     &          ' instead of the gif version? [y/n/a/q]:'
               call inkey(5,'ynaq',line,ans)
               if(ans.eq.'a')defact='a'
               if(ans.eq.'q')defact='q'
            endif
            if(defact.eq.'a')ans='y'
            if(defact.eq.'q')ans='n'
            if(ans.eq.'y')then
               jpgrep(jpg)=0
            else
               if (dd.eq.'\\')then
                  write(8,1610)jpgfil(jpg)(1:jpgfll(jpg)),
     &             jpgfil(jpg)(1:jpgfll(jpg))
 1610             format('del ',a,/,'if exist ',a,' goto errsys')
               else
                  write(8,1620)jpgfil(jpg)(1:jpgfll(jpg)),
     &             jpgfil(jpg)(1:jpgfll(jpg))
 1620             format('rm ',a,/,'if (-f ',a,') goto errsys')
               endif
            endif
         endif
c        read the identification string
         if(curage.eq.old)then
            call readfu(
     &       'savjpegs'//dd//jpgfil(jpg)(1:jpgfll(jpg)-3)//'jpd',
     &       jpgfll(jpg)+9,2,
     &       jpbdim-jpgend(jpg-1),i,jpbstr(jpgend(jpg-1)+1))
            jpgend(jpg)=jpgend(jpg-1)+i
         endif
 2000 continue
      call out('*** dojpegs: too many jpegs!')
 2010 continue

c     open the images.perl file
      if(prefix.eq.'*')then
         call openru('images.pl',2)
      else
         call openru(prefix//'images.pl',2)
      endif
      reccnt=-1

c     read the next record
 5000 rec=0
 5010 call getchr(2,chr,*5020,*5030)
      if(chr.eq.char(13))goto 5010
      goto 5050
c     end of file
 5020 call out('*** dojpegs: premature end to images.pl!')
c     read error
 5030 call out('*** dojpegs: error reading images.pl!')

c     process line end
 5050 if(chr.eq.char(10))then
c        end of file marker
         if(rec.eq.2 .and. recstr(1:2).eq.'1;')goto 5900
c        ignore blank record
         if(rec.eq.0)goto 5000
c        end of record
         if(recstr(rec:rec).eq.char(10))then
            reccnt=reccnt+1
c           ignore the header
            if(reccnt.le.0)goto 5000
c           go process the record
            goto 5100
         endif
      endif
c     add the character to the record
      rec=rec+1
      if(rec.gt.recdim)call out('*** dojpegs: record too long!')
      recstr(rec:rec)=chr
c     go read the next character
      goto 5010

c     read the file name from the record and remove unnecessary junk
 5100 recmax=rec
      if(recmax.le.51)call out('*** dojpegs: corrupt '//
     & 'record in images.pl: '//recstr(1:min(33,recmax))//'...!')
c     remove the invariant perl junk to save file space and effort
      if(recstr(1:8).ne.'$key = q')call out('*** dojpegs: '//
     & 'unrecognized record in images.pl: '//recstr(1:min(28,recmax))//
     & '...!')
      rec=index(recstr(1:recmax),char(10)//'$cached_env_img{$key} = q')
      if(rec.lt.12)call out('*** dojpegs: bad '//
     & 'record in images.pl: '//recstr(1:min(37,recmax))//'...!')
      do 5110 i=9,rec
         recstr(i-8:i-8)=recstr(i:i)
 5110 continue
      do 5120 i=rec+26,recmax
         recstr(i-33:i-33)=recstr(i:i)
 5120 continue
      recmax=recmax-33
c     find the image line
      rec=index(recstr(1:recmax),char(10)//' SRC="|."$dir".q|')
      if(rec.le.0)call out('*** dojpegs: missing '//
     & 'SRC in images.pl: '//recstr(1:min(36,recmax))//'...!')
c     find the file name
      recfll=index(recstr(rec+17:recmax),'"')-2
      if(recfll.lt.8)call out('*** dojpegs: missing '//
     & 'image in images.pl: '//recstr(1:min(34,recmax))//'...!')
      if(recfll.gt.fildim)call out('*** dojpegs: image '//
     & 'name too long in images.pl: '//recstr(1:min(28,recmax))//'...!')
      recfil(1:recfll)=recstr(rec+18:rec+17+recfll)
      if(recfil(recfll-3:recfll).ne.'.gif')call out('*** dojpegs: '//
     & 'image name corrupt in images.pl: '//recstr(1:min(29,recmax))//
     & '...!')
c     remove the entire spec
      i2=18+recfll+1
      do 5130 i=rec+i2,recmax
         recstr(i-i2:i-i2)=recstr(i:i)
 5130 continue
      recmax=recmax-i2
c     find file number
      do 5140 i=1,recfll-4
         recprl=i-1
         chr=recfil(i:i)
         if(chr.ge.'1' .and. chr.le.'9')goto 5141
 5140 continue
      call out('*** dojpegs: unnumbered gif file name "'//
     & recfil(1:recfll)//'" in images.pl!')
 5141 if(recprl.le.0)call out('*** dojpegs: gif file name "'//
     & recfil(1:recfll)//'" may not start with a digit!')
      recfno=0
      do 5150 i=recprl+1,recfll-4
         chr=recfil(i:i)
         if(chr.lt.'0' .or. chr.gt.'9')call out('*** dojpegs: '//
     &    'misnumbered gif file name "'//
     &    recfil(1:recfll)//'" in images.pl!')
         recfno=recfno*10+ichar(chr)-48
 5150 continue

c     for a thumbnail also remove the <A NAME="figure... HREF tag from recstr
 5200 timg=index(recfil(1:recprl),'Timg')
      if(timg.gt.0)then
         rec=index(recstr(1:recmax),'<A NAME="figure')
         if(rec.le.0)call out(
     &    '*** dojpegs: unrecognized thumbnail entry in images.pl')
         i2=index(recstr(rec:recmax),'<IMG')+4
         if(i2.lt.20)call out(
     &    '*** dojpegs: ununderstood thumbnail entry in images.pl')
         do 5210 i=rec+i2,recmax
            recstr(i-i2:i-i2)=recstr(i:i)
 5210    continue
         recmax=recmax-i2
      endif

c     find this image in the jpeg list
      do 5500 jpg=1,jpgmax
         if(jpgage(jpg).eq.new)then
            if(jpgrep(jpg).gt.0)goto 5500
            if(jpgfno(jpg).ne.recfno)goto 5500
            if(jpgfil(jpg)(1:jpgprl(jpg)).ne.recfil(1:recprl))goto 5500
            if(jpgrep(jpg).lt.0)goto 5510
            call writfu(jpgfil(jpg)(1:jpgfll(jpg)-4)//'.jpd',
     &       jpgfll(jpg),3,recdim,recmax,recstr)
            jpgrep(jpg)=recfno
            goto 5510
         else
            if(jpgrep(jpg).gt.0)goto 5500
            if(recmax.ne.jpgend(jpg)-jpgend(jpg-1))goto 5500
            if(jpgfil(jpg)(1:jpgprl(jpg)).ne.recfil(1:recprl))goto 5500
            do 5450 i=1,recmax
               if(recstr(i:i).ne.jpbstr(i+jpgend(jpg-1)))goto 5500
 5450       continue
            jpgrep(jpg)=recfno
            goto 5510
         endif
 5500 continue
 5510 continue

c     done if not a thumbnail
      if(timg.le.0)goto 5000

c     remove the T
      do 5550 i=timg+1,recfll
         recfil(i-1:i-1)=recfil(i:i)
 5550 continue
      recfll=recfll-1
      recprl=recprl-1

c     go process it
      goto 5200

c     done reading images.pl
 5900 close(2,err=5030)

c     skip the next step if no replaceable gifs

      do 5990 jpg=1,jpgmax
         if(jpgrep(jpg).gt.0)goto 6000
 5990 continue
      goto 6900

c     replace the appropriate images in all html files

c     read the file name
 6000 read(1,1,err=9910,end=6900)htmfil
c     blank lines mean the end
      if(htmfil.eq.' ')goto 6900
c     file length
      htmfll=index(htmfil,'.html')+4
c     ignore nonl2h html files that might be there
      if(htmfll.lt.6 .or. htmfll.gt.fildim)goto 6000

c     read the file into the buffer
      call readfu(htmfil(1:htmfll),htmfll,2,bufdim,bufmax,bufstr)
      use2=0

c     substitute all appropriate images
      totsct=0
      do 6500 jpg=1,jpgmax
         if(jpgrep(jpg).le.0)goto 6500
         call encint(jpgrep(jpg),cnuml,cnum)
         i=jpgprl(jpg)
         call trnit(
     &    '="'//jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.gif"',
     &    '="'//jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.jpg"',
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
         jpgsub(jpg)=jpgsub(jpg)+subct
         totsct=totsct+subct
 6500 continue

c     write the new html file
      if(totsct.gt.0)then
         if(use2.eq.0)then
            call writfu(htmfil(1:htmfll-4)//'newh',htmfll,
     &       2,bufdim,bufmax,bufstr)
         else
            call writfu(htmfil(1:htmfll-4)//'newh',htmfll,
     &       2,bufdim,bufmx2,bufst2)
         endif
         if (dd.eq.'\\')then
            write(8,6510)
     &       htmfil(1:htmfll-4),htmfil(1:htmfll),htmfil(1:htmfll-4)
 6510       format('move /y ',a,'newh ',a,/,
     &       'if exist ',a,'newh goto errsys')
         else
            write(8,6520)
     &       htmfil(1:htmfll-4),htmfil(1:htmfll),htmfil(1:htmfll-4)
 6520       format('mv ',a,'newh ',a,/,'if (-f ',a,'newh) goto errsys')
         endif
      endif

c     next
      goto 6000

 6900 continue

c     finish up the script files

c     move the reused old jpeg files in
      do 7500 jpg=1,jpgmax
         fll=jpgfll(jpg)
         if(jpgage(jpg).eq.new)then
            if(jpgrep(jpg).lt.0)goto 7500
            if(jpgrep(jpg).eq.0)call out('*** dojpegs: created "'//
     &       jpgfil(jpg)(1:fll)//'" with no matching gif?!')
         else
            if(jpgsub(jpg).le.0)then
               if (dd.eq.'\\')then
                  write(8,7410,err=9920)
     &             jpgfil(jpg)(1:fll),jpgfil(jpg)(1:fll),
     &             jpgfil(jpg)(1:fll-1)//'d',jpgfil(jpg)(1:fll-1)//'d'
 7410             format(
     &             'del savjpegs\\',a,/,
     &             'if exist savjpegs\\',a,' goto errsys',/,
     &             'del savjpegs\\',a,/,
     &             'if exist savjpegs\\',a,' goto errsys')
               else
                  write(8,7420,err=9920)
     &             jpgfil(jpg)(1:fll),jpgfil(jpg)(1:fll),
     &             jpgfil(jpg)(1:fll-1)//'d',jpgfil(jpg)(1:fll-1)//'d'
 7420             format(
     &             'rm savjpegs/',a,/,
     &             'if (-f savjpegs/',a,') goto errsys',/,
     &             'rm savjpegs/',a,/,
     &             'if (-f savjpegs/',a,') goto errsys')
               endif
            else
               call encint(jpgrep(jpg),cnuml,cnum)
               i=jpgprl(jpg)
               if (dd.eq.'\\')then
                  write(8,7430,err=9920)
     &             jpgfil(jpg)(1:fll),
     &             jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.jpg',
     &             jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.jpg',
     &             jpgfil(jpg)(1:fll-1)//'d',
     &             jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.jpd',
     &             jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.jpd'
 7430             format(
     &             'move savjpegs\\',a,' .\\',a,/,
     &             'if not exist ',a,' goto errsys',/,
     &             'move savjpegs\\',a,' .\\',a,/,
     &             'if not exist ',a,' goto errsys')
               else
                  write(8,7440,err=9920)
     &             jpgfil(jpg)(1:fll),
     &             jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.jpg',
     &             jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.jpg',
     &             jpgfil(jpg)(1:fll-1)//'d',
     &             jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.jpd',
     &             jpgfil(jpg)(1:i)//cnum(1:cnuml)//'.jpd'
 7440             format(
     &             'mv savjpegs/',a,' ./',a,/,
     &             'if !(-f ',a,') goto errsys',/,
     &             'mv savjpegs/',a,' ./',a,/,
     &             'if !(-f ',a,') goto errsys')
               endif
            endif
         endif
 7500 continue

c     write the trailer
      if (dd.eq.'\\')then
         write(8,7510)
 7510    format('echo OK | find "OK" > nul',/,
     &    'goto end',/,
     &    ':errsys',/,
     &    'echo *** dojpegs2: an unexplained error occurred.  '
     &    ,'Check disk and file attributes.',/,
     &    'echo exit | find "error" > nul',/,
     &    ':end')
      else
         write(8,7520)
 7520    format('goto end',/,
     &    'errsys:',/,
     &    'echo ''*** dojpegs2: an unexplained error occurred.  '
     &    ,'Check disk and file attributes.''',/,
     &    'exit 1',/,
     &    'end:')
      endif

c     all done
      call exit(0)

c     errors
 9900 call out('*** dojpegs: Unable to open input file @tmp.ind?!')
 9910 call out('*** dojpegs: Unable to read input file @tmp.ind?!')
 9920 call out('*** dojpegs: Unable to write to file?! Check disk!')

      end
