      subroutine dopair(
     & reinit,
     & orgdim,orgmax,orgstr,orgmod,trndim,trnmax,trnstr,
     & memdim,memmax,memstr,
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,
     & fndct,subct,
     & *,*)

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     Obtain an original/ translation pair from getsub and process it.
c     Takes exit 1 on end of the substitutions file and exit 2 if the
c     substitutions file must be re-opened.

c     Assumes that we are performing a sequence of substitutions on the
c     same data.  If the data changes, be sure to deselect all and/or to
c     reinitialize as appropriate.

c     avoid typos
      implicit none

c input

c     reinitialization: 0=no reinitialization, 1=reinitialize
      integer reinit

c work arrays

c     declared original string size
      integer orgdim
c     actual original string size
      integer orgmax
c     original string
      character*1 orgstr(orgdim)
c     modifiers
      character*1 orgmod(orgdim)

c     declared translation string size
      integer trndim
c     actual translation string size
      integer trnmax
c     translation string
      character*1 trnstr(trndim)

c     declared memory string size
      integer memdim
c     actual memory size (must be saved)
      integer memmax
c     memory string (must be saved)
      character*1 memstr(memdim)

c input/output

c     declared primary and secondary file buffer sizes
      integer bufdim
c     actual file buffer sizes
      integer bufmax,bufmx2
c     file buffers
      character*1 bufstr(bufdim),bufst2(bufdim)

c     whether the original is in bufst2 (may not change for specials.)
      integer use2

c output:

c     counter of matches found
      integer fndct
c     counter of nontrivial substitutions done
      integer subct

c common

c     declared substring selection levels
      integer levdim
      parameter (levdim=1000)
c     actual substring selection level
      integer lev
c     available bufdim at each level
      integer bufdml(0:levdim)
c     start of the post-selection string
      integer post(levdim)
c     line in the substitutions file at which the level was selected
      integer sellin(0:levdim)

c     after a successfull \f#, the #-th occurrence is from fnddat(2) to (3)
      integer fnddat(3)

c     total substitutions
      integer subtot
c     total found strings
      integer fndtot
c     total missed substitutions
      integer subnot
c     total missed searches
      integer fndnot
c     previous value of subtot at \x command
      integer oldsct

c     dopair block
      common/cmpair/lev,bufdml,post,sellin,
     & fnddat,
     & subtot,fndtot,subnot,fndnot,oldsct

c     verbosity
      integer verbos
c     substitions file
      character*132 subfil
      integer subfll
c     line count of the substitutions file
      integer lincts
c     text file
      character*132 txtfil
      integer txtfll
c     line count of the text file
      integer linctt

c     debug block
      common/cmdbg/verbos,subfil,subfll,lincts,txtfil,txtfll,linctt

c local

c     whether we need to initialize
      integer init
      save init
      data init/1/

c     target line of the substitions file to reach (-1 for next)
      integer tarlin
      save tarlin
      data tarlin/-1/

c     specials for the original optionally followed by numbers
      character*2 orgspn
      save orgspn
      data orgspn/'df'/
c     specials not followed by a translation
      character*3 notran
      save notran
      data notran/'drx'/

c     translation commands that convert the original
      character*19 trncnv
      save trncnv
      data trncnv/'"$&''()456<=>`hlu{|}'/
c     special translation commands that select a particular instance
      character*10 trnspn
      save trnspn
      data trnspn/'bcfsv^.,!?'/
c     whether this is a translation special that just selects inside dosubs
      integer trnsel

c     ascii value of blackslash
      integer asc_bs
      parameter (asc_bs=92)
c     ascii values of 0, 9, plus and minus
      integer asc_0,asc_9,asc_pl,asc_mn
      parameter (asc_0=48,asc_9=57,asc_pl=43,asc_mn=45)
c     ascii values of i, o, and x
      integer asc_i,asc_o,asc_x
      parameter (asc_i=105,asc_o=111,asc_x=120)
c     ascii index
      integer iasc

c     available buffer dimension
      integer bufdm

c     substring selection level index
      integer lev2

c     output line
      character*1 out(85)

c     integer as string
      character*12 cint
      integer cintl

c     character indices and maximum value
      integer i,i2,imax

c executable

c     formats
 10   format(a,a,a,a,a,74a1)

c     note where we are
      call insub('dopair')

c     initialize
      if(init.ne.0 .or. reinit.ne.0)then
c        make sure that no more than 123456788 substitutions are possible
         if(bufdim.gt.123456788/2-2)
     &    call fatal('Program error: bufdim is too large')
c        clear memory
         if(init.ne.0)memmax=-1
c        clear all selection levels
         lev=0
         bufdml(0)=bufdim
c        clear counts
         subtot=0
         fndtot=0
         subnot=0
         fndnot=0
         oldsct=-1
         init=0
      endif

c     active buffer size
      bufdm=bufdml(lev)

c     read the next original and translation
 100  call getsub(memdim,memmax,memstr,orgdim,trndim,
     & orgmax,orgstr,orgmod,trnmax,trnstr)

c     exit on end of file
      if(orgmax.eq.0)then
         if(tarlin.ne.-1)call fatal(
     &    'Program error: cannot return to start of loop')
         if(verbos.ge.1)then
            print10
            print10,'End of substitutions file'
         endif
         goto 9910
      endif

c     log it
      if(verbos.ge.2)then
         print10
         if(orgmax.lt.0)then
            cintl=0
            if(index(orgspn,orgmod(1)).ne.0 .and. orgmax.ne.-123456789)
     &       call inttos(-orgmax,cint,cintl)
            print10,'\\',orgmod(1),(cint(i:i),i=1,cintl)
            if(index(notran,orgmod(1)).ne.0)goto 1990
            goto 1950
         endif
         i2=0
         do 1930 i=1,orgmax
            if(orgmod(i).eq.char(0))then
               call encstr(1,orgstr(i),78-i2,imax,out(i2+1))
               i2=i2+imax
               if(i2.gt.77)goto 1931
            else
               i2=i2+1
               if(i2.gt.77)goto 1931
               out(i2)='\\'
               i2=i2+1
               if(i2.gt.77)goto 1931
               out(i2)=orgmod(i)
            endif
 1930    continue
 1931    if(i2.le.77)print10,'"',(out(i),i=1,i2),'"'
         if(i2.gt.77)print10,'"',(out(i),i=1,74),'.','.','.','"'
 1950    if(trnmax.lt.0)then
            if(index(trnspn,trnstr(1)).eq.0)goto 1970
            cintl=0
            if(trnmax.ne.-123456789)call inttos(-trnmax,cint,cintl)
            if(index('.,?!',trnstr(1)).eq.0)then
               print10,'"','\\',trnstr(1),(cint(i:i),i=1,cintl),'"'
            else
               do 1960 i=2,trndim
                  if(trnstr(i).eq.char(0))goto 1961
 1960          continue
               call fatal('Program error: \\'//trnstr(1)//
     &          ' message is missing')
 1961          i2=77-3-cintl
               call encstr(i-2,trnstr(2),i2+1,imax,out)
               if(imax.le.i2)then
                  print10,'"','\\',trnstr(1),(cint(i:i),i=1,cintl),':',
     &             (out(i),i=1,imax),'"'
               else
                  print10,'"','\\',trnstr(1),(cint(i:i),i=1,cintl),':',
     &             (out(i),i=1,i2-3),'.','.','.','"'
               endif
            endif
            goto 1990
 1970       call encstr(-trnmax,trnstr,77,imax,out)
            if(imax.le.76)print10,'"','\\',(out(i),i=1,imax),'"'
            if(imax.gt.76)print10,
     &       '"','\\',(out(i),i=1,73),'.','.','.','"'
            goto 1990
         endif
         i2=0
         i=0
 1980    i=i+1
         if(i.gt.trnmax)goto 1989
         i2=i2+1
         if(i2.ge.78)goto 1989
         iasc=ichar(trnstr(i))
         out(i2)=char(iasc)
         if(iasc.lt.32 .or. iasc.eq.asc_bs .or. iasc.gt.127)then
            call encstr(1,out(i2),78-i2+1,imax,out(i2))
            i2=i2+imax-1
            if(i2.ge.78)goto 1989
         endif
         if(iasc.ne.127)goto 1980
         if(i.ge.trnmax)
     &    call fatal('Program error: incomplete DEL sequence')
         i=i+1
         iasc=ichar(trnstr(i))
         if(iasc.eq.127)then
            out(i2)='\\'
            i2=i2+1
            if(i2.ge.78)goto 1989
            out(i2)='2'
         else if(iasc.ge.asc_0 .and. iasc.le.asc_9)then
            out(i2)='\\'
            i2=i2+1
            if(i2.ge.78)goto 1989
            out(i2)='#'
            i2=i2+1
            if(i2.ge.78)goto 1989
            out(i2)=char(iasc)
         else if(
     &       iasc.eq.asc_pl .or. iasc.eq.asc_mn .or.
     &       iasc.eq.asc_i .or. iasc.eq.asc_o .or. iasc.eq.asc_x
     &       )then
            out(i2)='\\'
            i2=i2+1
            if(i2.ge.78)goto 1989
            out(i2)=char(iasc)
         else if(iasc.gt.len(trncnv))then
            call fatal('Program error: undefined DEL sequence')
         else
            out(i2)='\\'
            i2=i2+1
            if(i2.ge.78)goto 1989
            out(i2)=trncnv(iasc:iasc)
         endif
         goto 1980
 1989    if(i2.le.77)print10,'"',(out(i),i=1,i2),'"'
         if(i2.gt.77)print10,'"',(out(i),i=1,74),'.','.','.','"'
 1990    continue
      endif

c     skip if we want to reach a target line
      if(tarlin.ge.0)then
         if(lincts.lt.tarlin)goto 100
         tarlin=-1
      endif

c     handle the original specials

      if(orgmax.gt.0)goto 2900

      if(orgmod(1).eq.'d')then

c        dump -orgmax selection levels
         orgmax=-orgmax
         if(orgmax.eq.123456789)orgmax=lev
         if(orgmax.gt.lev)
     &    call fatal('There are not so many selection levels')
         do 2300 lev2=lev,lev-orgmax+1,-1
            if(use2.eq.0)then
               if(sellin(lev2).ge.0 .and. bufmax.gt.0)
     &          tarlin=sellin(lev2)
               bufmx2=0
               do 2210 i=bufdml(lev2)+1,post(lev2)-1
                  bufmx2=bufmx2+1
                  bufst2(bufmx2)=bufstr(i)
 2210          continue
               do 2220 i=1,bufmax
                  bufmx2=bufmx2+1
                  bufst2(bufmx2)=bufstr(i)
 2220          continue
               do 2230 i=post(lev2),bufdml(lev2-1)
                  bufmx2=bufmx2+1
                  bufst2(bufmx2)=bufstr(i)
 2230          continue
            else
               if(sellin(lev2).ge.0 .and. bufmx2.gt.0)
     &          tarlin=sellin(lev2)
               bufmax=0
               do 2250 i=bufdml(lev2)+1,post(lev2)-1
                  bufmax=bufmax+1
                  bufstr(bufmax)=bufst2(i)
 2250          continue
               do 2260 i=1,bufmx2
                  bufmax=bufmax+1
                  bufstr(bufmax)=bufst2(i)
 2260          continue
               do 2270 i=post(lev2),bufdml(lev2-1)
                  bufmax=bufmax+1
                  bufstr(bufmax)=bufst2(i)
 2270          continue
            endif
            use2=1-use2
            if(tarlin.ge.0)then
                if(verbos.gt.1)then
                   i=bufmax
                   if(use2.ne.0)i=bufmx2
                   i2=lev-lev2+1
                   print2310,i2,i,' looping back'
               endif
               lev=lev2-1
               goto 9920
           endif
 2300    continue
         lev=lev-orgmax
         if(verbos.ge.1)then
            i=bufmax
            if(use2.ne.0)i=bufmx2
            print2310,orgmax,i
 2310       format('Dumped',i3,' levels.  New buffer size',i9,a)
         endif
         goto 9900

      else if(orgmod(1).eq.'r')then

c        go reread substitutions file
         if(verbos.ge.1)print10,'Restarting substitutions file.'
         goto 9920

      else if(orgmod(1).eq.'x')then

c        exit on failure to make changes after the last time we were here
         if(subtot.le.oldsct)then
            if(verbos.ge.1)print10,'No changes.  Exiting.'
            goto 9910
         endif
         oldsct=subtot
         if(verbos.ge.1)print10,'Changes were made.  Continuing.'
         goto 9900

      else if(orgmod(1).eq.'f')then

c        process a file name
         call dosubs(
     &    orgdim,orgmax,orgstr,orgmod,
     &    trndim,trnmax,trnstr,
     &    bufdm,bufmax,bufstr,
     &    bufmx2,bufst2,fnddat,subct)
         goto 9900

      else

         call fatal('Incorrect original special')

      endif

 2900 continue

c     see whether we are just selecting an occurrence
      trnsel=0
      if(trnmax.lt.0)then
         if(index(trnspn,trnstr(1)).gt.0)trnsel=1
      endif

c     do the substitutions
      if(use2.eq.0)then
         call dosubs(
     &    orgdim,orgmax,orgstr,orgmod,
     &    trndim,trnmax,trnstr,
     &    bufdm,bufmax,bufstr,
     &    bufmx2,bufst2,fnddat,subct)
      else
         call dosubs(
     &    orgdim,orgmax,orgstr,orgmod,
     &    trndim,trnmax,trnstr,
     &    bufdm,bufmx2,bufst2,
     &    bufmax,bufstr,fnddat,subct)
      endif
      if(trnsel.eq.0)use2=1-use2
      fndct=fnddat(1)
      subtot=subtot+subct
      fndtot=fndtot+fnddat(1)
      if(trnmax.eq.-123456789)then
         if(fnddat(1).le.0)fndnot=fndnot+1
      else if(trnsel.ne.0)then
         if(fnddat(1).lt.-trnmax)fndnot=fndnot+1
      else
         if(fnddat(1).le.0)fndnot=fndnot+1
         subnot=subnot+fnddat(1)-subct
      endif

c     log it
      if(verbos.ge.2)then
         print4910,fnddat(1),subct
 4910    format('found:',i9,'   changed:',i9)
      endif

c     handle special translations

      if(trnsel.eq.0)goto 5900

      if(index('bs',trnstr(1)).ne.0)then

c        next level of selection
         lev=lev+1
         if(lev.gt.levdim)call fatal('Too many selection levels')

c        set return point if the beginning of a loop
         sellin(lev)=-1
         if(trnstr(1).eq.'b')sellin(lev)=lincts

c        do the selection
         if(use2.eq.0)then

c           set new buffer size and position of the post-selection string
            bufdml(lev)=bufdml(lev-1)-bufmax+fnddat(3)-fnddat(2)+1
            bufdm=bufdml(lev)
            post(lev)=bufdm+fnddat(2)

c           initialize actual selected buffer size
            bufmx2=0

c           copy the selected string in the reduced buffer
            do 5110 i=fnddat(2),fnddat(3)
               bufmx2=bufmx2+1
               bufst2(bufmx2)=bufstr(i)
 5110       continue

c           save the pre-selection string
            do 5120 i=1,fnddat(2)-1
               bufst2(bufdm+i)=bufstr(i)
 5120       continue

c           save the post-selection string
            do 5130 i=fnddat(3)+1,bufmax
               bufst2(post(lev)-fnddat(3)-1+i)=bufstr(i)
 5130       continue

c           copy the saved area to the companion
            do 5140 i=bufdm+1,bufdml(lev-1)
               bufstr(i)=bufst2(i)
 5140       continue

         else

c           set new buffer size and position of the post-selection string
            bufdml(lev)=bufdml(lev-1)-bufmx2+fnddat(3)-fnddat(2)+1
            bufdm=bufdml(lev)
            post(lev)=bufdm+fnddat(2)

c           initialize actual selected buffer size
            bufmax=0

c           copy the selected string in the reduced buffer
            do 5150 i=fnddat(2),fnddat(3)
               bufmax=bufmax+1
               bufstr(bufmax)=bufst2(i)
 5150       continue

c           save the pre-selection string
            do 5160 i=1,fnddat(2)-1
               bufstr(bufdm+i)=bufst2(i)
 5160       continue

c           save the post-selection string
            do 5170 i=fnddat(3)+1,bufmx2
               bufstr(post(lev)-fnddat(3)-1+i)=bufst2(i)
 5170       continue

c           copy the saved area to the companion
            do 5180 i=bufdm+1,bufdml(lev-1)
               bufst2(i)=bufstr(i)
 5180       continue

         endif

         use2=1-use2

         if(verbos.ge.1)then
            if(use2.eq.0)print5190,bufmax,bufmx2
            if(use2.ne.0)print5190,bufmx2,bufmax
 5190       format('Selected',i9,' characters out of',i9)
         endif

      else if(trnstr(1).eq.'c')then

c        copy the selected area to memory
         if(fnddat(3)-fnddat(2)+1.gt.memdim)
     &    call fatal('Memory exceeded')
         memmax=0
         if(use2.eq.0)then
            do 5210 i=fnddat(2),fnddat(3)
               memmax=memmax+1
               memstr(memmax)=bufstr(i)
 5210       continue
         else
            do 5220 i=fnddat(2),fnddat(3)
               memmax=memmax+1
               memstr(memmax)=bufst2(i)
 5220       continue
         endif
         if(verbos.ge.1)then
            call inttos(memmax,cint,cintl)
            print10,'New memory size: ',cint(1:cintl)
         endif

      else if(trnstr(1).eq.'^')then

c        convert the selected area to upper case
         if(use2.eq.0)then
            do 5310 i=fnddat(2),fnddat(3)
               if(bufstr(i).ge.'a' .and. bufstr(i).le.'z')then
                  subct=1
                  bufstr(i)=char(ichar(bufstr(i))-32)
               endif
 5310       continue
         else
            do 5320 i=fnddat(2),fnddat(3)
               if(bufst2(i).ge.'a' .and. bufst2(i).le.'z')then
                  subct=1
                  bufst2(i)=char(ichar(bufst2(i))-32)
               endif
 5320       continue
         endif
         if(verbos.ge.2)print10,'Selection converted to upper case'

      else if(trnstr(1).eq.'v')then

c        convert the selected area to lower case
         if(use2.eq.0)then
            do 5350 i=fnddat(2),fnddat(3)
               if(bufstr(i).ge.'A' .and. bufstr(i).le.'Z')then
                  subct=1
                  bufstr(i)=char(ichar(bufstr(i))+32)
               endif
 5350       continue
         else
            do 5360 i=fnddat(2),fnddat(3)
               if(bufst2(i).ge.'A' .and. bufst2(i).le.'Z')then
                  subct=1
                  bufst2(i)=char(ichar(bufst2(i))+32)
               endif
 5360       continue
         endif
         if(verbos.ge.2)print10,'Selection converted to lower case'

      else if(trnstr(1).eq.'f')then

c        do nothing

      else if(index('.,?!',trnstr(1)).ne.0)then

c        ignore, handled in dosubs

      else

         call fatal('Incorrect translation special')

      endif

 5900 continue

c     next translation
 9900 call outsub('dopair')
      return
 9910 call outsub('dopair')
      return 1
 9920 call outsub('dopair')
      return 2

      end
