      subroutine dosubs(
     & orgdim,orgmax,orgstr,orgmod,
     & trndim,trnmax,trnstr,
     & bufdim,bufmax,bufstr,
     & bufmx2,bufst2,fnddat,subct)

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     Replace orgstr by trnstr in buffer bufstr.  Or just locate an
c     occurrence of orgstr.

      implicit none

c input:

c     declared original string size
      integer orgdim
c     actual original string size
      integer orgmax
c     original string
      character*1 orgstr(orgdim)
c     modifiers
      character*1 orgmod(orgdim)

c     declared translation string size
      integer trndim
c     actual translation string size, or coded command if negative
      integer trnmax
c     translation string
      character*1 trnstr(trndim)

c     declared buffer string size
      integer bufdim
c     actual buffer string size
      integer bufmax
c     buffer
      character*1 bufstr(bufdim)

c output:

c     actual output buffer string size, unless we are just selecting
      integer bufmx2
c     output buffer, unless we are just selecting
      character*1 bufst2(bufdim)
c     fnddat(1) is the counter of matches found.  If we are just
c     selecting, normally fnddat(2) is set to the start of occurrence
c     -trnmax of the search string, and fnddat(3) to the end.  However,
c     if trnmax equals -123456789, the last occurence is used.  If the
c     specified occurrence does not exist, an empty string at EOF is
c     returned.
      integer fnddat(3)
c     counter of nontrivial substitutions done
      integer subct

c common

c     verbosity
      integer verbos
c     substitions file
      character*132 subfil
      integer subfll
c     line count of the substitutions file
      integer lincts
c     text file
      character*132 txtfil
      integer txtfll
c     line count of the text file
      integer linctt

c     debug block
      common/cmdbg/verbos,subfil,subfll,lincts,txtfil,txtfll,linctt

c external

      integer utf8lt,utf16l,intllt
      external utf8lt,utf16l,intllt

c local:

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c     whether we have initialized
      integer init
      save init
      data init/0/

c     byte to unicode conversions
      integer byt2uc(0:255)
      save byt2uc
      data byt2uc(0)/-1/
c     first byte number where byt.ne.uc
      integer bytne
      save bytne

c     whether a numbers/strings mappings file has been read in
      integer havmap
      save havmap
      data havmap/0/
c     whether an hyphenation data file has been read in
      integer havhyp
      save havhyp
      data havhyp/0/

c     buffer positions in bufstr (0 is BOF, bufmax+1 is EOF)
      integer buf0,buf1,buf,bufhi
c     buffer positions in bufst2
      integer buf2,bufm20

c     position in the orginal string
      integer org

c     position in the translation string
      integer trn

c     ascii value of blackslash
      integer asc_bs
      parameter (asc_bs=92)
c     ascii values of single and double quotes
      integer asc_q,asc_dq
      parameter (asc_q=39,asc_dq=34)
c     ascii values of 0, 9, +, and -
      integer asc_0,asc_9,asc_pl,asc_mn
      parameter (asc_0=48,asc_9=57,asc_pl=43,asc_mn=45)
c     ascii values of tab, space, carriage return, and linefeed
      integer asc_tb,asc_sp,asc_cr,asc_lf
      parameter (asc_tb=9,asc_sp=32,asc_cr=13,asc_lf=10)
c     ascii values of a, h, i, n, o, and z
      integer asc_a,asc_i,asc_n,asc_o,asc_x,asc_z
      parameter
     & (asc_a=97,asc_i=105,asc_n=110,asc_o=111,asc_x=120,asc_z=122)
c     ascii values of A and Z
      integer asc_ca,asc_cz
      parameter (asc_ca=65,asc_cz=90)
c     ascii value of the internal word start and end
      integer asc_i1,asc_i2
      parameter (asc_i1=21,asc_i2=22)
c     ascii value of orgstr character
      integer iorg,iorgm,iorg0
c     ascii value of trnstr character
      integer itrn
c     ascii value
      integer iasc

c     list of characters that matches (-1 is start of file and 256 is end)
      integer good(-1:256)
c     number of characters to reset
      integer rstmax
c     reset index
      integer rst
c     characters to reset (for \-\(\ -\3\2\): 128 + 256  + 26 letters + 1 \ )
      integer rstval(411)

c     actual number of bracket pairs
      integer bpmax
c     bracket pair index, and the values for an opening or closing bracket
      integer bp,bpo,bpc
c     ascii values of the opening and closing brackets
      integer bpasco(256),bpascc(256)
c     declared number of active opening brackets
      integer aobdim
      parameter (aobdim=250)
c     actual number of active opening brackets
      integer aobmax
c     bracket pairs of the active opening brackets
      integer aobbp(aobdim)

c     translation commands that convert the original
      character*19 trncnv
      integer trncvl
      save trncnv,trncvl
      data trncnv/'"$&''()456<=>`hlu{|}'/
      integer cnvtyp
      integer cnvu,cnvl,cnvcap,cnv8bt,cnvhyp,
     & cnvamp,cnvut8,cnvu8a,cnvrtf,cnv6be,cnv6le,cnvbyt,cnvstr,
     & cnvh2d,cnvd2h,cnvo2d,cnvd2o,cnvb2d,cnvd2b
      save    cnvu,cnvl,cnvcap,cnv8bt,cnvhyp,
     & cnvamp,cnvut8,cnvu8a,cnvrtf,cnv6be,cnv6le,cnvbyt,cnvstr,
     & cnvh2d,cnvd2h,cnvo2d,cnvd2o,cnvb2d,cnvd2b
c     forgiveness of errors
      integer forgiv
c     special translation commands without instance selection
      character*2 trnspc
      save trnspc
      data trnspc/'er'/
      integer isspc
      integer spce,spcr
      save spce,spcr
c     character to look for
      integer ilook
c     special translation commands that select a particular instance
      character*10 trnspn
      save trnspn
      data trnspn/'bcfsv^.,!?'/
c     mode (0: normal, 1: trnspc translation, 2: trnspn, select only)
      integer trnmod

c     case sensitivity
      integer docase
c     obey case of original
      integer obeydm
      parameter (obeydm=10000)
c     case of the original
      character*1 obeyc(obeydm)
c     whether to obey case / count of characters obeyed
      integer obey
c     count of characters to obey
      integer obeymx
c     NOT is active
      integer havnot
c     ANY is active
      integer havany
c     whether we are in a group
      integer group
c     whether we have a match
      integer match

c     fast forward original:
c     0: no
c     1: all characters
c     2: matches only at BOF
c     3: matches only at EOF
      integer fforg

c     reusable starting part of the original
      integer reuse

c     whether we have a change
      integer change
c     precomputed value
      integer chang0

c     substrings
      integer sbsmax,sbs
      integer sbsb(0:9),sbse(0:9)

c     numeric label
      integer label,label0
      save label
      data label/0/

c     current line in subbinss.sub
      integer ssslin
      save ssslin
      data ssslin/0/
c     new line in subbinss.sub
      integer sssnew
c     whether we get the substitution from subbinss.sub
      integer frmsss

c     how many times the conversion command has been repeated
      integer repeat
c     whether we are in hexmode
      integer hexmod

c     whether a backslash is pending
      logical bsl
c     whether an 8 bit conversion is pending
      logical bit8

c     declared number of characters in the file names
      integer fildim
      parameter (fildim=132)
c     actual number of characters in the file names
      integer filmax
c     file name
      character*(fildim) filnam
c     file name of the substitutions file
      character*(fildim) sub
      integer subl
      save sub,subl

c     integer as string
      character*12 cint,cint2
      integer cintl,cint2l

c     encoded character(s)
      character*1 enc(128)
      integer enclt

c     user answer
      character*16 ans

c     character
      character*1 chr
      integer ichr

c     a number and its base
      integer num,b,b2

c     nonzero for little endian mode
      integer le

c     generic index
      integer i

c executable:

c     note where we are
      call insub('dosubs')

c     generic format
 10   format(a,78a1)
 20   format(2048a1)

c     initialize if needed
      if(init.eq.0)then
         init=1

c        substitutions file name
         subl=12
         sub(1:subl)='subbinss.sub'

c        mnemonics for the specials
         i=len(trnspc)
         spce=index(trnspc,'e')
         spcr=index(trnspc,'r')
         if(spce+spcr.ne.i*(i+1)/2)
     &    call fatal('Program error: trnspc not implemented correctly')

c        mnemonics for the conversions
         trncvl=len(trncnv)
         cnvu=index(trncnv,'u')
         cnvl=index(trncnv,'l')
         cnvcap=index(trncnv,'$')
         cnv8bt=index(trncnv,'4')
         cnvhyp=index(trncnv,'h')
         cnvamp=index(trncnv,'&')
         cnvut8=index(trncnv,'''')
         cnvu8a=index(trncnv,'"')
         cnvrtf=index(trncnv,'`')
         cnv6be=index(trncnv,'5')
         cnv6le=index(trncnv,'6')
         cnvbyt=index(trncnv,'|')
         cnvstr=index(trncnv,'=')
         cnvh2d=index(trncnv,'}')
         cnvd2h=index(trncnv,'{')
         cnvo2d=index(trncnv,')')
         cnvd2o=index(trncnv,'(')
         cnvb2d=index(trncnv,'>')
         cnvd2b=index(trncnv,'<')
         if(cnvu+cnvl+cnvcap+cnv8bt+cnvhyp+
     &    cnvamp+cnvut8+cnvu8a+cnvrtf+cnv6be+cnv6le+cnvbyt+cnvstr+
     &    cnvh2d+cnvd2h+cnvo2d+cnvd2o+cnvb2d+cnvd2b
     &    .ne. trncvl*(trncvl+1)/2)call fatal(
     &    'Program error: trncnv not implemented correctly')

      endif

c     handle negative orgmax
      if(orgmax.le.0)then

         if(orgmod(1).eq.'f')then

c           file input command
            if(trnmax.le.0)
     &       call fatal('Missing or illegal filename for \\f command')
            if(trnmax.gt.fildim)
     &       call fatal('Filename for \\f command is too long')
            filmax=trnmax
            do 110 trn=1,filmax
               if(trnstr(trn).eq.char(127))call fatal(
     &          'File names may not contain commands or DEL characters')
               filnam(trn:trn)=trnstr(trn)
 110        continue
            if(orgmax.eq.-1)then
               call rdb2uc(verbos,filnam(1:filmax),byt2uc,bytne)
            else if(orgmax.eq.-2)then
               call numstr(verbos,filnam(1:filmax),enclt,enc,i)
               havmap=1
            else if(orgmax.eq.-3)then
               if(ssslin.ne.0 .or. label.ne.0)call closu(4,sub(1:subl))
               subl=filmax
               sub(1:subl)=filnam(1:filmax)
               label=0
               ssslin=0
            else if(orgmax.eq.-4)then
               call hyphen(filnam(1:filmax),-1,chr,-1,-1,chr)
            else
               call fatal('Illegal number in \\f command')
            endif
            goto 9100

         else

            call fatal(
     &       'Program error: unimplemented original special in dosubs')

         endif

      endif

c     handle negative trnmax
      trnmod=0
      if(trnmax.lt.0)then
         if(index(trnspn,trnstr(1)).gt.0)then
            trnmod=2
         else
            trnmod=1
            isspc=index(trnspc,trnstr(1))
            if(isspc.le.0)
     &       call fatal('Program error: unknown translation special')
            ilook=0
            if(isspc.eq.spce .or. isspc.eq.spcr)then
               ilook=ichar(trnstr(2))
               chang0=1
               if(trnmax.eq.-3)then
                  if(trnstr(2).eq.trnstr(3))chang0=0
               endif
            else
               call fatal(
     &          'Program error: unimplemented translation special')
            endif
         endif
      endif

c     look for fast-forward originals
      fforg=0
      if(index('[]a',orgmod(1)).gt.0)then
         if(orgmax.eq.3 .and. orgmod(1)//orgmod(2)//orgmod(3).eq.'a?a')
     &    fforg=1
         if(orgmod(1).eq.'[')fforg=2
         if(orgmod(1).eq.']')fforg=3
      endif

c     find the reuse limit
      reuse=0
      org=0
      group=0
 200  org=org+1
      if(org.gt.orgmax)goto 290
      iorg=ichar(orgstr(org))
      if(iorg.eq.0)then
         iorgm=ichar(orgmod(org))
         if(iorgm.eq.0)goto 250
         if(index('''_+56alu[]',orgmod(org)).ne.0)goto 250
         if(index('=-~?!{',orgmod(org)).ne.0)goto 200
         if(iorgm.eq.ichar('&'))then
            if(group.ne.0)call fatal('Program error: bad group')
            group=1
            goto 200
         endif
         if(iorgm.eq.ichar('*'))then
            if(group.ne.1)call fatal('Program error: bad group')
            reuse=org
            goto 290
         endif
         goto 290
      endif
 250  if(group.eq.1)goto 200
      reuse=org
 290  continue

c     initialize the find and substitution counts
      fnddat(1)=0
      subct=0

c     initialize output buffer length
      bufmx2=0

c     array good must be fully reset
      rstmax=128

c     initialize the starting position of the matched string
      buf0=-1

c     initialize line count
      linctt=1

c     initialize the original to compare with
      org=0

c     Compare the text strings with the desired original

c     next starting position of a potential match
 1000 buf0=buf0+1
      if(buf0.gt.bufmax+1)goto 9000

c     initialize the end point of the matched string
      buf1=buf0-1

c     initialize the full string
      sbsb(0)=max(1,buf0)

c     increment line count and look for fast forward
      if(buf0.ge.1 .and. buf0.le.bufmax)then
         if(fforg.eq.0)then
         else if(fforg.eq.1)then
            sbsmax=0
            org=1
            docase=1
            obeymx=0
            obey=0
            buf1=bufmax
            linctt=0
            goto 8000
         else if(fforg.eq.2)then
            sbsmax=0
            org=1
            docase=1
            obeymx=0
            obey=0
            if(trnmod.ne.2)then
               if(bufmx2+bufmax-buf0+1.gt.bufdim)goto 9900
               do 1200 buf=buf0,bufmax
                  bufmx2=bufmx2+1
                  bufst2(bufmx2)=bufstr(buf)
 1200          continue
            endif
            goto 9000
         else if(fforg.eq.3)then
            sbsmax=0
            org=3
            docase=1
            obeymx=0
            obey=0
            if(trnmod.ne.2)then
               if(bufmx2+bufmax-buf0+1.gt.bufdim)goto 9900
               do 1300 buf=buf0,bufmax
                  bufmx2=bufmx2+1
                  bufst2(bufmx2)=bufstr(buf)
 1300          continue
            endif
            buf0=bufmax
            linctt=0
            goto 1000
         endif
         if(bufstr(buf0).eq.char(asc_lf))linctt=linctt+1
      endif

c     see whether we can reuse the current data
      if(reuse.gt.0 .and. org.eq.reuse)then
         if(sbsmax.gt.0)sbsb(1)=max(1,buf1+1)
         goto 4000
      endif

c     initialize the substring count
      sbsmax=0

c     initialize the original to compare with
      org=0
      docase=1
      obeymx=0
      obey=0

c     next (havnot,havany,good) to compare with
 2000 havnot=0
      havany=0
      group=0
      iorg0=-1
      if(rstmax.lt.128)then
         do 2002 rst=1,rstmax
            good(rstval(rst))=0
 2002    continue
      else
         do 2004 rst=-1,256
            good(rst)=0
 2004    continue
      endif
      rstmax=0

c     continue reading (havnot,havany,good)
 2010 org=org+1
      if(org.gt.orgmax)goto 8000
      rstmax=min(rstmax,128)

c     process the character
      iorg=ichar(orgstr(org))
      if(iorg.eq.0)then
         iorgm=ichar(orgmod(org))
c        plain ascii nul
         if(iorgm.eq.0)goto 3800
c        ranges must close
         if(iorg0.ne.-1)call fatal('Unclosed range in group')
c        case sensitivity
         if(iorgm.eq.ichar('='))then
            docase=1
            goto 2010
         else if(iorgm.eq.ichar('-'))then
            docase=0
            goto 2010
         else if(iorgm.eq.ichar('`'))then
            if(trnmax.lt.0)
     &       call fatal('Cannot obey case in a special translation')
            obey=1-obey
            if(obey.eq.1)docase=0
            goto 2010
c        matches
         else if(iorgm.eq.ichar('.'))then
            if(org.ge.orgmax)
     &       call fatal('Program error: unterminated \\. sequence')
            if(buf1.lt.buf0 .or. buf1.lt.1 .or. buf1.ge.bufmax)goto 4900
            iasc=ichar(bufstr(buf1))
            bpmax=1
            bpasco(1)=iasc
            bpascc(1)=iasc
            aobmax=1
            aobbp(1)=1
            havany=-9
 2210       org=org+1
            iorgm=ichar(orgmod(org))
            if(iorgm.eq.ichar('.'))goto 4000
            if(iorgm.ne.0 .or. org+2.gt.orgmax)
     &       call fatal('Program error: unterminated \\. sequence')
            iorg=ichar(orgstr(org))
            org=org+1
            iasc=ichar(orgstr(org))
            if(iasc.eq.0)then
               if(orgmod(org).ne.char(0))
     &          call fatal('Program error: corrupt \\. sequence')
            endif
            bpo=0
            bpc=0
            do 2250 bp=1,bpmax
               if(iorg.eq.bpasco(bp))then
                  bpo=bp
               else if(iasc.eq.bpascc(bp))then
                  bpc=bp
               endif
 2250       continue
            if(bpc.ne.0)call fatal(
     &       'A closing bracket is assigned to two opening brackets')
            if(bpo.eq.0)then
               bpmax=bpmax+1
               bpasco(bpmax)=iorg
               bpascc(bpmax)=iasc
            else
               bpascc(bpo)=iasc
            endif
            goto 2210
         else if(iorgm.eq.ichar(':'))then
            if(buf1.lt.buf0 .or. buf1.lt.1 .or. buf1.ge.bufmax)goto 4900
            iasc=ichar(bufstr(buf1))
            bpmax=1
            bpasco(1)=iasc
            i=iasc
            if(iasc.eq.ichar('('))i=ichar(')')
            if(iasc.eq.ichar('{'))i=ichar('}')
            if(iasc.eq.ichar('['))i=ichar(']')
            if(iasc.eq.ichar('<'))i=ichar('>')
            if(iasc.eq.ichar('`'))i=ichar('''')
            if(iasc.eq.ichar('&'))i=ichar(';')
            if(iasc.le.25 .and. mod(iasc,2).ne.0)i=iasc+1
            bpascc(1)=i
            aobmax=1
            aobbp(1)=1
            havany=-9
            goto 4000
c        not
         else if(iorgm.eq.ichar('~'))then
            havnot=1
            goto 2010
c        any
         else if(iorgm.eq.ichar('?'))then
            havany=1
            goto 2010
c        any
         else if(iorgm.eq.ichar('!'))then
            havany=-1
            goto 2010
c        start of group
         else if(iorgm.eq.ichar('&'))then
            group=1
            goto 2010
c        end of group
         else if(iorgm.eq.ichar('*'))then
            group=0
            goto 4000
c        start of substring
         else if(iorgm.eq.ichar('{'))then
            sbsmax=sbsmax+1
            sbsb(sbsmax)=max(1,buf1+1)
            goto 2010
c        end of substring
         else if(iorgm.eq.ichar('}'))then
            sbse(sbsmax)=min(bufmax,buf1)
            goto 2010
         endif
c        commands describing multiple characters, BOF, and EOF
         if(iorgm.eq.ichar(''''))then
            good(asc_q)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_q
            good(asc_dq)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_dq
         else if(iorgm.eq.ichar('_'))then
            good(asc_sp)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_sp
            good(asc_tb)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_tb
         else if(iorgm.eq.ichar('+'))then
            good(-1)=1
            rstmax=rstmax+1
            rstval(rstmax)=-1
            good(asc_sp)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_sp
            good(asc_tb)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_tb
            good(asc_cr)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_cr
            good(asc_lf)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_lf
            good(256)=1
            rstmax=rstmax+1
            rstval(rstmax)=256
         else if(iorgm.eq.ichar('5'))then
            do 3100 iasc=asc_0,asc_9
               good(iasc)=1
               rstmax=rstmax+1
               rstval(rstmax)=iasc
 3100       continue
         else if(iorgm.eq.ichar('6'))then
            good(asc_pl)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_pl
            good(asc_mn)=1
            rstmax=rstmax+1
            rstval(rstmax)=asc_mn
         else if(iorgm.eq.ichar('a'))then
            do 3200 iasc=0,255
               good(iasc)=1
 3200       continue
            rstmax=128
         else if(iorgm.eq.ichar('l'))then
            do 3300 iasc=asc_a,asc_z
               good(iasc)=1
               rstmax=rstmax+1
               rstval(rstmax)=iasc
 3300       continue
            if(docase.eq.0)then
               do 3350 iasc=asc_ca,asc_cz
                  good(iasc)=1
                  rstmax=rstmax+1
                  rstval(rstmax)=iasc
 3350          continue
            endif
         else if(iorgm.eq.ichar('u'))then
            do 3400 iasc=asc_ca,asc_cz
               good(iasc)=1
               rstmax=rstmax+1
               rstval(rstmax)=iasc
 3400       continue
            if(docase.eq.0)then
               do 3450 iasc=asc_a,asc_z
                  good(iasc)=1
                  rstmax=rstmax+1
                  rstval(rstmax)=iasc
 3450          continue
            endif
         else if(iorgm.eq.ichar('['))then
            good(-1)=1
            rstmax=rstmax+1
            rstval(rstmax)=-1
         else if(iorgm.eq.ichar(']'))then
            good(256)=1
            rstmax=rstmax+1
            rstval(rstmax)=256
         else
            print*,iorg,iorgm
            call fatal('Program error: no such original command')
         endif
         goto 3900
      endif

c     single character
 3800 if(group.ne.0)then
         if(orgstr(org+1).eq.'-')then
            if(orgmod(org+1).eq.char(0))then
               if(iorg0.ne.-1)call fatal('Invalid range in group')
               org=org+1
               iorg0=iorg
               goto 2010
            endif
         endif
      endif
      if(iorg0.eq.-1)then
         good(iorg)=1
         rstmax=rstmax+1
         rstval(rstmax)=iorg
         if(docase.eq.0)then
            if(iorg.ge.asc_a .and. iorg.le.asc_z)then
               good(iorg-32)=1
               rstmax=rstmax+1
               rstval(rstmax)=iorg-32
            else if(iorg.ge.asc_ca .and. iorg.le.asc_cz)then
               good(iorg+32)=1
               rstmax=rstmax+1
               rstval(rstmax)=iorg+32
            endif
         endif
      else
         do 3850 iorg=iorg0,iorg
            good(iorg)=1
            rstmax=rstmax+1
            rstval(rstmax)=iorg
            if(docase.eq.0)then
               if(iorg.ge.asc_a .and. iorg.le.asc_z)then
                  good(iorg-32)=1
                  rstmax=rstmax+1
                  rstval(rstmax)=iorg-32
               else if(iorg.ge.asc_ca .and. iorg.le.asc_cz)then
                  good(iorg+32)=1
                  rstmax=rstmax+1
                  rstval(rstmax)=iorg+32
               endif
            endif
 3850    continue
         iorg0=-1
      endif

c     if in a group, go read the next character
 3900 if(group.eq.1)goto 2010

c     do the comparison

 4000 continue

      if(havany.ne.0)then

c        find matching bracket
         if(havany.eq.-9)then
 4010       if(buf1.ge.bufmax)goto 4900
            iasc=ichar(bufstr(buf1+1))
            bpo=0
            bpc=0
            do 4020 bp=1,bpmax
               if(iasc.eq.bpasco(bp))bpo=bp
               if(iasc.eq.bpascc(bp))bpc=bp
 4020       continue
            if(bpo.ne.0 .or. bpc.ne.0)then
               if(bpc.eq.aobbp(aobmax))then
                  aobmax=aobmax-1
                  if(aobmax.eq.0)goto 2000
               else if(bpo.ne.0)then
                  aobmax=aobmax+1
                  if(aobmax.gt.aobdim)
     &             call fatal('bracket nesting is too deep')
                  aobbp(aobmax)=bpo
               else
                  goto 4900
               endif
            endif
            buf1=buf1+1
            if(obey.ne.0)then
               if(iasc.ge.asc_a .and. iasc.le.asc_z)then
                  obeymx=obeymx+1
                  if(obeymx.gt.obeydm)
     &             call fatal('Too many letters to obey case for')
                  obeyc(obeymx)='a'
               else if (iasc.ge.asc_ca .and. iasc.le.asc_cz)then
                  obeymx=obeymx+1
                  if(obeymx.gt.obeydm)
     &             call fatal('Too many letters to obey case for')
                  obeyc(obeymx)='A'
               endif
            endif
            goto 4010
         endif

c        compare the pending characters with the allowed ones
         if(buf1.lt.0)then
            match=good(-1)
            if(havnot.eq.1)match=1-match
            if(match.eq.1)then
               buf1=0
               if(havany.eq.-1)goto 4190
            endif
         endif
 4100    if(buf1.ge.0 .and. buf1.lt.bufmax)then
            iasc=ichar(bufstr(buf1+1))
            match=good(iasc)
            if(havnot.eq.1)match=1-match
            if(match.eq.1)then
               buf1=buf1+1
               if(obey.ne.0)then
                  if(iasc.ge.asc_a .and. iasc.le.asc_z)then
                     obeymx=obeymx+1
                     if(obeymx.gt.obeydm)
     &                call fatal('Too many letters to obey case for')
                     obeyc(obeymx)='a'
                  else if (iasc.ge.asc_ca .and. iasc.le.asc_cz)then
                     obeymx=obeymx+1
                     if(obeymx.gt.obeydm)
     &                call fatal('Too many letters to obey case for')
                     obeyc(obeymx)='A'
                  endif
               endif
               if(havany.eq.-1)goto 4190
               goto 4100
            endif
         endif
         if(buf1.eq.bufmax)then
            match=good(256)
            if(havnot.eq.1)match=1-match
            if(match.eq.1)buf1=bufmax+1
         endif
 4190    match=1

      else

c        compare the pending character with the allowed ones
         if(buf1.lt.0)then
            buf1=0
            match=good(-1)
         else if(buf1.lt.bufmax)then
            buf1=buf1+1
            iasc=ichar(bufstr(buf1))
            match=good(iasc)
            if(obey.ne.0)then
               if(iasc.ge.asc_a .and. iasc.le.asc_z)then
                  obeymx=obeymx+1
                  if(obeymx.gt.obeydm)
     &             call fatal('Too many letters to obey case for')
                  obeyc(obeymx)='a'
               else if (iasc.ge.asc_ca .and. iasc.le.asc_cz)then
                  obeymx=obeymx+1
                  if(obeymx.gt.obeydm)
     &             call fatal('Too many letters to obey case for')
                  obeyc(obeymx)='A'
               endif
            endif
         else if(buf1.lt.bufmax+1)then
            buf1=buf1+1
            match=good(256)
         else
            match=havnot
         endif
         if(havnot.eq.1)match=1-match

      endif

c     if a match so far, try more
      if(match.eq.1)goto 2000

c     no match, so copy the unchanged character to the output buffer
 4900 if(buf0.ge.1 .and. buf0.le.bufmax .and. trnmod.ne.2)then
         bufmx2=bufmx2+1
         if(bufmx2.gt.bufdim)goto 9900
         bufst2(bufmx2)=bufstr(buf0)
      endif
      goto 1000

c     substitute the translation

c     declare void if buf1 is before BOF
 8000 if(buf1.lt.0)goto 1000

c     increment the find counter
      fnddat(1)=fnddat(1)+1

c     set the full string end point
      sbse(0)=min(bufmax,buf1)

c     copy the translation into the output buffer
      if(trnmod.eq.0)then

c        put in the replacement string
         bufm20=bufmx2
         label0=label
         obey=0
         cnvtyp=0
         forgiv=0
         hexmod=0
         frmsss=0
         trn=0
 8100    trn=trn+1
         if(trn.gt.trnmax)goto 8600
c        start of a del sequence
         if(trnstr(trn).eq.char(127))then
            trn=trn+1
            itrn=ichar(trnstr(trn))
            if(itrn.eq.127)then
c              regular DEL character
               goto 8500
            else if(itrn.ge.asc_0 .and. itrn.le.asc_9)then
c              insert substring, with any selected conversion
               sbs=itrn-asc_0
               if(frmsss.ne.0)goto 8300
               if(cnvtyp.eq.0) then
c                 no conversion
                  buf=sbsb(sbs)
                  goto 8290
               else if(cnvtyp.eq.cnvl)then
c                 convert to lower case
                  do 8213 buf=sbsb(sbs),sbse(sbs)
                     bufmx2=bufmx2+1
                     if(bufmx2.gt.bufdim)goto 9900
                     iasc=ichar(bufstr(buf))
                     if(iasc.ge.asc_ca .and. iasc.le.asc_cz)iasc=iasc+32
                     bufst2(bufmx2)=char(iasc)
 8213             continue
               else if(cnvtyp.eq.cnvu)then
c                 convert to upper case
                  do 8215 buf=sbsb(sbs),sbse(sbs)
                     bufmx2=bufmx2+1
                     if(bufmx2.gt.bufdim)goto 9900
                     iasc=ichar(bufstr(buf))
                     if(iasc.ge.asc_a .and. iasc.le.asc_z)iasc=iasc-32
                     bufst2(bufmx2)=char(iasc)
 8215             continue
               else if(cnvtyp.eq.cnvcap)then
c                 convert to capitalized first letter
                  i=32
                  do 8217 buf=sbsb(sbs),sbse(sbs)
                     bufmx2=bufmx2+1
                     if(bufmx2.gt.bufdim)goto 9900
                     iasc=ichar(bufstr(buf))
                     if(iasc.ge.asc_a .and. iasc.le.asc_z)iasc=iasc-i
                     if(iasc.ge.asc_ca .and. iasc.le.asc_cz)i=0
                     bufst2(bufmx2)=char(iasc)
 8217             continue
               else if(cnvtyp.eq.cnv8bt)then
c                 convert 7 to 8 bit or vice-versa
                  do 8219 buf=sbsb(sbs),sbse(sbs)
                     bufmx2=bufmx2+1
                     if(bufmx2.gt.bufdim)goto 9900
                     iasc=ichar(bufstr(buf))
                     if(iasc.le.127)then
                        iasc=iasc+128
                     else
                        iasc=iasc-128
                     endif
                     bufst2(bufmx2)=char(iasc)
 8219             continue
               else if(cnvtyp.eq.cnvhyp)then
c                 convert to hyphenated
                  if(havhyp.eq.0)
     &             call hyphen('hyphenate.txt',-1,chr,-1,-1,chr)
                  havhyp=1
                  buf=sbsb(sbs)
                  bufhi=sbse(sbs)
                  call hyphen('&shy;',bufhi-buf+1,bufstr(buf),
     &             bufdim-bufmx2,enclt,bufst2(bufmx2+1))
                  bufmx2=bufmx2+enclt
               else if(
     &             cnvtyp.eq.cnvh2d .or. cnvtyp.eq.cnvd2h .or.
     &             cnvtyp.eq.cnvo2d .or. cnvtyp.eq.cnvd2o .or.
     &             cnvtyp.eq.cnvb2d .or. cnvtyp.eq.cnvd2b)then
c                 convert number base
                  buf=sbsb(sbs)
                  bufhi=sbse(sbs)
                  b=10
                  b2=10
                  if(cnvtyp.eq.cnvh2d)b=16
                  if(cnvtyp.eq.cnvo2d)b=8
                  if(cnvtyp.eq.cnvb2d)b=2
                  if(cnvtyp.eq.cnvd2h)b2=16
                  if(cnvtyp.eq.cnvd2o)b2=8
                  if(cnvtyp.eq.cnvd2b)b2=2
                  call rbaseb(b,48,bufhi-buf+1,bufstr(buf),num)
                  if(num.lt.0)then
                     if(forgiv.ge.2)goto 8290
                     call inttos(b,cint,cintl)
                     if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &                'Invalid base '//cint(1:cintl)//' number')
                     if(forgiv.lt.2)
     &                print10,'** Invalid base '//cint(1:cintl)//
     &                ' number found; replacement abandoned'
                     goto 8290
                  endif
                  call wbaseb(b2,48,num,128,enclt,enc)
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  do 8225 i=1,enclt
                     bufmx2=bufmx2+1
                     bufst2(bufmx2)=enc(i)
 8225             continue
               else if(cnvtyp.eq.cnv6be .or. cnvtyp.eq.cnv6le)then
c                 convert from or to UTF-16
                  le=1
                  if(cnvtyp.eq.cnv6be)le=0
                  buf=sbsb(sbs)
                  bufhi=sbse(sbs)
                  if(bufhi-buf+1.lt.3)goto 8235
                  if(ichar(bufstr(buf)).ne.asc_i1)goto 8235
                  if(ichar(bufstr(bufhi)).ne.asc_i2)goto 8235
                  do 8230 i=buf+1,bufhi-1
                     iasc=ichar(bufstr(i))
                     if(iasc.lt.asc_a .or. iasc.gt.asc_z)goto 8235
 8230             continue
                  call rbaseb(26,asc_a,bufhi-buf-1,bufstr(buf+1),num)
                  enclt=utf16l(num)
                  if(enclt.le.0)then
                     if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &                'Invalid character in internal format found')
                     if(forgiv.lt.2)
     &                print10,'** Invalid character in internal '//
     &                'format found; replacement abandoned'
                     goto 8290
                  endif
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  call wutf16(le,num,
     &             bufdim-bufmx2,enclt,bufst2(bufmx2+1))
                  bufmx2=bufmx2+enclt
                  goto 8239
 8235             if(buf.gt.bufhi)goto 8239
                  enclt=0
                  call rutf16(le,bufhi-buf+1,bufstr(buf),enclt,num)
                  if(num.lt.0 .or. num.gt.uclim)then
                     if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &                'Invalid UTF-16 character found')
                     if(forgiv.lt.2)
     &                print10,'** Invalid UTF-16 character found; '//
     &                'further replacements abandoned'
                     goto 8290
                  endif
                  buf=buf+enclt
                  enclt=intllt(num)
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  bufst2(bufmx2+1)=char(asc_i1)
                  call wbaseb(26,asc_a,num,
     &             bufdim-bufmx2-2,enclt,bufst2(bufmx2+2))
                  bufmx2=bufmx2+enclt+2
                  bufst2(bufmx2)=char(asc_i2)
                  goto 8235
 8239             continue
               else if(cnvtyp.eq.cnvut8 .or. cnvtyp.eq.cnvu8a)then
c                 convert from or to UTF-8
                  buf=sbsb(sbs)
                  bufhi=sbse(sbs)
                  if(bufhi-buf+1.lt.3)goto 8245
                  if(ichar(bufstr(buf)).ne.asc_i1)goto 8245
                  if(ichar(bufstr(bufhi)).ne.asc_i2)goto 8245
                  do 8240 i=buf+1,bufhi-1
                     iasc=ichar(bufstr(i))
                     if(iasc.lt.asc_a .or. iasc.gt.asc_z)goto 8245
 8240             continue
                  call rbaseb(26,asc_a,bufhi-buf-1,bufstr(buf+1),num)
                  enclt=utf8lt(num)
                  if(enclt.le.0)then
                     if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &                'Invalid character in internal format found')
                     if(forgiv.lt.2)
     &                print10,'** Invalid character in internal '//
     &                'format found; replacement abandoned'
                     goto 8290
                  endif
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  call wutf8(num,bufdim-bufmx2,enclt,bufst2(bufmx2+1))
                  bufmx2=bufmx2+enclt
                  goto 8249
 8245             if(buf.gt.bufhi)goto 8249
                  iasc=ichar(bufstr(buf))
                  if(iasc.gt.127)goto 8246
                  if(cnvtyp.eq.cnvu8a)goto 8246
                  bufmx2=bufmx2+1
                  if(bufmx2.gt.bufdim)goto 9900
                  bufst2(bufmx2)=bufstr(buf)
                  buf=buf+1
                  goto 8245
 8246             enclt=0
                  call rutf8(bufhi-buf+1,bufstr(buf),enclt,num)
                  if(num.lt.0 .or. num.gt.uclim)then
                     if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &                'Invalid UTF-8 character found')
                     if(forgiv.lt.2)
     &                print10,'** Invalid UTF-8 character found; '//
     &                'further replacements abandoned'
                     goto 8290
                  endif
                  buf=buf+enclt
                  enclt=intllt(num)
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  bufst2(bufmx2+1)=char(asc_i1)
                  call wbaseb(26,asc_a,num,
     &             bufdim-bufmx2-2,enclt,bufst2(bufmx2+2))
                  bufmx2=bufmx2+enclt+2
                  bufst2(bufmx2)=char(asc_i2)
                  goto 8245
 8249             continue
               else if(cnvtyp.eq.cnvbyt)then
c                 convert from or to 8 bit encoded
                  if(byt2uc(0).lt.0)
     &             call rdb2uc(verbos,'8bit_to_uc.sub',byt2uc,bytne)
                  buf=sbsb(sbs)
                  bufhi=sbse(sbs)
                  if(bufhi-buf+1.lt.3)goto 8255
                  if(ichar(bufstr(buf)).ne.asc_i1)goto 8255
                  if(ichar(bufstr(bufhi)).ne.asc_i2)goto 8255
                  do 8252 i=buf+1,bufhi-1
                     iasc=ichar(bufstr(i))
                     if(iasc.lt.asc_a .or. iasc.gt.asc_z)goto 8255
 8252             continue
                  call rbaseb(26,asc_a,bufhi-buf-1,bufstr(buf+1),num)
                  if(num.lt.0 .or. num.gt.uclim)then
                     if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &                'Invalid character in internal format found')
                     if(forgiv.lt.2)
     &                print10,'** Invalid character in internal '//
     &                'format found; replacement abandoned'
                     goto 8290
                  endif
                  if(num.le.255)then
                     i=num
                     if(byt2uc(i).eq.num)goto 8254
                  endif
                  do 8253 i=255,bytne,-1
                     if(byt2uc(i).eq.num)goto 8254
 8253             continue
                  if(forgiv.ge.2)goto 8290
                  call inttos(num,cint,cintl)
                  if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &             'Unicode character number '//cint(1:cintl)//
     &             ' has no 8 bit equivalent')
                  if(forgiv.lt.2)
     &             print10,'Unicode character number '//cint(1:cintl)//
     &             ' has no 8 bit equivalent'
                  goto 8290
 8254             bufmx2=bufmx2+1
                  if(bufmx2.gt.bufdim)goto 9900
                  bufst2(bufmx2)=char(i)
                  goto 8259
 8255             do 8257 buf=buf,bufhi
                     iasc=ichar(bufstr(buf))
                     num=byt2uc(iasc)
                     enclt=intllt(num)
                     if(bufmx2+enclt.gt.bufdim)goto 9900
                     bufst2(bufmx2+1)=char(asc_i1)
                     call wbaseb(26,asc_a,num,
     &                bufdim-bufmx2-2,enclt,bufst2(bufmx2+2))
                     bufmx2=bufmx2+enclt+2
                     bufst2(bufmx2)=char(asc_i2)
 8257             continue
 8259             continue
               else if(cnvtyp.eq.cnvstr)then
c                 convert between strings and numbers
                  if(havmap.eq.0)
     &             call numstr(verbos,'num_str-map.sub',enclt,enc,i)
                  havmap=1
                  buf=sbsb(sbs)
                  bufhi=sbse(sbs)
                  if(bufhi-buf+1.lt.3)goto 8265
                  if(ichar(bufstr(buf)).ne.asc_i1)goto 8265
                  if(ichar(bufstr(bufhi)).ne.asc_i2)goto 8265
                  do 8262 i=buf+1,bufhi-1
                     iasc=ichar(bufstr(i))
                     if(iasc.lt.asc_a .or. iasc.gt.asc_z)goto 8265
 8262             continue
                  call rbaseb(26,asc_a,bufhi-buf-1,bufstr(buf+1),num)
                  if(num.lt.0 .or. num.gt.uclim)then
                     if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &                'Invalid character in internal format found')
                     if(forgiv.lt.2)
     &                print10,'** Invalid character in internal '//
     &                'format found; replacement abandoned'
                     goto 8290
                  endif
                  call numstr(verbos,':',enclt,enc,num)
                  if(enclt.le.0)then
                     if(forgiv.ge.2)goto 8290
                     call inttos(num,cint,cintl)
                     if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &                'Number '//cint(1:cintl)//
     &                ' has no string equivalent')
                     print10,'Number '//cint(1:cintl)//
     &                ' has no string equivalent'
                     goto 8290
                  endif
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  do 8264 i=1,enclt
                     bufmx2=bufmx2+1
                     bufst2(bufmx2)=enc(i)
 8264             continue
                  goto 8269
 8265             if(repeat.gt.3)call fatal('The \\= command may not '//
     &             'be repeated more than 3 times')
                  enclt=0
                  i=mod(repeat,2)
                  do 8266 buf2=buf,bufhi
                     iasc=ichar(bufstr(buf2))
                     if(i.ge.1 .and. iasc.le.32)goto 8266
                     if(repeat.ge.2 .and.
     &                iasc.ge.asc_ca .and. iasc.le.asc_cz)iasc=iasc+32
                     enclt=enclt+1
                     if(enclt.gt.127)goto 8268
                     enc(enclt)=char(iasc)
 8266             continue
                  call numstr(verbos,'#',enclt,enc,num)
                  if(num.lt.0)goto 8268
                  enclt=intllt(num)
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  bufst2(bufmx2+1)=char(asc_i1)
                  call wbaseb(26,asc_a,num,
     &             bufdim-bufmx2-2,enclt,bufst2(bufmx2+2))
                  bufmx2=bufmx2+enclt+2
                  bufst2(bufmx2)=char(asc_i2)
                  goto 8269
 8268             if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &             'The selected string is not in the number/string '//
     &             'mapping file')
                  if(forgiv.lt.2)print10,
     &             'The selected string is not in the number/string '//
     &             'mapping file; replacement abandoned'
                  goto 8290
 8269             continue
               else if(cnvtyp.eq.cnvamp)then
c                 convert from or to html
                  if(repeat.gt.2)
     &             call fatal('Too many \\& commands in a row')
                  buf=sbsb(sbs)
                  bufhi=sbse(sbs)
                  if(bufhi-buf+1.lt.1)goto 8276
                  iasc=ichar(bufstr(buf))
                  if(iasc.eq.ichar('&'))goto 8275
                  if(iasc.ne.asc_i1)goto 8276
                  if(bufhi-buf+1.lt.3)goto 8278
                  if(ichar(bufstr(bufhi)).ne.asc_i2)goto 8278
                  call rbaseb(26,asc_a,bufhi-buf-1,bufstr(buf+1),num)
                  if(num.lt.0 .or. num.gt.uclim)goto 8278
                  call rwhtml(repeat,hexmod,num,128,enclt,enc)
                  if(enclt.le.0)goto 8278
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  do 8272 i=1,enclt
                     bufmx2=bufmx2+1
                     bufst2(bufmx2)=enc(i)
 8272             continue
                  goto 8279
 8275             num=-1
                  call rwhtml(repeat,hexmod,num,
     &             bufdim-buf+1,bufhi-buf+1,bufstr(buf))
                  enclt=intllt(num)
                  if(enclt.le.0)goto 8277
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  bufst2(bufmx2+1)=char(asc_i1)
                  call wbaseb(26,asc_a,num,
     &             bufdim-bufmx2-2,enclt,bufst2(bufmx2+2))
                  bufmx2=bufmx2+enclt+2
                  bufst2(bufmx2)=char(asc_i2)
                  goto 8279
 8276             if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &             'This is neither a html nor an internal '//
     &             'unicode character')
                  if(forgiv.lt.2)print10,'Found string is '//
     &             'neither a html nor an internal unicode character'
                  goto 8290
 8277             if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &             'This is not a valid html unicode character')
                  if(forgiv.lt.2)print10,
     &             'Found string is not a valid html unicode character'
                  goto 8290
 8278             if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &             'This is not a valid internal unicode character')
                  if(forgiv.lt.2)print10,'Found string is '//
     &             'not a valid internal unicode character'
                  goto 8290
 8279             continue
               else if(cnvtyp.eq.cnvrtf)then
c                 convert from or to RTF
                  buf=sbsb(sbs)
                  bufhi=sbse(sbs)
                  if(bufhi-buf+1.lt.1)goto 8286
                  iasc=ichar(bufstr(buf))
                  if(iasc.eq.ichar('\\'))goto 8285
                  if(iasc.ne.asc_i1)goto 8286
                  if(bufhi-buf+1.lt.3)goto 8288
                  if(ichar(bufstr(bufhi)).ne.asc_i2)goto 8288
                  call rbaseb(26,asc_a,bufhi-buf-1,bufstr(buf+1),num)
                  if(num.lt.0 .or. num.gt.uclim)goto 8288
                  if(num.ge.55296 .and. num.lt.57344)goto 8288
                  call wrtf(num,128,enclt,enc)
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  do 8282 i=1,enclt
                     bufmx2=bufmx2+1
                     bufst2(bufmx2)=enc(i)
 8282             continue
                  goto 8289
 8285             enclt=0                  
                  call rrtf(bufmax-buf+1,bufstr(buf),enclt,num)
                  if(num.lt.0 .or. num.gt.uclim)goto 8287
                  buf1=buf+enclt-1
                  enclt=intllt(num)
                  if(bufmx2+enclt.gt.bufdim)goto 9900
                  bufst2(bufmx2+1)=char(asc_i1)
                  call wbaseb(26,asc_a,num,
     &             bufdim-bufmx2-2,enclt,bufst2(bufmx2+2))
                  bufmx2=bufmx2+enclt+2
                  bufst2(bufmx2)=char(asc_i2)
                  goto 8289
 8286             if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &             'This is neither an rtf nor an internal '//
     &             'unicode character')
                  if(forgiv.lt.2)print10,'Found string is '//
     &             'neither an rtf or an internal unicode character'
                  goto 8290
 8287             if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &             'This is not a valid rtf unicode character')
                  if(forgiv.lt.2)print10,
     &             'Found string is not a valid rtf unicode character'
                  goto 8290
 8288             if(forgiv.eq.0)call fatals(buf,bufhi,bufstr,
     &             'This is not a valid internal unicode character')
                  if(forgiv.lt.2)print10,'Found string is '//
     &             'not a valid internal unicode character'
                  goto 8290
 8289             continue
               endif
               goto 8390
c              copy the substring unchanged to the output buffer
 8290          do 8295 buf=buf,sbse(sbs)
                  bufmx2=bufmx2+1
                  if(bufmx2.gt.bufdim)goto 9900
                  bufst2(bufmx2)=bufstr(buf)
 8295          continue
               goto 8390
c              insert the string from subbinss.sub with the substring label
 8300          if(sbse(sbs)-sbsb(sbs)+1.ne.9)call fatals(bufhi,buf,
     &          bufstr,'Invalid label for string to input')
               sssnew=0
               do 8310 i=sbsb(sbs),sbse(sbs)
                  iasc=ichar(bufstr(i))-asc_0
                  if(iasc.lt.0 .or. iasc.gt.9 .or. sssnew.ge.90000000)
     &             call fatal('Invalid label for string to input')
                  sssnew=sssnew*10+iasc
 8310          continue
               sssnew=sssnew-321000000
               if(sssnew.lt.1 .or. sssnew.gt.999999)call fatal(
     &          'Invalid label for string to input')
               if(ssslin.gt.sssnew)then
                  call closu(4,sub(1:subl))
                  call openru(sub(1:subl),4)
                  ssslin=1
               endif
 8320          if(ssslin.eq.sssnew)goto 8330
 8321          call getchr(4,chr,*8381,*8382)
               if(chr.ne.char(asc_lf))goto 8321
               ssslin=ssslin+1
               goto 8320
 8330          bsl=.false.
               bit8=.false.
 8340          call getchr(4,chr,*8381,*8382)
               ichr=ichar(chr)
               if(ichr.eq.asc_lf)then
                  if(bsl)call fatal(
     &             'Incomplete \\ sequence in '//sub(1:subl))
                  if(bit8)call fatal(
     &             'Incomplete \\3 sequence in '//sub(1:subl))
                  ssslin=ssslin+1
                  goto 8390
               else if(ichr.eq.asc_cr)then
                  goto 8340
               else
                  if(bsl)then
                     bsl=.false.
                     if(ichr.eq.asc_bs)then
                     else if(ichr.eq.asc_sp)then
                        ichr=0
                     else if(ichr.ge.asc_ca .and. ichr.le.asc_cz)then
                        ichr=ichr-asc_ca+1
                     else if(ichr.ge.asc_0+7 .and. ichr.le.asc_9)then
                        ichr=ichr-(asc_0+7)+27
                     else if(ichr.ge.asc_0 .and. ichr.le.asc_0+1)then
                        ichr=ichr-asc_0+30
                     else if(ichr.eq.asc_0+2)then
                        ichr=127
                     else if(bit8)then
                        call fatal(
     &                   'Incomplete "\\3" sequence in '//sub(1:subl))
                     else if(ichr.eq.asc_0+3)then
                        bit8=.true.
                        goto 8340
                     else
                        call fatal('Invalid command "\\'//chr//
     &                   '" in '//sub(1:subl))
                     endif
                  else
                     if(ichr.eq.asc_bs)then
                        bsl=.true.
                        goto 8340
                     endif
                  endif
                  bufmx2=bufmx2+1
                  if(bufmx2.gt.bufdim)goto 9900
                  if(bit8)then
                     bit8=.false.
                     if(ichr.gt.127)call fatal(
     &                'Corrupt "\\3" sequence in '//sub(1:subl))
                     ichr=ichr+128
                  endif
                  if(cnvtyp.eq.cnv8bt)then
                     if(ichr.le.127)then
                        ichr=ichr+128
                     else
                        ichr=ichr-128
                     endif
                  endif
                  bufst2(bufmx2)=char(ichr)
                  goto 8340
               endif
 8381          call inttos(sssnew+321000000,cint,cintl) 
               call fatal('Premature end to file '//sub(1:subl)//
     &          ' while looking for label '//cint(1:cintl))
 8382          call fatal('Error reading file '//sub(1:subl))
c              end inserting the substring
 8390          frmsss=0
               cnvtyp=0
            else if(itrn.eq.asc_pl)then
c              increase forgiveness
               forgiv=forgiv+1
            else if(itrn.eq.asc_mn)then
c              decrease forgiveness
               forgiv=forgiv-1
            else if(itrn.eq.asc_n)then
               call inttos(linctt,cint,cintl)
               if(bufmx2+cintl.gt.bufdim)goto 9900
               do 8420 i=1,cintl
                  bufmx2=bufmx2+1
                  bufst2(bufmx2)=cint(i:i)
 8420          continue
            else if(itrn.eq.asc_x)then
c              toggle hex mode
               hexmod=1-hexmod
            else if(itrn.eq.asc_i)then
c              set insert-from-subbinss.sub mode
               if(frmsss.ne.0)call fatal('Duplicate \\i command')
               if(label.ne.0)call fatal('Cannot mix \\o and \\i '//
     &          'commands in the same substitutions file')
               if(ssslin.eq.0)then
                  call openru(sub(1:subl),4)
                  ssslin=1
               endif
               frmsss=1
            else if(itrn.eq.asc_o)then
c              put the original in subbinss.sub, put a label in the replacement
               if(label0.ne.label)call fatal(
     &          'Only one \\o label can be generated in a substitution')
               if(ssslin.ne.0)call fatal('Cannot mix \\o and \\i '//
     &          'commands in the same substitutions file')
               if(label.eq.0)then
                  call openwu(sub(1:subl),4)
                  label=321000000
               endif
               label0=label
               label=label+1
               if(label-321000000.gt.999999)
     &          call fatal('Too many numerical labels')
               call inttos(label,cint,cintl)
               if(index(cint(1:cintl),'*').ne.0)
     &          call fatal('Currupt label')
               do 8450 i=1,cintl
                  bufmx2=bufmx2+1
                  if(bufmx2.gt.bufdim)goto 9900
                  bufst2(bufmx2)=cint(i:i)
 8450          continue
               enclt=-1
               do 8470 buf=max(1,buf0),min(bufmax,buf1)
                  iasc=ichar(bufstr(buf))
                  if(cnvtyp.eq.cnv8bt)then
                     if(iasc.le.127)then
                        iasc=iasc+128
                     else
                        iasc=iasc-128
                     endif
                  endif
                  if(iasc.ne.asc_bs .and.
     &             iasc.ne.asc_cr .and. iasc.ne.asc_lf)then
                     call putchr(4,char(iasc),*8480)
                  else
                     enc(1)=char(iasc)
                     call encstr(1,enc,4,enclt,enc)
                     do 8460 i=1,enclt
                        call putchr(4,enc(i),*8480)
 8460                continue
                  endif
 8470          continue
               call putchr(4,char(asc_cr),*8480)
               call putchr(4,char(asc_lf),*8480)
               goto 8490
 8480          call fatal('Error writing to '//sub(1:subl))
 8490          cnvtyp=0
            else if(itrn.gt.trncvl)then
c              any other del sequence must be conversion type
               call fatal('Program error: invalid DEL string found')
            else
c              set conversion type
               if(cnvtyp.eq.0)then
                  cnvtyp=itrn
                  repeat=0
               else
                  if(cnvtyp.eq.itrn)then
                     repeat=repeat+1
                  else
                     call fatal('Only one conversion type '//trncnv//
     &                ' can be active at a time')
                  endif
               endif
            endif
c           done with the del sequence
            goto 8100
         endif
c        single replacement character
 8500    bufmx2=bufmx2+1
         if(bufmx2.gt.bufdim)goto 9900
         bufst2(bufmx2)=trnstr(trn)
         if(obeymx.gt.0)then
            iasc=ichar(trnstr(trn))
            if(iasc.ge.asc_a .and. iasc.le.asc_z)then
               obey=obey+1
               if(obeyc(min(obeymx,obey)).eq.'A')
     &          bufst2(bufmx2)=char(iasc-32)
            else if(iasc.ge.asc_ca .and. iasc.le.asc_cz)then
               obey=obey+1
               if(obeyc(min(obeymx,obey)).eq.'a')
     &          bufst2(bufmx2)=char(iasc+32)
            endif
         endif
         goto 8100
c        done replacing with the translation
 8600    if(frmsss.ne.0)
     &    call fatal('Command \\i has no [sub]string label to insert')
         if(cnvtyp.ne.0)call fatal('Command \\'//trncnv(cnvtyp:cnvtyp)//
     &    ' has no [sub]string label to convert')
         change=1
         if(bufmx2-bufm20 .ne. min(bufmax,buf1)-max(1,buf0)+1)goto 8611
         buf=max(1,buf0)-1
         do 8610 buf2=bufm20+1,bufmx2
            buf=buf+1
            if(bufst2(buf2).ne.bufstr(buf))goto 8611
 8610    continue
         change=0
 8611    continue

      else if(trnmod.eq.1)then

         change=0
         buf=max(1,buf0)-1
 8710    buf=buf+1
         if(buf.gt.buf1 .or. buf.gt.bufmax)goto 8790
         iasc=ichar(bufstr(buf))
         if(iasc.eq.ilook)then
            if(isspc.eq.spcr)then
               if(chang0.ne.0)change=1
               do 8750 trn=3,-trnmax
                  bufmx2=bufmx2+1
                  if(bufmx2.gt.bufdim)goto 9900
                  bufst2(bufmx2)=trnstr(trn)
 8750          continue
               goto 8710
            else if(isspc.eq.spce)then
               if(min(bufmax,buf1)-buf+1.lt.-trnmax-2)goto 8780
               do 8760 trn=3,-trnmax-1
                  if(trnstr(trn).ne.bufstr(buf+trn-2))goto 8780
 8760          continue
               if(chang0.ne.0)change=1
               bufmx2=bufmx2+1
               if(bufmx2.gt.bufdim)goto 9900
               bufst2(bufmx2)=trnstr(-trnmax)
               buf=buf-trnmax-3
               goto 8710
            else
               call fatal('Program error: isspc is invalid')
            endif
         endif
 8780    bufmx2=bufmx2+1
         if(bufmx2.gt.bufdim)goto 9900
         bufst2(bufmx2)=char(iasc)
         goto 8710
 8790    continue

      else if(trnmod.eq.2)then

         change=0
         buf=sbsb(sbsmax)
         bufhi=sbse(sbsmax)
         fnddat(2)=buf
         fnddat(3)=bufhi
         if(fnddat(1).eq.-trnmax)then
c           found
            if(index(',!?',trnstr(1)).ne.0)goto 9200
            goto 9100
         endif

      endif

c     update the nontrivial substitution count
      subct=subct+change

c     set the new position in bufstr to proceed from
      if(buf1.ge.buf0)then
         do 8910 buf=buf0+1,buf1
            if(bufstr(buf).eq.char(asc_lf))linctt=linctt+1
 8910    continue
         buf0=buf1
      else
         if(buf0.ge.1 .and. buf0.le.bufmax .and. trnmod.ne.2)then
            bufmx2=bufmx2+1
            if(bufmx2.gt.bufdim)goto 9900
            bufst2(bufmx2)=bufstr(buf0)
         endif
      endif

c     go examine the next starting character in bufstr
      goto 1000

c     no more characters in bufstr
 9000 continue

c     set the final location as a zero length string at eof if not found
      if(trnmod.eq.2)then
         if(trnmax.ne.-123456789 .or. fnddat(1).le.0)then
c           not found
            if(trnstr(1).eq.'.')goto 9200
            fnddat(2)=bufmax+1
            fnddat(3)=bufmax
         else if(index(',!?',trnstr(1)).ne.0)then
c           found
            buf=fnddat(2)
            bufhi=fnddat(3)
            goto 9200
         endif
      endif

c     all done
 9100 linctt=0
      call outsub('dosubs')
      return

c     handle the case that a warning or forbidden string is found
 9200 do 9210 trn=2,trndim
         if(trnstr(trn).eq.char(0))goto 9220
 9210 continue
      call fatal('Program error: \\'//trnstr(1)//
     & ' string not NUL terminated')
 9220 if(trn.gt.2)then
         call inttos(linctt,cint,cintl)
         if(index('?!',trnstr(1)).gt.0)then
            print10,'Problem in '//
     &       txtfil(1:txtfll)//' ('//cint(1:cintl)//'):'
            print20,(trnstr(trn),trn=2,trn-1)
         else
            print20,(txtfil(i:i),i=1,txtfll),':',' ',
     &       (trnstr(trn),trn=2,trn-1)
         endif
      endif
      if(trnstr(1).eq.'.' .or. trnstr(1).eq.',')then
         if(ssslin.ne.0 .or. label.ne.0)call closu(4,sub(1:subl))
         call exit(0)
      endif
      if(trnstr(1).eq.'!')then
         if(bufhi.ge.buf)then
            call fatals(bufhi,buf,bufstr,
     &       'The text file string above is not allowed')
         else
            call fatal('A forbidden string was found')
         endif
      endif
      if(bufhi.ge.buf)then
         print10,'Offending string:'
         call prnstr(bufhi,buf,bufstr)
      endif
 9280 print9281,'Hit Enter to continue, or q and Enter to quit: '
 9281 format(a,$)
      read(*,9282)ans
 9282 format(a16)
      if(ans.eq.'q')call fatal('Aborted by user on problem')
      if(ans.ne.' ')goto 9280
      goto 9100

c     out of storage
 9900 call inttos(bufdim,cint,cintl)
      call fatal('Out of storage.  Buffer size is '//cint(1:cintl))
      end
