c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

      subroutine fatal(msg)

c     Terminate execution on error, printing error message msg.

      implicit none

c input:

c     message to show
      character*(*) msg

c local

c     character
      character*1 chr(1)

c executable

c     let fatalg do it
      call insub('fatalg')
      call fatalg(0,0,chr,msg,'!',' ')

      end



      subroutine fatals(str1,str2,strchr,msg)

c     Terminate execution on error, printing the string strchr between
c     str1 and str2, and then error message msg.  If the string, after
c     conversion to printable ASCII, exceeds 77 characters, the part at
c     the str2 side is truncated.

      implicit none

c input:

c     limits of the string to show; str1 may exceed str2
      integer str1,str2
c     string characters
      character*1 strchr(*)

c     message to show
      character*(*) msg

c executable

c     let fatalg do it
      call insub('fatalg')
      call fatalg(str1,str2,strchr,msg,'!',' ')

      end



      subroutine fatal2(msg1,msg2)

c     Terminate execution on error, printing error message msg1msg2.

      implicit none

c input:

c     message to show
      character*(*) msg1
      character*(*) msg2

c local

c     character
      character*1 chr(1)

c executable

c     let fatalg do it
      call insub('fatalg')
      call fatalg(0,0,chr,msg1,msg2,' ')

      end



      subroutine fatalg(str1,str2,strchr,msg1,msg2,msg3)

c     This is an internal subroutine to terminate execution on error.
c     It is used by the more user-friendly ones.

c     Warning: this subroutine *must* be called as:
c        call insub('fatalg')
c        call fatalg(str1,str2,strchr,msg1,msg2,msg3)
c     This subroutine itself may *not* call insub.

      implicit none

c input:

c     start and end of the string to show
      integer str1,str2
c     string
      character*1 strchr(*)

c     message to show
      character*(*) msg1,msg2,msg3

c common

c     verbosity
      integer verbos
c     substitions file
      character*132 subfil
      integer subfll
c     line count of the substitutions file
      integer lincts
c     text file
      character*132 txtfil
      integer txtfll
c     line count of the text file
      integer linctt

c     debug block
      common/cmdbg/verbos,subfil,subfll,lincts,txtfil,txtfll,linctt

c     buffer of subroutine names
      integer subdim
      parameter (subdim=10)
      character*8 subnam(subdim+1)
      integer sublt(subdim+1),sub

c     subroutine block
      common/cmsubs/subnam,sublt,sub

c local

c     integer as string
      character*12 cint
      integer cintl

c     generic index
      integer i

c executable

c     generic formats
 10   format(a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a,a)
 20   format(a)

c     skip all but the error if verbos is negative
      if(verbos.lt.0)goto 500

c     dump the subroutine history
      if(sub.eq.0)then
         print10,
     &    '*** The subroutine traceback has not been initialized!'
      else
         if(sub.lt.0)then
            print10,
     &       '*** The subroutine traceback has been corrupted!'
            print10,
     &       'The last traceback before corruption was:'
            sub=-sub
         endif
         if(sublt(sub).le.0)sub=sub-1
         print10,subnam(1)(1:sublt(1)),
     &    (' ',subnam(i)(1:sublt(i)),i=2,sub)
      endif

c     data on substitutions file
      if(subfll.gt.0)then
         if(lincts.le.0)then
            print10,'Substitutions file ',subfil(1:subfll)
         else
            call inttos(lincts,cint,cintl)
            print10,'Substitutions file ',subfil(1:subfll),
     &       ' line ',cint(1:cintl)
         endif
      endif

c     data on text file
      if(txtfll.gt.0)then
         if(linctt.le.0)then
            print10,'Text file ',txtfil(1:txtfll)
         else
            call inttos(linctt,cint,cintl)
            print10,'Text file ',txtfil(1:txtfll),
     &       ' line ',cint(1:cintl),' (modified)'
         endif
      endif

c     print the string
 500  call prnstr(str1,str2,strchr)

c     print the error message
      if(msg3.eq.' ')then
         print10,char(7),'*** ',msg1,msg2
      else
         print10,char(7),'*** ',msg1,msg2,msg3
      endif

c     exit in disgrace
      if(index(msg1,'Out of storage').eq.0)then
         call fatal0(1)
      endif
      if(verbos.ge.0)print20,
     & 'There should be a bigger version of subbin.exe in the bin',
     & 'subfolder of the system-files folder inside l2h.  Look for',
     & 'subbin_big.exe.  Move this file in with your document.  Or',
     & 'move it into the bin subfolder of l2h, deleting the subbin.exe',
     & 'that is there.  Either way, rename subbin_big.exe subbin.exe.',
     & ' ',
     & 'If that is not enough, you may need to edit subbn0_big.f and',
     & 'hyphen_big.f and build a bigger subbin_big.f yourself.  This',
     & 'requires the g77 compiler.'
      call fatal0(2)

      end


      subroutine prnstr(str1,str2,strchr)

c     Print the string strchr between str1 and str2.  If the string,
c     after conversion to printable ASCII, exceeds 77 characters, the
c     part at the str2 side is truncated.

      implicit none

c input:

c     start and end of the string to show
      integer str1,str2
c     string
      character*1 strchr(*)

c local

c     encoded characters of string
      character*1 enc(312)
      integer enclt

c     string index
      integer i

c executable

c     print format
 10   format(79a1)

c     encode the string and print it out as far as possible
      if(str1.gt.0 .and. str2.gt.0)then
         if(str1.le.str2)then
            call encstr(str2-str1+1,strchr(str1),78,enclt,enc)
            if(enclt.le.77)then
               print10,'"',(enc(i),i=1,enclt),'"'
            else
               print10,'"',(enc(i),i=1,74),'.','.','.','"'
            endif
         else
            i=max(str2,str1-77)
            call encstr(str1-i+1,strchr(i),312,enclt,enc)
            if(enclt.le.77)then
               print10,'"',(enc(i),i=1,enclt),'"'
            else
               print10,'"','.','.','.',(enc(i),i=enclt-73,enclt),'"'
            endif
         endif
      endif

      return
      end


      subroutine fatal0(stat)

c     Terminate execution with an error code.

      implicit none

c input

c     status to return (1: error, 2: out of storage')
      integer stat

c executable

      call exit(stat)

      end



      subroutine insub(subrtn)

c     Remember the fact that we are now in subroutine subrtn.

c     The argument subrtn must be a lower case string of up to 8
c     characters that uniquely identifies the calling subroutine.
c     Before the calling subroutine exits, it must call outsub.

c     Subroutine insub must be initialized from the main program as:
c           call insub('main')

      implicit none

c input:

c     subroutine name
      character*(*) subrtn

c common

c     buffer of subroutine names
      integer subdim
      parameter (subdim=10)
      character*8 subnam(subdim+1)
      integer sublt(subdim+1),sub

c     subroutine block
      common/cmsubs/subnam,sublt,sub

c local

c     whether we have been initialized
      integer init
      data init/0/
      save init

c     local copy of the data base
      character*8 copnam(subdim+1)
      integer coplt(subdim+1),cop
      save copnam,coplt,cop

c     second subroutine index
      integer sub2

c     character string
      character*1 chr(1)

c     character index
      integer i

c executable

c     initialize
      if(init.eq.0)then
         if(subrtn.eq.'main')then
            init=1
            sub=1
            subnam(sub)='main'
            sublt(sub)=4
            goto 900
         endif
      endif

c     check whether the data base is initialized
      if(init.ne.1)then
         sub=0
         if(subrtn.eq.'fatalg')return
         call fatalg(0,0,chr,'Program error in insub: ',
     &    'subroutine insub has not been initialized!',' ')
      endif

c     check whether the data base is corrupted
      if(sub.gt.cop)goto 110
      if(sub.lt.1)goto 110
      do 100 cop=1,sub
         if(coplt(cop).ne.sublt(cop))goto 110
         if(copnam(cop).ne.subnam(cop))goto 110
 100  continue
      goto 190
c     corrupt
 110  do 150 sub=1,cop
         subnam(sub)=copnam(sub)
         sublt(sub)=coplt(sub)
 150  continue
      sub=-cop
      if(subrtn.eq.'fatalg')return
      call fatalg(0,0,chr,'Program error in insub: ',
     & 'the subroutine traceback is corrupt!',' ')
 190  continue

c     that is all we do for fatalg and outsub
      if(subrtn.eq.'fatalg')return
      if(subrtn.eq.'outsub')return

c     convert the name to local
      sub=sub+1
      subnam(sub)=subrtn
      sublt(sub)=0
      do 300 i=1,8
         if((subnam(sub)(i:i).ge.'a' .and. subnam(sub)(i:i).le.'z') .or.
     &    (subnam(sub)(i:i).ge.'0' .and. subnam(sub)(i:i).le.'9'))then
            if(sublt(sub).lt.i-1)call fatalg(0,0,chr,
     &       'Program error in insub: subroutine name ',
     &          subnam(sub)(1:i),'... may not contain spaces!')
            sublt(sub)=i
         else if(subnam(sub)(i:i).ne.' ')then
            if(subnam(sub)(i:i).lt.' ' .or. subnam(sub)(i:i).gt.'~')
     &       call fatalg(0,0,chr,'Program error in insub: '//
     &       'subroutine name ',subnam(sub)(1:i),
     &       '... contains a nonprintable character!')
            call fatalg(0,0,chr,'Program error in insub: '//
     &       'subroutine name ',subnam(sub)(1:i),
     &       '... contains illegal character "'//subnam(sub)(i:i)//'"!')
         endif
 300  continue
      if(subrtn.ne.subnam(sub))call fatalg(0,0,chr,
     & 'Program error in insub: subroutine name ',subnam(sub),
     & '... is too long!')
      if(sublt(sub).le.0)call fatalg(0,0,chr,'Program error in insub: ',
     & 'the subroutine name is blank!',' ')

c     check for buffer overflow
      if(sub.gt.subdim)
     & call fatalg(0,0,chr,
     & 'Program error inside insub: ','too many subroutines!',' ')

c     see whether it is already there
      do 500 sub2=1,sub-1
         if(subnam(sub2).eq.subnam(sub))
     &    call fatalg(0,0,chr,'Program error in insub: subroutine ',
     &    subnam(sub)(1:sublt(sub)),' cannot call itself!')
 500  continue

c     copy the new name
 900  cop=sub
      copnam(sub)=subnam(sub)
      coplt(sub)=sublt(sub)

      return
      end



      subroutine outsub(subrtn)

c     Remember the fact that we are no longer in subroutine subrtn.

c     This subroutine should not be called by the main program.

      implicit none

c input:

c     subroutine
      character*(*) subrtn

c common

c     buffer of subroutine names
      integer subdim
      parameter (subdim=10)
      character*8 subnam(subdim+1)
      integer sublt(subdim+1),sub

c     subroutine block
      common/cmsubs/subnam,sublt,sub

c local

c     cleaned name
      character*12 cln
      integer clnlt

c     index
      integer i

c executable

c     check for buffer corruption
      call insub('outsub')

c     OK if subrtn is marked as current and not main
      if(subrtn.eq.subnam(sub))then
         if(sub.eq.1)
     &    call fatal('Program error in outsub: cannot get out of main')
         sub=sub-1
         return
      endif

c     check for missing name
      if(subrtn.eq.' ')call fatal(
     & 'Program error in outsub: the subroutine name is blank')

c     logic is wrong
      cln=subrtn
      if(cln(1:8).ne.subrtn)cln(9:12)='... '
      clnlt=1
      do 100 i=1,12
         if(cln(i:i).ne.' ')clnlt=i
         if(cln(i:i).lt.' ' .or. cln(i:i).gt.'~')cln(i:i)='?'
 100  continue
      call fatal('Program error in outsub: either '//cln(1:clnlt)//
     & ' did not enter or '//subnam(sub)(1:sublt(sub))//' did not exit')

      end



      subroutine inttos(int,cint,cintl)

c     Convert an integer to a string

      implicit none

c input:

c     integer to show
      integer int

c     integer as string
      character*(*) cint
      integer cintl

c local:

c     printed integer as string
      character*12 ci
c     string limits
      integer i1,i2
c     character index
      integer i

c executable

c     print the integer
      write(ci,10)int
 10   format(i12)

c     find the boundaries
      do 20 i=1,12
         i1=i
         if(ci(i:i).gt.' ')goto 21
 20   continue
      goto 90
 21   do 30 i=12,1,-1
         i2=i
         if(ci(i:i).gt.' ')goto 31
 30   continue
      goto 90
 31   continue

c     copy it to output
      cintl=i2-i1+1
      if(cintl.gt.len(cint))goto 90
      cint=ci(i1:i2)      
      return

c     error
 90   cint='************'
      cintl=min(12,len(cint))
      return
      end



      subroutine encstr(strmax,strchr,outdim,outmax,outstr)

c     Encode the strmax characters in str for printing.

c     Note that this will never shorten a string, and expand it by at
c     most a factor 4.

      implicit none

c input:

c     string characters
      integer strmax
      character*1 strchr(*)

c     declared encoded string size
      integer outdim

c output:

c     encoded string
      integer outmax
      character*1 outstr(outdim)

c local

c     ascii code
      integer iasc

c     indices
      integer i,i2

c executable

c     encode the string
      i2=0
      do 100 i=1,strmax
         iasc=ichar(strchr(i))
 50      i2=i2+1
         if(i2.gt.outdim)goto 110
         if(iasc.ge.32 .and. iasc.lt.127 .and. iasc.ne.ichar('\\'))then
            outstr(i2)=char(iasc)
            goto 100
         endif
         outstr(i2)='\\'
         i2=i2+1
         if(i2.gt.outdim)goto 110
         if(iasc.gt.127)then
            outstr(i2)='3'
            iasc=iasc-128
            goto 50
         endif
         if(iasc.eq.0)then
            outstr(i2)=' '
         else if(iasc.ge.1 .and. iasc.le.26)then
            outstr(i2)=char(iasc-1+ichar('A'))
         else if(iasc.ge.27 .and. iasc.le.29)then
            outstr(i2)=char(iasc-27+ichar('7'))
         else if(iasc.ge.30 .and. iasc.le.31)then
            outstr(i2)=char(iasc-30+ichar('0'))
         else if(iasc.eq.127)then
            outstr(i2)='2'
         else
            outstr(i2)='\\'
         endif
 100  continue
      outmax=i2
      return
 110  outmax=outdim
      return

      end
