      subroutine getsub(
     & memdim,memmax,memstr,orgdim,trndim,
     & orgmax,orgstr,orgmod,trnmax,trnstr)

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     Get an original and translation pair from the substitutions file
c     on unit 4.

c     Returns orgmax=0 on end of file.  Otherwise returns orgmax and/or
c     trnmax negative if they are a command that requires special
c     handling.

c     Each item in the original is represented by both an orgstr and an
c     orgmod character.  If the item is just a plain character, then
c     orgstr is that character and orgmod is ascii null.  On the other
c     hand, if the item is a character type specification or a command,
c     then orgmod is the name of the specification or command and orgstr
c     is ascii null.

c     Normal translation (i.e. trnmax not negative) consists of
c     translation characters and commands.  Commands are of the form of
c     a DEL delete character followed by another character.  For the
c     precise second character, see the source below.  DEL DEL is the
c     single delete character.

      implicit none

c input:

c     declared memory string size
      integer memdim
c     actual memory size
      integer memmax
c     memory string
      character*1 memstr(memdim)

c     declared original string size
      integer orgdim

c     declared translation string size
      integer trndim

c output:

c     actual original string size, or 0 for eof, or negative for a special
      integer orgmax
c     original string
      character*1 orgstr(orgdim)
c     modifiers
      character*1 orgmod(orgdim)

c     actual translation string size, or negative for a special
      integer trnmax
c     translation string
      character*1 trnstr(trndim)

c common

c     verbosity
      integer verbos
c     substitions file
      character*132 subfil
      integer subfll
c     line count of the substitutions file
      integer lincts
c     text file
      character*132 txtfil
      integer txtfll
c     line count of the text file
      integer linctt

c     debug block
      common/cmdbg/verbos,subfil,subfll,lincts,txtfil,txtfll,linctt

c local:

c     whether we have reached end of file
      logical eof
      save eof
      data eof/.false./

c     single character (from a set) match specifications and char(0)
      character*11 scms
      save scms
      data scms/' ''_+56alu[]'/
c     specials for the original not followed by numbers
      character*2 orgspc
      save orgspc
      data orgspc/'rx'/
c     specials for the original optionally followed by numbers
      character*2 orgspn
      save orgspn
      data orgspn/'df'/
c     specials not followed by a translation
      character*3 notran
      save notran
      data notran/'drx'/

c     translation commands that convert
      character*19 trncnv
      save trncnv
      data trncnv/'"$&''()456<=>`hlu{|}'/
c     special translation commands without instance selection
      character*2 trnspc
      save trnspc
      data trnspc/'er'/
c     special translation commands that select a particular instance
      character*10 trnspn
      save trnspn
      data trnspn/'bcfsv^.,!?'/

c     whether this is a special
      logical isspc

c     whether we are in a comment
      logical incom
c     whether a carriage-return is pending
      logical cr
c     whether a backslash is pending
      logical bsl
c     whether an 8 bit conversion is pending
      logical bit8
c     whether a substring is pending
      logical subs
c     whether we are in a group
      logical group

c     number of characters in the match group
      integer mgroup

c     ascii value of backslash
      integer asc_bs
      parameter (asc_bs=92)
c     ascii value of percent
      integer asc_pt
      parameter (asc_pt=37)
c     ascii values of 0, 9, +, and -
      integer asc_0,asc_9,asc_pl,asc_mn
      parameter (asc_0=48,asc_9=57,asc_pl=43,asc_mn=45)
c     ascii values of space, carriage return, and linefeed
      integer asc_sp,asc_cr,asc_lf
      parameter (asc_sp=32,asc_cr=13,asc_lf=10)
c     ascii values of i, n, o, and x
      integer asc_i,asc_n,asc_o,asc_x
      parameter (asc_i=105,asc_n=110,asc_o=111,asc_x=120)
c     ascii values of A and Z
      integer asc_ca,asc_cz
      parameter (asc_ca=65,asc_cz=90)

c     number value
      integer value

c     position in orginal string
      integer org,org2

c     position in translated string
      integer trn,trn2

c     position in memory string
      integer mem

c     substring count
      integer sbs

c     saved value of the text file length
      integer txtfl0

c     character
      character*1 chr
      integer ichr

c executable:

c     note where we are
      call insub('getsub')

c     save the text file name length and disable its output
      txtfl0=txtfll
      txtfll=0

c     return on end of file
      orgmax=0
      if(eof)goto 9000

c     initialize
      if(lincts.le.1)then

c        complete scms
         scms(1:1)=char(0)

c        set line number
         lincts=1

      endif

c     read the original

c     initialize the original
 1000 orgmax=0
      cr=.false.
      bsl=.false.
      bit8=.false.
      incom=.false.

c     read from the file
 1100 call getchr(1,chr,*1110,*1120)
      ichr=ichar(chr)
      goto 1130
 1110 eof=.true.
      if(cr)call fatal('Illegal carriage-return')
      if(bsl)call fatal('Incomplete \\ sequence at EOF')
      if(bit8)call fatal('Incomplete "\\3" sequence at EOF')
      if(orgmax.le.0)goto 9000
      goto 1900
 1120 call fatal('Disk error reading the substitutions file')
 1130 continue

c     check for end of line
      if(ichr.eq.asc_lf)then
         if(bsl)call fatal('Incomplete \\ sequence at end of line')
         if(bit8)call fatal('Incomplete "\\3" sequence at end of line')
         lincts=lincts+1
         goto 1900
      endif

c     carriage-return character check
      if(cr)call fatal('Illegal carriage-return')

c     ignore all characters in comments
      if(incom)goto 1100

c     check for loose carriage-return characters
      if(ichr.eq.asc_cr)then
         cr=.true.
         goto 1100
      endif

c     increment the buffer
      orgmax=orgmax+1
      if(orgmax.gt.orgdim)call fatal('String to translate too long')

c     process the character based on whether we are in a backslash sequence
      orgmod(orgmax)=char(0)
      if(bsl)then
         bsl=.false.
c        create special characters
         if(ichr.eq.asc_bs)then
         else if(ichr.eq.asc_sp)then
            ichr=0
         else if(ichr.ge.asc_ca .and. ichr.le.asc_cz)then
            ichr=ichr-asc_ca+1
         else if(ichr.ge.asc_0+7 .and. ichr.le.asc_9)then
            ichr=ichr-(asc_0+7)+27
         else if(ichr.ge.asc_0 .and. ichr.le.asc_0+1)then
            ichr=ichr-asc_0+30
         else if(ichr.eq.asc_0+2)then
            ichr=127
         else if(bit8)then
            call fatal('Incomplete "\\3" sequence')
         else if(ichr.eq.asc_0+3)then
            bit8=.true.
            orgmax=orgmax-1
            goto 1100
         else if(ichr.eq.asc_pt)then
            incom=.true.
            orgmax=orgmax-1
            goto 1100
         else if(ichr.eq.ichar('m'))then
            if(memmax.lt.0)call fatal('Memory string is not defined')
            orgmax=orgmax-1
            if(memmax.gt.0)then
               if(orgmax+memmax.gt.orgdim)
     &          call fatal('String to translate too long')
               do 1510 mem=1,memmax
                  orgstr(orgmax+mem)=memstr(mem)
                  orgmod(orgmax+mem)=char(0)
 1510          continue
               orgmax=orgmax+memmax
            else
               if(orgmax+6.gt.orgdim)
     &          call fatal('String to translate too long')
               orgmod(orgmax+1)='~'
               orgmod(orgmax+2)='&'
               orgmod(orgmax+3)='a'
               orgmod(orgmax+4)='['
               orgmod(orgmax+5)=']'
               orgmod(orgmax+6)='*'
               do 1520 mem=1,6
                  orgstr(orgmax+mem)=char(0)
 1520          continue
               orgmax=orgmax+6
            endif
            goto 1100
         else
            orgmod(orgmax)=char(ichr)
            ichr=0
         endif
      else
c        check for a new backslash sequence
         if(ichr.eq.asc_bs)then
            bsl=.true.
            orgmax=orgmax-1
            goto 1100
         endif
      endif

c     add the character
      if(bit8)then
         bit8=.false.
         ichr=ichr+128
         if(ichr.gt.255)call fatal('Illegal 8 bit character')
      endif
      orgstr(orgmax)=char(ichr)
      goto 1100

c     ignore empty lines before originals
 1900 if(orgmax.le.0)goto 1000

c     check the original for consistency
      lincts=lincts-1
      subs=.false.
      sbs=0
      group=.false.
      mgroup=-1
      do 1950 org=1,orgmax
         chr=orgmod(org)
c        in match groups, only normal characters count
         if(mgroup.ge.0)then
            if(chr.eq.char(0))then
               mgroup=mgroup+1
            else
               if(chr.ne.'.')call fatal('Inside \\. ... \\. no '//
     &          'commands or character classes are allowed ')
            endif
         endif
c        single character match specifications are fine
         if(index(scms,chr).gt.0)goto 1950
c        translate aliases
         if(index('()',chr).gt.0)then
            if(chr.eq.'(')chr='&'
            if(chr.eq.')')chr='*'
            orgmod(org)=chr
         endif
c        case changes are fine
         if(index('-=`',chr).gt.0)goto 1950
c        no number specials are only fine alone.
         if(index(orgspc,chr).gt.0)then
            if(orgmax.ne.1)call fatal('Illegal original special: '//chr)
            orgmax=-1
            goto 1951
         endif
c        number specials are only fine at the start and followed by digits
         if(index(orgspn,chr).gt.0)then
            if(org.ne.1)call fatal('Illegal original special: '//chr)
            value=0
            do 1920 org2=2,orgmax
               if(orgstr(org2).lt.'0' .or. orgstr(org2).gt.'9')
     &          call fatal(
     &          'Invalid number behind original special '//chr)
               value=value*10-48+ichar(orgstr(org2))
               if(value.ge.123456789 .or. value.lt.0)call fatal(
     &          'Original special '//chr//' has a too large count')
 1920       continue
            orgmax=-value
            if(orgmax.eq.0)orgmax=-123456789
            goto 1951
         endif
c        check matches
         if(chr.eq.'.')then
            if(group)
     &       call fatal('Command "\\." cannot be inside a group')
            if(mgroup.ge.0)then
               if(mod(mgroup,2).ne.0)call fatal(
     &          '\\. ... \\. characters must come in pairs')
               mgroup=-1
            else
               mgroup=0
            endif
         else if(chr.eq.':')then
            if(group)
     &       call fatal('Command "\\." cannot be inside a group')
c        check \? and \! for preceding a \~, group, or character specification
         else if(chr.eq.'?' .or. chr.eq.'!')then
            if(group)call fatal(
     &       'Command "\\'//chr//'" cannot be inside a group')
            if(index('~&('//scms,orgmod(min(orgmax,org+1))).le.0)
     &       call fatal('Incomplete command "\\'//chr//'"')
c        check \~ for preceding a group or character specification
         else if(chr.eq.'~')then
            if(group)
     &       call fatal('Command "\\~" cannot be inside a group')
            if(index('&('//scms,orgmod(min(orgmax,org+1))).le.0)
     &       call fatal('Incomplete command "\\~"')
c        check group start
         else if(chr.eq.'&')then
            if(group)call fatal('Already in a group')
            group=.true.
c        check group end
         else if(chr.eq.'*')then
            if(.not.group)call fatal('No group to end')
            group=.false.
c        check substring start
         else if(chr.eq.'{')then
            if(subs)call fatal('Already in a substring')
            if(group)call fatal('Cannot start substring in a group')
            subs=.true.
            sbs=sbs+1
c        check substring end
         else if(chr.eq.'}')then
            if(.not.subs)call fatal('No substring to end')
            if(group)call fatal('Cannot end substring in a group')
            subs=.false.
         else
            call fatal('Illegal command "\\'//chr//'"')
         endif
 1950 continue
 1951 if(group)call fatal('Unterminated "\\&" group')
      if(mgroup.ne.-1)call fatal('Unterminated "\\." group')
      if(subs)call fatal('Unterminated "\\{" substring')
      if(sbs.gt.9)call fatal('Too many "\\{" substrings')
      lincts=lincts+1

c     return if there is no translation
      if(orgmod(1).eq.'r')eof=.true.
      if(index(notran,orgmod(1)).gt.0)goto 9000
      if(eof)call fatal('Missing translation line at end of file')

c     read the translation

c     initialize the translation
      trnmax=0
      cr=.false.
      bsl=.false.
      bit8=.false.
      incom=.false.
      subs=.false.
      isspc=.false.

c     read from the file
 2100 call getchr(1,chr,*2110,*2120)
      ichr=ichar(chr)
      goto 2130
 2110 eof=.true.
      if(cr)call fatal('Illegal carriage-return')
      if(bsl)call fatal('Incomplete \\ sequence')
      if(bit8)call fatal('Incomplete "\\3" sequence')
      if(subs)call fatal('Incomplete "\\#" sequence')
      goto 2900
 2120 call fatal('Disk error reading the substitutions file')
 2130 continue

c     check for end of line
      if(ichr.eq.asc_lf)then
         if(bsl)call fatal('Incomplete \\ sequence')
         if(bit8)call fatal('Incomplete "\\3" sequence')
         if(subs)call fatal('Incomplete "\\#" sequence')
         lincts=lincts+1
         goto 2900
      endif

c     carriage-return character check
      if(cr)call fatal('Illegal carriage-return')

c     ignore all characters in comments
      if(incom)goto 2100

c     check for loose carriage-return characters
      if(ichr.eq.asc_cr)then
         cr=.true.
         goto 2100
      endif

c     increment the buffer
      trnmax=trnmax+1
      if(trnmax.gt.trndim)call fatal('Substitution string too long')

c     process the character based on whether we are in a backslash sequence
      if(bsl)then
         bsl=.false.
c        create special characters
         if(ichr.eq.asc_bs)then
         else if(ichr.eq.asc_sp)then
            ichr=0
         else if(ichr.ge.asc_ca .and. ichr.le.asc_cz)then
            ichr=ichr-asc_ca+1
         else if(ichr.ge.asc_0+7 .and. ichr.le.asc_9)then
            ichr=ichr-(asc_0+7)+27
         else if(ichr.ge.asc_0 .and. ichr.le.asc_0+1)then
            ichr=ichr-asc_0+30
         else if(ichr.eq.asc_0+2)then
            ichr=127
         else if(bit8)then
            call fatal('Incomplete "\\3" sequence')
         else if(ichr.eq.asc_0+3)then
            bit8=.true.
            trnmax=trnmax-1
            goto 2100
         else if(subs)then
            call fatal('Incomplete substring')
         else if(ichr.eq.asc_pt)then
            incom=.true.
            trnmax=trnmax-1
            goto 2100
         else if(ichr.eq.ichar('m'))then
            if(memmax.lt.0)call fatal('Memory string is not defined')
            trnmax=trnmax-1
            do 2510 mem=1,memmax
               trnmax=trnmax+1
               if(trnmax.gt.trndim)
     &          call fatal('Substitution string too long')
               trnstr(trnmax)=memstr(mem)
               if(memstr(mem).eq.char(127) .and. .not.isspc)then
                  trnmax=trnmax+1
                  if(trnmax.gt.trndim)
     &             call fatal('Substitution string too long')
                  trnstr(trnmax)=memstr(mem)
               endif
 2510       continue
            goto 2100
         else if(ichr.eq.ichar('#'))then
            if(isspc)
     &       call fatal('Cannot use substring in translation special')
            subs=.true.
            trnstr(trnmax)=char(127)
            goto 2100
         else if(
     &       ichr.eq.asc_pl .or. ichr.eq.asc_mn .or. ichr.eq.asc_i .or.
     &       ichr.eq.asc_n .or. ichr.eq.asc_o .or. ichr.eq.asc_x
     &       )then
            trnstr(trnmax)=char(127)
            trnmax=trnmax+1
            if(trnmax.gt.trndim)
     &       call fatal('translation string too long')
            trnstr(trnmax)=char(ichr)
            goto 2100
         else if(index(trncnv,chr).gt.0)then
            if(chr.eq.'`' .and. trnmax.ne.1)
     &       call fatal('Command "\\`" must be alone')
            trnstr(trnmax)=char(127)
            trnmax=trnmax+1
            if(trnmax.gt.trndim)
     &       call fatal('Substitution string too long')
            trnstr(trnmax)=char(index(trncnv,chr))
            goto 2100
         else if(index(trnspc//trnspn,chr).gt.0)then
            if(trnmax.ne.1)
     &       call fatal('Misplaced substitution special: '//chr)
            isspc=.true.
         else if(index(scms//'-=?~&*(){}',chr).gt.0)then
            call fatal(
     &       'Command "\\'//chr//'" cannot be used in translations')
         else
            call fatal('Invalid command "\\'//chr//'"')
         endif
      else
c        check for a new backslash sequence
         if(ichr.eq.asc_bs)then
            bsl=.true.
            trnmax=trnmax-1
            goto 2100
         endif
      endif

c     add the character
      if(bit8)then
         bit8=.false.
         ichr=ichr+128
         if(ichr.gt.255)call fatal('Illegal 8 bit character')
      endif
      trnstr(trnmax)=char(ichr)

c     if the previous command was an substring introducer, check the number
      if(subs)then
         subs=.false.
         if(ichr.lt.asc_0 .or. ichr.gt.asc_9)
     &    call fatal('Substring number out of range')
         if(ichr.gt.asc_0+sbs)
     &    call fatal('Substring number exceeds substrings')
      endif

c     double a delete character
      if(ichr.eq.127 .and. .not.isspc)then
         trnmax=trnmax+1
         if(trnmax.gt.trndim)call fatal('Substitution string too long')
         trnstr(trnmax)=char(127)
      endif

c     go read the next character
      goto 2100

c     append \#0 to a conversion command that occurs at the end of a line
 2900 if(trnmax.ge.2)then
         if(trnstr(1).eq.char(127) .and. trnstr(2).eq.'`')then
            if(trnmax.ne.2)call fatal('Command "\\`" must be alone')
            if(sbs.ne.0)
     &       call fatal('Command "\\`" does not take substrings')
         endif
         if(trnstr(trnmax-1).eq.char(127))then
            ichr=ichar(trnstr(trnmax))
            if(ichr.ge.1 .and. ichr.le.len(trncnv))then
               trnmax=trnmax+2
               if(trnmax.gt.trndim)
     &          call fatal('Substitution string too long')
               trnstr(trnmax-1)=char(127)
               trnstr(trnmax)='0'
            endif
         endif
      endif

c     check specials
      if(isspc)then
         chr=trnstr(1)
         if(trnmax.ge.123456789)call fatal(
     &    'Too many characters behind substitution special '//chr)
         if(index(trnspc,chr).gt.0)then
            if(chr.eq.'e')then
               if(trnmax.lt.3)call fatal('Substitution special e '//
     &          'must be followed by at least 2 characters')
            else if(chr.eq.'r')then
               if(trnmax.lt.2)call fatal('Substitution special r '//
     &          'must be followed by at least one character')
            else if(trnmax.ne.1)then
               call fatal(
     &          'Substitution special '//chr//' must be alone')
            endif
            trnmax=-trnmax
         else if(index(trnspn,chr).gt.0)then
            if(sbs.gt.1)call fatal(
     &       'Only one substring allowed for command \\'//chr)
            value=0
            do 2950 trn=2,trnmax
               if(index('.,!?',chr).ne.0 .and. trnstr(trn).eq.':')then
                  do 2945 trn2=trn+1,trnmax
                     if(trnstr(trn2).eq.char(127) .or.
     &                trnstr(trn2).eq.char(0))call fatal(
     &                'No NUL, DEL or commands allowed in a \\'//
     &                chr//' string')
                     trnstr(trn2-trn+1)=trnstr(trn2)
 2945             continue
                  trnstr(trnmax+1-trn+1)=char(0)
                  goto 2951
               endif
               if(trnstr(trn).lt.'0' .or. trnstr(trn).gt.'9')call fatal(
     &          'Invalid number behind substitution special '//chr)
               value=value*10-48+ichar(trnstr(trn))
               if(value.ge.12345679 .or. value.lt.0)call fatal(
     &          'Substitution special '//chr//' has too large a count')
 2950       continue
            if(index('.,!?',chr).ne.0)trnstr(2)=char(0)
 2951       trnmax=-value
            if(trnmax.eq.0)trnmax=-123456789
         else
            call fatal('Program error: unknown special slipped through')
         endif
      endif

 9000 txtfll=txtfl0
      call outsub('getsub')
      return
      end
