      subroutine subbn0(outfil)

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     The actual, subroutine implementation, of subbin.

c     Parameter bufdim below sets the maximum file size, in bytes, that
c     can be processed.  Note that the memory requirement of this
c     program is about twice that amount.  For startup speed, and to
c     reduce swapping, do not make bufdim larger than needed.

c     Parameter orgdim sets the maximum size of original and translation
c     strings.  Reasonably speaking, this can be small compared to the
c     maximum file size.

c     Note that there is additional storage needed for hyphenation.
c     This is set in subroutine hyphen.  Fortunately, the amount needed
c     there is probably small.

c     The following I/O units are used
c     1: substitutions file
c     2: input text file
c     3: output text file
c     4: subbinss.sub
c     7: used by subroutine hyphen to read in the hyphenation tables,
c        and by dosubs to read in the byte conversion table.

c     avoid typos
      implicit none

c input

c     output file
      character*(*) outfil

c common

c     declared substring selection levels
      integer levdim
      parameter (levdim=1000)
c     actual substring selection level
      integer lev
c     available bufdim at each level
      integer bufdml(0:levdim)
c     start of the post-selection string
      integer post(levdim)
c     line in the substitutions file at which the level was selected
      integer sellin(0:levdim)

c     after a successfull \f#, the #-th occurrence is from fnddat(2) to (3)
      integer fnddat(3)

c     total substitutions
      integer subtot
c     total found strings
      integer fndtot
c     total missed substitutions
      integer subnot
c     total missed searches
      integer fndnot
c     previous value of subtot at \x command
      integer oldsct

c     dopair block
      common/cmpair/lev,bufdml,post,sellin,
     & fnddat,
     & subtot,fndtot,subnot,fndnot,oldsct

c     verbosity
      integer verbos
c     substitions file
      character*132 subfil
      integer subfll
c     line count of the substitutions file
      integer lincts
c     text file
      character*132 txtfil
      integer txtfll
c     line count of the text file
      integer linctt

c     debug block
      common/cmdbg/verbos,subfil,subfll,lincts,txtfil,txtfll,linctt

c local

c     declared primary and secondary file buffer sizes
      integer bufdim
      parameter (bufdim=10000000)
c     actual file buffer sizes
      integer bufmax,bufmx2
c     initial buffer size
      integer bufmx0
c     file buffers
      character*1 bufstr(bufdim),bufst2(bufdim)

c     declared original string size
      integer orgdim
      parameter (orgdim=100000)
c     actual original string size
      integer orgmax
c     original string
      character*1 orgstr(orgdim)
c     modifiers
      character*1 orgmod(orgdim)

c     declared translation string size
      integer trndim
      parameter (trndim=orgdim)
c     actual translation string size
      integer trnmax
c     translation string
      character*1 trnstr(trndim)

c     declared memory string size
      integer memdim
      parameter (memdim=orgdim)
c     actual memory size
      integer memmax
c     memory string
      character*1 memstr(memdim)
      save memstr,memmax

c     counter of matches found for this pair
      integer fndct
c     counter of nontrivial substitutions done for this pair
      integer subct

c     whether we use buffer 2
      integer use2

c     whether to write te new file
      integer dowrt

c executable

c     note where we are
      call insub('subbn0')

c     initialize
      call inibuf

c     read the file to change into the first storage buffer
      call readfu(txtfil,txtfll,2,bufdim,bufmax,bufstr)
      linctt=1
      bufmx0=bufmax

c     initialize the storage buffer to use
      use2=0

c     open the orginal/translation table file
      goto 900
 880  close(1,err=890)
      goto 900
 890  call fatal('Unable to close the substitution file?!')
 900  call openru(subfil(1:subfll),1)
      lincts=1

c     start point for the next string translation
 1000 continue

c     do the translation
      call dopair(
     & 0,
     & orgdim,orgmax,orgstr,orgmod,trndim,trnmax,trnstr,
     & memdim,memmax,memstr,
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,
     & fndct,subct,
     & *9000,*880)

c     next translation
      goto 1000

c     write the new file
 9000 dowrt=0
      if(subtot.gt.0)dowrt=1
      if(use2.eq.0)then
         if(bufmax.ne.bufmx0)dowrt=1
         if(bufmax.eq.0)dowrt=0
         if(dowrt.ne.0)
     &    call writfu(outfil,len(outfil),3,bufdim,bufmax,bufstr)
      else
         if(bufmx2.ne.bufmx0)dowrt=1
         if(bufmx2.eq.0)dowrt=0
         if(dowrt.ne.0)
     &    call writfu(outfil,len(outfil),3,bufdim,bufmx2,bufst2)
      endif

c     print statistics
      if(verbos.ge.1)then
         print9110,fndtot,subtot,fndnot,subnot
 9110    format(
     &    'subbin: Total strings found:  ',i10/
     &    '        Total strings changed:',i10/
     &    '        String find failures: ',i10/
     &    '        Unchanged strings:    ',i10)
      endif

c     all done
      call outsub('subbn0')
      return
      end
