c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

      function utf8lt(chrno)

c     Length of the UTF-8 string of unicode character chrno.

c     Returns -1 if chrno is not a valid unicode character number.

      implicit none

c input

c     character number
      integer chrno

c output

c     length of the UTF-8 string
      integer utf8lt

c local

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c executable

      utf8lt=-1
      if(chrno.lt.0 .or. chrno.gt.uclim)goto 990
      utf8lt=1
      if(chrno.ge.128)utf8lt=2
      if(chrno.ge.2048)utf8lt=3
      if(chrno.ge.65536)utf8lt=4

 990  return
      end



      function utf16l(chrno)

c     Length of the UTF-16 string of a character number.

      implicit none

c input

c     character number
      integer chrno

c output

c     length of the UTF-16 string
      integer utf16l

c local

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c executable

      utf16l=-1
      if(chrno.lt.0 .or. chrno.gt.uclim)goto 990
      if(chrno.ge.55296 .and. chrno.lt.57344)goto 990
      utf16l=2
      if(chrno.ge.65536)utf16l=4

 990  return
      end



      function intllt(chrno)

c     Complete length of the internal string of unicode character chrno.

c     Returns -1 if chrno is not a valid unicode character number.

      implicit none

c input

c     character number
      integer chrno

c output

c     length of the UTF-8 string
      integer intllt

c local

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c executable

      intllt=-1
      if(chrno.lt.0 .or. chrno.gt.uclim)goto 990
      intllt=1
      if(chrno.ge.26)intllt=2
      if(chrno.ge.26**2)intllt=3
      if(chrno.ge.26**3)intllt=4
      if(chrno.ge.26**4)intllt=5
      intllt=intllt+2

 990  return
      end



      subroutine wutf8(chrno,strdim,strmax,strchr)

c     Write a utf 8 character to string strchr.

      implicit none

c input

c     character number to write
      integer chrno

c     declared dimension of the string
      integer strdim

c output

c     number of characters in string
      integer strmax

c     string
      character*1 strchr(strdim)

c local

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c     local value of chrno
      integer ichr
c     subparts of ichr
      integer i1,i2,i3

c     temporary
      integer itemp

c executable

c     note where we are
      call insub('wutf8')

c     copy the number to local variable
      ichr=chrno

c     check validity
      if(ichr.lt.0 .or. ichr.gt.uclim)
     & call fatal('UTF-8 number to write is invalid')

c     standard ASCII
      if(ichr.le.127)then
         strmax=1
         if(strmax.gt.strdim)
     &    call fatal('Out of storage writing a UTF-8 number')
         strchr(1)=char(ichr)
         goto 990
      endif

c     put the last 6 bits in i1 and remove them from ichr
      itemp=ichr/64
      i1=ichr-itemp*64
      ichr=itemp
c     a two-byte string can hold 5 more bits, i.e. up to 31
      if(ichr.le.31)then
         strmax=2
         if(strmax.gt.strdim)
     &    call fatal('Out of storage writing a UTF-8 number')
         strchr(1)=char(ichr+192)
         strchr(2)=char(i1+128)
         goto 990
      endif

c     put the next 6 bits in i2 and remove them from ichr
      itemp=ichr/64
      i2=ichr-itemp*64
      ichr=itemp
c     a three-byte string can hold 4 more bits, i.e. up to 15
      if(ichr.le.15)then
         strmax=3
         if(strmax.gt.strdim)
     &    call fatal('Out of storage writing a UTF-8 number')
         strchr(1)=char(ichr+224)
         strchr(2)=char(i2+128)
         strchr(3)=char(i1+128)
         goto 990
      endif

c     put the next 6 bits in i3 and remove them from ichr
      itemp=ichr/64
      i3=ichr-itemp*64
      ichr=itemp
c     a four-byte string can hold 3 more bits, i.e. up to 7
      if(ichr.gt.7)call fatal('UTF-8 number to write is invalid')
      strmax=4
      if(strmax.gt.strdim)
     & call fatal('Out of storage writing a UTF-8 number')
      strchr(1)=char(ichr+240)
      strchr(2)=char(i3+128)
      strchr(3)=char(i2+128)
      strchr(4)=char(i1+128)

 990  call outsub('wutf8')
      return
      end



      subroutine rutf8(strlim,strchr,strmax,chrno)

c     Read a utf 8 character from string strchr.

c     Returns -1 if strmax is zero and the string is invalid as a
c     unicode character number.

      implicit none

c input

c     defined length of the string
      integer strlim

c     string
      character*1 strchr(*)

c input/output

c     length of the string (initialize to zero for it to be computed)
      integer strmax

c output

c     character number
      integer chrno

c local

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c     number of bytes in the utf-8 string
      integer bytes
c     byte index
      integer b

c     ascii value
      integer iasc

c executable

c     note where we are
      call insub('rutf8')

c     check storage
      if(strmax.lt.0 .or. strlim.lt.0)
     & call fatal('Invalid string length')
      if(strmax.gt.strlim)call fatal('String too long')

c     examine the first character
      if(strlim.eq.0)goto 900
      chrno=ichar(strchr(1))

c     ASCII is unchanged in UTF-8
      if(chrno.le.127)then
         if(strmax.eq.0)then
            strmax=1
         else if(strmax.ne.1)then
            call fatal('Wrong UTF-8 length for ASCII')
         endif
         goto 990
      endif

c     remove the two leading bits
      if(chrno.lt.192)then
         if(strmax.eq.0)goto 900
         call fatal('Invalid UTF-8 character')
      endif
      chrno=chrno-192

c     find out the additional bytes from the header and remove the header
      if(chrno.lt.32)then
         bytes=1
      else
         chrno=chrno-32
         if(chrno.lt.16)then
            bytes=2
         else
            chrno=chrno-16
            bytes=3
            if(chrno.gt.7)then
               if(strmax.eq.0)goto 900
               call fatal('Invalid UTF-8 character')
            endif
         endif
      endif

c     check against the defined length
      if(strmax.eq.0)then
         if(1+bytes.gt.strlim)goto 900
      else if(strmax.ne.1+bytes)then
         call fatal('Invalid UTF-8 character string length')
      endif

c     read the remaining bytes
      do 100 b=1,bytes
         iasc=ichar(strchr(1+b))-128
         if(iasc.lt.0 .or.iasc.gt.63)then
            if(strmax.eq.0)goto 900
            call fatal('Invalid UTF-8 character read')
         endif
         chrno=chrno*64+iasc
 100  continue
      if(chrno.lt.0 .or. chrno.gt.uclim)then
         if(strmax.eq.0)goto 900
         call fatal('Invalid Unicode character read')
      endif

c     set the length
      if(strmax.eq.0)strmax=1+bytes

c     done
      goto 990

c     invalid UTF-8 string
 900  chrno=-1
 990  call outsub('rutf8')
      return
      end



      subroutine rwhtml(wnum,whex,num,strdim,strmax,strchr)

c     Read or write a unicode character in html format.
c
c     Read mode input:
c        num=-1
c        strmax: the string length
c     Read mode output:
c        num=chrno, or -1 if invalid
c
c     Write mode input:
c        num nonnegative
c     Write mode output:
c        strmax: the length of the string or 0 if num is invalid or -1 if
c                the character string does not fit
c        Whether names are used if they exist, and whether numbers are
c        written in hex instead of decimal depends on wnum and whex.

      implicit none

c input

c     whether to write numbers instead of names (0: no, 1: yes, 2: if shorter)
      integer wnum
c     whether to write hex (nonzero for yes)
      integer whex

c input/output

c     unicode character or -1
      integer num

c     declared number of characters in the string
      integer strdim

c     actual number of characters in the string
      integer strmax

c     the string characters
      character*1 strchr(strdim)

c local

c     number of character entities
      integer cedim
      parameter (cedim=253)
c     character entity index
      integer ce
c     binary search values
      integer celo,cehi
c     character entity names
      character*8 cenam(cedim)
      save cenam
      data cenam/
     &'quot    ','amp     ','apos    ','lt      ','gt      ','nbsp    ',
     &'iexcl   ','cent    ','pound   ','curren  ','yen     ','brvbar  ',
     &'sect    ','uml     ','copy    ','ordf    ','laquo   ','not     ',
     &'shy     ','reg     ','macr    ','deg     ','plusmn  ','sup2    ',
     &'sup3    ','acute   ','micro   ','para    ','middot  ','cedil   ',
     &'sup1    ','ordm    ','raquo   ','frac14  ','frac12  ','frac34  ',
     &'iquest  ','Agrave  ','Aacute  ','Acirc   ','Atilde  ','Auml    ',
     &'Aring   ','AElig   ','Ccedil  ','Egrave  ','Eacute  ','Ecirc   ',
     &'Euml    ','Igrave  ','Iacute  ','Icirc   ','Iuml    ','ETH     ',
     &'Ntilde  ','Ograve  ','Oacute  ','Ocirc   ','Otilde  ','Ouml    ',
     &'times   ','Oslash  ','Ugrave  ','Uacute  ','Ucirc   ','Uuml    ',
     &'Yacute  ','THORN   ','szlig   ','agrave  ','aacute  ','acirc   ',
     &'atilde  ','auml    ','aring   ','aelig   ','ccedil  ','egrave  ',
     &'eacute  ','ecirc   ','euml    ','igrave  ','iacute  ','icirc   ',
     &'iuml    ','eth     ','ntilde  ','ograve  ','oacute  ','ocirc   ',
     &'otilde  ','ouml    ','divide  ','oslash  ','ugrave  ','uacute  ',
     &'ucirc   ','uuml    ','yacute  ','thorn   ','yuml    ','OElig   ',
     &'oelig   ','Scaron  ','scaron  ','Yuml    ','fnof    ','circ    ',
     &'tilde   ','Alpha   ','Beta    ','Gamma   ','Delta   ','Epsilon ',
     &'Zeta    ','Eta     ','Theta   ','Iota    ','Kappa   ','Lambda  ',
     &'Mu      ','Nu      ','Xi      ','Omicron ','Pi      ','Rho     ',
     &'Sigma   ','Tau     ','Upsilon ','Phi     ','Chi     ','Psi     ',
     &'Omega   ','alpha   ','beta    ','gamma   ','delta   ','epsilon ',
     &'zeta    ','eta     ','theta   ','iota    ','kappa   ','lambda  ',
     &'mu      ','nu      ','xi      ','omicron ','pi      ','rho     ',
     &'sigmaf  ','sigma   ','tau     ','upsilon ','phi     ','chi     ',
     &'psi     ','omega   ','thetasym','upsih   ','piv     ','ensp    ',
     &'emsp    ','thinsp  ','zwnj    ','zwj     ','lrm     ','rlm     ',
     &'ndash   ','mdash   ','lsquo   ','rsquo   ','sbquo   ','ldquo   ',
     &'rdquo   ','bdquo   ','dagger  ','Dagger  ','bull    ','hellip  ',
     &'permil  ','prime   ','Prime   ','lsaquo  ','rsaquo  ','oline   ',
     &'frasl   ','euro    ','image   ','weierp  ','real    ','trade   ',
     &'alefsym ','larr    ','uarr    ','rarr    ','darr    ','harr    ',
     &'crarr   ','lArr    ','uArr    ','rArr    ','dArr    ','hArr    ',
     &'forall  ','part    ','exist   ','empty   ','nabla   ','isin    ',
     &'notin   ','ni      ','prod    ','sum     ','minus   ','lowast  ',
     &'radic   ','prop    ','infin   ','ang     ','and     ','or      ',
     &'cap     ','cup     ','int     ','there4  ','sim     ','cong    ',
     &'asymp   ','ne      ','equiv   ','le      ','ge      ','sub     ',
     &'sup     ','nsub    ','sube    ','supe    ','oplus   ','otimes  ',
     &'perp    ','sdot    ','lceil   ','rceil   ','lfloor  ','rfloor  ',
     &'lang    ','rang    ','loz     ','spades  ','clubs   ','hearts  ',
     &'diams   '/
c     character entity numbers
      integer cenum(cedim)
      save cenum
      data cenum/
     &  34,  38,  39,  60,  62, 160, 161, 162, 163, 164, 165, 166,
     & 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178,
     & 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190,
     & 191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202,
     & 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214,
     & 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226,
     & 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238,
     & 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250,
     & 251, 252, 253, 254, 255, 338, 339, 352, 353, 376, 402, 710,
     & 732, 913, 914, 915, 916, 917, 918, 919, 920, 921, 922, 923,
     & 924, 925, 926, 927, 928, 929, 931, 932, 933, 934, 935, 936,
     & 937, 945, 946, 947, 948, 949, 950, 951, 952, 953, 954, 955,
     & 956, 957, 958, 959, 960, 961, 962, 963, 964, 965, 966, 967,
     & 968, 969, 977, 978, 982,8194,8195,8201,8204,8205,8206,8207,
     &8211,8212,8216,8217,8218,8220,8221,8222,8224,8225,8226,8230,
     &8240,8242,8243,8249,8250,8254,8260,8364,8465,8472,8476,8482,
     &8501,8592,8593,8594,8595,8596,8629,8656,8657,8658,8659,8660,
     &8704,8706,8707,8709,8711,8712,8713,8715,8719,8721,8722,8727,
     &8730,8733,8734,8736,8743,8744,8745,8746,8747,8756,8764,8773,
     &8776,8800,8801,8804,8805,8834,8835,8836,8838,8839,8853,8855,
     &8869,8901,8968,8969,8970,8971,9001,9002,9674,9824,9827,9829,
     &9830/

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c     buffer of last-used character entities
      integer bufdim
      parameter (bufdim=7)
      integer buf
      integer bufce(bufdim)
      save bufce
      data bufce/1,2,3,4,5,6,19/

c     name to find
      character*8 nam

c     number as string
      character*1 nschr(8)
      integer ns,nsmax

c     character index
      integer str

c executable

c     read mode

c     jump if the number is given
      if(num.ge.0)goto 500

c     assume invalid until proven valid
      num=-1

c     no strings less than 3 or more than 10 characters long
      if(strmax.lt.3 .or. strmax.gt.10)goto 990
c     must start with & and end with ;
      if(strchr(1).ne.'&' .or. strchr(strmax).ne.';')goto 990

c     handle numeric strings
      if(strchr(2).eq.'#')then
         if(strmax.lt.4)goto 990
         if(strchr(3).eq.'x' .or. strchr(3).eq.'X')then
            if(strmax.lt.5)goto 990
            call rbaseb(16,48,strmax-4,strchr(4),num)
            if(num.gt.uclim)num=-1
         else
            call rbaseb(10,48,strmax-3,strchr(3),num)
            if(num.gt.uclim)num=-1
         endif
         goto 990
      endif

c     copy the name to local
      nam=' '
      do 10 str=1,strmax-2
         nam(str:str)=strchr(str+1)
 10   continue

c     try the last used names first
      do 100 buf=1,bufdim
         if(cenam(bufce(buf)).eq.nam)then
            num=cenum(bufce(buf))
            goto 990
         endif
 100  continue

c     search through the names
      do 200 ce=1,cedim
         if(cenam(ce).eq.nam)then
            num=cenum(ce)
            do 150 buf=bufdim,2,-1
               bufce(buf)=bufce(buf-1)
 150        continue
            bufce(1)=ce
            goto 990
         endif
 200  continue

c     failed
      goto 990

c     case that the number is given

c     entry point
 500  continue

c     check validity
      if(num.lt.0 .or. num.gt.uclim)then
         strmax=0
         goto 990
      endif
      if(strdim.lt.3)then
         strmax=-1
         goto 990
      endif

c     write the number to string
      if(whex.eq.0)then
         call wbaseb(10,48,num,8,nsmax,nschr)
      else
         nschr(1)='x'
         call wbaseb(16,48,num,7,nsmax,nschr(2))
         nsmax=nsmax+1
      endif

c     use if wnum is 1
      if(wnum.eq.1)goto 800

c     initialize lower search limit
      ce=1
      if(num.lt.cenum(ce))goto 800
      if(num.eq.cenum(ce))goto 900
      celo=ce

c     initialize upper search limit
      ce=cedim
      if(num.gt.cenum(ce))goto 800
      if(num.eq.cenum(ce))goto 900
      cehi=ce

c     do a binary search
 700  ce=celo+(cehi-celo)/2
      if(ce.eq.celo)goto 800
      if(num.gt.cenum(ce))then
         celo=ce
         goto 700
      else if(num.lt.cenum(ce))then
         cehi=ce
         goto 700
      else
         goto 900
      endif

c     did not find the number as string
 800  strmax=nsmax+3
      if(strmax.gt.strdim)then
         strmax=-1
      else
         strchr(1)='&'
         strchr(2)='#'
         do 810 ns=1,nsmax
            strchr(ns+2)=nschr(ns)
 810     continue
         strchr(strmax)=';'
      endif
      goto 990

c     found the number as string ce
 900  strchr(1)='&'
      do 910 str=1,8
         if(cenam(ce)(str:str).le.' ')goto 920
         if(wnum.eq.2 .and. str.gt.nsmax+1)goto 800
         if(str+2.gt.strdim)then
            strmax=-1
            goto 990
         endif
         strchr(str+1)=cenam(ce)(str:str)
 910  continue
      str=9
 920  strmax=str+1
      strchr(strmax)=';'

 990  return
      end



      subroutine wbaseb(b,ichr0,number,strdim,strmax,strchr)

c     Write a nonnegative number to a character string in base b.  The
c     "digits" are the characters starting from ASCII code ichr0.
c     Exception: if ichr0=48 and b.le.36, conventional hex-style
c     digits will be used.


      implicit none

c input

c     base
      integer b
c     character to use for 0 as an ASCII number
      integer ichr0

c     number to write
      integer number

c     declared dimension of strchr
      integer strdim

c output

c     number of characters in strchr
      integer strmax

c     the number as a base b string
      character*1 strchr(strdim)

c local

c     local value of number
      integer num

c     digits
      integer digval(128),dig

c     temporary
      integer itemp

c     string index
      integer str

c executable

c     note where we are
      call insub('wbaseb')

c     copy to local
      num=number

c     check
      if(b.lt.2)call fatal('Invalid number base')
      if(ichr0.lt.0 .or. ichr0+b-1.gt.255)
     & call fatal('Invalid digit characters')
      if(num.lt.0)call fatal('Negative number')

c     find the digits
      dig=0
 100  itemp=num/b
      dig=dig+1
      digval(dig)=num-itemp*b
      num=itemp
      if(num.gt.0 .and. dig.lt.128)goto 100
      if(num.gt.0)
     & call fatal('Program error: number too large')
      strmax=dig

c     use hex-style where appropriate
      if(b.gt.10 .and. b.le.36 .and. ichr0.eq.48)then
         do 200 dig=1,strmax
            if(digval(dig).gt.9)digval(dig)=digval(dig)+7
 200     continue
      endif

c     output to string
      if(strmax.gt.strdim)call fatal('String is too short')
      str=0
      do 300 dig=strmax,1,-1
         str=str+1
         strchr(str)=char(digval(dig)+ichr0)
 300  continue

      call outsub('wbaseb')
      return
      end



      subroutine rbaseb(b,ichr0,strmax,strchr,number)

c     Read a base b number as produced by wbaseb.  (Lower case is
c     allowed in hex-style numbers.)  Returns -1 if the number is
c     invalid or overflows.

      implicit none

c input

c     base
      integer b
c     character to use for 0 as an ASCII number
      integer ichr0

c     number of characters in strchr
      integer strmax

c     the number as a base b string
      character*1 strchr(*)

c output

c     the number, or -1 if the number is invalid
      integer number

c local

c     digits
      integer digval(128),dig

c     temporary
      integer itemp

c executable

c     note where we are
      call insub('rbaseb')

c     check sanity
      if(b.lt.2)call fatal('Invalid number base')
      if(ichr0.lt.0 .or. ichr0+b-1.gt.255)
     & call fatal('Invalid digit characters')
      if(strmax.le.0)goto 900

c     find the digits
      if(b.gt.10 .and. b.le.36 .and. ichr0.eq.48)then
         do 100 dig=1,strmax
            itemp=ichar(strchr(dig))
            if(itemp.ge.ichar('a') .and. itemp.le.ichar('z'))
     &       itemp=itemp-32
            itemp=itemp-ichr0
            if(itemp.gt.9 .and. itemp.lt.17)goto 900
            if(itemp.gt.9)itemp=itemp-7
            if(itemp.lt.0 .or. itemp.gt.b-1)goto 900
            digval(dig)=itemp
 100     continue
      else
         do 200 dig=1,strmax
            itemp=ichar(strchr(dig))-ichr0
            if(itemp.lt.0 .or. itemp.gt.b-1)goto 900
            digval(dig)=itemp
 200     continue
      endif

c     form the number
      number=0
      do 500 dig=1,strmax
         itemp=(2147483647-digval(dig))/b
         if(number.gt.itemp)goto 900
         number=number*b+digval(dig)
 500  continue

c     done
      goto 990

c     invalid number
 900  number=-1
 990  call outsub('rbaseb')
      return
      end



      subroutine rutf16(le,strlim,strchr,strmax,chrno)

c     Read a utf 16 character from string strchr.

c     Returns -1 if strmax is zero and the string is invalid as a
c     unicode character number.

      implicit none

c input

c     0 for big endian byte order of the 16 bit ints, nonzero for little endian
      integer le

c     defined length of the string
      integer strlim

c     string
      character*1 strchr(*)

c input/output

c     length of the string (initialize to zero for it to be computed)
      integer strmax

c output

c     character number
      integer chrno

c local

c     the two 16 bit integers
      integer int1,int2

c executable

c     note where we are
      call insub('rutf16')

c     check storage
      if(strmax.lt.0 .or. strlim.lt.0)
     & call fatal('Invalid UTF-16 string size')
      if(strmax.gt.strlim)call fatal('UTF-16 string too long')

c     check the string size
      if(strlim.lt.2)then
         if(strmax.eq.0)goto 900
         call fatal('String is too short for a utf-16 number')
      endif

c     read the first 16 bit integer
      if(le.eq.0)then
         int1=ichar(strchr(1))*256+ichar(strchr(2))
      else
         int1=ichar(strchr(2))*256+ichar(strchr(1))
      endif

c     done if there is no second 16 bit integer
      if(int1.lt.55296 .or. int1.ge.57344)then
         if(strmax.eq.0)then
            strmax=2
         else
            if(strmax.ne.2)call fatal('Wrong UTF-16 string length')
         endif
         chrno=int1
         goto 990
      endif

c     check the string size
      if(strlim.lt.4)then
         if(strmax.eq.0)goto 900
         call fatal('String is too short for a 4 byte utf-16 number')
      endif

c     check that this is indeed a first int
      if(int1.ge.56320)then
         if(strmax.eq.0)goto 900
         call fatal('UTF-16 string starts with a second int')
      endif

c     read the second int
      if(le.eq.0)then
         int2=ichar(strchr(3))*256+ichar(strchr(4))
      else
         int2=ichar(strchr(4))*256+ichar(strchr(3))
      endif

c     check that this is indeed a second int
      if(int2.lt.56320 .or. int2.ge.57344)then
         if(strmax.eq.0)goto 900
         call fatal('UTF-16 string ends with a first int')
      endif
      if(strmax.eq.0)then
         strmax=4
      else
         if(strmax.ne.4)call fatal('Wrong UTF-16 string length')
      endif

c     find the number
      chrno=(int1-55296)*1024+(int2-56320)+65536
      goto 990

c     invalid UTF-16 string
 900  chrno=-1
 990  call outsub('rutf16')
      return
      end



      subroutine wutf16(le,chrno,strdim,strmax,strchr)

c     Write a utf 16 character.

      implicit none

c input

c     0 for big endian byte order of the 16 bit ints, nonzero for little endian
      integer le

c     character number
      integer chrno

c     declared length of the string
      integer strdim

c output

c     length of the string
      integer strmax

c     string
      character*1 strchr(strdim)

c local

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c     the two 16 bit integers
      integer int1,int2

c     the two 16 bit bytes
      integer byt1,byt2

c     local value of chrno
      integer num

c executable

c     note where we are
      call insub('wutf16')

c     convert number to local
      num=chrno

c     check sanity of number
      if(num.lt.0 .or. num.gt.uclim)call fatal(
     & 'Unicode character number is invalid')
      if(num.ge.55296 .and. num.lt.57344)call fatal(
     & 'Unicode character number is invalid (reserved)')

c     jump for 4-byte case
      if(num.ge.65536)goto 500

c     check storage
      if(strdim.lt.2)call fatal('Out of storage')

c     find 2-byte integers
      byt1=num/256
      byt2=num-byt1*256

c     put them in
      if(le.eq.0)then
         strchr(1)=char(byt1)
         strchr(2)=char(byt2)
      else
         strchr(1)=char(byt2)
         strchr(2)=char(byt1)
      endif
      strmax=2

c     done
      goto 990

c     check storage
 500  if(strdim.lt.4)call fatal('Out of storage')

c     subtract the 2-byte range
      num=num-65536

c     take apart into two
      int1=num/1024
      int2=num-int1*1024

c     put int1 into two bytes with offset
      int1=int1+55296
      byt1=int1/256
      byt2=int1-byt1*256
      if(le.eq.0)then
         strchr(1)=char(byt1)
         strchr(2)=char(byt2)
      else
         strchr(1)=char(byt2)
         strchr(2)=char(byt1)
      endif

c     put int2 into two bytes with offset
      int2=int2+56320
      byt1=int2/256
      byt2=int2-byt1*256
      if(le.eq.0)then
         strchr(3)=char(byt1)
         strchr(4)=char(byt2)
      else
         strchr(3)=char(byt2)
         strchr(4)=char(byt1)
      endif
      strmax=4

c     jump back
 990  call outsub('wutf16')
      return
      end



      subroutine rrtf(strlim,strchr,strmax,chrno)

c     Read an RTF character from string strchr.  There must be 1
c     nondigit character terminating each \u[number] (sub)string.

c     Returns -1 if strmax is zero and the string is invalid as a
c     unicode character number.

      implicit none

c input

c     defined length of the string
      integer strlim

c     string
      character*1 strchr(*)

c input/output

c     length of the string (initialize to zero for it to be computed)
      integer strmax

c output

c     character number
      integer chrno

c local

c     end of the string
      integer strend
c     string indices
      integer str,str2

c     the two 16 bit integers
      integer int1,int2

c     sign of the integers
      integer sgn

c     digit
      integer i

c executable

c     note where we are
      call insub('rrtf')

c     check storage
      if(strmax.lt.0 .or. strlim.lt.0)
     & call fatal('Invalid RTF string size')
      if(strmax.gt.strlim)call fatal('RTF string too long')

c     end of the readable string
      strend=strlim
      if(strmax.gt.0)strend=strmax

c     initialize position
      str=0

c     read the first 16 bit integer
      str=str+1
      if(str.gt.strend)goto 810
      if(strchr(str).ne.'\\')goto 820
      str=str+1
      if(str.gt.strend)goto 810
      if(strchr(str).ne.'u')goto 820
      str=str+1
      if(str.gt.strend)goto 810
      sgn=1
      if(strchr(str).eq.'-')then
         sgn=-1
         str=str+1
         if(str.gt.strend)goto 810
      endif
      do 100 str2=str,strend
         i=ichar(strchr(str2))-48
         if(i.lt.0 .or. i.gt.9)goto 110
 100  continue
      goto 810
 110  str2=str2-1
      if(str2.lt.str)goto 820
      call rbaseb(10,48,str2-str+1,strchr(str),int1)
      if(int1.lt.0)goto 820
      if(sgn.eq.1)then
c        this should really be 32768
         if(int1.ge.65536)goto 820
      else
         int1=65536-int1
c        0 should really be 32768
         if(int1.lt.0 .or. int1.ge.65536)goto 820
      endif
      str=str2+1

c     done if there is no second 16 bit integer
      if(int1.lt.55296 .or. int1.ge.57344)then
         if(strmax.eq.0)then
            strmax=str
         else
            if(strmax.ne.str)call fatal('Wrong RTF string length')
         endif
         chrno=int1
         goto 990
      endif

c     check that this is indeed a first int
      if(int1.ge.56320)then
         if(strmax.eq.0)goto 900
         call fatal('RTF string starts with a second int')
      endif

c     read the second 16 bit integer
      str=str+1
      if(str.gt.strend)goto 810
      if(strchr(str).ne.'\\')goto 820
      str=str+1
      if(str.gt.strend)goto 810
      if(strchr(str).ne.'u')goto 820
      str=str+1
      if(str.gt.strend)goto 810
      sgn=1
      if(strchr(str).eq.'-')then
         sgn=-1
         str=str+1
         if(str.gt.strend)goto 810
      endif
      do 200 str2=str,strend
         i=ichar(strchr(str2))-48
         if(i.lt.0 .or. i.gt.9)goto 210
 200  continue
      goto 810
 210  str2=str2-1
      if(str2.lt.str)goto 820
      call rbaseb(10,48,str2-str+1,strchr(str),int2)
      if(int2.lt.0)goto 820
      if(sgn.eq.1)then
c        this should really be 32768
         if(int2.ge.65536)goto 820
      else
         int2=65536-int2
c        0 should really be 32768
         if(int2.lt.0 .or. int2.ge.65536)goto 820
      endif
      str=str2+1

c     check that this is indeed a second int
      if(int2.lt.56320 .or. int2.ge.57344)then
         if(strmax.eq.0)goto 900
         call fatal('RTF string ends with a first int')
      endif
      if(strmax.eq.0)then
         strmax=str
      else
         if(strmax.ne.str)call fatal('Wrong RTF string length')
      endif

c     find the number
      chrno=(int1-55296)*1024+(int2-56320)+65536
      goto 990

c     premature end
 810  if(strmax.eq.0)goto 900
      call fatal('Premature end of RTF unicode character')
c     invalid RTF string
 820  if(strmax.eq.0)goto 900
      call fatal('Invalid RTF unicode character')

c     mark as invalid
 900  chrno=-1
 990  call outsub('rrtf')
      return
      end



      subroutine wrtf(chrno,strdim,strmax,strchr)

c     Write an RTF character.  A non-unicode replacement character ?
c     will be added.

      implicit none

c input

c     character number
      integer chrno

c     declared length of the string
      integer strdim

c output

c     length of the string
      integer strmax

c     string
      character*1 strchr(strdim)

c local

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c     the two 16 bit integers
      integer int1,int2

c     local value of chrno
      integer num

c     encoded number
      character*1 enc(5)
      integer enclt

c executable

c     note where we are
      call insub('wrtf')

c     convert number to local
      num=chrno

c     check sanity of number
      if(num.lt.0 .or. num.gt.uclim)call fatal(
     & 'Unicode character number is invalid')
      if(num.ge.55296 .and. num.lt.57344)call fatal(
     & 'Unicode character number is invalid (reserved)')

c     jump for 2-int case
      if(num.ge.65536)goto 500

c     encode the number
      if(num.lt.32768)then
         call wbaseb(10,48,num,5,enclt,enc)
         strmax=3+enclt
         if(strmax.gt.strdim)call fatal('Out of storage')
         call wbaseb(10,48,num,5,enclt,strchr(3))
      else
         num=65536-num
         call wbaseb(10,48,num,5,enclt,enc)
         strmax=4+enclt
         if(strmax.gt.strdim)call fatal('Out of storage')
         call wbaseb(10,48,num,5,enclt,strchr(4))
         strchr(3)='-'
      endif
      strchr(1)='\\'
      strchr(2)='u'
      strchr(strmax)='?'

c     done
      goto 990

c     initialize strmax
 500  strmax=0

c     subtract the 2-byte range
      num=num-65536

c     take apart into two
      int1=num/1024
      int2=num-int1*1024

c     write first int
      int1=int1+55296
      int1=65536-int1
      call wbaseb(10,48,int1,5,enclt,enc)
      strmax=4+enclt
      if(strmax+5.gt.strdim)call fatal('Out of storage')
      call wbaseb(10,48,int1,5,enclt,strchr(4))
      strchr(1)='\\'
      strchr(2)='u'
      strchr(3)='-'
      strchr(strmax)='?'

c     write second int
      int2=int2+56320
      int2=65536-int2
      call wbaseb(10,48,int2,5,enclt,enc)
      if(strmax+4+enclt.gt.strdim)call fatal('Out of storage')
      call wbaseb(10,48,int2,5,enclt,strchr(strmax+4))
      strchr(strmax+1)='\\'
      strchr(strmax+2)='u'
      strchr(strmax+3)='-'
      strmax=strmax+4+enclt
      strchr(strmax)='?'

c     jump back
 990  call outsub('wrtf')
      return
      end



      subroutine rdb2uc(verbos,fil,byt2uc,bytne)

c     Read a byte to unicode conversion file.

      implicit none

c input:

c     verbosity
      integer verbos

c     file to read
      character*(*) fil

c output:

c     byte to unicode conversions
      integer byt2uc(0:255)

c     first byte number where byt.ne.uc
      integer bytne

c local:

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)

c     byte number
      integer byt

c     unicode number
      integer num

c executable:

c     generic formats
 10   format(a,a,i11)
 20   format(a,i4,a,i7)

c     note where we are
      call insub('rdb2uc')

c     initialize the mapping to the identity
      do 100 byt=0,255
         byt2uc(byt)=byt
 100  continue
      bytne=256

c     done if the file name is nil
      if(fil.eq.'nil')goto 901

c     open the file
      if(verbos.gt.0)print10,'Reading byte to UC mapping ',fil
      open(7,file=fil,status='old',err=910)

c     read the lines
 500  read(7,*,end=900,err=920)byt,num
      if(byt.lt.0 .or. byt.gt.255)goto 930
      if(num.lt.0 .or. num.gt.uclim)goto 940
      byt2uc(byt)=num
      if(verbos.gt.0 .and. byt.le.127 .and. byt.ne.num)
     & print20,'   ascii',byt,' converts to unicode',num
      if(byt.ne.num .and. byt.lt.bytne)bytne=byt
      goto 500

c     done
 900  close(7)
 901  call outsub('rdb2uc')
      return

c     error opening the file
 910  if(verbos.le.0)print10,'Reading byte to UC mapping ',fil
      call fatal('Unable to open the byte mapping file')
c     error reading the file
 920  if(verbos.le.0)print10,'Reading byte to UC mapping ',fil
      call fatal('Error reading the byte mapping file')
 930  if(verbos.le.0)print10,'Reading byte to UC mapping ',fil,byt
      call fatal('Invalid byte number in the byte mapping file')
 940  if(verbos.le.0)print10,'Reading byte to UC mapping ',fil,num
      call fatal('Invalid unicode number in the byte mapping file')
      end
