      program askfol

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     Ask for an existing DOS folder specification.

c     Usage:
c        [Show explanatory text]
c        askfol VARIABLE [default]
c        call asktmp.bat
c     This will set VARIABLE to the users input.  There will be no
c     trailing \; C:\ will be C:\. on exit.

c     If the user returns an existing directory, the errorlevel will be 0.
c     Otherwise the error level will be 1.

c     avoid typos
      implicit none

c     declared number of arguments
      integer argdim
      parameter (argdim=2)
c     actual number of arguments
      integer argmx
c     arguments
      character*80 argt(argdim)
      integer argtl(argdim)

c     input line
      character*80 line
      integer linelt
c     position of the colon in line
      integer icol
c     position in line
      integer i,i0,i1

c     whether the folder exists
      logical exists

c     buffer of saved previous inputs
      integer bufdim
      parameter (bufdim=5)
      character*80 buffer(bufdim)

c executable

c     allow I/O to use units 7 and on
      call libini(7)

c     set the error exit status to 1 in case the user aborts
      call errset(1)

c     get the command line arguments
      call args(argdim,argmx,argt,argtl)
      if(argmx.lt.1)goto 990
      if(index(argt(1)(1:argtl(1)),' ').ne.0) goto 990
      if(argt(min(argmx,2))(1:argtl(min(argmx,2))).eq.' ') goto 990

c     remove any \..
      if(argmx.ge.2)then
 110     if(argtl(2).ge.7 .and. argt(2)(2:2).eq.':')then
            i1=index(argt(2)(1:argtl(2)),'\\..')
            if(i1.lt.5)goto 190
            do 150 i=i1-2,3,-1
               i0=i
               if(argt(2)(i:i).eq.'\\')goto 160
 150        continue
            goto 190
 160        do 170 i=i1+3,argtl(2)
               argt(2)(i-i1-3+i0:i-i1-3+i0)=argt(2)(i:i)
 170        continue
            argtl(2)=argtl(2)-i1-3+i0
            goto 110
         endif
 190     continue
      endif

c     try again messages
      goto 300
 210  call out('** Path must start with the disk; eg, C:')
      call out('   Look in My Computer for example letters.')
      call out('   (Use the Esc key, or Ctrl-C, to abort.)')
      goto 300
 220  call out('** Do not double backslashes')
      goto 300
 230  call out('** Folder does not exist: '//line(1:linelt)//'\\.')
      call out('   Suggestion: browse down to the desired folder '//
     & 'in My Computer.')
      call out('   Then right-click a file inside that folder and '//
     & 'select "Properties".')
      call out('   The Location of the file is what you want here.')
      call out('   (Use the Esc key, or Ctrl-C, to abort.)')
      goto 300

c     create the prompt
 300  line='Enter folder specification:'
      icol=index(line,':')
      if(argmx.eq.2)then
         if(icol+1+argtl(2).gt.79)
     &    call out('*** askfol: default too long!')
         line(icol+2:)=argt(2)(1:argtl(2))
      endif

c     get the response by the user
      call incone(bufdim,buffer,line,icol+2,linelt)

c     get rid of trailing spaces and refuse empty inputs
      do 400 i=linelt,1,-1
         if(line(i:i).gt.' ')goto 410
         linelt=i-1
 400  continue
      goto 300
c     clear the screen line
 410  call outcon(0,0,char(13))

c     check that it starts with a disk
      if(line(1:1).ge.'a' .and. line(1:1).le.'z')
     & line(1:1)=char(ichar(line(1:1))-32)
      if(linelt.lt.2 .or. line(2:2).ne.':'
     & .or. line(1:1).lt.'A' .or. line(1:1).gt.'Z')goto 210
      if(index(line(1:linelt),'\\\\').ne.0)goto 220

c     remove any final backslash
      if(line(linelt:linelt).eq.'\\')linelt=linelt-1

c     add \. if a pure disk
      if(linelt.eq.2)then
         linelt=4
         line(3:4)='\\.'
      endif

c     check for existence
      inquire(file=line(1:linelt)//'\\.',exist=exists)
      if(.not.exists)goto 230

c     create a batch file
      call opennw(1,'asktmp.bat')
      write(1,910,err=999)
     & argt(1)(1:argtl(1)),line(1:linelt),
     & argt(1)(1:argtl(1)),line(1:linelt)
 910  format(
     & '@echo off'/
     & 'set ',a,'=',a/
     & 'if "%',a,'%" == "',a,'" goto end'/
     & 'echo exit | find "error" > nul'/
     & ':end')
      close(1,err=999)

      call exit(0)

 990  call out('Usage: askfol VARIABLE [default]')
      call out('       call asktmp.bat')
      call out('VARIABLE cannot contain spaces')
      call out('Aborted!')

 999  call out(
     & '*** Unable to write to file asktmp.bat.  Check disk space and')
      call out(
     & '    folder ownership and protections!')

      end
