      program pnmscl

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     Purpose: Substitute colors in a pnm file.
c              Essentially the same as pnmchange at this time.

c     Usage: See subroutine usage at the end of this file for usage details.

c     Programming notes:

c     If a color in the input file is not an original in the
c     substitutions list, the closest pixel is picked.  Maybe what we
c     should do instead is create traces from each color where the hue
c     is kept the same, but the saturation varies from a bit larger (eg
c     50%) to zero.  Also the lightness could be allowed to vary a bit
c     in each direction, but not going too far away from either white or
c     black.  Add an extrapolation weigting factor to that?
c
c     Suppose we create traces from each given color associated with a
c     confidence level that the substitution color is reasonable.  Then
c     select the highest confidence level?

c     avoid typos
      implicit none

c common:

c     debug option
      character*256 debug
      integer debugl
c     input file
      character*256 inf
      integer infl
c     output file
      character*256 outf
      integer outfl
c     substitutions file
      character*256 subf
      integer subfl

c     common block
      common debugl,infl,outfl,subfl,debug,inf,outf,subf

c local:

c     codes for the pnm formats recognized
      integer pbm,pgm,ppm
      parameter (pbm=4,pgm=5,ppm=6)
      integer ppbm,ppgm,pppm
      parameter (ppbm=1,ppgm=2,pppm=3)
c     actual format
      integer pid
c     desired output format
      integer pidw
      data pidw/0/

c     input picture width
      integer imax
c     index in the width direction
      integer i

c     input picture height
      integer jmax
c     index in the height direction
      integer j

c     input maximum color value
      integer cmax
c     input maximum color value as real
      real rcmax
c     output maximum color value
      integer cmaxw

c     pbm bit counter
      integer bit

c     pixel values as character indices
      integer icr,icg,icb

c     declared number of color substitutions
      integer sdim
      parameter (sdim=10000)
c     actual number of color substitutions
      integer smax
c     substitution index
      integer s
c     original rgb values from 0 to 255
      integer icro(sdim),icgo(sdim),icbo(sdim)
c     translation rgb values from 0 to 255
      integer icrt(sdim),icgt(sdim),icbt(sdim)

c     number of command arguments
      integer iargc
      external iargc
      integer argmax
c     argument index
      integer arg
c     argument string
      character*256 argstr
c     argument length
      integer argl

c     distance between colors
      integer dis
c     its minimum value and the corresponding s value
      integer dismin,smin
c     number of pixels that were not found
      integer notfnd

c     substitutions file line
      character*16 sublin
c     the digit values in it
      integer digs(13)

c     ascii values
      integer asc,asc0,ascca,asca

c     whether a file exist
      logical exists

c     error message
      character*66 errmsg

c     number as string
      character*20 cnum

c     input file character
      character*1 chr

c     character index
      integer ic

c executable:

c initialize the common block
      debugl=0
      infl=0
      outfl=0
      subfl=0

c process the command line parameters:

c     read the command line parameters and process them

c     get number of arguments
      argmax=iargc()
      if(argmax.eq.0)call usage(' ')
      if(argmax.lt.4)call usage('too few parameters specified; need 4')
      if(argmax.gt.4)call usage('too many parameters specified; need 4')

      do 100 arg=1,argmax

c        get the argument
         argstr=' '
         call getarg(arg,argstr)
c        eliminate trailing whitespace
         do 50 ic=256,1,-1
            argl=ic
            if(argstr(argl:argl).gt.' ')goto 51
 50      continue
         call usage('parameter '//char(arg+48)//' is blank')
c        convert to lowercase
 51      if(arg.ne.1)goto 61
         do 60 ic=1,argl
            if(argstr(ic:ic).ge.'A' .and. argstr(ic:ic).le.'Z')
     &       argstr(ic:ic)=char(ichar(argstr(ic:ic))+32)
 60      continue
 61      continue

c        process the parameter

         if(arg.eq.1)then

c           argument 1 is the debug option

c           set the debug option
            debugl=argl
            debug(1:debugl)=argstr(1:argl)
            call out(debug(1:debugl),'Debug option: '//debug(1:debugl))
c           check it
            if(debug(1:debugl).ne.'-d' .and. debug(1:debugl).ne.'-n')
     &       call usage('debug parameter can either be "-d" or "-n"')

         elseif(arg.eq.2)then

c           argument 2 is the substitutions file

c           set the substitutions file name
            subfl=argl
            subf(1:subfl)=argstr(1:argl)
            call out(debug(1:debugl),
     &       'Substitutions file: '//subf(1:subfl))
c           see whether it exists
            errmsg='error looking for substitutions file'
            inquire(file=subf(1:subfl),exist=exists,err=9900)
            if(.not. exists)call usage('substitutions file not found')
c           open it for reading
            errmsg='error opening the substitutions file'
            open(3,file=subf(1:subfl),status='old',err=9900)

         elseif(arg.eq.3)then

c           argument 3 is the input file

c           set the input file name
            infl=argl
            inf(1:infl)=argstr(1:argl)
            call out(debug(1:debugl),'Input file: '//inf(1:infl))
c           see whether it exists
            errmsg='error looking for input file'
            inquire(file=inf(1:infl),exist=exists,err=9900)
            if(.not. exists)call usage('input file not found')
c           open it for reading
            errmsg='error opening the input file'
            open(1,file=inf(1:infl),
     &       status='old',form='unformatted',err=9900)

         elseif(arg.eq.4)then

c           argument 4 is the output file

c           set the output file name
            outfl=argl
            outf(1:outfl)=argstr(1:argl)
            call out(debug(1:debugl),'Output file: '//outf(1:outfl))
c           see whether an old output file exists, and delete it if it does
            errmsg='error looking for the output file'
            inquire(file=outf(1:outfl),exist=exists,err=9900)
            if(exists)then
               errmsg='error opening the old output file'
               open(2,file=outf(1:outfl),status='old',err=9900)
               errmsg='error deleting the old output file'
               close(2,status='delete',err=9900)
            endif
c           open the output file
            errmsg='error opening the output file'
            open(2,file=outf(1:outfl),
     &       status='unknown',form='unformatted',err=9900)

         endif

 100  continue

c     read the substitutions file
      s=0
      errmsg='error reading the substitutions file'
 500  s=s+1
      if(s.gt.sdim)call fatal('too many color substitutions')
 505  read(3,510,end=590,err=9900)sublin
 510  format(a16)
      if(sublin.eq.' ')goto 505
      if(sublin(1:1).eq.'#')goto 505
      if(sublin(14:16).ne.' ')
     & call fatal('invalid color substitution line')
      if(sublin(7:7).ne.' ')
     & call fatal('invalid color substitution line')
      asc0=ichar('0')
      ascca=ichar('A')
      asca=ichar('a')
      do 550 ic=1,13
         if(ic.eq.7)goto 550
         asc=ichar(sublin(ic:ic))
         if(asc.ge.asc0 .and. asc.le.asc0+9)then
            digs(ic)=asc-asc0
         else if(asc.ge.ascca .and. asc.le.ascca+5)then
            digs(ic)=asc-ascca+10
         else if(asc.ge.asca .and. asc.le.asca+5)then
            digs(ic)=asc-asca+10
         else
            call fatal('invalid color substitution line')
         endif
 550  continue
      icro(s)=digs(1)*16+digs(2)
      icgo(s)=digs(3)*16+digs(4)
      icbo(s)=digs(5)*16+digs(6)
      icrt(s)=digs(8)*16+digs(9)
      icgt(s)=digs(10)*16+digs(11)
      icbt(s)=digs(12)*16+digs(13)
      goto 500
 590  smax=s-1
      if(smax.lt.2)call fatal('need at least two color substitutions')

c read the input file header:

c     read the initial P
      call getchr(chr)
      if(chr.ne.'P')call fatal('input file is not a pnm file')
      call out(debug(1:debugl),'Have P in input file')

c     read the type ID
      call getchr(chr)
      pid=ichar(chr)-48
      call outi(debug(1:debugl),'Have pnm ID: ',pid)
      if(pid.eq.ppbm .or. pid.eq.ppgm .or. pid.eq.pppm)call fatal(
     & 'plain pnm encoding not supported; who uses them?')
      if(pid.ne.pbm .and. pid.ne.pgm .and. pid.ne.ppm)call fatal(
     & 'unrecognized input file format')
      if(pid.eq.pbm)call out(debug(1:debugl),'Input format: pbm')
      if(pid.eq.pgm)call out(debug(1:debugl),'Input format: pgm')
      if(pid.eq.ppm)call out(debug(1:debugl),'Input format: ppm')

c     read the figure width
      imax=0
      call gobble(chr)
      if(chr.lt.'0' .or. chr.gt.'9')call fatal(
     & 'missing width in the input file')
 1310 imax=imax*10+ichar(chr)-48
      call getchr(chr)
      if(chr.ge.'0' .and. chr.le.'9')goto 1310
      call outi(debug(1:debugl),'Width: ',imax)
      if(chr.gt.' ' .or. imax.eq.0)call fatal(
     & 'corrupt width in the input file')

c     read the height
      jmax=0
      call gobble(chr)
      if(chr.lt.'0' .or. chr.gt.'9')call fatal(
     & 'missing height in the input file')
 1410 jmax=jmax*10+ichar(chr)-48
      call getchr(chr)
      if(chr.ge.'0' .and. chr.le.'9')goto 1410
      call outi(debug(1:debugl),'Height: ',jmax)
      if(chr.gt.' ' .or. jmax.eq.0)call fatal(
     & 'corrupt height specification in the input file')

c     read the maximum color value cmax
      cmax=1
      if(pid.eq.pbm)goto 1590
      cmax=0
      call gobble(chr)
      if(chr.lt.'0' .or. chr.gt.'9')call fatal(
     & 'missing maximum color value in the input file')
 1510 cmax=cmax*10+ichar(chr)-48
      if(cmax.gt.255)call fatal('program maximum color value exceeded')
      call getchr(chr)
      if(chr.ge.'0' .and. chr.le.'9')goto 1510
      if(chr.gt.' ' .or. cmax.eq.0)call fatal(
     & 'corrupt end of header in input file')
 1590 call outi(debug(1:debugl),'Maximum color value: ',cmax)
      rcmax=cmax

c     force the output format to be ppm
      pidw=ppm
      call out(debug(1:debugl),'Output format: ppm')
      cmaxw=255
      call outi(debug(1:debugl),'Maximum color value out: ',cmaxw)

c     write the output file header

      call putchr('P')
      call putchr(char(48+pidw))
      call putchr(char(10))
 1710 format(i20)
      write(cnum,1710)imax
      do 1720 ic=1,20
         if(cnum(ic:ic).ge.'0' .and. cnum(ic:ic).le.'9')
     &    call putchr(cnum(ic:ic))
 1720 continue
      call putchr(' ')
      write(cnum,1710)jmax
      do 1730 ic=1,20
         if(cnum(ic:ic).ge.'0' .and. cnum(ic:ic).le.'9')
     &    call putchr(cnum(ic:ic))
 1730 continue
      call putchr(char(10))
      if(pidw.eq.pbm)goto 1750
      write(cnum,1710)cmaxw
      do 1740 ic=1,20
         if(cnum(ic:ic).ge.'0' .and. cnum(ic:ic).le.'9')
     &    call putchr(cnum(ic:ic))
 1740 continue
      call putchr(char(10))
 1750 continue

c read and convert the rest of the input file:
      notfnd=0
      icr=0
      icg=0
      icb=0

      do 3000 j=1,jmax

c        the bit of the input file character we are in if we are reading pbm
         bit=8

         do 2900 i=1,imax

c           get the next pixel from the file

            if(pid.eq.pbm)then

c              for PBM (B & W), read the blackness out of the correct bit
               bit=bit+1
               if(bit.gt.8)then
                  call getchr(chr)
                  bit=1
               endif
               icg=ichar(chr)/2**(8-bit)
               icg=icg-icg/2*2
               icg=cmax-icg
               icg=nint(icg*cmaxw/rcmax)
               icr=icg
               icb=icg

            elseif(pid.eq.pgm)then

c              for PPM (grey), read the lightness
               call getchr(chr)
               icg=ichar(chr)
               if(icg.gt.cmax)
     &          call fatal('pixel exceeds maximum color value')
               icg=nint(icg*cmaxw/rcmax)
               icr=icg
               icb=icg

            elseif(pid.eq.ppm)then

c              for PPM, read the RGB values
               call getchr(chr)
               icr=ichar(chr)
               call getchr(chr)
               icg=ichar(chr)
               call getchr(chr)
               icb=ichar(chr)
               if(icr.gt.cmax .or. icg.gt.cmax .or. icb.gt.cmax)
     &          call fatal('color exceeds maximum color value')
               icr=nint(icr*cmaxw/rcmax)
               icg=nint(icg*cmaxw/rcmax)
               icb=nint(icb*cmaxw/rcmax)

            else

               call fatal('program error in read')

            endif

c           find the closest pixel in the list
            dismin=3*255**2+1
            smin=1
            do 2500 s=1,smax
               dis=(icr-icro(s))**2+(icg-icgo(s))**2+(icb-icbo(s))**2
               if(dis.lt.dismin)then
                  dismin=dis
                  smin=s
               endif
 2500       continue
            if(dismin.ne.0)notfnd=notfnd+1

c           write the closest pixel
            call putchr(char(icrt(smin)))
            call putchr(char(icgt(smin)))
            call putchr(char(icbt(smin)))

 2900    continue

 3000 continue

      if(notfnd.ne.0)call outi('-d','** pixels not found: ',notfnd)

c     done
      call exit(0)

c     error exit
 9900 call fatal(errmsg)

      end



      subroutine getchr(chr)

c     Gets the next character from the input file.

c     avoid typos
      implicit none

c output:

c     character read
      character*1 chr

c local:

c     number of characters read
      integer ccnt
      data ccnt/0/

c     error status
      integer stat

c executable:

c     get the character
      call fgetc(1,chr,stat)

c     check for problems
      if(stat.eq.-1)then
         call outi('-d',
     &    '             Character count: ',ccnt)
         call fatal('premature end of input file')
      endif
      if(stat.ne.0)then
         call outi('-d',
     &    '             Character count: ',ccnt)
         call outi('-d',
     &    '             Input file I/O error code: ',stat)
         call fatal('error reading input file')
      endif

c     count the characters read so far
      ccnt=ccnt+1

c     all OK
      return
      end

      subroutine gobble(chr)

c     Gets the next character from the input file.  Gobbles white
c     space and comments.

c     avoid typos
      implicit none

c output:

c     character read
      character*1 chr

c local

c     number of characters in the current comment
      integer coml

c executable:

c     not in a comment yet
      coml=0

c     get the next character
 10   call getchr(chr)
      if(coml.gt.0 .and. chr.ne.char(10))goto 10
      coml=0
      if(chr.eq.'#')then
         coml=1
         goto 10
      endif
      if(chr.le.' ')goto 10

c     got it
      return
      end



      subroutine putchr(chr)

c     Write a character to the output file.

c     avoid typos
      implicit none

c input:

c     character to write
      character*1 chr

c local:

c     number of characters written
      integer ccnt
      data ccnt/0/

c     error status
      integer stat

c executable:

c     write the character
      call fputc(2,chr,stat)

c     check for problems
      if(stat.ne.0)then
         call outi('-d',
     &    '             Character count: ',ccnt)
         call outi('-d',
     &    '             Output file I/O error code: ',stat)
         call fatal('error writing to output file')
      endif

c     count the characters written so far
      ccnt=ccnt+1

c     all OK
      return
      end



      subroutine fatal(msg)

c     Prints the error message msg and aborts.

c     avoid typos
      implicit none

c input:

c     error message, 66 characters or less
      character*(*) msg

c common:

c     debug option
      character*256 debug
      integer debugl
c     input file
      character*256 inf
      integer infl
c     output file
      character*256 outf
      integer outfl
c     substitutions file
      character*256 subf
      integer subfl

c     common block
      common debugl,infl,outfl,subfl,debug,inf,outf,subf

c executable:

c     print the message
      print10,char(7)
 10   format(a1,'*** pnmsubcol: fatal error occurred')
      if(debugl.gt.0)
     & print20,'debug option: "',debug(1:min(debugl,50)),'"'
      if(subfl.gt.0)
     & print20,' substitutions file: "',subf(1:min(subfl,50)),'"'
      if(infl.gt.0)
     & print20,'  input file: "',inf(max(1,infl-49):infl),'"'
      if(outfl.gt.0)
     & print20,' output file: "',outf(max(1,outfl-49):outfl),'"'
      print20,msg
 20   format('             ',3a)

c     return error status 1
      call exit(1)
      end


      subroutine out(debug,msg)

c     Prints the message msg.

c     avoid typos
      implicit none

c input:

c     debug option
      character*(*) debug

c     message
      character*(*) msg

c executable:

c     print the message
      if(debug.eq.'-d')print10,msg
 10   format(a)

c     done
      return
      end

      subroutine outi(debug,msg,i)

c     Prints the message msg and the integer value i.

c     avoid typos
      implicit none

c input:

c     debug option
      character*(*) debug

c     message
      character*(*) msg

c     integer value
      integer i

c local

c     integer as string
      character*20 cnum
c     starting position
      integer start

c executable:

c     print the message and integer value
      write(cnum,10)i
 10   format(i20)
      do 20 start=1,19
         if(cnum(start:start).gt.' ')goto 21
 20   continue
 21   if(debug.eq.'-d')print50,msg,cnum(start:20)
 50   format(a,a)

c     done
      return
      end



      subroutine usage(msg)

c     Explains usage and aborts.

c     avoid typos
      implicit none

c input:

c     error message, 59 characters or less
      character*(*) msg

c common:

c     debug option
      character*256 debug
      integer debugl
c     input file
      character*256 inf
      integer infl
c     output file
      character*256 outf
      integer outfl
c     substitutions file
      character*256 subf
      integer subfl

c     common block
      common debugl,infl,outfl,subfl,debug,inf,outf,subf

c executable:

c     print the message
      print10,'Usage:'
      print11,'pnmsubcol.exe -<debug> subsfile infile outfile',
     & 'where:',
     & ' ',
     & '<debug> can be "d" for debugging output or "n" for not.',
     & ' ',
     & 'SUBSFILE contains in each line an original/substitution '//
     & 'color pair as',
     & 'RRGGBB RRGGBB',
     & ' ',
     & 'On success, the return status is 0.  Everything else is',
     & 'failure.',
     & ' ',
     & '(If the above scrolled off your screen, use "pnmsubcol.exe '//
     & char(124)//' more")'
 10   format('*** pnmsubcol: ',a)
 11   format('               ',a)

      if(debugl.le.0)goto 50
      print20
      print20,'Parameters specified:'
      print20, '   debug option: "',debug(1:min(debugl,50)),'"'
      if(subfl.gt.0)
     & print20,'    substitutions file: "',subf(1:min(subfl,50)),'"'
      if(infl.gt.0)
     & print20,'     input file: "',inf(max(1,infl-49):infl),'"'
      if(outfl.gt.0)
     & print20,'    output file: "',outf(max(1,outfl-49):outfl),'"'
 20   format('             ',3a)
 50   if(msg.ne.' ')then
         print20
         print20,'Error: ',msg
      endif

c     return error status 1
      call exit(1)
      end
