      program readlg

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     Reads a latex log file.  Summarizes the results in a @rl file.
c
c     Usage: readlog - datapath file.log task lines
c            -  can be followed by some debugging characters:
c               s to dump the strings as read
c               u to show unrecognized log lines with lower case characters
c            datapath is the location of readlog.ind and should end in \ or /
c               in DOS use a space between the final \ and the closing "
c            file.log is the log file; the error summary will be in file.@rl
c            task can be:
c               "ini" to read a new log file and display the first problem.
c               "all" to briefly display all problems
c               "prev" to display the previous problem
c               "next" to display the next problem
c               "nextdiff" to display the next different problem
c               "nexttype" to display the next type of problem
c               "same" to display the same problem
c            lines is the number of screen lines:
c               0 not to prompt for "more?";
c               positive to prompt for "more?" after that many lines;
c               negative to prompt for "more?" after -lines, and print
c               at least -lines lines, and erase the "more?" lines, and the
c               last line printed will be a dot-dash one.
c     If lines is unspecified, 0 is assumed.  If task is also unspecified,
c     "all" is assumed

c     avoid typos
      implicit none

c external

c     number of command arguments
      integer iargc
      external iargc

c     start next line of screen output
      integer nxtlin
      external nxtlin

c local

c     declared number of characters in the log file
      integer logdim
      parameter (logdim=2000000)
c     actual number of characters
      integer logmax
c     character index
      integer log,log2
c     the characters
      character*(logdim) logch

c     declared number of words in the database of log words
      integer lwdim
      parameter (lwdim=200000)
c     actual number of words
      integer lwmax
c     word index
      integer lw
c     word offset and length in logch
      integer lwlogo(lwdim),lwlogl(lwdim)

c     declared number of characters in the database of search string characters
      integer scdim
      parameter (scdim=100000)
c     actual number of characters
      integer scmax
c     character index
      integer sc,sc2
c     the characters
      character*(scdim) scch

c     declared number of words in the database of search string words
      integer swdim
      parameter (swdim=20000)
c     actual number of words
      integer swmax
c     word index
      integer sw
c     pattern indicator
c     0: literal string
c     1: anything goes
c     2: check in detail: ASCII 0-2 are special, see findit for details
      integer swpat(swdim)
c     word location in scch
      integer swpos(swdim)
c     word length
      integer swlt(swdim)

c     declared number of search strings
      integer sdim
      parameter (sdim=2000)
c     actual number of search strings
      integer smax
c     location of the last error
      integer smaxe
c     location of the last major warning
      integer smaxw
c     locations where the '!' errors begin and end
      integer sexbeg,sexend
c     location of the "! Emergency stop." error
      integer sestop
c     search string index
      integer s
c     nonzero if the string is incomplete
      integer sinc(sdim)
c     1 if the string is verified to end/not end in a point; or else 0 or -1
      integer sver(sdim)
c     whether the string has the input line number
      integer shasln(sdim)
c     the number of lines to skip after the string
      integer sskip(sdim)
c     whether a rerun is requested
      integer srerun(sdim)
c     errors at this s are equivalent if snorpt=-1 or snorpt>0 words are equal
      integer snorpt(sdim)
c     whether to reset parentheses
      integer snorst(sdim)
c     the number of the last error/warning of this type
      integer slast(sdim)
c     end of the string in the sw database
      integer send(0:sdim)
      data send(0)/0/
c     offset and length of the label in the search string characters database
      integer slabo(sdim),slabl(sdim)

c     declared number of lines in the log file
      integer ldim
      parameter (ldim=80000)
c     actual number of lines
      integer lmax
c     line indices
      integer l,l2
c     ending position of the line in logch
      integer lend(0:ldim)
      data lend(0)/0/

c     declared number of open parentheses
      integer opndim
      parameter (opndim=1000)
c     actual number of open parentheses
      integer opnmax
c     open parentheses index
      integer opn
c     position of the open parenthesis in logch
      integer opnl(opndim),opnlog(opndim)
c     data backup
      integer opnmx0
      integer opnl0(opndim),opnlg0(opndim)

c     declared number of errors
      integer errdim
      parameter (errdim=20000)
c     actual number of errors
      integer errmax
c     error index
      integer erri
c     number of the error
      integer errs(errdim)
c     line of the error
      integer errl(errdim)
c     final line of the error description
      integer errend(errdim)
c     page of the error in the pdf file
      integer errpag(errdim)
c     offset and length of the file name in logch
      integer errflo(errdim),errfll(errdim)
c     location of the error in the tex source file
      integer errloc(errdim)
c     location of the previous error of this type
      integer errprv(errdim)
c     error offset and end in the log words database
      integer errlwo(errdim),errlwe(errdim)
c     whether the error is hidden
      integer errhid(errdim)

c     declared number of warnings
      integer wrndim
      parameter (wrndim=20000)
c     actual number of warnings
      integer wrnmax
c     warning indices
      integer wrn,wrn2
c     number of the warning
      integer wrns(wrndim)
c     line of the warning
      integer wrnl(wrndim)
c     page of the warning in the pdf file
      integer wrnpag(wrndim)
c     offset and length of the file name in logch
      integer wrnflo(wrndim),wrnfll(wrndim)
c     location of the warning in the tex source file
      integer wrnloc(wrndim)
c     location of the previous warning of this type
      integer wrnprv(wrndim)
c     warning offset and end in the log words database
      integer wrnlwo(wrndim),wrnlwe(wrndim)
c     whether the warning is hidden
      integer wrnhid(wrndim)
c     number of major warnings
      integer badwrn

c     declared number of words in the current line
      integer wdim
      parameter (wdim=50)
c     actual number of words
      integer wmax
c     word index
      integer w
c     word position in the line (not the logch buffer)
      integer wpos(wdim)
c     word length
      integer wlt(wdim)

c     whether a rerun is requested
      integer rerun

c     help web page
      character*80 page
      integer pagel
      data pagel/0/

c     declared number of arguments
      integer argdim
      parameter (argdim=5)
c     actual number of arguments
      integer argmax
c     argument string
      character*256 argch(argdim)
      integer arglt(argdim)

c     ASCII codes
      integer asclf,asccr,ascspc,ascrq,ascopn,
     & asccls,ascpt,asc0,asc9,asclb,ascrb,asclq,
     & asca,ascn,ascz
      parameter (asclf=10,asccr=13,ascspc=32,ascrq=39,ascopn=40,
     & asccls=41,ascpt=46,asc0=48,asc9=57,asclb=91,ascrb=93,asclq=96,
     & asca=97,ascn=110,ascz=122)

c     current ascii code, previous one, and second previous
      integer asc,ascp,ascpp

c     whether to dump the strings as read
      integer debugs
c     whether to show unrecognized log lines with lower case letters
      integer debugu

c     whether we are in DOS
      integer dos

c     task to perform
      integer task
      integer ini,all,prev,next,nextd,nextt,same
      parameter (ini=1,all=2,prev=3,next=4,nextd=5,nextt=6,same=7)
c     whether we could find the requested problem
      integer notfnd
      data notfnd/0/

c     lines to print to the screen before "more?" (< 0 erases the "more?" line)
      integer screen
c     number of characters in a line
      integer linelt
      parameter (linelt=79)
c     minum number of lines to write
      integer linmin

c     whether the program ran out of storage
      integer prgerr

c     generic line count
      integer linect

c     total lines written
      integer lintot

c     problem index (either erri or ermax+wrn)
      integer prb
c     current problem being displayed
      integer curprb

c     location in logch that has the line(s) in the tex source file
      integer loc
c     length of the string that has the line(s)
      integer loclt
      data loclt/0/

c     current page number and backup value
      integer pageno,pagen0
c     next page number
      integer newno

c     whether there are lowercase characters in the line
      integer lowcas

c     whether we are in a comment line
      integer incom
c     whether we are in a qualifier line
      integer inqual

c     whether we are in a quited string
      integer inquot
c     whether we are at a word boundary
      integer wbound

c     whether we need to check for a pattern
      integer chkpat
      data chkpat/0/

c     whether we are in an error listing
      integer inerr

c     count of the major warnings
      integer wrnct

c     the number of lines to skip
      integer skip

c     input file name offset and length inlogch
      integer fileo,filel
c     maximum file length that can be shown
      integer filelt

c     number of characters in the actual log file line so far
      integer inlinl

c     generic line
      character*80 line
      integer linel

c     integer as string
      character*20 ci
      integer cil

c     whether a file exists
      logical exists

c     generic indices
      integer i,i2

c executable

c     output line format
 1    format(80a1)
c     check size
      if(80.lt.linelt+1)call fatal('Format 10 is too small.')
c     character string format
 2    format(a)
c     debug line format
 3    format(79a1)

c read the command line arguments

c     number of command line arguments
      argmax=iargc()
      if(argmax.lt.3 .or. argmax.gt.5)goto 9000

c     get the options string
      argch(1)=' '
      call getarg(1,argch(1))
      do 8 i=256,1,-1
         arglt(1)=i
         if(argch(1)(i:i).gt.' ')goto 9
 8    continue
      call fatal('Options string is blank')
 9    if(argch(1)(1:1).ne.'-')
     & call fatal('Options string must start with a "-"')
      debugs=0
      if(index(argch(1)(1:arglt(1)),'s').ne.0)debugs=1
      debugu=0
      if(index(argch(1)(1:arglt(1)),'u').ne.0)debugu=1

c     get the path to readlog.ind
      argch(2)=' '
      call getarg(2,argch(2))
      dos=1
      do 10 i=256,1,-1
         arglt(2)=i
         if(argch(2)(i:i).gt.' ')goto 11
 10   continue
      call fatal('Path to readlog.ind is blank')
c     see whether we are in dos or unix
 11   if(argch(2)(arglt(2):arglt(2)).eq.'\\')then
      else if(argch(2)(arglt(2):arglt(2)).eq.'/')then
         dos=0
      else
         call fatal('Path does not end in \\ or /: "'//
     &    argch(2)(1:arglt(2))//'"')
      endif

c     get the log file to examine
      argch(3)=' '
      call getarg(3,argch(3))
      do 20 i=256,1,-1
         arglt(3)=i
         if(argch(3)(i:i).gt.' ')goto 21
 20   continue
      call fatal('Log file to read is not specified')
c     check that it ends in .xxx
 21   if(arglt(3).lt.4)call fatal('Log file to read must end in .XXX')
      if(argch(3)(arglt(3)-3:arglt(3)-3).ne.'.')
     & call fatal('Log file to read must end in .XXX')

c     get the task to perform
      task=all
      if(argmax.lt.4)goto 39
      argch(4)=' '
      call getarg(4,argch(4))
      do 30 i=256,1,-1
         arglt(4)=i
         if(argch(4)(i:i).gt.' ')goto 31
 30   continue
      call fatal('The task is blank')
 31   if(argch(4)(1:arglt(4)).eq.'ini')then
         task=ini
      else if(argch(4)(1:arglt(4)).eq.'all')then
         task=all
      else if(argch(4)(1:arglt(4)).eq.'prev')then
         task=prev
      else if(argch(4)(1:arglt(4)).eq.'next')then
         task=next
      else if(argch(4)(1:arglt(4)).eq.'nextdiff')then
         task=nextd
      else if(argch(4)(1:arglt(4)).eq.'nexttype')then
         task=nextt
      else if(argch(4)(1:arglt(4)).eq.'same')then
         task=same
      else
         call fatal('Invalid task: "'//argch(4)(1:arglt(4))//'"')
      endif
 39   continue

c     get the number of screen lines
      screen=0
      linmin=0
      if(argmax.lt.5)goto 49
      argch(5)=' '
      call getarg(5,argch(5))
      do 40 i=256,1,-1
         arglt(5)=i
         if(argch(5)(i:i).gt.' ')goto 41
 40   continue
      call fatal('The number of screen lines is blank')
 41   screen=0
      i2=1
      if(argch(5)(1:1).eq.'-')then
         if(arglt(5).le.1)
     &    call fatal('The number of screen lines is missing')
         i2=2
      endif
      do 45 i=i2,arglt(5)
         asc=ichar(argch(5)(i:i))
         if(asc.lt.asc0 .or. asc.gt.asc9)
     &    call fatal('Invalid number of screen lines "'//
     &    argch(5)(1:arglt(5))//'"')
         screen=screen*10-asc0+asc
 45   continue
      if(screen.lt.2 .and. screen.ne.0)
     & call fatal('The number of screen lines must be at least 2')
      if(i2.eq.2)then
         linmin=screen
         screen=-screen
      endif
 49   continue

c     skip reading any old file.@rl if task is ini or all
      curprb=0
      if(task.eq.ini .or. task.eq.all)goto 90

c     skip reading if the file does not exist
      inquire(file=argch(3)(1:arglt(3)-3)//'@rl',exist=exists,err=9010)
      if(.not.exists)goto 90

c     open file.@rl
      open(1,file=argch(3)(1:arglt(3)-3)//'@rl',err=9015)

c     read the current problem number
      read(1,60,err=9020)line
 60   format(a80)
      i=index(line,':')
      if(i.ne.17)goto 9023
      if(line(1:17).ne.'Selected problem:')goto 9023
      read(line(i+1:80),*,err=9023)curprb
      if(curprb.lt.0)goto 9023

c     close file.@rl
      close(1,err=9030)

c     end reading file.@rl
 90   continue

c read the error strings

c     open readlog.ind
      open(1,file=argch(2)(1:arglt(2))//'readlog.ind',
     & status='old',form='unformatted',err=9110)

c     initialize the string characters buffer
      scmax=0

c     initialize the string words buffer
      swmax=0

c     initialize the string limits
      smax=0
      smaxe=-1
      smaxw=-1

c     initialize the locations where the '!' errors begin and end
      sexbeg=0
      sexend=-1

c     initialize the location of the emergency stop
      sestop=0

c     not in a comment or qualifier line yet
      incom=0
      inqual=0

c     initialize the first string
      s=1
      sinc(s)=0
      sver(s)=0
      shasln(s)=0
      sskip(s)=0
      srerun(s)=0
      snorpt(s)=0
      snorst(s)=0
      slast(s)=0
      send(s)=0
      slabl(s)=0

c     initialize the first line
      wbound=1
      asc=0
      ascp=0

c     initialize the line count
      linect=1

c     read the next character in readlog.ind
 1000 if(scmax.ge.scdim)then
         call inttos(linect,ci,cil)
         call fatal('Too many characters '//
     &    'in readlog.ind line '//ci(1:cil))
      endif
      sc=scmax+1
      call getchr(1,scch(sc:sc),*9121,*9120)
      ascpp=ascp
      ascp=asc
      asc=ichar(scch(sc:sc))
      if(asc.eq.13)goto 1000
      if(incom.ne.0)then
         if(asc.ne.asclf)goto 1000
         incom=0
         wbound=1
         asc=0
         ascp=0
         linect=linect+1
         goto 1000
      endif
      if(inqual.ne.0)then
         if(asc.ne.asclf)then
            if(i.ge.80)then
               call inttos(linect,ci,cil)
               call fatal('Qualifier line too long '//
     &          'in readlog.ind line '//ci(1:cil))
            endif
            i=i+1
            line(i:i)=char(asc)
            goto 1000
         endif
         i2=index(line(1:i),'<incomplete<')
         if(i2.gt.0)then
            sinc(smax)=1
            line(i2+1:i2+10)=' '
         endif
         i2=index(line(1:i),'<rerun<')
         if(i2.gt.0)then
            srerun(smax)=1
            line(i2+1:i2+5)=' '
         endif
         i2=index(line(1:i),'<hasline<')
         if(i2.gt.0)then
            shasln(smax)=1
            line(i2+1:i2+7)=' '
         endif
         i2=index(line(1:i),'<noreset<')
         if(i2.gt.0)then
            snorst(smax)=1
            line(i2+1:i2+7)=' '
         endif
         i2=index(line(1:i),'<norepeat<')
         if(i2.gt.0)then
            snorpt(smax)=-1
            line(i2+1:i2+8)=' '
         endif
         i2=index(line(1:i),'<norepeat=')
         if(i2.gt.0)then
            line(i2+1:i2+9)=' '
            snorpt(smax)=0
            do 1040 i2=i2+10,i
               asc=ichar(line(i2:i2))
               if(asc.eq.ichar('<'))goto 1041
               if(asc.lt.asc0 .or. asc.gt.asc9)then
                  call inttos(linect,ci,cil)
                  call fatal('Invalid norepeat word count '//
     &             'in readlog.ind line '//ci(1:cil))
               endif
               snorpt(smax)=snorpt(smax)*10+asc-asc0
               line(i2:i2)=' '
 1040       continue
            call inttos(linect,ci,cil)
            call fatal('Unterminated norepeat word count '//
     &       'in readlog.ind line '//ci(1:cil))
 1041       if(snorpt(smax).le.0)then
               call inttos(linect,ci,cil)
               call fatal('Missing norepeat word count '//
     &          'in readlog.ind line '//ci(1:cil))
            endif
         endif
         if(snorpt(smax).gt.send(smax)-send(smax-1))then
            call inttos(linect,ci,cil)
            call fatal('Norepeat word count exceeds words '//
     &       'in readlog.ind line '//ci(1:cil))
         endif
 1050    i2=index(line(1:i),'<skip<')
         if(i2.gt.0)then
            sskip(smax)=sskip(smax)+1
            line(i2+1:i2+4)=' '
            goto 1050
         endif
         do 1060 i2=1,i
            if(line(i2:i2).ne.' ' .and. line(i2:i2).ne.'<')then
               call inttos(linect,ci,cil)
               call fatal('Invalid modifier line for search string '//
     &          'in readlog.ind line '//ci(1:cil))
            endif
 1060    continue
         inqual=0
         wbound=1
         asc=0
         ascp=0
         linect=linect+1
         goto 1000
      endif

c     check whether currently at a word boundary
      if(wbound.eq.0)goto 1500
c     no changes if another space or tab or control character
      if(asc.eq.asclf)goto 1700
      if(asc.le.ascspc)goto 1000
c     check for a comment or qualifier line or for verification
      if(send(s).le.send(s-1))then
         if(asc.eq.ichar('%'))then
            incom=1
            goto 1000
         endif
         if(asc.eq.ichar('<') .and. ascp.ne.ascspc)then
            if(smax.le.0)then
               call inttos(linect,ci,cil)
               call fatal('Qualifier line before all search strings '//
     &          'in readlog.ind line '//ci(1:cil))
            endif
            inqual=1
            line(1:1)='<'
            i=1
            goto 1000
         endif
         if(ascp.eq.ascspc)sver(s)=1
      endif
c     start a new word
      scmax=scmax+1
      swmax=swmax+1
      if(swmax.gt.swdim)then
         call inttos(linect,ci,cil)
         call fatal('Too many words in readlog.ind line '//ci(1:cil))
      endif
      send(s)=swmax
      swpat(swmax)=0
      swpos(swmax)=scmax
      swlt(swmax)=1
      wbound=0
      chkpat=0
      goto 1000

c     inside a word
 1500 if(asc.le.ascspc)goto 1600
c     add to the word
      scmax=scmax+1
      swlt(swmax)=swlt(swmax)+1
c     check for indications of patterns
      if(asc.eq.ascpt)then
         if(ascp.eq.ascpt .and. ascpp.eq.ascpt)chkpat=1
         if(ascp.eq.ascn .and. ascpp.eq.ascn)chkpat=1
      else if(asc.eq.ascn)then
         if(ascp.eq.ascn .and. ascpp.eq.ascn)chkpat=1
      endif
      goto 1000

c     found the end of a word
 1600 wbound=1
c     check for accepted patterns
      if(chkpat.eq.0)goto 1690
      sc=swpos(swmax)
      sc2=sc+swlt(swmax)-1
      if(sc2-sc+1.eq.3)then
         if(scch(sc:sc2).eq.'...')then
c           '...': assume: anything goes
            swpat(swmax)=1
         else if(scch(sc:sc2).eq.'nnn')then
c           'nnn': assume: digits go
            swpat(swmax)=2
            scch(sc:sc2)=char(0)//char(0)//char(2)
         endif
      else
 1610    i=index(scch(sc:sc2),'...')
         if(i.gt.0)then
c           '...': assume: anything not equal to the following character goes
            swpat(swmax)=2
            i=sc+i-1
            scch(i:i+2)=char(0)//char(0)//char(1)
            goto 1610
         endif
 1620    i=index(scch(sc:sc2),'nnn')
         if(i.gt.0)then
c           'nnn': assume: digits go
            swpat(swmax)=2
            i=sc+i-1
            scch(i:i+2)=char(0)//char(0)//char(2)
            goto 1620
         endif
 1630    i=index(scch(sc:sc2),'nn.')
         if(i.gt.0)then
c           'nn.': assume: digits followed by a decimal point go
            swpat(swmax)=2
            i=sc+i-1
            scch(i:i+1)=char(0)//char(2)
            goto 1630
         endif
      endif
 1690 if(asc.ne.asclf)goto 1000

c     check for a blank line
 1700 if(send(s).le.send(s-1))then
         if(s.eq.1) then
         else if(smaxe.lt.0)then
            smaxe=smax
         else if(smaxw.lt.0)then
            smaxw=smax
         else
            goto 1900
         endif
         linect=linect+1
         goto 1000
      endif
c     check for too many words
      if(send(s)-send(s-1).ge.wdim)then
         call inttos(linect,ci,cil)
         call fatal('Parameter wdim is too small for the problem '//
     &    'in readlog.ind line '//ci(1:cil))
      endif
c     add the string to the database
      smax=s
c     move the start of the first word past a "#label"> string if any
      sw=send(s-1)+1
      sc=swpos(sw)
      sc2=sc+swlt(sw)-1
      if(sc2.gt.sc .and. scch(sc:min(scdim,sc+1)).eq.'"#')then
         i=index(scch(sc:sc2),'">')
         if(i.lt.4 .or. i.ge.swlt(sw)-1)then
            call inttos(linect,ci,cil)
            call fatal('Corrupt name in readlog.ind line '//ci(1:cil))
         endif
         slabo(s)=sc+1
         slabl(s)=i-3
         swlt(sw)=swlt(sw)-(i+1)
         swpos(sw)=swpos(sw)+(i+1)
      endif
c     check for an initial '!'
      sw=send(s-1)+1
      if(smaxe.lt.0 .and. swlt(sw).eq.1)then
         if(scch(swpos(sw):swpos(sw)).eq.'!')then
            if(sexbeg.eq.0)sexbeg=s
            sexend=s
            if(send(s)-send(s-1).eq.3)then
               if(swlt(sw+1).eq.9 .and. swlt(sw+2).eq.5)then
                  if(scch(swpos(sw):swpos(sw)+14)
     &             .eq.'!Emergencystop.')sestop=s
               endif
            endif
         endif
      endif
c     if unverified, check whether there is a final point given
      if(sver(s).eq.0)then
         sw=send(s)
         sc=swpos(sw)+swlt(sw)-1
         if(scch(sc:sc).ne.'.')sver(s)=-1
c        words that are a single point do not qualify
         if(swlt(sw).le.1+sver(s))sver(s)=1
c        incomplete strings do not qualify
         if(sinc(s).ne.0)sver(s)=1
      endif
c     initialize the next string
      s=smax+1
      if(s.gt.sdim)then
         call inttos(linect,ci,cil)
         call fatal('Too many search strings '//
     &    'in readlog.ind line '//ci(1:cil))
      endif
      sinc(s)=0
      sver(s)=0
      shasln(s)=0
      sskip(s)=0
      srerun(s)=0
      snorpt(s)=0
      snorst(s)=0
      slast(s)=0
      send(s)=send(s-1)
      slabl(s)=0
c     initialize a new line
      wbound=1
      asc=0
      ascp=0
c     increment line count
      linect=linect+1

      goto 1000

c     close readlog.ind
 1900 close(1,err=9130)

c     check
      if(smaxe.le.0)call fatal('No errors in file readlog.ind')
      if(smaxw.le.smaxe)
     & call fatal('No major warnings in file readlog.ind')
      if(smax.le.smaxw)
     & call fatal('No minor warnings in file readlog.ind')
      if(sexend.lt.sexbeg)
     & call fatal('No errors starting with ! in file readlog.ind')


c     dump the search strings
      if(debugs.ne.0)then
         do 1950 s=1,smax
            call inttos(s,ci,cil)
            if(s.eq.sexbeg)print2,'Start of ! strings:'
            print3,(ci(i:i),i=1,cil),
     &       (' ',(char(max(32,ichar(scch(sc:sc)))),
     &       sc=swpos(sw),swpos(sw)+swlt(sw)-1),
     &       sw=send(s-1)+1,send(s))
            if(slabl(s).le.0)print2,' no label'
            if(s.eq.sexend)print2,'End of ! strings'
            if(s.eq.smaxe .or. s.eq. smaxw .or. s.eq.smax)
     &       print3,('-',i=1,78)
 1950    continue
      endif

c read the log file

c     open the log file
      open(1,file=argch(3)(1:arglt(3)),
     & status='old',form='unformatted',err=9210)

c     initialize the list of document errors
      errmax=0

c     initialize the list of document warnings
      wrnmax=0

c     initialize the bad warnings count
      badwrn=0

c     no rerun requested yet
      rerun=0

c     initialize the log file buffer
      logmax=0
c     initialize its words database
      lwmax=0

c     initialize the buffer of open parentheses
      opnmax=0

c     initialize the page number
      pageno=1
      newno=-1

c     we have not recently seen an error
      inerr=0
c     not skipping a line
      skip=0

c     initialize the first line
      lmax=0
      l=1
      lend(l)=0
      wmax=0
      lowcas=0
      inquot=0
      wbound=1
      inlinl=0

c     initialize the open parentheses backup
      opnmx0=0

c     initialize the page number backup
      pagen0=1

c     read the next character in the log file
 2000 if(logmax.ge.logdim)goto 6000
      log=logmax+1
      call getchr(1,logch(log:log),*6000,*9220)
      asc=ichar(logch(log:log))

c     ignore carriage controls
      if(asc.eq.asccr)goto 2000

c     new position in the line
      inlinl=inlinl+1

c     look for end of line
      if(asc.eq.asclf)then
c        ignore a lf in column 80
         if(inlinl.eq.80 .and. inerr.ne.1)then
            inlinl=0
            goto 2000
         endif
c        go process the end of the line
         goto 3000
      endif

c     add the character to the current line
      logmax=log
      lend(l)=log

c     update the parentheses database
      if(asc.eq.ascopn .and. inquot.eq.0)then
         if(opnmax.ge.opndim)goto 6000
         opnmax=opnmax+1
         opnl(opnmax)=l
         opnlog(opnmax)=log
      endif
      if(asc.eq.asccls .and. inquot.eq.0)then
         if(opnmax.gt.0)opnmax=opnmax-1
      endif

c     update the current page number
      if(newno.ge.0)then
         if(asc.ge.asc0 .and. asc.le.asc9 .and. newno.le.99999)then
            newno=newno*10+asc-asc0
         else if(asc.eq.ascrb)then
            pageno=newno+1
            newno=-1
         else
            newno=-1
         endif
      endif
      if(asc.eq.asclb .and. inquot.eq.0)newno=0

c     handle case that we are at a word boundary
      if(wbound.eq.0)goto 2500
c     no changes if another space or tab or control character
      if(asc.le.ascspc)then
         if(asc.eq.9)logch(logmax:logmax)=' '
         goto 2000
      endif
c     start a new word
      wmax=min(wdim,wmax+1)
      wpos(wmax)=logmax-lend(l-1)
      wlt(wmax)=1
      wbound=0
      if(asc.ge.asca .and. asc.le.ascz)lowcas=1
      if(asc.eq.asclq)inquot=1
      goto 2000

c     handle case that we are inside a word
 2500 if(asc.le.ascspc .and. inquot.eq.0)then
c        control characters end a word
         wbound=1
         if(asc.eq.9)logch(logmax:logmax)=' '
      else
c        add to the word
         wlt(wmax)=wlt(wmax)+1
         if(asc.ge.asca .and. asc.le.ascz)lowcas=1
         if(asc.eq.9)logch(logmax:logmax)=' '
         if(asc.eq.ascrq)inquot=0
      endif
      goto 2000

c     analyze the read line
 3000 continue

c     skip the line as needed
      if(skip.gt.0)then
         if(inerr.gt.0)goto 3090
c        search for a trailing 'on input line nnn.'
         if(wmax.lt.5)goto 3090
         if(wlt(wmax-3).eq.2 .and. wlt(wmax-2).eq.5
     &    .and. wlt(wmax-1).eq.4)then
            log=wpos(wmax-3)+lend(l-1)
            if(logch(log:log+12).eq.'on input line')goto 3050
         endif
         goto 3090
c        set the location of the input file line
 3050    log=wpos(wmax)+lend(l-1)
         asc=ichar(logch(log:log))
         if(asc.le.asc0 .or. asc.gt.asc9)goto 3090
c        location of last warning?
         if(wrnmax.le.0)goto 3080
         if(l-wrnl(wrnmax).gt.skip)goto 3080
         if(wrnloc(wrnmax).gt.0)goto 3090
         wrnloc(wrnmax)=log
         goto 3090
c        location of last error?
 3080    if(errmax.le.0)goto 3090
         if(l-errl(errmax).gt.skip)goto 3090
         if(errloc(errmax).gt.0)goto 3090
         errloc(errmax)=log
 3090    skip=skip-1
         goto 4900
      endif

c     handle case that we are already in an error description
      if(inerr.eq.0)goto 4000
c     find the info and end of the message
      if(inerr.eq.1)then
c        ignore blank lines
         if(lend(l).le.lend(l-1))goto 5000
c        look for a l.nnn line and set inerr to 2 if found
         if(lend(l)-lend(l-1).lt.4)goto 3200
         log=lend(l-1)+1
         if(logch(log:log+1).ne.'l.')goto 3200
         log=log+2
         asc=ichar(logch(log:log))
         if(asc.le.asc0 .or. asc.gt.asc9)goto 3200
         do 3100 log=log+1,logmax
            asc=ichar(logch(log:log))
            if(asc.ge.asc0 .and. asc.le.asc9)then
            else if(asc.ne.ascspc)then
               goto 3200
            else
               errloc(errmax)=lend(l-1)+3
               inerr=2
               goto 4900
            endif
 3100    continue
c        allow some reasonable number of lines for l.nnn to show up
 3200    if(l-errl(errmax).gt.10)inerr=0
         goto 4000
      else if(inerr.eq.2)then
c        the second source line has been read
         inerr=3
         goto 4900
      else if(inerr.eq.3)then
c        look for an empty line and set the error end from it
         if(lend(l).le.lend(l-1))then
            errend(errmax)=l-1
            inerr=0
         endif
         goto 4000
      endif

c     look for an error or warning
 4000 continue

c     let findit go down the list
      s=0
      if(lowcas.gt.0)then
         call findit(
     &    sdim,smax,sexbeg,sexend,sinc,sver,send,
     &    swdim,swpat,swpos,swlt,
     &    scch,
     &    wdim,wmax,wpos,wlt,logch(lend(l-1)+1:lend(l)),
     &    s)
c        show strings that are not found if the debug option is set
         if(s.eq.0 .and. debugu.ne.0)then
c           skip useless stuff
            if(lend(l)-lend(l-1).ge.43)then
               if(logch(lend(l-1)+1:lend(l-1)+27).eq.
     &          '   defining Unicode char U+')goto 4010
            endif
c           print the string
            print3,'?',(logch(log:log),log=lend(l-1)+1,lend(l))
 4010       continue
         endif
      endif

c     if not found go to the next line
      if(s.le.0)goto 5000

c     if already in an error, (but not emergency stop), end it
      if(inerr.ne.0)then
         if(s.eq.sestop)goto 5000
         inerr=0
         errend(errmax)=l-1
      endif

c     process based on type
      if(s.le.smaxe)then
c        handle the case that it is an error
         if(s.ge.sexbeg .and. s.le.sexend)inerr=1
         if(errmax.ge.errdim)goto 6000
         erri=errmax+1
         errs(erri)=s
         errl(erri)=l
         errend(erri)=l
         errpag(erri)=pageno
         errfll(erri)=0
         opn=opnmax
 4100    if(opn.le.0)goto 4190
         errflo(erri)=opnlog(opn)
         errfll(erri)=0
         do 4110 log=errflo(erri)+1,lend(opnl(opn))
            if(index('[]()*?',logch(log:log)).gt.0)goto 4120
            errfll(erri)=errfll(erri)+1
 4110    continue
 4120    if(errfll(erri).le.0)goto 4190
         log=errflo(erri)+errfll(erri)
         if(logch(log:log).le.' ')then
            errfll(erri)=errfll(erri)-1
            goto 4120
         endif
         log=errflo(erri)+errfll(erri)
         if(logch(log:log).eq.'"')then
            errfll(erri)=errfll(erri)-1
            if(errfll(erri).le.0)goto 4190
         endif
         log=errflo(erri)+1
         if(logch(log:log).eq.'"')then
            errflo(erri)=errflo(erri)+1
            errfll(erri)=errfll(erri)-1
            if(errfll(erri).le.0)goto 4190
         endif
         if(errfll(erri).ge.4)then
            if(logch(log-3:log-3).eq.'.')then
               if(index('.eps.EPS.jpg.JPG.pdf.PDF',
     &          logch(log-3:log)).gt.0)then
                  opn=opn-1
                  goto 4100
               endif
            endif
         endif
 4190    errloc(erri)=0
         if(shasln(s).ne.0)then
            log=wpos(wmax)+lend(l-1)
            asc=ichar(logch(log:log))
            if(asc.gt.asc0 .and. asc.le.asc9)errloc(erri)=log
         endif
         errprv(erri)=0
         if(slast(s).gt.0)errprv(erri)=slast(s)
         slast(s)=erri
         errhid(erri)=0
         errlwo(erri)=lwmax
         do 4200 w=1,wmax
            if(lwmax.ge.lwdim)goto 6000
            lwmax=lwmax+1
            lw=lwmax
            lwlogo(lw)=wpos(w)+lend(l-1)-1
            lwlogl(lw)=wlt(w)
 4200    continue
         errlwe(erri)=lwmax
         errmax=errmax+1
      else
c        handle the case that it is a warning
         if(wrnmax.ge.wrndim)goto 6000
         wrn=wrnmax+1
         wrns(wrn)=s
         wrnl(wrn)=l
         wrnpag(wrn)=pageno
         wrnfll(wrn)=0
         opn=opnmax
 4300    if(opn.le.0)goto 4390
         wrnflo(wrn)=opnlog(opn)
         wrnfll(wrn)=0
         do 4310 log=wrnflo(wrn)+1,lend(opnl(opn))
            if(index('[]()*?',logch(log:log)).gt.0)goto 4320
            wrnfll(wrn)=wrnfll(wrn)+1
 4310    continue
 4320    if(wrnfll(wrn).le.0)goto 4390
         log=wrnflo(wrn)+wrnfll(wrn)
         if(logch(log:log).le.' ')then
            wrnfll(wrn)=wrnfll(wrn)-1
            goto 4320
         endif
         log=wrnflo(wrn)+wrnfll(wrn)
         if(logch(log:log).eq.'"')then
            wrnfll(wrn)=wrnfll(wrn)-1
            if(wrnfll(wrn).le.0)goto 4390
         endif
         log=wrnflo(wrn)+1
         if(logch(log:log).eq.'"')then
            wrnflo(wrn)=wrnflo(wrn)+1
            wrnfll(wrn)=wrnfll(wrn)-1
            if(wrnfll(wrn).le.0)goto 4390
         endif
         if(wrnfll(wrn).ge.4)then
            if(logch(log-3:log-3).eq.'.')then
               if(index('.eps.EPS.jpg.JPG.pdf.PDF',
     &          logch(log-3:log)).gt.0)then
                  opn=opn-1
                  goto 4300
               endif
            endif
         endif
 4390    wrnloc(wrn)=0
         if(shasln(s).ne.0)then
            log=wpos(wmax)+lend(l-1)
            asc=ichar(logch(log:log))
            if(asc.gt.asc0 .and. asc.le.asc9)wrnloc(wrn)=log
         endif
         wrnprv(wrn)=0
         if(slast(s).gt.0)wrnprv(wrn)=slast(s)
         slast(s)=wrn
         wrnhid(wrn)=0
         wrnlwo(wrn)=lwmax
         do 4400 w=1,wmax
            if(lwmax.ge.lwdim)goto 6000
            lwmax=lwmax+1
            lw=lwmax
            lwlogo(lw)=wpos(w)+lend(l-1)-1
            lwlogl(lw)=wlt(w)
 4400    continue
         wrnlwe(wrn)=lwmax
         if(s.le.smaxw)badwrn=badwrn+1
         wrnmax=wrnmax+1
      endif
      skip=sskip(s)
      if(srerun(s).ne.0)rerun=1
      if(snorst(s).ne.0)goto 5000

c     reset the parentheses data that may have gotten messed up
 4900 opnmax=opnmx0
      do 4950 opn=1,opnmax
         opnl(opn)=opnl0(opn)
         opnlog(opn)=opnlg0(opn)
 4950 continue

c     reset the page number to what it was at the start of the line
      pageno=pagen0

c     go read the next line
 5000 lmax=l
      if(lmax.ge.ldim)goto 6000
      l=l+1
      lend(l)=logmax
      wmax=0
      lowcas=0
      inquot=0
      wbound=1
      inlinl=0
      opnmx0=opnmax
      do 5100 opn=1,opnmax
         opnl0(opn)=opnl(opn)
         opnlg0(opn)=opnlog(opn)
 5100 continue
      pagen0=pageno
      goto 2000

c output the results

c     end of file or out of storage
 6000 continue

c     mark program errors
      prgerr=0
      if(logmax.ge.logdim)prgerr=1
      if(lwmax.ge.lwdim)prgerr=1
      if(lmax.ge.ldim)prgerr=1
      if(opnmax.ge.opndim)prgerr=1
      if(errmax.ge.errdim)prgerr=1
      if(wrnmax.ge.wrndim)prgerr=1

c     close the log file
      close(1,err=9230)

c     case that there are no problems
      if(errmax+wrnmax.le.0)then
         curprb=0
         if(task.ne.ini .and. task.ne.same .and. task.ne.all)notfnd=1
         if(notfnd.ne.0 .or. prgerr.ne.0)goto 7900
         if(screen.eq.0)goto 6090
         if(screen.lt.0)then
            do 6050 i=1,(-screen-1)/2
               if(nxtlin(screen,linect,lintot).eq.1)goto 8000
               print2
 6050       continue
         endif
         if(nxtlin(screen,linect,lintot).eq.1)goto 8000
         if(arglt(3)+30.le.linelt-1)then
            i=(linelt+1-(arglt(3)+30))/2
            line(1:i)=' '
            print2,line(1:i)//'No problems found in log file '//
     &       argch(3)(1:arglt(3))
         else if(arglt(3)+30.eq.linelt)then
            print2,'No problems found in log file '//
     &       argch(3)(1:arglt(3))
         else
            print2,'No problems found in log file ...'//
     &       argch(3)(arglt(3)-(linelt-33)+1:arglt(3))
         endif
 6090    goto 8000
      endif

c     briefly show all problems

c     skip if task is not all
      if(task.ne.all)goto 7000

c     hide errors that are repeated
      do 6300 erri=2,errmax
         call hiderp(
     &    logch,
     &    lwdim,lwlogo,lwlogl,
     &    swdim,swpat,
     &    sdim,snorpt,send,
     &    errdim,erri,errs,errprv,errlwo,errlwe,errhid)
 6300 continue

c     show all errors that are not hidden
      do 6500 erri=1,errmax
         if(errhid(erri).eq.1)goto 6500
         l=errl(erri)
         log=lend(l-1)
         log2=lend(l)
 6450    if(nxtlin(screen,linect,lintot).eq.1)goto 8000
         if(log2-log.le.linelt)then
            print2,logch(log+1:log2)
         else
            print2,logch(log+1:log+linelt)
            log=log+linelt
            goto 6450
         endif
         if(errhid(erri).eq.-1)then
            if(nxtlin(screen,linect,lintot).eq.1)goto 8000
            print2,'(There are additional errors like this '//
     &       'that will not be shown.)'
         endif
 6500 continue

c     hide warnings that are repeated
      do 6600 wrn=wrnmax,1,-1
         call hiderp(
     &    logch,
     &    lwdim,lwlogo,lwlogl,
     &    swdim,swpat,
     &    sdim,snorpt,send,
     &    wrndim,wrn,wrns,wrnprv,wrnlwo,wrnlwe,wrnhid)
 6600 continue

c     show all warnings that are not hidden
      do 6800 wrn=1,wrnmax
         if(wrnhid(wrn).eq.1)goto 6800
         l=wrnl(wrn)
         log=lend(l-1)
         log2=lend(l)
 6750    if(nxtlin(screen,linect,lintot).eq.1)goto 8000
         if(log2-log.le.linelt)then
            print2,logch(log+1:log2)
         else
            print2,logch(log+1:log+linelt)
            log=log+linelt
            goto 6750
         endif
         if(wrnhid(wrn).eq.-1)then
            if(nxtlin(screen,linect,lintot).eq.1)goto 8000
            print2,'(There are additional warnings like this '//
     &       'that will not be shown.)'
         endif
 6800 continue

c     go show the summary line
      goto 7900

c     show a single error or warning

c     start point
 7000 continue

c     check the current problem number for sanity
      if(curprb.lt.0)curprb=0
      if(curprb.gt.errmax+wrnmax)curprb=errmax+wrnmax

c     find the problem to display
      if(task.eq.ini)then
         curprb=1
      else if(task.eq.same)then
         curprb=max(1,curprb)
      else if(task.eq.prev)then
         if(curprb.lt.2)notfnd=1
         curprb=max(1,curprb-1)
      else if(task.eq.next)then
         if(curprb.gt.errmax+wrnmax-1)notfnd=1
         curprb=min(errmax+wrnmax,curprb+1)
      else if(task.eq.nextd)then
         if(curprb.gt.errmax+wrnmax-1)notfnd=1
         curprb=min(errmax+wrnmax,curprb+1)
         do 7200 prb=curprb,errmax+wrnmax
            if(prb.le.errmax)then
               erri=prb
               call hiderp(
     &          logch,
     &          lwdim,lwlogo,lwlogl,
     &          swdim,swpat,
     &          sdim,snorpt,send,
     &          errdim,erri,errs,errprv,errlwo,errlwe,errhid)
               if(errhid(erri).eq.1)goto 7200
               curprb=erri
               goto 7210
            else
               wrn=prb-errmax
               call hiderp(
     &          logch,
     &          lwdim,lwlogo,lwlogl,
     &          swdim,swpat,
     &          sdim,snorpt,send,
     &          wrndim,wrn,wrns,wrnprv,wrnlwo,wrnlwe,wrnhid)
               if(wrnhid(wrn).eq.1)goto 7200
               curprb=errmax+wrn
               goto 7210
            endif
 7200    continue
         notfnd=1
 7210    continue
      else if(task.eq.nextt)then
         if(curprb.gt.errmax+wrnmax-1)notfnd=1
         curprb=min(errmax+wrnmax,curprb+1)
         do 7400 prb=curprb,errmax+wrnmax
            if(prb.le.errmax)then
               erri=prb
               snorpt(errs(erri))=-1
               call hiderp(
     &          logch,
     &          lwdim,lwlogo,lwlogl,
     &          swdim,swpat,
     &          sdim,snorpt,send,
     &          errdim,erri,errs,errprv,errlwo,errlwe,errhid)
               if(errhid(erri).eq.1)goto 7400
               curprb=erri
               goto 7410
            else
               wrn=prb-errmax
               snorpt(wrns(wrn))=-1
               call hiderp(
     &          logch,
     &          lwdim,lwlogo,lwlogl,
     &          swdim,swpat,
     &          sdim,snorpt,send,
     &          wrndim,wrn,wrns,wrnprv,wrnlwo,wrnlwe,wrnhid)
               if(wrnhid(wrn).eq.1)goto 7400
               curprb=errmax+wrn
               goto 7410
            endif
 7400    continue
         notfnd=1
 7410    continue
      endif
      prb=curprb

c     find the data on the current error or warning
      if(prb.le.errmax)then
         erri=prb
         fileo=errflo(erri)
         filel=errfll(erri)
         pageno=errpag(erri)
         loc=errloc(erri)
         s=errs(erri)
         i=slabl(s)
         if(i.gt.0)then
            pagel=12+i
            sc=slabo(s)
            page(1:pagel)='errors.html#'//scch(sc+1:sc+i)
         else
            pagel=20
            page(1:pagel)='errors.html#contents'
         endif
      else
         wrn=prb-errmax
         fileo=wrnflo(wrn)
         filel=wrnfll(wrn)
         pageno=wrnpag(wrn)
         loc=wrnloc(wrn)
         s=wrns(wrn)
         i=slabl(s)
         if(i.gt.0)then
            pagel=14+i
            sc=slabo(s)
            page(1:pagel)='warnings.html#'//scch(sc+1:sc+i)
         else
            pagel=22
            page(1:pagel)='warnings.html#contents'
         endif
      endif

c     print the error location
      filelt=linelt-(19+18+1)
      if(loc.gt.0)then
         loclt=0
         do 7510 log=loc,logmax
            asc=ichar(logch(log:log))
            if((asc.lt.asc0 .or. asc.gt.asc9) .and. asc.ne.ichar('-'))
     &       goto 7511
            loclt=loclt+1
 7510    continue
 7511    if(loclt.le.0)then
            loc=0
         else
            filelt=filelt-6-min(12,loclt)
            if(index(logch(loc:loc+loclt-1),'-').ne.0)filelt=filelt-1
         endif
      endif
      call inttos(pageno,ci,cil)
      filelt=filelt-cil
      linel=0
      line(linel+1:linel+19)='Presumably in file '
      linel=linel+19
      log2=lend(l)
      i=0
      if(filel.le.4)then
         i=3
      else if(logch(fileo+filel-3:fileo+filel-3).ne.'.')then
         i=3
      endif
      filelt=filelt-i
      if(filel.gt.filelt)then
         line(linel+1:linel+filelt)=
     &    '...'//logch(fileo+filel-filelt+4:fileo+filel)
         linel=linel+filelt
      else if(filel.gt.0)then
         line(linel+1:linel+filel)=logch(fileo+1:fileo+filel)
         linel=linel+filel
      endif
      if(i.ne.0)then
         line(linel+1:linel+3)='???'
         linel=linel+3
      endif
      if(loc.gt.0)then
         if(index(logch(loc:loc+loclt-1),'-').eq.0)then
            line(linel+1:linel+6)=' line '
            linel=linel+6
         else
            line(linel+1:linel+7)=' lines '
            linel=linel+7
         endif
         if(loclt.le.12)then
            line(linel+1:linel+loclt)=logch(loc:loc+loclt-1)
            linel=linel+loclt
         else
            line(linel+1:linel+12)=logch(loc:loc+8)//'...'
            linel=linel+12
         endif
      endif
      line(linel+1:linel+18)=', and on pdf page '
      linel=linel+18
      line(linel+1:linel+cil)=ci(1:cil)
      linel=linel+cil
      line(linel+1:linel+1)=':'
      linel=linel+1
      if(nxtlin(screen,linect,lintot).eq.1)goto 8000
      print2,line(1:linel)

c     put the error or warning itself on the screen
      if(prb.le.errmax)then
         l2=errend(erri)
         s=errs(erri)
         if(s.lt.sexbeg .or. s.gt.sexend)
     &    l2=min(lmax,errl(erri)+sskip(s))
         do 7700 l=errl(erri),l2
            log=lend(l-1)
            log2=lend(l)
            if(log2.le.log)then
               if(nxtlin(screen,linect,lintot).eq.1)goto 8000
               print2
               goto 7700
            endif
            if(l.eq.errl(erri))goto 7680
            if(s.lt.sexbeg .or. s.gt.sexend)goto 7680
            if(index(logch(log+1:log2),'<return>').gt.0)goto 7690
            if(index(logch(log+1:log2),'<RETURN>').gt.0)goto 7690
            if(index(logch(log+1:log2),'<Return>').gt.0)goto 7690
            if(index(logch(log+1:log2),'Type ').gt.0)goto 7690
            if(index(logch(log+1:log2),' type ').gt.0)goto 7690
 7680       if(nxtlin(screen,linect,lintot).eq.1)goto 8000
            if(log2-log.le.linelt)then
               print2,logch(log+1:log2)
               goto 7700
            else
               print2,logch(log+1:log+linelt)
               log=log+linelt
               goto 7680
            endif
 7690       if(nxtlin(screen,linect,lintot).eq.1)goto 8000
            print2,'THIS LINE ERASED AS INAPPROPRIATE FOR '//
     &       'NONINTERACTIVE MODE.'
 7700    continue
      else
         do 7800 l=wrnl(wrn),min(lmax,wrnl(wrn)+sskip(wrns(wrn)))
            log=lend(l-1)
            log2=lend(l)
            if(log2.le.log)then
               if(nxtlin(screen,linect,lintot).eq.1)goto 8000
               print2
               goto 7800
            endif
 7780       if(nxtlin(screen,linect,lintot).eq.1)goto 8000
            if(log2-log.le.linelt)then
               print2,logch(log+1:log2)
               goto 7800
            else
               print2,logch(log+1:log+linelt)
               log=log+linelt
               goto 7780
            endif
 7800    continue
      endif

c     warn if not found
      if(pagel.ge.9)then
         if(page(pagel-8:pagel).eq.'#contents')then
            if(nxtlin(screen,linect,lintot).eq.1)goto 8000
            print2,'** Help for this specific problem could '//
     &       'not be located. **'
         endif
      endif

c     show a summary line
 7900 linel=0
      line(linel+1:linel+7)='Total: '
      linel=linel+7
      call inttos(errmax,ci,cil)
      line(linel+1:linel+cil)=ci(1:cil)
      linel=linel+cil
      line(linel+1:linel+7)=' errors'
      linel=linel+7
      wrnct=0
      if(curprb.gt.0 .and. curprb.le.errmax)then
         call inttos(curprb,ci,cil)
         line(linel+1:linel+2+cil+12)=' ('//ci(1:cil)//' is above), '
         linel=linel+2+cil+12
      else
         line(linel+1:linel+2)=', '
         linel=linel+2
         wrn=curprb-errmax
         if(wrn.gt.0 .and. wrn.le.wrnmax)then
            do 7920 wrn2=1,wrn
               if(wrns(wrn2).le.smaxw)wrnct=wrnct+1
 7920       continue
            if(wrns(wrn).gt.smaxw)wrnct=-(wrn-wrnct)
         endif
      endif
      call inttos(badwrn,ci,cil)
      line(linel+1:linel+cil)=ci(1:cil)
      linel=linel+cil
      line(linel+1:linel+15)=' major warnings'
      linel=linel+15
      if(wrnct.gt.0)then
         call inttos(wrnct,ci,cil)
         line(linel+1:linel+2+cil+16)=
     &    ' ('//ci(1:cil)//' is above), and '
         linel=linel+2+cil+16
      else
         line(linel+1:linel+6)=', and '
         linel=linel+6
      endif
      call inttos(wrnmax-badwrn,ci,cil)
      line(linel+1:linel+cil)=ci(1:cil)
      linel=linel+cil
      line(linel+1:linel+15)=' minor warnings'
      linel=linel+15
      if(wrnct.lt.0)then
         call inttos(-wrnct,ci,cil)
         line(linel+1:linel+2+cil+11)=' ('//ci(1:cil)//' is above).'
         linel=linel+2+cil+11
      else
         line(linel+1:linel+1)='.'
         linel=linel+1
      endif
      if(nxtlin(screen,linect,lintot).eq.1)goto 8000
      print2,line(1:min(linelt,linel))

c     warn about problems that were not found
      if(notfnd.ne.0)then
         if(nxtlin(screen,linect,lintot).eq.1)goto 8000
         if(task.eq.all)print2,'** There are no problems to display'
         if(task.eq.prev)print2,'** There is no previous problem'
         if(task.eq.next)print2,'** There is no next problem'
         if(task.eq.nextd)
     &    print2,'** There is no next different problem'
         if(task.eq.nextt)
     &    print2,'** There is no next different type of problem'
      endif

c     warn about program problems, if any
      if(prgerr.ne.0)then
         if(nxtlin(screen,linect,lintot).eq.1)goto 8000
         print2,'*** Log file exceeds program limits: '//
     &    'there might be additional errors!'
      endif

c     finish the screen
 8000 do 8010 i=lintot+1,linmin-1
         print2
 8010 continue
      if(screen.lt.0)print2,' '//
     & '---------------------------------------'//
     & '---------------------------------------'

c write the summary file

      call opennw(2,argch(3)(1:arglt(3)-3)//'@rl')

      call inttos(curprb,ci,cil)
      write(2,2,err=9510)'Selected problem: '//ci(1:cil)//' '

      call inttos(errmax+wrnmax-curprb,ci,cil)
      write(2,2,err=9510)'Problems left: '//ci(1:cil)

      call inttos(errmax,ci,cil)
      write(2,2,err=9510)'Errors: '//ci(1:cil)

      call inttos(badwrn,ci,cil)
      write(2,2,err=9510)'Major warnings: '//ci(1:cil)

      call inttos(wrnmax-badwrn,ci,cil)
      write(2,2,err=9510)'Minor warnings: '//ci(1:cil)

      if(prgerr.ne.0)then
         if(logmax.ge.logdim)write(2,2,err=9510)
     &    '*** Too many characters in log file '//argch(3)(1:arglt(3))
         if(lwmax.ge.lwdim)write(2,2,err=9510)
     &    '*** Problem descriptions have too many words in log file '//
     &    argch(3)(1:arglt(3))
         if(lmax.ge.ldim)write(2,2,err=9510)
     &    '*** Too many lines in log file '//argch(3)(1:arglt(3))
         if(opnmax.ge.opndim)write(2,2,err=9510)
     &    '*** Too many unclosed parentheses in log file '//
     &    argch(3)(1:arglt(3))
         if(errmax.ge.errdim)write(2,2,err=9510)
     &    '*** Too many error messages in log file '//
     &    argch(3)(1:arglt(3))
         if(wrnmax.ge.wrndim)write(2,2,err=9510)
     &    '*** Too many warnings in log file '//argch(3)(1:arglt(3))
      endif

      if(rerun.ne.0)write(2,2,err=9510)'Rerun latex'

      if(pagel.gt.0)write(2,2,err=9510)page(1:pagel)

c     close file
      close(2,err=9520)

c done

c     exit
 8900 call exit(0)

c error messages

 9000 call fatal('Usage: readlog - data_path log '//
     & '{ini|all|prev|next|nextdiff|nexttype|same} lines')

 9010 call fatal('Error looking for '//argch(3)(1:arglt(3)-3)//'@rl')
 9015 call fatal('Unable to open file '//argch(3)(1:arglt(3)-3)//'@rl')
 9020 call fatal('Error reading file '//argch(3)(1:arglt(3)-3)//'@rl')
 9023 do 9025 i=80,1,-1
         if(line(i:i).gt.' ')call fatal('File '//argch(3)(1:arglt(3)-3)
     &    //'@rl is corrupt; first line is: '//line(1:i))
 9025 continue
      call fatal('File '//argch(3)(1:arglt(3)-3)//
     & '@rl is corrupt; first line is blank')
 9030 call fatal('Error closing file '//argch(3)(1:arglt(3)-3)//'@rl')

 9110 call fatal('Error opening file "'//
     & argch(2)(1:arglt(2))//'readlog.ind"')
 9120 call fatal('Error reading file "'//
     & argch(2)(1:arglt(2))//'readlog.ind"')
 9121 call fatal('Premature end to file "'//
     & argch(2)(1:arglt(2))//'readlog.ind"')
 9130 call fatal('Error closing file "'//
     & argch(2)(1:arglt(2))//'readlog.ind"')

 9210 call fatal('Unable to open log file "'//argch(3)(1:arglt(3))//'"')
 9220 call fatal('Error reading log file "'//argch(3)(1:arglt(3))//'"')
 9230 call fatal('Error closing log file "'//argch(3)(1:arglt(3))//'"')

 9510 call fatal('unable to write to summary file '//
     & argch(3)(1:arglt(3)-3)//'@rl')
 9520 call fatal('unable to close summary file '//
     & argch(3)(1:arglt(3)-3)//'@rl')

      end



      subroutine findit(
     & sdim,smax,sexbeg,sexend,sinc,sver,send,
     & swdim,swpat,swpos,swlt,
     & scch,
     & wdim,wmax,wpos,wlt,logch,
     & s)

c     Find an error or warning message in the current line.

c     avoid typos
      implicit none

c input

c     declared number of search strings
      integer sdim
c     actual number of search strings
      integer smax
c     locations where the '!' errors begin and end
      integer sexbeg,sexend
c     nonzero if the string is incomplete
      integer sinc(sdim)
c     1 if the string is verified to end/ not end in a point; or else 0 or -1
      integer sver(sdim)
c     end of the string in the sw database
      integer send(0:sdim)

c     declared number of words in the database of search string words
      integer swdim
c     pattern indicator
c     0: literal string
c     1: anything goes
c     2: check in detail whether the pattern fits the error string word:
c        ASCII 0 in the pattern is ignored.
c        ASCII 1 at the end of the pattern matches the rest of the word.
c        ASCII 1 otherwise matches all except the character following ASCII 1.
c        ASCII 2 matches a nonnegative number.
c        Any other pattern character matches the same character in the word.
      integer swpat(swdim)
c     word location in scch
      integer swpos(swdim)
c     word length
      integer swlt(swdim)

c     the search string characters
      character*(*) scch

c     declared number of words in the current line
      integer wdim
c     actual number of words
      integer wmax
c     word position in the line
      integer wpos(wdim)
c     word length
      integer wlt(wdim)
c     the line in the log file
      character*(*) logch

c output

c     search string index, or 0 if not found
      integer s

c local

c     search string word index
      integer sw,sw0,sw1,sw2

c     search string character index
      integer sc,sc1,sc2

c     log file line word index
      integer w

c     log file line character index
      integer log,log1,log2

c     whether the first character is !
      integer isex

c     whether the last character is a point
      integer haspt

c     ascii values of characters in the search string and log
      integer asc,ascl
c     ascii values of zero and 9
      integer asc0,asc9
      parameter (asc0=48,asc9=57)

c executable

c     debug: print out what we are looking for
c10   format(a,' ',a,' ',a,' ',a,' ',a,' ',a,' ',a,' ',a,' ',a,' ',a)
c     print*,'findit:'
c     print10,(logch(wpos(w):wpos(w)+wlt(w)-1),w=1,wmax)

c     check that there is something to search for
      if(wmax.le.0) goto 900

c     see whether it starts with !
      isex=0
      if(wlt(1).eq.1)then
         if(logch(wpos(1):wpos(1)).eq.'!')then
            isex=1
         endif
      endif

c     see whether the last character is a point
      haspt=0
      log=wpos(wmax)+wlt(wmax)-1
      if(logch(log:log).eq.'.')haspt=1

c     loop over the search strings
      s=0
      if(isex.ne.0)s=sexbeg-1
 100  s=s+1
      if(isex.eq.0)then
         if(s.eq.sexbeg)s=sexend+1
         if(s.gt.smax)goto 900
      else
         if(s.gt.sexend)goto 900
      endif

c     find the first and last word
      sw0=send(s-1)
      sw2=send(s)

c see whether the string can be rejected without doing character ops

c     check the number of words
      if(sinc(s).eq.0)then
         if(sw2-sw0.ne.wmax)goto 100
      else
         if(sw2-sw0.gt.wmax)goto 100
      endif
      sw1=sw0+1

c     check the word lengths
      do 200 sw=sw1,sw2-1
         if(swlt(sw).ne.wlt(sw-sw0) .and. swpat(sw).eq.0)goto 100
 200  continue
      if(swpat(sw2).eq.0)then
         if(sver(s).eq.1 .or. sw-sw0.lt.wmax)then
            if(swlt(sw2).ne.wlt(sw2-sw0))goto 100
         else
            if(swlt(sw2)-(sver(s)+1).ne.wlt(sw2-sw0)-haspt)goto 100
         endif
      endif

c crunch it out

c     loop over the words and see whether they match
      do 500 sw=sw1,sw2
         sc1=swpos(sw)
         sc2=sc1+swlt(sw)-1
         w=sw-sw0
         log1=wpos(w)
         log2=log1+wlt(w)-1

c        drop any uncertain final point
         if(w.eq.wmax)then
            if(sver(s).ne.1)then
               if(sver(s).eq.0)sc2=sc2-1
               if(haspt.eq.1)log2=log2-1
               if(log2.lt.log1)then
                  if(scch(sc1:sc2).ne.char(0)//char(0)//char(1))goto 100
                  goto 500
               endif
            endif
         endif

         if(swpat(sw).eq.0)then

c           compare the words literally
            if(scch(sc1:sc2).ne.logch(log1:log2))goto 100

         else

c           OK if anything goes
            if(swpat(sw).eq.1)goto 500

c           start looking at the individual characters
            sc=sc1-1
            log=log1-1

c           next pattern character
 300        sc=sc+1
            if(sc.gt.sc2)goto 390

c           get ascii value of the character
            asc=ichar(scch(sc:sc))

c           a normal character must be matched
            if(asc.ge.3)then
               if(log.ge.log2)goto 100
               log=log+1
               if(logch(log:log).ne.scch(sc:sc))goto 100
               goto 300
            endif

c           char(0) is ignored
            if(asc.eq.0)goto 300

c           char(2) gobbles up one or more following digits
            if(asc.eq.2)then
               if(log.ge.log2)goto 100
               log=log+1
               ascl=ichar(logch(log:log))
               if(ascl.lt.asc0 .or. ascl.gt.asc9)goto 100
 350           if(log.ge.log2)goto 300
               ascl=ichar(logch(log+1:log+1))
               if(ascl.ge.asc0 .and. ascl.le.asc9)then
                  log=log+1
                  goto 350
               endif
               goto 300
            endif

c           char(1) at the end of the word gobbles up all the rest
            if(sc.eq.sc2)goto 500

c           char(1) gobbles up everything unequal to the following character
            asc=ichar(scch(sc+1:sc+1))
 370        if(log.ge.log2)goto 300
            ascl=ichar(logch(log+1:log+1))
            if(ascl.ne.asc)then
               log=log+1
               goto 370
            endif
            goto 300

c           if there are characters left in the word, it does not match
 390        if(log.lt.log2)goto 100

         endif

 500  continue

c     everything matches
c     print*,'found s:',s
      return

c     failed to find it
 900  s=0
c     print*,'not found'
      return
      end


      function nxtlin(screen,linect,lintot)

c     Start the next line of terminal output.

c     avoid typos
      implicit none

c input

c     number of screen lines or 0 for any.
c     if negative, the "more" line will be erased after the prompt
      integer screen

c input/output

c     screen line count
      integer linect

c     total lines written
      integer lintot

c output

c     0 to continue or 1 to exit program
      integer nxtlin

c local

c     whether we initialized
      integer init
      data init/0/
      save init

c     prompt line
      character*80 line

c     user answer
      character*1 ans

c executable

c     format to write without new line
 10   format(a,$)

c     initialize
      if(init.eq.0)then
c        zero the line counts
         linect=0
         lintot=0
c        done initializing
         init=1
      endif

c     assume no exit requested
      nxtlin=0

c     increment the line counts
      linect=linect+1
      lintot=lintot+1

c     when screen is zero, nothing else is needed
      if(screen.eq.0)return

c     done if not at the last line
      if(linect.lt.abs(screen))return

c     if we reached the last available line, ask whether to continue

c     set the prompt line
      line='Hit Space or m for more, or q to quit: m'

c     get the user's key
      call inconk(0,line,ans)

c     process answer
 100  if(index(' mMqQ',ans).ne.0)then
         if(screen.ge.0)then
            call inconk(2,line,ans)
         else
            print10,
     &       char(13)//'                                        '//
     &       '                                       '//char(13)
         endif
         if(index('qQ',ans).ne.0)nxtlin=1
         linect=1
         return
      else
         line=
     &    'Hit Space or m for more, or q to quit (key was invalid): m'
         call inconk(1,line,ans)
         goto 100
      endif

      end




      subroutine hiderp(
     & logch,
     & lwdim,lwlogo,lwlogl,
     & swdim,swpat,
     & sdim,snorpt,send,
     & idim,i,is,iprv,ilwo,ilwe,ihid)

c     Compare problem i to earlier problems of the same type and see
c     whether there is an earlier problem i2 that is "equivalent" (as
c     determined by snorpt) to problem i.

c     More precisely if ihid(i) is 1 (i.e. hide the problem), it stays
c     1.  Else if there is a previous equivalent problem i2 with
c     ihid(i2) not equal to 1, then ihid(i) is set to 1 and ihid(i2) to
c     -1 (i.e. show the problem but not the next ones).

c     avoid typos
      implicit none

c input

c     the log file characters
      character*(*) logch

c     declared number of words in the database of log words
      integer lwdim
c     word offset and length in logch
      integer lwlogo(lwdim),lwlogl(lwdim)

c     declared number of words in the database of search string words
      integer swdim
c     pattern indicator
c     0: literal string
c     1: anything goes
c     2: check in detail:  ASCII 0-2 are special, see findit for details
      integer swpat(swdim)

c     declared number of search strings
      integer sdim
c     problems at this s are equivalent if snorpt=-1 or snorpt>0 words are equal
      integer snorpt(sdim)
c     end of the string in the sw database
      integer send(0:sdim)

c     declared number of problems
      integer idim
c     problem index
      integer i
c     number of the problem
      integer is(idim)
c     location of the previous problem of this type
      integer iprv(idim)
c     problem offset and end in the log words database
      integer ilwo(idim),ilwe(idim)

c input/output

c     whether the problem is hidden
      integer ihid(idim)

c local

c     word offsets in lw
      integer lwo,lwo2

c     word offsets and lengths in logch
      integer logo,logo2,logl

c     search string word offset
      integer swo

c     search string index
      integer s

c     comparison problem index
      integer i2

c     word number
      integer w

c executable

c     if snorpt is zero, problems of this type are never equivalent.
      s=is(i)
      if(snorpt(s).eq.0)goto 900

c     if the problem is already hidden, it stays hidden
      if(ihid(i).eq.1)goto 900

c     find the previous problem of this type
      i2=i
 100  i2=iprv(i2)

c     no action if there is no previous problem of this type
      if(i2.le.0)goto 900

c     ignore problems already marked hidden
      if(ihid(i2).eq.1)goto 100

c     if snorpt is -1, all problems of this type are equivalent
      if(snorpt(s).eq.-1)goto 800

c     otherwise see whether the first snorpt words are equal
      swo=send(s-1)
      lwo=ilwo(i)
      lwo2=ilwo(i2)
      do 500 w=1,snorpt(s)
         if(swpat(swo+w).eq.0)goto 500
         logl=lwlogl(lwo+w)
         if(logl.ne.lwlogl(lwo2+w))goto 100
         logo=lwlogo(lwo+w)
         logo2=lwlogo(lwo2+w)
         if(logch(logo+1:logo+logl).ne.logch(logo2+1:logo2+logl))
     &    goto 100
 500  continue

c     mark the problems as equivalent
 800  ihid(i)=1
      ihid(i2)=-1

 900  return
      end


      subroutine getchr(iunit,chr,*,*)

c     Gets the next character from the input file open on unit iunit.

c     Takes exit 1 on end of file and 2 on other error.

c     This version for g77.  Unit iunit must have been opened using
c     "form='unformatted'".

c     avoid typos
      implicit none

c input:

c     I/O unit to read from
      integer iunit

c output:

c     read character
      character*1 chr

c local:

c     error status
      integer stat

c executable:

c     read a character from the file
      call fgetc(iunit,chr,stat)

c     check for problems
      if(stat.eq.-1)then
c        end of file
         return 1
      else if(stat.ne.0)then
c        other error
         return 2
      endif

c     all OK
      return

      end
