      program sellist

c     Ask the user to choose an item from a list.  Puts the selected
c     item in a file.  Optionally adds a line
c        echon.exe "ITEM"
c     where ITEM is the selected item truncated or padded to a fixed
c     number of characters.

c     Items can be up to 79 characters long, but only 22 will be shown
c     in the menu.  Watch for characters that are special to the DOS or
c     tcsh shells, *including trailing digits in DOS*.

c     Usage: sellist.exe "ROOT" "TYPE" "DEFAULT" CHARS [FIELD SEP]
c            ROOT is the root input and output file specification
c               The list of items must be in ROOT.lst
c               The selected item will be in ROOT.tmp
c                  If quit is selected no ROOT.tmp is created
c            TYPE is the type of items, eg. "language" or "theme"
c            DEFAULT is the default; specify a space for none
c            CHARS, if nonzero, is the number of characters echoed.
c            FIELD if nonblank is the field of the list items to select.
c                It can be a number from 1 to 9.
c                Fields are numbered from right to left
c            SEP are the field separators if FIELD is specified.
c            Blank or missing fields are ignored.

c     avoid typos
      implicit none

c local

c     declared number of arguments
      integer argdim
      parameter (argdim=6)
c     actual number of arguments
      integer argmx
c     arguments
      character*80 argt(argdim)
      integer argtl(argdim)

c     number of characters specified
      integer chars

c     selectable items
      integer sdim,smax,s
      parameter (sdim=460)
      character*80 sit(sdim)
      integer sitl(sdim)

c     screen lines
      integer l
      character*79 lin(18)

c     menu counter
      integer menu,menumx

c     keys
      integer key,keymax
      character*62 keychr
      integer keys(62)
c     default key character
      character*1 def

c     prompt line
      character*80 line

c     input line
      character*256 inlin

c     field
      integer fld,fld2

c     ascii code and the value of 'a'
      integer asc,asca

c     generic character
      character*1 chr

c     generic indices
      integer i,i1,i2

c executable

c     formats
 10   format(a,a,a)
 20   format(79a1)

c     set the ascii code for 'a'
      asca=ichar('a')

c     allow I/O to use units 7 and on
      call libini(7)

c     set the error exit status to 1
      call errset(1)

c     get the command line arguments
      call args(argdim,argmx,argt,argtl)
      if(argmx.lt.4)goto 990
      if(argmx.eq.5)goto 990
      if(argt(1)(1:argtl(1)).eq.' ')
     & call out('*** sellist: Program error: ROOT cannot be blank!')
      if(argt(2)(1:argtl(2)).eq.' ')
     & call out('*** sellist: Program error: TYPE cannot be blank!')
      if(argtl(2).gt.29)
     & call out('*** sellist: Program error: TYPE is too long!')
      if(argtl(3).gt.79)
     & call out('*** sellist: Program error: DEFAULT is too long!')
      chars=0
      do 50 i=1,argtl(4)
         if(argt(4)(i:i).lt.'0' .or. argt(4)(i:i).gt.'9')call out(
     &    '*** sellist: Program error: CHARS must be a number!')
         chars=chars*10+ichar(argt(4)(i:i))-48
         if(chars.gt.79)
     &    call out('*** sellist: Program error: CHARS is too big!')
 50   continue
      if(argmx.ge.5)then
         if(argt(5)(1:argtl(5)).eq.' ')argmx=4
      endif
      fld=0
      if(argmx.eq.6)then
         if(argtl(5).gt.2)goto 990
         if(argtl(5).eq.2 .and. argt(5)(2:2).ne.'-')goto 990
         fld=ichar(argt(5)(1:1))-48
         if(fld.lt.1 .or. fld.gt.9)goto 990
         if(argt(6)(1:argtl(6)).eq.' ')call out(
     &    '*** sellist: SEP cannot be blank if FIELD is specified!')
      endif

c     read the input file
      open(1,file=argt(1)(1:argtl(1))//'.lst',status='old',err=910)
      s=0
 100  read(1,110,end=190,err=920)inlin
 110  format(a256)
c     get rid of trailing spaces
      i1=1
      i2=256
      do 120 i=256,1,-1
         if(inlin(i:i).gt.' ')goto 121
         i2=i-1
 120  continue
      call out('*** sellist: There is a blank item in the list!')
 121  continue
      if(fld.ne.0)then
         fld2=1
         if(fld2.eq.fld .and. argtl(5).eq.2)goto 131
         do 130 i=i2,1,-1
            if(index(argt(6)(1:argtl(6)),inlin(i:i)).le.0)goto 130
            fld2=fld2+1
            if(fld2.gt.fld)then
               i1=i+1
               goto 131
            endif
            i2=i-1
            if(fld2.eq.fld .and. argtl(5).eq.2)goto 131
 130     continue
         if(fld2.lt.fld)goto 100
 131     continue
      endif
      if(i2-i1+1.le.0)goto 100
      if(i2-i1+1.gt.79)call out(
     & '*** sellist: An item in the list is too long!')
      s=s+1
      if(s.gt.sdim)call out('*** sellist: Too many menu items!')
      sitl(s)=i2-i1+1
      sit(s)=inlin(i1:i2)
c     i=sitl(s)
c     if(index(sit(s)(1:i),'\\').gt.0)goto 9030
c     if(index(sit(s)(1:i),'>').gt.0)goto 9030
c     if(index(sit(s)(1:i),'<').gt.0)goto 9030
c     if(index(sit(s)(1:i),'&').gt.0)goto 9030
c     if(index(sit(s)(1:i),'|').gt.0)goto 9030
c     if(index(sit(s)(1:i),'%').gt.0)goto 9030
c     if(index(sit(s)(1:i),'"').gt.0)goto 9030
c     if(index(sit(s)(1:i),'''').gt.0)goto 9030
c     if(index(sit(s)(1:i),'!').gt.0)goto 9030
c     if(sit(s)(i:i).ge.'0' .and. sit(s)(i:i).le.'9')goto 9030
      goto 100
 190  close(1)
      smax=s
      if(smax.lt.1)call out('*** sellist: No menu items to select!')

c     number of menus
      menumx=(smax+51-1)/51
      if(menumx.gt.9)call out('*** sellist: too many menus!')

c     initialize
      menu=1
      s=0

c     write the current menu
 200  def=' '
      s=(menu-1)*51
      key=0
      asc=asca
      do 210 l=1,18
         lin(l)=' '
         s=s+1
         if(s.le.smax)then
            key=key+1
            keys(key)=s
            keychr(key:key)=char(asc)
            if(sit(s)(1:sitl(s)).eq.argt(3)(1:argtl(3)))def=char(asc)
            lin(l)(1+1:1+3)=char(asc)//': '
            lin(l)(4+1:1+25)=sit(s)
            if(asc.ge.asca)then
               asc=asc-32
            else
               asc=asc+32+1
            endif
            if(asc.eq.ichar('q'))asc=asc-32
         endif
 210  continue
      do 220 l=1,18
         s=s+1
         if(s.le.smax)then
            key=key+1
            keys(key)=s
            keychr(key:key)=char(asc)
            if(sit(s)(1:sitl(s)).eq.argt(3)(1:argtl(3)))def=char(asc)
            lin(l)(1+27:27+3)=char(asc)//': '
            lin(l)(4+27:27+25)=sit(s)
            if(asc.ge.asca)then
               asc=asc-32
            else
               asc=asc+32+1
            endif
            if(asc.eq.ichar('q'))asc=asc-32
         endif
 220  continue
      do 230 l=1,15
         s=s+1
         if(s.le.smax)then
            key=key+1
            keys(key)=s
            keychr(key:key)=char(asc)
            if(sit(s)(1:sitl(s)).eq.argt(3)(1:argtl(3)))def=char(asc)
            lin(l)(1+53:53+3)=char(asc)//': '
            lin(l)(4+53:53+26)=sit(s)
            if(asc.ge.asca)then
               asc=asc-32
            else
               asc=asc+32+1
            endif
            if(asc.eq.ichar('q'))asc=asc-32
         endif
 230  continue
      if(menumx.gt.1)then
         if(menu.gt.1)then
            lin(16)(1+53:79)='1: first screen'
            key=key+1
            keys(key)=0
            keychr(key:key)='1'
         else
            lin(16)(1+53:79)='-- first screen'
         endif
         lin(17)(1+53:79)=' : next screen'
         key=key+1
         keys(key)=0
         keychr(key:key)=' '
         do 240 i=2,menumx
            if(i.eq.menu)goto 240
            key=key+1
            keys(key)=0
            keychr(key:key)=char(i+48)
 240     continue
         line='or press Space for the next screen or q to quit: '//def
      else
         line='or press q to quit: '//def
      endif
      lin(18)(1+53:79)='q: quit'
      key=key+1
      keys(key)=0
      keychr(key:key)='q'
      keymax=key

c     write menu
      print10
      print10
      print10
      print20,('=',i=1,79)
      i=79-10-argtl(2)
      print20,(' ',i=1,i/2),(argt(2)(i:i),i=1,argtl(2)),' ',
     & 's','e','l','e','c','t','i','o','n'
      print10
      do 300 l=1,18
         do 250 i=79,1,-1
            if(lin(l)(i:i).gt.' ')then
               print10,lin(l)(1:i)
               goto 300
            endif
 250     continue
         print10
 300  continue
      print10
      print10,'Press the *case sensitive* letter of the desired '//
     & argt(2)(1:argtl(2))//','

c     get the response by the user
      call inkey(4,keychr(1:keymax),line,chr)

c     process it
      if(chr.eq.'q')call exit(0)
      if(chr.eq.' ')then
         menu=menu+1
         if(menu.gt.menumx)menu=1
         goto 200
      endif
      if(chr.ge.'1' .and. chr.le.'9')then
         menu=min(menumx,ichar(chr)-48)
         goto 200
      endif
      key=index(keychr(1:keymax),chr)
      if(key.le.0)call out('*** sellist: Program error: bad key!')
      s=keys(key)
      if(s.lt.1 .or. s.gt.smax)
     & call out('*** sellist: Program error: bad item!')

c     write the key to the output file
      open(2,file=argt(1)(1:argtl(1))//'.tmp',err=960)
      write(2,10,err=970)sit(s)(1:sitl(s))
      if(chars.gt.0)then
         if(sit(s)(chars:chars).eq.' ')then
            write(2,10,err=970)'echon.exe "'//sit(s)(1:chars)//'*"'
         else
            write(2,10,err=970)'echon.exe "'//sit(s)(1:chars)//'"'
         endif
      endif
      close(2,err=980)

c     done
      call exit(0)

c     errors
 910  call out('*** sellist: File '//
     & argt(1)(1:argtl(1))//'.lst not found!')
 920  call out('*** sellist: File '//
     & argt(1)(1:argtl(1))//'.lst read error!')
 960  call out('*** sellist: File '//
     & argt(1)(1:argtl(1))//'.tmp cannot be created!')
 970  call out('*** sellist: File '//
     & argt(1)(1:argtl(1))//'.tmp cannot be written to!')
 980  call out('*** sellist: File '//
     & argt(1)(1:argtl(1))//'.tmp cannot be finished!')
 990  call out(
     & 'Usage: sellist.exe "ROOT" "TYPE" "DEFAULT" CHARS [FIELD SEP]')
      call exit(1)
      end
