      program reconc

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     Reconcile the tags and whitespace in NEWFILE with that in OLDFILE.
c     This uses a quasi-probabilistic approach.

c     Usage: reconcile.exe OLDFILE NEWFILE RCNFILE [d]
c            OLDFILE is the enc file produced by tex_enc.sub.
c            NEWFILE is the same file mangled by the spell/grammar checker.
c            RCNFILE is the reconciled file to be created.

c     Note that ASCII characters other than &, <, and > should not be
c     html-encoded.

c declarations

c     avoid typos
      implicit none

c local

c     rough fraction of words changed in the spell/grammar check
      real fchang
      parameter (fchang=0.1)
c     rough fraction of words removed in the spell/grammar check
      real fremov
      parameter (fremov=0.05)

c     maximum iterations to find what old word corresponds to what new one
      integer itermx
      parameter (itermx=100)
c     iteration at which the last change occurred
      integer iterch
c     iteration index
      integer iter

c     character buffer sizes, OLDFILE and NEWFILE
      integer codim,cndim
      parameter (codim=8000000,cndim=200000)
c     actual character buffer sizes
      integer comax,cnmax
c     character indices
      integer co,co0,co2,cn,cn0,cn2
c     characters
      character*1 coch(codim),cnch(cndim)

c     declared number of paragraphs in OLDFILE
      integer podim
      parameter (podim=200000)
c     actual number of paragraphs
      integer pomax
c     paragraph index
      integer po
c     paragraph numbers
      integer ponum(podim)
c     paragraph start characters
      integer poco(podim)
c     paragraph number in NEWFILE
      integer pnnum

c     declared number of words in the paragraph, OLDFILE and NEWFILE
      integer wodim,wndim
      parameter (wodim=30000,wndim=30000)
c     actual number of words in the paragraph
      integer womax,wnmax
c     word indices
      integer wo,wo0,wo2,woinit,wolast,wostep,wn,wn0,wn2,wnold
c     whether the word is a ^N sequence
      integer wo_n(wodim),wn_n(wndim)
c     word start and end in the file buffer
      integer wobeg(0:wodim+1),woend(0:wodim+1)
      integer wnbeg(0:wndim+1),wnend(0:wndim+1)
c     start and end of the inner word (excluding surrounding punctuation)
      integer wobegi(wodim),woendi(wodim)
      integer wnbegi(wndim),wnendi(wndim)
c     word length of the inner word in characters
      integer wolt(wodim),wnlt(wndim)
c     assignment from wo to wn and vice-versa
      integer wown(0:wodim+1),wnwo(0:wndim+1)
c     nonzero if assigned to an interstitial location (in between wn-1 and wn)
      integer wowni(0:wodim+1),wniold
c     nonzero if the assigned words are equal
      integer wowne(0:wodim+1),wnwoe(0:wndim+1)
c     probability of good assigment
      real wog(0:wodim+1),wng(0:wndim+1)
c     pointers to the next item to print
      integer wonext(0:wodim+1),wnnext(0:wndim+1)
c     offset of the wo values in the pointers
      integer wooff

c     declared number of encoded words
      integer edim
      parameter (edim=wodim+wndim)
c     actual number of encoded words
      integer emax
c     encoded word index
      integer e
c     location of the encoded word in the encoded words buffer
      integer ebuf(edim)
c     index of the sorted encoded words
      integer es,es2
c     assignment from e to es and vice-versa
      integer ees(edim),ese(edim)
c     first sorted encoded word that the current word is equal to
      integer eses0(edim),es0,es02
c     word multiplicities in terms of wo and wn
      integer es0mo(edim),es0mn(edim),mo,mo2,mn,mn2

c     declared number of integers in the encoded words buffer
      integer bufdim
      parameter (bufdim=cndim)
c     buffer integer index
      integer buf,buf1,buf2
c     the integers
      integer bufi(bufdim)
c     multiplier of the next character in an integer
      integer mul
c     multiplier of the last character in an integer
      integer mulend
      parameter (mulend=256**3)

c     command line arguments
      integer argdim
      parameter (argdim=4)
      integer argmx
      character*132 argt(argdim)
      integer argtl(argdim)
      external args

c     read mode
      integer rdmode
c     reading the paragraph marker
      integer parmrk
      parameter (parmrk=1)
c     reading whitespace
      integer white
      parameter (white=2)
c     simply copying the unmatched paragraph
      integer copy
      parameter (copy=3)
c     reading the punctuation at the beginning of a word
      integer punctb
      parameter (punctb=4)
c     reading the actual word
      integer inword
      parameter (inword=5)
c     reading the punctuation at the end of a word
      integer puncte
      parameter (puncte=8)

c     character type
      integer chrtyp
c     possible values besides 0 (ASCII)
      integer htmlch,utf8ch
      parameter (htmlch=1,utf8ch=2)

c     maximum unicode character number
      integer uclim
      parameter (uclim=1114111)
c     number of unicode characters that we keep data on
      integer ucdim
      parameter (ucdim=65535)
c     data (1: uses html, 2: uses utf-8; punctuation: 4=opening 8=closing)
      integer ucdata(0:ucdim)
      data ucdata/65536*0/
c     unicode character number and previous value
      integer uc,uc0

c     character in utf8 format
      character*1 utf8(4)
      integer utf8l
c     number of bytes in the utf-8 string
      integer bytes

c     ascii characters
      integer asc_tb,asc_lf,asc_n,asc_p,asc_sp
      parameter (asc_tb=9,asc_lf=10,asc_n=14,asc_p=16,asc_sp=32)
      integer asc_am,asc_pd,asc_pt,asc_sc,asc_bs

c     number of erroneous characters in the header
      integer errhd
c     number of missing paragraphs
      integer pargon
c     number of duplicated paragraphs
      integer pardup
c     number of paragraphs that pop up out of nothing
      integer parpop
c     number of missing latex strings
      integer latgon
c     number of latex strings that pop up out of nothing
      integer latpop

c     estimated probability that the old word is unchanged
      real psame
c     probability that the old word is changed
      real pchang
c     probability that the old word is removed
      real premov
c     probability of a word match being right before considering context
      real pbare
c     probability of a word match being right
      real p,p2
c     best probability so far
      real pbest
c     sum of estimated probabilities
      real psum

c     whether we are in debug mode
      integer debug

c     number as character string
      character*20 cnum
      integer cnuml

c     character number
      integer ichr

c     character count
      integer cnt

c     position of the last ^N character
      integer pos_n

c     sort interval
      integer int

c     nonzero if a change occurred
      integer change

c     real temporary
      real temp

c     generic integers
      integer i,i1,i2

c executable

c     generic format
 10   format(a,i10,a,i10)
 20   format(79a1)
 30   format(a,10a1,a)

c     set ascii codes
      asc_am=ichar('&')
      asc_pd=ichar('#')
      asc_pt=ichar('.')
      asc_sc=ichar(';')
      asc_bs=ichar('\\')

c     set the recognized punctutation characters
      ucdata(ichar('!'))=puncte
      ucdata(ichar('"'))=punctb+puncte
      ucdata(ichar(''''))=punctb+puncte
      ucdata(ichar('('))=punctb
      ucdata(ichar(')'))=puncte
      ucdata(ichar(','))=puncte
      ucdata(ichar('.'))=puncte
      ucdata(ichar(':'))=puncte
      ucdata(ichar(';'))=puncte
      ucdata(ichar('?'))=puncte
      ucdata(ichar('['))=punctb
      ucdata(ichar(']'))=puncte
      ucdata(ichar('`'))=punctb
      ucdata(ichar('{'))=punctb
      ucdata(ichar('}'))=puncte
      ucdata(8216)=punctb
      ucdata(8217)=puncte
      ucdata(8220)=punctb
      ucdata(8221)=puncte
      do 110 i=0,37
         ucdata(i)=ucdata(i)+utf8ch
 110  continue
      ucdata(38)=ucdata(38)+htmlch
      do 120 i=39,59
         ucdata(i)=ucdata(i)+utf8ch
 120  continue
      ucdata(60)=ucdata(60)+htmlch
      ucdata(61)=ucdata(61)+utf8ch
      ucdata(62)=ucdata(62)+htmlch
      do 130 i=63,127
         ucdata(i)=ucdata(i)+utf8ch
 130  continue



c     get the command line arguments
      call args(argdim,'Usage: reconcile.exe OLDFILE NEWFILE RCNFILE',
     & argmx,argt,argtl)
      if(argmx.lt.3)goto 9990
      debug=0
      if(argmx.eq.4)then
         if(argt(4)(1:argtl(4)).ne.'d')goto 9990
         debug=1
      endif

c     read in OLDILE

c     open the file
      call openru(argt(1)(1:argtl(1)),1)

c     initialize
      po=0
      co=0
      rdmode=0
      chrtyp=0
      uc=0
      cnum='321000000.'//char(asc_lf)//char(asc_p)

c     read the next character
 1000 co0=co
 1010 co=co+1
      if(co.gt.codim)goto 9911
      call getchr(1,coch(co),*1900,*9910)
      if(co.le.12)then
         if(coch(co).ne.cnum(co:co))call fatal('Reconcile: '//
     &    'corrupt header in '//argt(1)(1:argtl(1)))
         if(co.le.11)goto 1000
      endif
      ichr=ichar(coch(co))

c     see how to find the unicode value
      if(chrtyp.eq.0)then
         uc0=uc
         uc=ichr
c        see whether we need to read a html string
         if(ichr.eq.asc_am)then
            chrtyp=htmlch
            cnt=1
            goto 1010
         endif
c        see whether we need to read a utf-8 string
         if(ichr.ge.128)then
            chrtyp=utf8ch
            cnt=1
            bytes=1
            if(ichr.ge.128+64)bytes=2
            if(ichr.ge.128+64+32)bytes=3
            if(ichr.ge.128+64+32+16)bytes=4
            if(ichr.ge.128+64+32+16+8)bytes=5
            if(bytes.lt.2 .or. bytes.gt.4)call fatal('Reconcile: '//
     &       'corrupt UTF-8 character in '//argt(1)(1:argtl(1)))
            goto 1010
         endif
      endif

c     handle case that we are in a decimal html string
      if(chrtyp.eq.htmlch)then
         cnt=cnt+1
         if(cnt.eq.2)then
            if(ichr.ne.asc_pd)call fatal('Reconcile: '//
     &       'corrupt or nondecimal html in '//argt(1)(1:argtl(1)))
            uc=0
            goto 1010
         endif
         i=ichr-48
         if(i.ge.0 .and. i.le.9)then
            uc=uc*10+i
            if(uc.gt.uclim)call fatal('Reconcile: '//
     &       'corrupt decimal html in '//argt(1)(1:argtl(1)))
            goto 1010
         endif
         if(cnt.le.3 .or. ichr.ne.asc_sc)call fatal('Reconcile: '//
     &    'corrupt decimal html in '//argt(1)(1:argtl(1)))
         if(uc.le.127 .and. uc.ne.38 .and. uc.ne.60 .and. uc.ne.62)
     &    call fatal('Reconcile: found '//
     &    'html-encoded normal ASCII in '//argt(1)(1:argtl(1)))
         if(uc.le.ucdim)then
            if(mod(ucdata(uc),2*htmlch).lt.htmlch)
     &       ucdata(uc)=ucdata(uc)+htmlch
         endif
         chrtyp=0
      endif

c     handle case that we are in a UTF-8 string
      if(chrtyp.eq.utf8ch)then
         cnt=cnt+1
         if(ichr.le.127)call fatal('Reconcile: '//
     &    'corrupt UTF-8 character in '//argt(1)(1:argtl(1)))
         if(cnt.lt.bytes)goto 1010 
         call rutf8(cnt,coch(co0+1),uc)
         if(uc.lt.0)call fatal('Reconcile: '//
     &    'corrupt UTF-8 character in '//argt(1)(1:argtl(1)))
         if(uc.le.ucdim)then
            if(mod(ucdata(uc),2*utf8ch).lt.utf8ch)
     &       ucdata(uc)=ucdata(uc)+utf8ch
         endif
         chrtyp=0
      endif

c     handle case that we are in a ^PNNN paragraph marker string
      if(rdmode.eq.parmrk)then
         i=uc-48
         if(i.ge.0 .and. i.le.9)then
            if(ponum(po).ge.32200000)call fatal('Reconcile: '//
     &       'paragraph number too big in file '//argt(1)(1:argtl(1)))
            ponum(po)=ponum(po)*10+i
            goto 1000
         endif
         if(uc.ne.asc_lf .and. uc.ne.asc_tb .and. uc.ne.asc_sp)
     &    call fatal('Reconcile: paragraph number '//
     &    'not terminated by whitespace in '//argt(1)(1:argtl(1)))
         if(ponum(po).le.0)call fatal('Reconcile: '//
     &    'missing or zero paragraph number in '//argt(1)(1:argtl(1)))
         rdmode=0
         if(po.eq.1)goto 1000
         if(ponum(po).le.ponum(po-1))call fatal('Reconcile: '//
     &    'paragraph number in '//argt(1)(1:argtl(1))//' out of order')
         goto 1000
      endif

c     ^P starts a ^PNNN paragraph marker
      if(uc.eq.asc_p)then
         rdmode=parmrk
         po=po+1
         if(po.gt.podim)call fatal('Reconcile: file '
     &    //argt(1)(1:argtl(1))//' has too many paragraphs')
         poco(po)=co0+1
         ponum(po)=0
         goto 1000
      endif

c     go read next character
      goto 1000

c     finish up
 1900 uc=-1
      if(chrtyp.eq.htmlch)call fatal('Reconcile: '//
     &'incomplete decimal html in '//argt(1)(1:argtl(1)))
      if(chrtyp.eq.utf8ch)call fatal('Reconcile: '//
     & 'incomplete UTF-8 character in '//argt(1)(1:argtl(1)))
      comax=co-1
      pomax=po
      if(pomax.le.0)call fatal('Reconcile: file '
     & //argt(1)(1:argtl(1))//' has no paragraph markers')
      po=1

c     close the file
      close(1,err=9919)



c     initialize the other files

c     open RCNFILE
      call openwu(argt(3)(1:argtl(3)),3)

c     write the header to it
      do 2100 co=1,poco(1)-1
         call putchr(3,coch(co),*9930)
 2100 continue

c     open NEWFILE
      call openru(argt(2)(1:argtl(2)),2)

c     find the first paragraph marker
      
c     initialize
      cn=0
      errhd=0
      cnum='321000000.'//char(asc_lf)

c     read to the first paragraph marker
 2300 cn=cn+1
      if(cn.gt.cndim)goto 9921
      call getchr(2,cnch(cn),*2380,*9920)
      if(cn.le.11)then
         if(cnch(cn).ne.cnum(cn:cn))call fatal('Reconcile: '//
     &    'corrupt start marker in file '//argt(2)(1:argtl(2)))
         goto 2300
      endif
      ichr=ichar(cnch(cn))
      if(ichr.eq.asc_p)goto 2390
      if(ichr.ne.asc_lf .and. ichr.ne.asc_tb .and. ichr.ne.asc_sp)
     & errhd=errhd+1
      goto 2300
 2380 call fatal('Reconcile: file '
     & //argt(2)(1:argtl(2))//' has no paragraph markers')
 2390 cnmax=cn
      if(errhd.ne.0)then
         do 2395 cn2=12,cn-1
            call putchr(3,cnch(cn2),*9930)
 2395    continue
         call encint(errhd,cnuml,cnum)
         print10,'*** Reconcile: there are '//
     &    cnum(1:cnuml)//' nonblank characters before the first'
         print10,'    paragraph marker in the file returned by the '//
     &    'spell/grammar checker.'
         call pauseq
      endif

c     initialize the error counts
      parpop=0
      pardup=0
      latpop=0
      latgon=0



c     read in the next paragraph of NEWFILE

c     done if at end of file
 2500 if(cnch(cnmax).ne.char(asc_p))goto 8000

c     initialize the encoded words buffer
      e=0
      buf=0

c     initialize the file reading
      cn=1
      cnch(cn)=char(asc_p)
      pnnum=0
      wn=0
      rdmode=parmrk
      chrtyp=0
      pos_n=-11
      uc=0

c     read the next character
 3000 cn0=cn
 3010 cn=cn+1
      if(cn.gt.cndim)goto 9921
      call getchr(2,cnch(cn),*3900,*9920)
      ichr=ichar(cnch(cn))

c     handle case that we are just copying to the output file
      if(rdmode.eq.copy)then
         if(ichr.eq.asc_p)then
            cnmax=cn
            goto 2500
         endif
         call putchr(3,cnch(cn),*9930)
         goto 3000
      endif

c     see how to find the unicode value
      if(chrtyp.eq.0)then
         uc0=uc
         uc=ichr
c        see whether we need to read a html string
         if(ichr.eq.asc_am)then
            chrtyp=htmlch
            cnt=1
            goto 3010
         endif
c        see whether we need to read a utf-8 string
         if(ichr.ge.128)then
            chrtyp=utf8ch
            cnt=1
            bytes=1
            if(ichr.ge.128+64)bytes=2
            if(ichr.ge.128+64+32)bytes=3
            if(ichr.ge.128+64+32+16)bytes=4
            if(ichr.ge.128+64+32+16+8)bytes=5
            if(bytes.lt.2 .or. bytes.gt.4)call fatal('Reconcile: '//
     &       'corrupt UTF-8 character in '//argt(2)(1:argtl(2)))
            goto 3010
         endif
      endif

c     handle case that we are in a decimal html string
      if(chrtyp.eq.htmlch)then
         cnt=cnt+1
         if(cnt.eq.2)then
            if(ichr.ne.asc_pd)call fatal('Reconcile: '//
     &       'corrupt or nondecimal html in '//argt(2)(1:argtl(2)))
            uc=0
            goto 3010
         endif
         i=ichr-48
         if(i.ge.0 .and. i.le.9)then
            uc=uc*10+i
            if(uc.gt.uclim)call fatal('Reconcile: '//
     &       'corrupt decimal html in '//argt(2)(1:argtl(2)))
            goto 3010
         endif
         if(cnt.le.3 .or. ichr.ne.asc_sc)call fatal('Reconcile: '//
     &    'corrupt decimal html in '//argt(2)(1:argtl(2)))
         if(uc.le.127 .and. uc.ne.38 .and. uc.ne.60 .and. uc.ne.62)
     &    call fatal('Reconcile: found '//
     &    'html-encoded normal ASCII in '//argt(2)(1:argtl(2)))
         chrtyp=0
      endif

c     handle case that we are in a UTF-8 string
      if(chrtyp.eq.utf8ch)then
         cnt=cnt+1
         if(ichr.le.127)call fatal('Reconcile: '//
     &    'corrupt UTF-8 character in '//argt(2)(1:argtl(2)))
         if(cnt.lt.bytes)goto 3010
         call rutf8(cnt,cnch(cn0+1),uc)
         if(uc.lt.0)call fatal('Reconcile: '//
     &    'corrupt UTF-8 character in '//argt(2)(1:argtl(2)))
         chrtyp=0
      endif

c     handle case that we are in a ^PNNN paragraph marker string
      if(rdmode.eq.parmrk)then
         i=uc-48
         if(i.ge.0 .and. i.le.9)then
            if(pnnum.ge.32200000)call fatal('Reconcile: '//
     &       'paragraph number too big in file '//argt(2)(1:argtl(2)))
            pnnum=pnnum*10+i
            goto 3000
         endif
c        find the paragraph number in the reference file
         if(pnnum.lt.ponum(1) .or. pnnum.gt.ponum(pomax))goto 3150
 3120    if(ponum(po).le.pnnum)goto 3130
         po=po-1
         goto 3120
 3130    if(ponum(po).ge.pnnum)goto 3140
         po=po+1
         goto 3130
 3140    if(ponum(po).eq.pnnum)goto 3190
 3150    if(parpop.eq.0)then
            call encint(pnnum,cnuml,cnum)
            print10,'*** Reconcile: corrupt paragraph number '//
     &       cnum(1:cnuml)//' appears in the file returned'
            print10,'    by the spell/grammar checker.  This '//
     &       'paragraph number does not exist'
            print10,'    in the original html file.'
            call pauseq
         endif
         parpop=parpop+1
         do 3160 cn2=1,cn0
            call putchr(3,cnch(cn2),*9930)
 3160    continue
         call putchr(3,'?',*9930)
         call putchr(3,'?',*9930)
         call putchr(3,'?',*9930)
         do 3170 cn2=cn0+1,cn
            call putchr(3,cnch(cn2),*9930)
 3170    continue
         rdmode=copy
         goto 3000
 3190    if(ichar(coch(poco(po))).ne.asc_p)then
            if(cn+2.gt.cndim)goto 9921
            if(pardup.eq.0)then
               call encint(pnnum,cnuml,cnum)
               print10,'*** Reconcile: the same paragraph number '//
     &          cnum(1:cnuml)//' appears twice in the file'
               print10,'    returned by the spell/grammar checker.'
               call pauseq
            endif
            pardup=pardup+1
            do 3195 cn2=cn,cn0+1,-1
               cnch(cn2+2)=cnch(cn2)
 3195       continue
            cnch(cn0+1)='?'
            cnch(cn0+2)='?'
            cn0=cn0+2
            cn=cn+2
         endif
         coch(poco(po))=char(0)
         wnbeg(0)=cn0+1
         wnend(0)=cn0
         rdmode=white
      endif

c     check validity if we are in an ^N?321NNNNNN encoded LaTeX string
      if(pos_n.ne.-11)then
         cnt=cn-pos_n+1
         if(cnt.eq.2)then
            if(uc.eq.asc_n .or. uc.gt.127)call fatal('Reconcile: '//
     &       'wrongly encoded LaTeX marker in '//argt(2)(1:argtl(2)))
         else
            i=uc-48
            if(cnt.gt.11 .or. i.lt.0 .or. i.gt.9
     &       .or. (cnt.eq.3 .and. i.ne.3) .or. (cnt.eq.4 .and. i.ne.2)
     &       .or. (cnt.eq.5 .and. i.ne.1))call fatal('Reconcile: '//
     &       'corrupt encoded LaTeX marker in '//argt(2)(1:argtl(2)))
         endif
      endif

c     count iterations to prevent infinite loops
      iter=0
 3500 iter=iter+1
      if(iter.gt.2)
     & call fatal('Reconcile: program error -- infinite loop')

c     handle the pending character if we are currently in whitespace mode
      if(rdmode.eq.white)then
c        EOF or ^P ends the current paragraph
         if(uc.eq.-1 .or. uc.eq.asc_p)goto 3910
c        go read the next character if more whitespace
         if(uc.le.asc_sp .and. uc.ne.asc_n)goto 3000
c        start the next word
         wn=wn+1
         if(wn.gt.wndim)call fatal('Reconcile: '//
     &    'paragraph has too many words in '//argt(2)(1:argtl(2)))
         wnbeg(wn)=cn0+1
         wn_n(wn)=0
         wnlt(wn)=0
c        next encoded word
         e=e+1
         if(e.gt.edim)goto 9941
c        initialize its first buffer integer
         buf=buf+1
         if(buf.gt.bufdim)goto 9942
         ebuf(e)=buf
         bufi(buf)=0
         mul=1
c        we are now in the beginning punctuation
         rdmode=punctb
      endif

c     handle the pending character if we are currently in prepunctuation mode
      if(rdmode.eq.punctb)then
c        go read the next character if more prefix punctuation
         if(uc.gt.asc_sp .and. uc.le.ucdim)then
            if(mod(ucdata(uc),2*punctb).ge.punctb)goto 3000
         endif
c        start the inner word
         wnbegi(wn)=cn0+1
c        we are now in the inner word
         rdmode=inword
      endif

c     handle the pending character if we are currently in inner word mode
      if(rdmode.eq.inword)then
c        ^N not at the start of the word starts a new one
         if(uc.eq.asc_n .and. wnbegi(wn).ne.cn0+1)goto 3790
c        whitespace ends the inner word
         if(uc.le.asc_sp .and. uc.ne.asc_n)goto 3790
c        any punctuation following a backslash is part of the inner word
         if(uc0.eq.asc_bs)goto 3710
c        anything following ^N is part of the inner word
         if(uc0.eq.asc_n)goto 3710
c        pre or post punctuation ends the word
         if(uc.le.asc_sp .or. uc.gt.ucdim)goto 3710
         if(ucdata(uc).ge.punctb)goto 3790
c        add the character to the encoded word
 3710    if(cn.eq.pos_n+1)goto 3719
         if(uc.le.127)then
            bufi(buf)=bufi(buf)+uc*mul
            if(mul.eq.mulend)then
               buf=buf+1
               if(buf.gt.bufdim)goto 9942
               bufi(buf)=0
               mul=1
            else
               mul=mul*256
            endif
         else
            call wutf8(uc,4,utf8l,utf8)
            do 3715 i=1,utf8l
               if(mul.eq.mulend)then
                  bufi(buf)=bufi(buf)+asc_sp*mul
                  buf=buf+1
                  if(buf.gt.bufdim)goto 9942
                  bufi(buf)=0
                  mul=1
               endif
               bufi(buf)=bufi(buf)+ichar(utf8(i))*mul
               if(mul.eq.mulend)then
                  buf=buf+1
                  if(buf.gt.bufdim)goto 9942
                  bufi(buf)=0
                  mul=1
               else
                  mul=mul*256
               endif
 3715       continue
         endif
 3719    continue
c        update the inner word length
         wnlt(wn)=wnlt(wn)+1
c        special handling of ^N
         if(uc.eq.asc_n)then
c           mark as a ^N word
            wn_n(wn)=1
c           mark the position for checking the next characters
            pos_n=cn
         endif
c        end of ^N word ends the inner word
         if(cn-pos_n+1.eq.11)then
            wnendi(wn)=cn
            rdmode=puncte
            pos_n=-11
         endif
c        go read the next character
         goto 3000
c        end the inner word
 3790    wnendi(wn)=cn0
c        we are now in the postfix punctuation
         rdmode=puncte
      endif

c     handle the pending character if we are currently in postpunctuation mode
      if(rdmode.eq.puncte)then
c        go read the next character if more postfix punctuation
         if(uc.gt.asc_sp .and. uc.le.ucdim)then
            if(ucdata(uc).ge.puncte)goto 3000
         endif
c        end the word
         wnend(wn)=cn0
c        we are now in whitespace
         rdmode=white
      endif

c     return to processing whitespace mode
      goto 3500

c     finish up
 3900 uc=-1
      if(chrtyp.eq.htmlch)call fatal('Reconcile: '//
     &'incomplete decimal html in '//argt(2)(1:argtl(2)))
      if(chrtyp.eq.utf8ch)call fatal('Reconcile: '//
     & 'incomplete UTF-8 character in '//argt(2)(1:argtl(2)))
      if(pos_n.ne.-11)call fatal('Reconcile: '//
     & 'incomplete encoded LaTeX marker in '//argt(2)(1:argtl(2)))
      if(rdmode.eq.copy)goto 8000
      iter=0
      goto 3500
 3910 cnmax=cn
      if(uc.eq.-1)cnmax=cn-1
      wnmax=wn
      wnbeg(wnmax+1)=cn0+1
      wnend(wnmax+1)=cn0

c     debug output
      if(debug.ne.0)then
         print10,'========================================'//
     &    '======================================='
         print10,'Read paragraph',po,', label',pnnum
         print10,'New words:'
         do 3950 wn=1,wnmax
            print3940,wn,wnlt(wn),wnbeg(wn),wnbegi(wn)-wnbeg(wn)+1,
     &       wnendi(wn)-wnbeg(wn)+1,wnend(wn)-wnbeg(wn)+1,
     &       '[',(cnch(cn),cn=wnbeg(wn),wnend(wn)),']'
 3940       format(2i3,i8,' 1',3i3,' ',1024a1)
 3950    continue
      endif

c     read the corresponding paragraph of OLDFILE

c     initialize
      co=poco(po)
      wo=0
      rdmode=parmrk
      chrtyp=0
      pos_n=-11
      uc=0

c     process the next character
 4000 co0=co
 4010 co=co+1
      if(co.gt.comax)goto 4900
      ichr=ichar(coch(co))

c     see how to find the unicode value
      if(chrtyp.eq.0)then
         uc0=uc
         uc=ichr
c        see whether we need to read a html string
         if(ichr.eq.asc_am)then
            chrtyp=htmlch
            cnt=1
            goto 4010
         endif
c        see whether we need to read a utf-8 string
         if(ichr.ge.128)then
            chrtyp=utf8ch
            cnt=1
            bytes=1
            if(ichr.ge.128+64)bytes=2
            if(ichr.ge.128+64+32)bytes=3
            if(ichr.ge.128+64+32+16)bytes=4
            if(ichr.ge.128+64+32+16+8)bytes=5
            if(bytes.lt.2 .or. bytes.gt.4)call fatal('Reconcile: '//
     &       'corrupt UTF-8 character in '//argt(1)(1:argtl(1)))
            goto 4010
         endif
      endif

c     handle case that we are in a decimal html string
      if(chrtyp.eq.htmlch)then
         cnt=cnt+1
         if(cnt.eq.2)then
            if(ichr.ne.asc_pd)call fatal('Reconcile: '//
     &       'corrupt or nondecimal html in '//argt(1)(1:argtl(1)))
            uc=0
            goto 4010
         endif
         i=ichr-48
         if(i.ge.0 .and. i.le.9)then
            uc=uc*10+i
            if(uc.gt.uclim)call fatal('Reconcile: '//
     &       'corrupt decimal html in '//argt(1)(1:argtl(1)))
            goto 4010
         endif
         if(cnt.le.3 .or. ichr.ne.asc_sc)call fatal('Reconcile: '//
     &    'corrupt decimal html in '//argt(1)(1:argtl(1)))
         if(uc.le.127 .and. uc.ne.38 .and. uc.ne.60 .and. uc.ne.62)
     &    call fatal('Reconcile: found '//
     &    'html-encoded normal ASCII in '//argt(1)(1:argtl(1)))
         chrtyp=0
      endif

c     handle case that we are in a UTF-8 string
      if(chrtyp.eq.utf8ch)then
         cnt=cnt+1
         if(ichr.le.127)call fatal('Reconcile: '//
     &    'corrupt UTF-8 character in '//argt(1)(1:argtl(1)))
         if(cnt.lt.bytes)goto 4010
         call rutf8(cnt,coch(co0+1),uc)
         if(uc.lt.0)call fatal('Reconcile: '//
     &    'corrupt UTF-8 character in '//argt(1)(1:argtl(1)))
         chrtyp=0
      endif

c     handle case that we are in a ^PNNN paragraph marker string
      if(rdmode.eq.parmrk)then
         i=uc-48
         if(i.ge.0 .and. i.le.9)goto 4000
         wobeg(0)=co0+1
         woend(0)=co0
         rdmode=white
      endif

c     check validity if we are in an ^N?321NNNNNN encoded LaTeX string
      if(pos_n.ne.-11)then
         cnt=co-pos_n+1
         if(cnt.eq.2)then
            if(uc.eq.asc_n .or. uc.gt.127)call fatal('Reconcile: '//
     &       'wrongly encoded LaTeX marker in '//argt(1)(1:argtl(1)))
         else
            i=uc-48
            if(cnt.gt.11 .or. i.lt.0 .or. i.gt.9
     &       .or. (cnt.eq.3 .and. i.ne.3) .or. (cnt.eq.4 .and. i.ne.2)
     &       .or. (cnt.eq.5 .and. i.ne.1))call fatal('Reconcile: '//
     &       'corrupt encoded LaTeX marker in '//argt(1)(1:argtl(1)))
         endif
      endif

c     count iterations to prevent infinite loops
      iter=0
 4500 iter=iter+1
      if(iter.gt.2)
     & call fatal('Reconcile: program error -- infinite loop')

c     handle the pending character if we are currently in whitespace mode
      if(rdmode.eq.white)then
c        EOF or ^P ends the current paragraph
         if(uc.eq.-1 .or. uc.eq.asc_p)goto 4910
c        go read the next character if more whitespace
         if(uc.le.asc_sp .and. uc.ne.asc_n)goto 4000
c        start the next word
         wo=wo+1
         if(wo.gt.wodim)call fatal('Reconcile: '//
     &    'paragraph has too many words in '//argt(1)(1:argtl(1)))
         wobeg(wo)=co0+1
         wo_n(wo)=0
         wolt(wo)=0
c        next encoded word
         e=e+1
         if(e.gt.edim)goto 9941
c        initialize its first buffer integer
         buf=buf+1
         if(buf.gt.bufdim)goto 9942
         ebuf(e)=buf
         bufi(buf)=0
         mul=1
c        we are now in the beginning punctuation
         rdmode=punctb
      endif

c     handle the pending character if we are currently in prepunctuation mode
      if(rdmode.eq.punctb)then
c        go read the next character if more prefix punctuation
         if(uc.gt.asc_sp .and. uc.le.ucdim)then
            if(mod(ucdata(uc),2*punctb).ge.punctb)goto 4000
         endif
c        start the inner word
         wobegi(wo)=co0+1
c        we are now in the inner word
         rdmode=inword
      endif

c     handle the pending character if we are currently in inner word mode
      if(rdmode.eq.inword)then
c        ^N not at the start of the word starts a new one
         if(uc.eq.asc_n .and. wobegi(wo).ne.co0+1)goto 4790
c        whitespace ends the inner word
         if(uc.le.asc_sp .and. uc.ne.asc_n)goto 4790
c        any punctuation following a backslash is part of the inner word
         if(uc0.eq.asc_bs)goto 4710
c        anything following ^N is part of the inner word
         if(uc0.eq.asc_n)goto 4710
c        pre or post punctuation ends the word
         if(uc.le.asc_sp .or. uc.gt.ucdim)goto 4710
         if(ucdata(uc).ge.punctb)goto 4790
c        add the character to the encoded word
 4710    if(co.eq.pos_n+1)goto 4719
         if(uc.le.127)then
            bufi(buf)=bufi(buf)+uc*mul
            if(mul.eq.mulend)then
               buf=buf+1
               if(buf.gt.bufdim)goto 9942
               bufi(buf)=0
               mul=1
            else
               mul=mul*256
            endif
         else
            call wutf8(uc,4,utf8l,utf8)
            do 4715 i=1,utf8l
               if(mul.eq.mulend)then
                  bufi(buf)=bufi(buf)+asc_sp*mul
                  buf=buf+1
                  if(buf.gt.bufdim)goto 9942
                  bufi(buf)=0
                  mul=1
               endif
               bufi(buf)=bufi(buf)+ichar(utf8(i))*mul
               if(mul.eq.mulend)then
                  buf=buf+1
                  if(buf.gt.bufdim)goto 9942
                  bufi(buf)=0
                  mul=1
               else
                  mul=mul*256
               endif
 4715       continue
         endif
 4719    continue
c        update the inner word length
         wolt(wo)=wolt(wo)+1
c        special handling of ^N
         if(uc.eq.asc_n)then
c           mark as a ^N word
            wo_n(wo)=1
c           mark the position for checking the next characters
            pos_n=co
         endif
c        end of ^N word ends the inner word
         if(co-pos_n+1.eq.11)then
            woendi(wo)=co
            rdmode=puncte
            pos_n=-11
         endif
c        go read the next character
         goto 4000
c        end the inner word
 4790    woendi(wo)=co0
c        we are now in the postfix punctuation
         rdmode=puncte
      endif

c     handle the pending character if we are currently in postpunctuation mode
      if(rdmode.eq.puncte)then
c        go read the next character if more postfix punctuation
         if(uc.gt.asc_sp .and. uc.le.ucdim)then
            if(ucdata(uc).ge.puncte)goto 4000
         endif
c        end the word
         woend(wo)=co0
c        we are now in whitespace
         rdmode=white
      endif

c     return to processing whitespace mode
      goto 4500

c     finish up
 4900 uc=-1
      if(chrtyp.eq.htmlch)call fatal('Reconcile: '//
     &'incomplete decimal html in '//argt(1)(1:argtl(1)))
      if(chrtyp.eq.utf8ch)call fatal('Reconcile: '//
     & 'incomplete UTF-8 character in '//argt(1)(1:argtl(1)))
      if(pos_n.ne.-11)call fatal('Reconcile: '//
     & 'incomplete encoded LaTeX marker in '//argt(1)(1:argtl(1)))
      iter=0
      goto 4500
 4910 womax=wo
      wobeg(womax+1)=co0+1
      woend(womax+1)=co0
      emax=e

c     debug output
      if(debug.ne.0)then
         print10,'Old words:'
         do 4950 wo=1,womax
            print4940,wo,wolt(wo),wobeg(wo),wobegi(wo)-wobeg(wo)+1,
     &       woendi(wo)-wobeg(wo)+1,woend(wo)-wobeg(wo)+1,
     &       '[',(coch(co),co=wobeg(wo),woend(wo)),']'
 4940       format(2i3,i8,' 1',3i3,' ',1024a1)
 4950    continue
      endif

c     sort the encoded words

c     initialize the sort mapping and lowest of group to trivial
      do 5010 es=1,emax
         ese(es)=es
 5010 continue
      eses0(1)=1

c     initialize the sort interval
      int=emax

c     sort at the next sort interval
 5100 int=int/2
      if(int.eq.0)goto 5190
c     initialize the next sort
 5110 change=0
      if(int.eq.1)then
         do 5120 es=1,emax
            eses0(es)=es
 5120    continue
      endif
c     sort
      do 5150 es=1,emax-int
         buf1=ebuf(ese(es))
         buf2=ebuf(ese(es+int))
 5130    i1=bufi(buf1)
         i2=bufi(buf2)
         if(i1.lt.i2)goto 5150
         if(i1.gt.i2)goto 5140
         if(i1.ge.mulend)then
            buf1=buf1+1
            buf2=buf2+1
            goto 5130
         endif
         if(ese(es).gt.ese(es+int))goto 5140
         eses0(es+int)=eses0(es)
         goto 5150
 5140    change=ese(es)
         ese(es)=ese(es+int)
         ese(es+int)=change
 5150 continue
c     repeat this sort
      if(change.ne.0)goto 5110
c     go sort at the next interval
      goto 5100
c     end sorting
 5190 continue

c     find the inverse mapping and multiplicities
      es0=0
      do 5200 es=1,emax
         ees(ese(es))=es
         if(eses0(es).ne.es0)then
            es0=eses0(es)
            es0mo(es0)=0
            es0mn(es0)=0
         endif
         if(ese(es).le.wnmax)then
            es0mn(es0)=es0mn(es0)+1
         else
            es0mo(es0)=es0mo(es0)+1
         endif
 5200 continue

c     debug output
      if(debug.ne.0)then
         print10,'Sorted words:'
         do 5220 es=1,emax
            e=ese(es)
            es0=eses0(es)
            if(e.le.wnmax)then
               wn=e
               print5210,es,es0mn(es0),es0mo(es0),es0,wn,'n',
     &          '[',(cnch(cn),cn=wnbeg(wn),wnend(wn)),']'
            else
               wo=e-wnmax
               print5210,es,es0mn(es0),es0mo(es0),es0,wo,'o',
     &          '[',(coch(co),co=wobeg(wo),woend(wo)),']'
            endif
 5210       format(i4,2i3,2i4,a1,' ',1024a1)
 5220    continue
      endif

c     initialize the mapping from new to old
      wnwo(0)=0
      wnwoe(0)=1
      wng(0)=1.
      wnwo(wnmax+1)=womax+1
      wnwoe(wnmax+1)=1
      wng(wnmax+1)=1.
      do 5300 wn=1,wnmax
         e=wn
         es=ees(e)
         es0=eses0(es)
         mn=es0mn(es0)
         if(mn.lt.1)goto 9900
         mo=es0mo(es0)
         if(es.eq.es0 .and. wn_n(wn).ne.0)then
            if(mn.gt.mo)then
               if(latpop.eq.0)then
                  print30,'*** Reconcile: LaTeX string ',
     &             (cnch(cn),cn=wnbegi(wn)+1,wnbegi(wn)+10),
     &             ' is misplaced/corrupt in the file',
     &             '    returned by the spell/grammar checker.'
                  call pauseq
               endif
               latpop=latpop+mn-mo
            endif
         endif
         wo=(wn*(womax+1)+wnmax)/(wnmax+1)
         if(mo.le.0)then
            wnwo(wn)=wo
            wnwoe(wn)=0
            wng(wn)=.5/max(womax,wnmax)
         else
            i=3*womax+3
            do 5250 es2=es0+mn,es0+mn+mo-1
               wo2=ese(es2)-wnmax
               if(abs(wo2-wo).ge.i)goto 5250
               i=abs(wo2-wo)
               wnwo(wn)=wo2
 5250       continue
            wnwoe(wn)=1
            wng(wn)=.5/max(mn,mo)
            if(wn_n(wn).ne.0)wng(wn)=1.
         endif
 5300 continue

c     initialize the mapping from old to new and the probability of being right
      wown(0)=0
      wowni(0)=0
      wowne(0)=1
      wog(0)=1.
      wown(womax+1)=wnmax+1
      wowni(womax+1)=0
      wowne(womax+1)=1
      wog(womax+1)=1.
      change=0
      if(womax.ne.wnmax)change=1
      do 5400 wo=1,womax
         e=wo+wnmax
         es=ees(e)
         es0=eses0(es)
         mo=es0mo(es0)
         if(mo.lt.1)goto 9900
         mn=es0mn(es0)
         if(wo_n(wo).ne.0)then
            if(mo.ne.1)call fatal('Reconcile: '//
     &       'duplicate LaTeX marker in '//argt(1)(1:argtl(1)))
            if(mn.le.0)then
               if(latgon.eq.0)then
                  print30,'*** Reconcile: LaTeX string ',
     &             (coch(co),co=wobegi(wo)+1,wobegi(wo)+10),
     &             ' is lost in the file returned by',
     &             '    the spell/grammar checker.'
                  call pauseq
               endif
               latgon=latgon+1
            endif
         endif
         wn=(wo*(wnmax+1)+womax)/(womax+1)
         if(mn.le.0)then
            wown(wo)=wn
            wowni(wo)=0
            if(wn*(womax+1).ne.wo*(wnmax+1))wowni(wo)=1
            wowne(wo)=0
            wog(wo)=.5/max(womax,wnmax)
            change=1
         else
            i=3*wnmax+3
            do 5350 es2=es0,es0+mn-1
               wn2=ese(es2)
               if(abs(wn2-wn).ge.i)goto 5350
               i=abs(wn2-wn)
               wown(wo)=wn2
 5350       continue
            wowni(wo)=0
            wowne(wo)=1
            wog(wo)=.5/max(mo,mn)
            if(wo_n(wo).ne.0)wog(wo)=1./mn
            if(wown(wo).ne.wo)change=1
         endif
 5400 continue

c     if no change, accept at face value
      if(change.eq.0)then
         if(debug.ne.0)print10,'No changes in paragraph',pnnum
         change=-1
         goto 7000
      endif

c     iterate to determine what word wo corresponds to what word wn

c     initialize the iterations
      iter=0
      iterch=0
      wostep=-1

c     next iteration
 5500 iter=iter+1
      change=0
c     use symmetric Gauss-Seidel iteration for faster propagation both ways
      if(wostep.eq.-1)then
         woinit=1
         wolast=womax
         wostep=1
      else
         woinit=womax
         wolast=1
         wostep=-1
      endif

c     find the best mapping from old to new
      do 5600 wo=woinit,wolast,wostep
c        initialize
         e=wo+wnmax
         es=ees(e)
         es0=eses0(es)
         mn=es0mn(es0)
         mo=es0mo(es0)
         pbest=0.
         wnold=wown(wo)
         wniold=wowni(wo)
c        initialize the probability that word wo equals some word wn
         psame=0.
c        try the words in the new file we are equal to
         if(mn.ne.0)then
c           probability that the word is unchanged, ignoring context
            psame=1.-fchang-fremov
            if(mn.lt.mo)psame=psame*(mn+0.)/mo
c           probability that a matching word is the right one, ignoring context
            pbare=psame/mn*(.5+wolt(wo))/(2.+wolt(wo))
c           look at all words that we could be equal to
            psum=0.
            do 5520 es=es0,es0+mn-1
               wn=ese(es)
c              initial probability that wn corresponds to wo
               p=pbare
c              compare to the previous word
               wn2=wown(wo-1)
               if(wn2.eq.wn-1)then
c                 increased confidence if we are in sequence
                  p=1.-(1.-p)/(1.+wog(wo-1))
               else
c                 decreased confidence if we are out of sequence
                  temp=abs(wn-1-wn2)
                  p=p*(1.+wog(wo-1)*temp)/(1.+2.*wog(wo-1)*temp)
               endif
c              compare to the next word
               wn2=wown(wo+1)
               if(wn2.eq.wn+1)then
c                 increased confidence if we are in sequence
                  p=1.-(1.-p)/(1.+wog(wo+1))
               else
c                 decreased confidence if we are out of sequence
                  temp=abs(wn+1-wn2)
                  p=p*(1.+wog(wo+1)*temp)/(1.+2.*wog(wo+1)*temp)
               endif
c              see whether wn is assigned to another
               if(wnwo(wn).ne.wo)then
c                 decreased confidence if the other has higher probability
                  if(wng(wn).gt.p)p=p/wng(wn)*p
               endif
c              accept this assignment if the most likely one yet
               if(p.gt.pbest)then
                  pbest=p
                  wown(wo)=wn
                  wowni(wo)=0
                  wowne(wo)=1
                  wog(wo)=p
               endif
               psum=psum+p
 5520       continue
c           modify psame based on the contextual probabilities
            psame=psum/(1.-psame+psum)
c           handle matched ^N words specially
            if(wo_n(wo).ne.0)then
c              if ^N words are uniquely matched take the assignment to be right
               if(mn.eq.1)wog(wo)=1.
c              either way, we always assign them to a match
               goto 5590
            endif
         endif
c        ^N words cannot be changed, only destroyed
         if(wo_n(wo).ne.0)then
            premov=1.
            goto 5571
         endif
c        find the probability that this is a changed word
         pchang=(1.-psame)*fchang/(fchang+fremov)
c        one likely location for a changed word is after the preceding word
         wn=wown(wo-1)+1
         if(wowni(wo-1).ne.0)wn=wn-1
c        initialize the probability to none
         p=0.
c        if not a valid new word, that is it
         if(wn.lt.1 .or. wn.gt.wnmax)goto 5540
c        not applicable if we are equal
         es02=eses0(ees(wn))
         if(es02.eq.es0)goto 5540
c        assume reasonably that this is the one if wn is unassigned
         p=pchang*wog(wo-1)
c        if the new word is probably equal to an old one, confidence plunges
         mo2=es0mo(es02)
         mn2=es0mn(es02)
         if(mo2.ge.mn2)then
            p=p*(fchang+fremov)
         else
            temp=(1.-fchang-fremov)*mo2/mn2
            p=p*(1.-temp)
         endif
c        another likely location for a changed word is before the next word
 5540    wn2=wown(wo+1)-1
c        initialize the probability to none
         p2=0.
c        if not a valid new word, that is it
         if(wn2.lt.1 .or. wn2.gt.wnmax)goto 5550
c        not applicable if we are equal
         es02=eses0(ees(wn2))
         if(es02.eq.es0)goto 5550
c        assume reasonably that this is the one if wn2 is unassigned
         p2=pchang*wog(wo+1)
c        if the new word is probably equal to an old one, confidence plunges
         mo2=es0mo(es02)
         mn2=es0mn(es02)
         if(mo2.ge.mn2)then
            p2=p2*(fchang+fremov)
         else
            temp=(1.-fchang-fremov)*mo2/mn2
            p2=p2*(1.-temp)
         endif
c        figure out the best possibility
 5550    if(p.le.0. .and. p2.le.0.)then
         else if(wn.eq.wn2)then
c           seems to fit in nicely, multiply probabilities of being bad
            p=1.-(1.-p)*(1.-p2)
c           decreased confidence if another has higher probability
            if(wnwo(wn).ne.wo)then
               if(wng(wn).gt.p)p=p/wng(wn)*p
            endif
         else
c           take the better one
            if(p2.gt.p)then
               wn=wn2
               p=p2
            endif
c           decreased confidence if another has higher probability
            if(wnwo(wn).ne.wo)then
               if(wng(wn).gt.p)p=p/wng(wn)*p
            endif
         endif
c        accept this assignment if the most likely one yet
         if(p.gt.pbest)then
            pbest=p
            wown(wo)=wn
            wowni(wo)=0
            wowne(wo)=0
            wog(wo)=p
         endif
c        find the probability that this is a removed word
         premov=(1.-psame)*fremov/(fchang+fremov)
c        one likely location for a removed word is after the preceding word
 5571    wn=wown(wo-1)+1
         if(wowni(wo-1).ne.0)wn=wn-1
c        another is before the following word
         wn2=wown(wo+1)
         if(wn.eq.wn2)then
c           seems to fit in nicely, assume it is the one
            p=premov*(0.5*wog(wo-1)+0.5*wog(wo+1))
         else
c           give each possibility a maximum 25% probability
            p=premov*0.25*wog(wo-1)
            p2=premov*0.25*wog(wo+1)
            if(p2.gt.p)then
               wn=wn2
               p=p2
            endif
         endif
c        accept this assignment if the most likely one yet
         if(p.gt.pbest)then
            pbest=p
            wown(wo)=wn
            wowni(wo)=1
            wowne(wo)=0
            wog(wo)=p
         endif
c        see whether the assignment has changed
 5590    if(wown(wo).ne.wnold .or. wowni(wo).ne.wniold)change=change+1
 5600 continue

c     find the best mapping from new back to old
      do 5700 wn=1,wnmax
         wnwo(wn)=womax+1
         wnwoe(wn)=0
         wng(wn)=0.
 5700 continue
      do 5750 wo=1,womax
         if(wowni(wo).ne.0)goto 5750
         wn=wown(wo)
         if(wog(wo).le.wng(wn))goto 5750
         wnwo(wn)=wo
         wnwoe(wn)=wowne(wo)
         wng(wn)=wog(wo)
 5750 continue

c     done if no further changes seem to occur
      if(change.eq.0 .and. iter-iterch.ge.4)goto 7000
c     done if too many iterations
      if(iter.gt.itermx)then
         print10,'** Reconciliation not converged for paragraph',pnnum
         goto 7000
      endif
      if(change.ne.0)iterch=iter
      goto 5500



c     write the new paragraph to file with corrections as needed
 7000 continue

c     debug output
      if(debug.ne.0)then
         print10,'Associations old --> new'
         do 7020 wo=1,womax
            wn=wown(wo)
            if(wowni(wo).ne.0)then
               print7011,' ',
     &          (' ',co=wobeg(wo),woend(wo)),' ','v',' ',
     &          (cnch(cn),cn=wnbeg(wn-1),wnend(wn-1)),']'
               print7010,wo,wn,wog(wo),'[',
     &          (coch(co),co=wobeg(wo),woend(wo)),' ','^',' ',
     &          (cnch(cn),cn=wnbeg(wn),wnend(wn)),']'
            else
               print7010,wo,wn,wog(wo),'[',
     &          (coch(co),co=wobeg(wo),woend(wo)),'-','-','>',
     &          (cnch(cn),cn=wnbeg(wn),wnend(wn)),']'
            endif
 7010       format(i3,i4,f6.3,' ',1024a1)
 7011       format('              ',1024a1)
 7020    continue
         print10,'Associations old <-- new'
         do 7040 wn=1,wnmax
            wo=wnwo(wn)
            print7010,wo,wn,wng(wn),'[',
     &       (coch(co),co=wobeg(wo),woend(wo)),'<','-','-',
     &       (cnch(cn),cn=wnbeg(wn),wnend(wn)),']'
 7040    continue
      endif

c     initialize the write sequence pointers
      do 7100 wn=0,wnmax
         wnnext(wn)=wn+1
 7100 continue

c     offset of the wo values in the pointers
      wooff=wnmax+2

c     insert any lost labels at the most likely point
      do 7200 wo=womax,1,-1
         if(wo_n(wo).ne.1)goto 7200
         e=wnmax+wo
         es=ees(e)
         es0=eses0(es)
         mn=es0mn(es0)
         do 7150 es2=es0,es0+mn-1
            wn=ese(es2)
            if(wnwo(wn).eq.wo .and. wnwoe(wn).ne.0)goto 7200
 7150    continue
         wn=wown(wo)
         if(wn.lt.1 .or. wog(wo).lt.2./(2.+wnmax))wn=wnmax+1
         wonext(wo)=wnnext(wn-1)
         wnnext(wn-1)=wo+wooff
 7200 continue

c     write the paragraph with corrections
      wn=0
      wo=wn-wooff

c     next word to write
 7500 wn0=wn
      wo0=wo
      if(wn0.le.wnmax+1)then
         wn=wnnext(wn0)
      else
         wn=wonext(wo0)
      endif
      wo=wn-wooff

c     write the preceding whitespace
      if(wn0.eq.0)then
         call putchr(3,char(asc_p),*9930)
         do 7520 co=poco(po)+1,wobeg(1)-1
            call putchr(3,coch(co),*9930)
 7520    continue
      else if(wn.eq.wnmax+1)then
         do 7521 co=woend(womax)+1,wobeg(womax+1)-1
            call putchr(3,coch(co),*9930)
 7521    continue
         goto 7590
      else if(wn.gt.wnmax+1)then
         do 7522 co=woend(wo-1)+1,wobeg(wo)-1
            call putchr(3,coch(co),*9930)
 7522    continue
      else if(wn0.gt.wnmax+1)then
         do 7523 co=woend(wo0)+1,wobeg(wo0+1)-1
            call putchr(3,coch(co),*9930)
 7523    continue
      else
         wo2=wnwo(wn)
         if(wo2.le.womax .and. wnwo(wn-1).eq.wo2-1)then
            do 7524 co=woend(wo2-1)+1,wobeg(wo2)-1
               call putchr(3,coch(co),*9930)
 7524       continue
         else
            do 7525 cn=wnend(wn-1)+1,wnbeg(wn)-1
               call putchr(3,cnch(cn),*9930)
 7525       continue
         endif
      endif

c     write the word
      if(wn.gt.wnmax+1)then
         do 7540 co=wobeg(wo),woend(wo)
            call putchr(3,coch(co),*9930)
 7540    continue
      else
         wo2=wnwo(wn)
         if(wo2.le.0)wo2=womax+1
         cn=wnbeg(wn)
         cn2=wnbegi(wn)
         co=wobeg(wo2)
         co2=co
         if(wo2.le.womax)co2=wobegi(wo2)
         if(cn2.gt.cn)call putnew(cn2-cn,cnch(cn),co2-co,coch(co),
     &    ucdata,ucdim,htmlch,utf8ch,*9930)
         cn=cn2
         cn2=wnendi(wn)+1
         co=co2
         if(wo2.le.womax)co2=woendi(wo2)+1
         if(wnwoe(wn).ne.0)then
            do 7550 co=wobegi(wo2),woendi(wo2)
               call putchr(3,coch(co),*9930)
 7550       continue
         else
            call putnew(cn2-cn,cnch(cn),co2-co,coch(co),
     &       ucdata,ucdim,htmlch,utf8ch,*9930)
         endif
         cn=cn2
         cn2=wnend(wn)+1
         co=co2
         if(wo2.le.womax)co2=woend(wo2)+1
         if(cn2.gt.cn)call putnew(cn2-cn,cnch(cn),co2-co,coch(co),
     &    ucdata,ucdim,htmlch,utf8ch,*9930)
      endif

c     next word
      goto 7500

c     done writing
 7590 continue

c     debug output
      if(debug.ne.0)then
         if(change.eq.-1)then
            print10,'Press return at end of unchanged paragraph',pnnum
         else
            print10,'Press return at end of paragraph',pnnum
         endif
         read*
      endif

c     go read the next paragraph
      goto 2500



c     no more paragraphs in NEWFILE
 8000 continue

c     write any paragraphs that were not found to the end of the file
      pargon=0
      do 8300 po=1,pomax
         co=poco(po)
         if(coch(co).ne.char(asc_p))goto 8300
         if(pargon.eq.0)then
            call encint(ponum(po),cnuml,cnum)
            print10,'*** Reconcile: paragraph number '//
     &       cnum(1:cnuml)//' has disappeared in the file returned'
            print10,'    by the spell/grammar checker.'
            call pauseq
         endif
         pargon=pargon+1
         co2=comax
         if(po.lt.pomax)co2=poco(po+1)-1
         call putchr(3,char(asc_lf),*9930)
         call putchr(3,char(asc_lf),*9930)
         do 8250 co=co,co2
            call putchr(3,coch(co),*9930)
 8250    continue
 8300 continue

c     warn about additional missing paragraph markers
      if(pargon.gt.1)then
         call encint(pargon,cnuml,cnum)
         print10,'*** Reconcile: there were a total of '//
     &    cnum(1:cnuml)//' missing paragraph markers'
         print10,
     &    '    in the file returned by the spell/grammar checker.'
      endif

c     warn about additional duplicated paragraph markers
      if(pardup.gt.1)then
         call encint(pardup,cnuml,cnum)
         print10,'*** Reconcile: there were a total of '//
     &    cnum(1:cnuml)//' duplicated paragraph markers'
         print10,
     &    '    in the file returned by the spell/grammar checker.'
      endif

c     warn about additional corrupt paragraph markers
      if(parpop.gt.1)then
         call encint(parpop,cnuml,cnum)
         print10,'*** Reconcile: there were a total of '//
     &    cnum(1:cnuml)//' corrupt paragraph markers'
         print10,
     &    '    in the file returned by the spell/grammar checker.'
      endif

c     warn about additional missing LaTeX markers
      if(latgon.gt.1)then
         call encint(latgon,cnuml,cnum)
         print10,'*** Reconcile: there were a total of '//
     &    cnum(1:cnuml)//' LaTeX strings that were lost'
         print10,
     &    '    in the file returned by the spell/grammar checker.'
      endif

c     warn about additional misplaced or corrupt LaTeX markers
      if(latpop.gt.1)then
         call encint(latpop,cnuml,cnum)
         print10,'*** Reconcile: there were a total of '//
     &    cnum(1:cnuml)//' misplaced/corrupt LaTeX strings'
         print10,
     &    '    in the file returned by the spell/grammar checker.'
      endif

c     done
 9000 close(2,err=9929)
      close(3,err=9939)
      call exit(0)

c     errors
 9900 call fatal('Reconcile: program error')
 9910 call fatal('Reconcile: error reading reference file '//
     & argt(1)(1:argtl(1)))
 9911 call fatal('Reconcile: reference file '
     & //argt(1)(1:argtl(1))//' is too big to process')
 9919 call fatal('Reconcile: unable to close reference file '//
     & argt(1)(1:argtl(1)))
 9920 call fatal('Reconcile: error reading modified file '//
     & argt(2)(1:argtl(2)))
 9921 call fatal('Reconcile: a paragraph in modified file '
     & //argt(2)(1:argtl(2))//' is too big to process')
 9929 call fatal('Reconcile: unable to close modified file '//
     & argt(2)(1:argtl(2)))
 9930 call fatal('Reconcile: error writing to reconciled file '//
     & argt(3)(1:argtl(3)))
 9939 call fatal('Reconcile: unable to close reconciled file '//
     & argt(3)(1:argtl(3)))
 9941 call fatal('Reconcile: a paragraph has too many words')
 9942 call fatal('Reconcile: a paragraph is too big for the buffer')
c     print usage
 9990 call fatal('Usage: reconcile.exe OLDFILE NEWFILE RCNFILE [d]')
      end



      subroutine putnew(nmax,nstr,omax,ostr,
     & ucdata,ucdim,htmlch,utf8ch,*)

c     Write a string from the new file, trying to maintain the old file
c     character format.

c     avoid typos
      implicit none

c input

c     the new string
      integer nmax
      character*1 nstr(*)

c     the old string
      integer omax
      character*1 ostr(*)

c     number of unicode characters that we keep data on
      integer ucdim
c     data
      integer ucdata(0:ucdim)

c     character types besides ASCII
      integer htmlch,utf8ch

c local

c     character types besides ASCII
      integer html,utf8
      parameter (html=1,utf8=2)

c     ascii characters
      integer asc_am,asc_sc
      parameter (asc_am=38,asc_sc=59)

c     character indices
      integer n,n0,o,o2

c     unicode character number
      integer uc

c     alternate representation of the character
      character*1 alt(16)
      integer altl

c     number of bytes in the utf-8 string
      integer bytes

c     number as character string
      character*20 cnum
      integer cnuml

c     character number
      integer ichr

c     generic integer
      integer i

c executable

c     check definitions
      if(htmlch.ne.html .or. utf8ch.ne.utf8)goto 9900

c     initialize the character writing
      n=0

c     next character
 1000 n=n+1
      if(n.gt.nmax)return

c     character number
      ichr=ichar(nstr(n))

c     write it if plain
      if(ichr.le.127 .and. ichr.ne.asc_am)then
         call putchr(3,nstr(n),*9930)
         goto 1000
      endif

c     handle nonascii characters
      n0=n
      if(ichr.eq.asc_am)then
c        read the complete html character
         n=n+1
         if(n.gt.nmax)goto 9900
         if(nstr(n).ne.'#')goto 9900
         uc=0
 1100    n=n+1
         if(n.gt.nmax)goto 9900
         i=ichar(nstr(n))-48
         if(i.ge.0 .and. i.le.9)then
            uc=uc*10+i
            goto 1100
         endif
         if(i+48.ne.asc_sc .or. n-n0.lt.3)goto 9900
c        form the alternate version
         call wutf8(uc,4,altl,alt)
c        anything above ucdim is assumed to have no LaTeX version
         if(uc.gt.ucdim)goto 1600
c        assume there is no LaTeX version if no html form in the old file
         if(mod(ucdata(uc),2*html).lt.html)goto 1600
c        if no utf-8 version in the old file, write it as is
         if(mod(ucdata(uc),2*utf8).lt.utf8)goto 1500
c        both forms exist in the old file
         goto 1400
      else
c        read the complete utf-8 character
         bytes=1
         if(ichr.ge.128+64)bytes=2
         if(ichr.ge.128+64+32)bytes=3
         if(ichr.ge.128+64+32+16)bytes=4
         if(ichr.ge.128+64+32+16+8)bytes=5
         n=n0+bytes-1
         if(bytes.lt.2 .or. bytes.gt.4 .or. n.gt.nmax)goto 9900
         call rutf8(bytes,nstr(n0),uc)
         if(uc.lt.0)goto 9900
c        anything above ucdim is assumed to have no LaTeX version
         if(uc.gt.ucdim)goto 1500
c        assume there is no LaTeX version if no html form in the old file
         if(mod(ucdata(uc),2*html).lt.html)goto 1500
c        form the alternate version
         alt(1)='&'
         alt(2)='#'
         call encint(uc,cnuml,cnum)
         altl=cnuml+3
         do 1200 i=1,cnuml
            alt(i+2)=cnum(i:i)
 1200    continue
         alt(altl)=';'
c        if no utf-8 version in the old file, write the html one
         if(mod(ucdata(uc),2*utf8).lt.utf8)goto 1600
c        both forms exist in the old file
         goto 1400
      endif

c     try to find the alternate version in the old string
 1400 do 1450 o=1,omax-(altl-1)
         do 1440 o2=o,o+(altl-1)
            if(ostr(o2).ne.alt(1+(o2-o)))goto 1450
 1440    continue
         goto 1600
 1450 continue

c     write the character as is
 1500 do 1550 i=n0,n
         call putchr(3,nstr(i),*9930)
 1550 continue

c     next character
      goto 1000

c     write in the alternate format
 1600 do 1650 i=1,altl
         call putchr(3,alt(i),*9930)
 1650 continue

c     next character
      goto 1000

 9900 call fatal('Reconcile: program error in putnew')
 9930 return 1
      end
