      program wizhlp

c     Copyright 2012 Leon van Dommelen.  This software is made available
c     under the GNU Public License version 3.

c     Helper for the wizard utility.  Does things DOS simply won't do.

c     Note that no real argument chacking is performed, use with care.

c     Usage: wizhlp.exe L2HPATH TASK [PARAMETERS]
c            L2HPATH is the path to l2h terminated in \ or /
c               the normal l2h folder structure is assumed.
c               in DOS use a space between the final \ and the closing ".
c            TASK [PARAMETERS] is one of:
c               fntsiz
c                  changes web page font sizes
c                  creates index@new.css and settings@new.pl
c               colors
c                  changes web page colors
c                  creates index@new.css
c               setatt
c                  changes colors and font sizes of individual web page items.
c                  creates index@new.css
c               myicons
c                  swaps in the set of icons in local folder buttons@new
c                  reads their sizes and puts it in settings.pl
c                  creates settings@new.pl
c               chicons
c                  changes the colors of the icons in the buttons@new folder.
c                  creates @colors.sub, @colors_g.sub and settings@new.pl
c               askcol PROMPT VARIABLE DEFAULT
c                  asks for a color and converts it to a six digit hex string
c                  PROMPT should have no colon, can be up to 66 characters long
c                  VARIABLE must be lower case letters
c                  DEFAULT must be an uppercase 6 digit hex string
c                  creates @tmp.bat that sets VARIABLE to the user input
c               askint PROMPT VARIABLE MIN MAX DEFAULT
c                  asks for an integer in the range from MIN to MAX
c                  PROMPT should have no colon
c                  VARIABLE must be lower case letters
c                  DEFAULT plus PROMPT can be up to 77 characters long
c                  creates @tmp.bat that sets VARIABLE to the user input
c               asknum PROMPT VARIABLE MIN MAX DEFAULT
c                  asks for a real number in the range from MIN to MAX
c                  PROMPT should have no colon
c                  VARIABLE must be lower case letters
c                  DEFAULT plus PROMPT can be up to 77 characters long
c                  creates @tmp.bat that sets VARIABLE to the user input

c declarations

c     avoid typos
      implicit none

c common path to the wiz files

c     path string
      character*256 path
c     path string length
      integer pathl
      common /cmpath/pathl,path

c common data from dopair

c     declared substring selection levels
      integer levdim
      parameter (levdim=1000)
c     actual substring selection level
      integer lev
c     available bufdim at each level
      integer bufdml(0:levdim)
c     start of the post-selection string
      integer post(levdim)
c     after a successfull \f#, the #-th occurrence is from fnddat(2) to (3)
      integer fnddat(3)
c     total substitutions
      integer subtot
c     total found strings
      integer fndtot
c     total missed substitutions
      integer subnot
c     total missed searches
      integer fndnot
c     previous value of subtot at \x command
      integer oldsct

      common/cmpair/lev,bufdml,post,
     & fnddat,
     & subtot,fndtot,subnot,fndnot,oldsct

c local

c     command line arguments
      integer argdim
      parameter (argdim=9)
      integer argmx
      character*132 argt(argdim)
      integer argtl(argdim)
      external args

c     folder separator
      character*1 sep

c     number of style sheet tags that are editable
      integer ssemax
      parameter (ssemax=38)
c     editable style sheet tag index
      integer sse
c     declared editable style sheet tag name length
      integer ssetgd
      parameter (ssetgd=18)
c     actual editable style sheet tag name length
      integer ssetgl
c     editable style sheet tag names
      character*(ssetgd) ssetag(ssemax)
      data ssetag/
     & 'BODY',
     & 'H1','H2','H3','H4','H5','H6',
     & 'CAPTION',
     & 'TT','code','pre',
     & 'SMALL.XTINY','SMALL.TINY','SMALL.SCRIPTSIZE',
     & 'SMALL.FOOTNOTESIZE','SMALL.SMALL','BIG.LARGE',
     & 'BIG.XLARGE','BIG.XXLARGE','BIG.HUGE','BIG.XHUGE',
     & 'HR',
     & 'TD','TD.eqno','TD.runner','TD.runnerb',
     & 'a:link','a:visited','a:active','a:hover',
     & 'a.runner:link','a.runner:visited',
     & 'a.runner:active','a.runner:hover',
     & 'a.runnerb:link','a.runnerb:visited',
     & 'a.runnerb:active','a.runnerb:hover'/
c     declared editable style sheet description name length
      integer ssedsd
      parameter (ssedsd=28)
c     editable style sheet tag descriptions
      character*(ssedsd) ssedsc(ssemax)
      data ssedsc/
     & 'body (normal)',
     & 'H1 (part title)','H2 (chapter title)',
     & 'H3 (section title)','H4 (subsection title)',
     & 'H5 (subsubsection title)','H6 (paragraph title)',
     & 'caption',
     & 'tt (typewriter)','code (verb)','pre (verbatim)',
     & 'extra tiny','tiny','scriptsize',
     & 'footnotesize','small','large',
     & 'Large','LARGE','huge','Huge',
     & 'HTML ruler',
     & 'TD','TD.eqno','top bar','bottom bar',
     & 'unvisited link','visited link','active link','link under mouse',
     & 'unvisited link, top bar','visited link, top bar',
     & 'active link, top bar','link under mouse, top bar',
     & 'unvisited link, bottom bar','visited link, bottom bar',
     & 'active link, bottom bar','link under mouse, bottom bar'/
c     font size magscales, powers of 1.2 (cheat for numbers 9-11)
      integer ssemag(ssemax)
      data ssemag/
     & 0,
     & 5,4,3,2,1,0,
     & 0,
     & 0,0,0,
     & -5,-4,-3,-2,-1,1,2,3,4,5,
     & 99,
     & 0,0,0,0,
     & 99,99,99,99,
     & 99,99,99,99,
     & 99,99,99,99/
c     color codes (blwh is black/white, depending on background)
      integer cwhite,cblack,ctitle,cblue,cblueb,cblwh
      parameter (cwhite=1,cblack=2,ctitle=3,cblue=4,cblueb=5,cblwh=6)
      integer mblue,mblueb
      parameter (mblue=-4,mblueb=-5)
c     foreground color classifications
      integer ssecol(ssemax)
      data ssecol/
     & cblack,
     & ctitle,ctitle,ctitle,ctitle,ctitle,ctitle,
     & cblack,
     & cblack,cblack,cblack,
     & 0,0,0,0,0,0,0,0,0,0,
     & cblue,
     & cblack,cblack,cblwh,cblwh,
     & cblack,cblack,ctitle,ctitle,
     & cblwh,cblwh,cblwh,cblwh,
     & cblwh,cblwh,cblwh,cblwh/
c     background color classifications
      integer ssebcl(ssemax)
      data ssebcl/
     & cwhite,
     & 0,0,0,0,0,0,
     & 0,
     & 0,0,0,
     & 0,0,0,0,0,0,0,0,0,0,
     & cblue,
     & 0,0,cblue,cblueb,
     & 0,0,0,0,
     & mblue,mblue,mblue,mblue,
     & mblueb,mblueb,mblueb,mblueb/
c     selectors
      integer ssemrk(ssemax)

c     number of navigation icons we are worried about
      integer navmax
      parameter (navmax=15)
c     navigation icon index
      integer nav
c     navigation icon names in iconsizes, plus a space
      character*20 navnam(navmax)
      data navnam/'up','next','previous','next_group','next_inactive',
     & 'previous_group','change_begin','change_begin_right',
     & 'change_end','change_end_right','change_delete',
     & 'change_delete_right','contents','index','image'/
c     navigation icon name length
      integer navnml
c     corresponding navigation icon file names, plus a space
      character*9 navfil(navmax)
      data navfil/'up','next','prev','nx_grp','next_g',
     & 'pv_grp','ch_begin','ch_beg_r',
     & 'ch_end','ch_end_r','ch_delet',
     & 'ch_del_r','contents','index','image'/
c     navigation file length
      integer navfll
c     picture size
      integer xsiz,ysiz
c     icon size string
      character*1600 icons
      integer iconsl

c     declared primary and secondary file buffer sizes
      integer bufdim
      parameter (bufdim=500000)
c     actual file buffer sizes
      integer bufmax,bufmx2
c     file buffers
      character*1 bufstr(bufdim),bufst2(bufdim)

c     counter of matches found
      integer fndct
c     counter of nontrivial substitutions done
      integer subct

c     whether we use buffer 2
      integer use2

c     whether we must write new files
      integer wrtidx,wrtl2h
      data wrtidx/0/,wrtl2h/0/

c     what needs to be done to any existing web-pages folder
      integer webpag
      data webpag/0/
c     copy in the new buttons, remake, delete and remake
      integer copbut,remake,delete
      parameter (copbut=1,remake=2,delete=4)

c     font size from the body tag
      integer fntsz0
c     entered body font size (float)
      real fntszb
c     font size
      integer fntsiz
c     scale factor
      real scale

c     document body font size (float)
      real fntszd

c     default colors, from index.css
      character*6 dwhite,dblack,dtitle,dblue,dblueb
c     replacement colors
      character*6 rwhite,rblack,rtitle,rblue,rblueb,rblwh,rcolor
c     color distance
      integer coldis
      external coldis

c     count of failures
      integer failct

c     nonzero if we have a DOS file
      integer crlf

c     hex characters
      character*16 hex
      parameter (hex='0123456789ABCDEF')

c     generic lines
      character*80 line
      integer linelt

c     answer
      character*1 ans

c     numeric limits
      integer intmin,intmax
      real valmin,valmax

c     number as character string
      character*20 cnum
      integer cnuml

c     whether a file exists
      logical exists

c     iteration index
      integer iter

c     generic index
      integer i

c     temporaries
      integer itemp
      real temp

c executable

c     generic character formats
 10   format(a,a,a)
 11   format(a)

c     allow I/O to use units 7 and on
      call libini(7)

c     set the error exit status to 9 to allow some passing of pointers
      call errset(9)

c     get the command line arguments
      call args(argdim,'Usage: wizhlp.exe L2HPATH TASK [PARAMETERS]',
     & argmx,argt,argtl)
      if(argmx.lt.2)goto 9990

c     set path to wiz files
      i=argtl(1)
      sep=argt(1)(i:i)
      if(sep.ne.'\\' .and. sep.ne.'/')call out(
     & '*** wizhlp: the last character of L2HPATH must be \\ or /!')
      path=argt(1)(1:i)//'system-files'//sep//'wiz'//sep
      pathl=i+12+1+3+1

c     Process the functions

      if(argt(2)(1:argtl(2)).ne.'fntsiz')goto 1200

c *****************************************************************************
c     fntsiz: change font size
c *****************************************************************************

c     change index.css

c     read the style file to change into storage
      call readfu('index.css',9,2,bufdim,bufmax,bufstr)
      use2=0

c     convert crlf into lf
      call trnit('\\M\\J','\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,crlf)
      call trnit('\\M\\J','\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,itemp)

c     find body font size
      call getval('BODY','font-size',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
      fntsz0=0
      if(fndct.eq.-1)then
         call type('nobody')
         call out(
     &    '*** The theme is corrupt!  Please restore, then try again!')
      else if(linelt.le.0)then
         call out(
     &    '*** index.css is corrupt: no body font size specified')
         call out(
     &    '    The theme is corrupt!  Please restore, then try again.')
      else
         do 1130 i=1,linelt
            if(line(i:i).lt.'0' .or. line(i:i).gt.'9')then
               if(line(i:linelt).eq.'px')goto 1132
               goto 1131
            endif
            fntsz0=fntsz0*10+ichar(line(i:i))-48
 1130    continue
 1131    fntsz0=0
         call out(
     &    '*** The body font size in index.css is corrupt')
         call out(
     &    '    The theme is corrupt!  Please restore, then try again.')
 1132    continue
      endif

c     ask for the new size
      call type('askbfsz')
 1140 if(fntsz0.eq.0)fntsz0=15
      fntsz0=max(10,min(30,fntsz0))
      call encint(fntsz0,cnuml,cnum)
      line='Enter the new main font size: '//cnum(1:cnuml)
      call in2(line,linelt)
      if(line(1:linelt).eq.'q')goto 9980
      call readvl(line(1:linelt),fntszb,*1140)
      if(fntszb.lt.9.6 .or. fntszb.gt.40.4)then
         call out('** Accepted range is 10 to 40')
         goto 1140
      endif

c     convert all tags with font sizes
      failct=0
      do 1150 sse=1,ssemax
         ssemrk(sse)=0
         if(ssemag(sse).eq.99)goto 1150
         fntsiz=nint(fntszb*1.2**ssemag(sse))
         if(sse.ge.9 .and. sse.le.11)fntsiz=nint(fntszb*1.07)
         fntsiz=max(7,min(99,fntsiz))
         call encint(fntsiz,cnuml,cnum)
         ssetgl=index(ssetag(sse),' ')-1
         if(ssetgl.le.0)ssetgl=ssetgd
         call setval(ssetag(sse)(1:ssetgl),'font-size',
     &    cnum(1:cnuml)//'px',0,
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
         if(fndct.le.0)ssemrk(sse)=1
         if(fndct.le.0)failct=failct+1
         if(subct.gt.0)wrtidx=1
 1150 continue

c     warn about failures
      if(failct.gt.0)then
         call type('badfsiz')
         call lsttag(ssemax,ssedsc,ssemrk)
         call pause
      endif

c     convert lf into crlf
      if(crlf.ne.0)call trnit('\\J','\\M\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)

c     write the new index.css file
      if(wrtidx.ne.0)then
         if(use2.eq.0)then
            call writfu('index@new.css',13,2,bufdim,bufmax,bufstr)
         else
            call writfu('index@new.css',13,2,bufdim,bufmx2,bufst2)
         endif
      endif

c     change settings.pl

c     read the file to change into storage
      call readfu('settings.pl',11,2,bufdim,bufmax,bufstr)
      use2=0

c     ask for document font size
      call type('askdfsiz')
 1160 line='Enter document font size in points: 12'
      call in2(line,linelt)
      if(line(1:linelt).eq.'q')goto 9980
      call readvl(line(1:linelt),fntszd,*1160)
      if(fntszd.lt.6.6)then
         call out('** Think again.')
         goto 1160
      endif
      if(fntszd.lt.6.6 .or. fntszd.gt.15.4)then
         call out('** Accepted range is 7 to 15')
         goto 1160
      endif

c     compute new scale factor
      scale=fntszb*0.11256*10./fntszd
      write(cnum(1:6),1170)scale
 1170 format(f6.3)
      i=1
      if(cnum(1:1).eq.' ')i=2

c     replace the scale factor
      call trnit(
     & '$MATH_SCALE_FACTOR\\?\\+=\\?\\+\\?\\&\\5.\\*\\?\\+;',
     & '$MATH_SCALE_FACTOR = '//cnum(i:6)//';',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,wrtl2h)
      if(fndct.le.0)then
         call out(
     &    '*** Unable to change the math images size in settings.pl')
         call type('restl2hi')
         call out(
     &    '    The theme is corrupt!  Please restore, then try again.')
      endif

c     ask for display magnification
      call type('askdmag')
 1180 line='Enter the magnification factor for displayed formulae: 1'
      call in2(line,linelt)
      if(line(1:linelt).eq.'q')goto 9980
      call readvl(line(1:linelt),scale,*1180)
      if(scale.lt.0.5 .or. scale.gt.2.)then
         call out('** Accepted range is 0.5 to 2')
         goto 1180
      endif

c     replace the magnification factor
      write(cnum(1:6),1170)scale
      i=1
      if(cnum(1:1).eq.' ')i=2
      call trnit(
     & '$DISP_SCALE_FACTOR\\?\\+=\\?\\+\\?\\&\\5.\\*\\?\\+;',
     & '$DISP_SCALE_FACTOR = '//cnum(i:6)//';',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
      wrtl2h=wrtl2h+subct
      if(fndct.le.0)then
         call out('*** Unable to change the display scale factor '//
     &    'in settings.pl')
         call type('restl2hi')
         call out(
     &    '    The theme is corrupt!  Please restore, then try again.')
      endif

c     ask for figure magnification
      call type('askfmag')
 1190 line='Enter the magnification factor for figures: 1.81'
      call in2(line,linelt)
      if(line(1:linelt).eq.'q')goto 9980
      call readvl(line(1:linelt),scale,*1190)
      if(scale.lt.0.25 .or. scale.gt.4.)then
         call out('** Accepted range is 0.25 to 4')
         goto 1190
      endif

c     replace the magnification factor
      write(cnum(1:6),1170)scale
      i=1
      if(cnum(1:1).eq.' ')i=2
      call trnit(
     & '$FIGURE_SCALE_FACTOR\\?\\+=\\?\\+\\?\\&\\5.\\*\\?\\+;',
     & '$FIGURE_SCALE_FACTOR = '//cnum(i:6)//';',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
      wrtl2h=wrtl2h+subct
      if(fndct.le.0)then
         call out('*** Unable to change the figure scale factor '//
     &    'in settings.pl')
         call type('restl2hi')
         call out(
     &    '    The theme is corrupt!  Please restore, then try again.')
      endif

c     write the new settings.pl file
      if(wrtl2h.ne.0)then
         if(use2.eq.0)then
            call writfu('settings@new.pl',15,2,bufdim,bufmax,bufstr)
         else
            call writfu('settings@new.pl',15,2,bufdim,bufmx2,bufst2)
         endif
         webpag=delete
      endif

c     done
      goto 9900

 1200 if(argt(2)(1:argtl(2)).ne.'colors')goto 1400

c *****************************************************************************
c     colors: change color scheme
c *****************************************************************************

c     change index.css

c     read the style file to change into storage
      call readfu('index.css',9,2,bufdim,bufmax,bufstr)
      use2=0

c     convert crlf into lf
      call trnit('\\M\\J','\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,crlf)
      call trnit('\\M\\J','\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,itemp)

c     get the background color from index.css and ask for the replacement
      goto 1310
 1309 call type('nobody')
      call out(
     & '*** The theme is corrupt!  Please restore, then try again!')
 1310 call getval('BODY','background-color',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
      if(fndct.eq.-1)goto 1309
      if(linelt.le.0)then
         call warn(
     &    'No background color specified in index.css; frames version?')
         dwhite='FFFFFF'
         call askcol('Enter the new web page background color',
     &    dwhite,rwhite)
         dwhite=rwhite
      else if(linelt.eq.7)then
         if(line(1:1).ne.'#')goto 1309
         do 1313 i=2,7
            if(line(i:i).ge.'a' .and. line(i:i).le.'z')
     &       line(i:i)=char(ichar(line(i:i))-32)
            if(index(hex,line(i:i)).le.0)goto 1309
 1313    continue
         call typep('ghost')
         dwhite=line(2:7)
         call askcol('Enter the new web page background color',
     &    dwhite,rwhite)
         if(dwhite.ne.rwhite)webpag=delete
         dwhite=rwhite
      else
         goto 1309
      endif

c     get the text color from index.css and ask for the replacement
      call getval('BODY','color',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
      if(fndct.eq.-1)goto 1309
      if(linelt.le.0)then
         call warn(
     &    'No text color specified in index.css; frames version?')
         dblack='000000'
      else if(linelt.eq.7)then
         if(line(1:1).ne.'#')goto 1309
         do 1322 i=2,7
            if(line(i:i).ge.'a' .and. line(i:i).le.'z')
     &       line(i:i)=char(ichar(line(i:i))-32)
            if(index(hex,line(i:i)).le.0)goto 1309
 1322    continue
         dblack=line(2:7)
      else
         goto 1309
      endif
      call askcol('Enter the new main text color',dblack,rblack)
      dblack=rblack
      if(coldis(rwhite,rblack).lt.150)then
         call out('** Text and background are hard to distinguish.')
         line='Are you sure about those colors? [y/n]:'
         call inkey(5,'yn',line,ans)
         if(ans.ne.'y')goto 1310
      endif

c     get the title color from index.css and ask for the replacement
      call getval('H3','color',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
      dtitle='000066'
      if(linelt.eq.7)then
         if(line(1:1).ne.'#')goto 1332
         do 1331 i=2,7
            if(line(i:i).ge.'a' .and. line(i:i).le.'z')
     &       line(i:i)=char(ichar(line(i:i))-32)
            if(index(hex,line(i:i)).le.0)goto 1332
 1331    continue
         dtitle=line(2:7)
 1332    continue
      endif
      call askcol('Enter the color for section titles',dtitle,rtitle)
      dtitle=rtitle

c     get the top bar color from index.css and ask for the replacement
      call getval('TD.runner','background-color',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
      dblue='0033AA'
      if(linelt.eq.7)then
         if(line(1:1).ne.'#')goto 1342
         do 1341 i=2,7
            if(line(i:i).ge.'a' .and. line(i:i).le.'z')
     &       line(i:i)=char(ichar(line(i:i))-32)
            if(index(hex,line(i:i)).le.0)goto 1342
 1341    continue
         dblue=line(2:7)
 1342    continue
      endif
      call askcol('Enter the new color of the header bar',
     & dblue,rblue)
      dblue=rblue

c     get the bottom bar color from index.css and ask for the replacement
      call getval('TD.runnerb','background-color',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
      dblueb='0033AA'
      if(linelt.eq.7)then
         if(line(1:1).ne.'#')goto 1347
         do 1346 i=2,7
            if(line(i:i).ge.'a' .and. line(i:i).le.'z')
     &       line(i:i)=char(ichar(line(i:i))-32)
            if(index(hex,line(i:i)).le.0)goto 1347
 1346    continue
         dblueb=line(2:7)
 1347    continue
      endif
      call askcol('Enter the new color of the footer bar',
     & dblueb,rblueb)
      dblueb=rblueb

c     set the colors
      failct=0
      do 1350 sse=1,ssemax
         ssemrk(sse)=0
         ssetgl=index(ssetag(sse),' ')-1
         if(ssetgl.le.0)ssetgl=ssetgd
         if(ssecol(sse).eq.cwhite)then
            call setval(ssetag(sse)(1:ssetgl),'color',
     &       '#'//rwhite,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssebcl(sse).eq.cwhite)then
            call setval(ssetag(sse)(1:ssetgl),'background-color',
     &       '#'//rwhite,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssecol(sse).eq.cblack)then
            call setval(ssetag(sse)(1:ssetgl),'color',
     &       '#'//rblack,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssebcl(sse).eq.cblack)then
            call setval(ssetag(sse)(1:ssetgl),'background-color',
     &       '#'//rblack,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssecol(sse).eq.ctitle)then
            call setval(ssetag(sse)(1:ssetgl),'color',
     &       '#'//rtitle,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssebcl(sse).eq.ctitle)then
            call setval(ssetag(sse)(1:ssetgl),'background-color',
     &       '#'//rtitle,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssecol(sse).eq.cblue)then
            call setval(ssetag(sse)(1:ssetgl),'color',
     &       '#'//rblue,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssebcl(sse).eq.cblue)then
            call setval(ssetag(sse)(1:ssetgl),'background-color',
     &       '#'//rblue,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssecol(sse).eq.cblueb)then
            call setval(ssetag(sse)(1:ssetgl),'color',
     &       '#'//rblueb,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssebcl(sse).eq.cblueb)then
            call setval(ssetag(sse)(1:ssetgl),'background-color',
     &       '#'//rblueb,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
         if(ssecol(sse).eq.cblwh)then
            rblwh=rblack
            i=abs(ssebcl(sse))
            if(i.eq.cblack)then
               rblwh=rwhite
            else if(i.eq.cblue)then
               if(coldis(rwhite,rblue).gt.coldis(rblack,rblue))
     &          rblwh=rwhite
            else if(i.eq.cblueb)then
               if(coldis(rwhite,rblueb).gt.coldis(rblack,rblueb))
     &          rblwh=rwhite
            else if(i.eq.ctitle)then
               if(coldis(rwhite,rtitle).gt.coldis(rblack,rtitle))
     &          rblwh=rwhite
            endif
            call setval(ssetag(sse)(1:ssetgl),'color',
     &       '#'//rblwh,0,
     &       bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
            if(fndct.le.0)ssemrk(sse)=1
            if(fndct.le.0)failct=failct+1
            if(subct.gt.0)wrtidx=1
         endif
 1350 continue

c     warn about failures
      if(failct.gt.0)then
         call type('badcols')
         call lsttag(ssemax,ssedsc,ssemrk)
         call pause
      endif

c     convert lf into crlf
      if(crlf.ne.0)call trnit('\\J','\\M\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)

c     write the new index.css file
      if(wrtidx.ne.0)then
         if(use2.eq.0)then
            call writfu('index@new.css',13,2,bufdim,bufmax,bufstr)
         else
            call writfu('index@new.css',13,2,bufdim,bufmx2,bufst2)
         endif
      endif

c     done
      goto 9900

 1400 if(argt(2)(1:argtl(2)).ne.'setatt')goto 1600

c *****************************************************************************
c     setatt: set individual style sheet attributes
c *****************************************************************************

c     change index.css

c     read the style file to change into storage
      call readfu('index.css',9,2,bufdim,bufmax,bufstr)
      use2=0

c     convert crlf into lf
      call trnit('\\M\\J','\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,crlf)
      call trnit('\\M\\J','\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,itemp)

c     get the tag to change
 1510 do 1520 sse=1,ssemax
         ssemrk(sse)=1
 1520 continue
      call type('seltag')
      call lsttgn(ssemax,ssedsc,ssemrk)
      call out(' ')
      goto 1531
 1530 call out('** Invalid tag number')
 1531 line='Enter tag number (0 if done, q to quit): 0'
      call in2(line,linelt)
      if(line(1:linelt).eq.'q')goto 9980
      if(linelt.lt.1 .or. linelt.gt.3)goto 1530
      call readiv(line(1:linelt),sse,*1530)
      if(sse.eq.0)goto 1590
      if(sse.le.0 .or. sse.gt.ssemax)goto 1530
      ssetgl=index(ssetag(sse),' ')-1
      if(ssetgl.le.0)ssetgl=ssetgd

c     get attributes
 1540 call getval(ssetag(sse)(1:ssetgl),'font-size',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
      if(fndct.lt.0)then
         call out('** tag was not found in index.css')
         goto 1531
      endif
      call out(' ')
      call out('Select an attribute to change:')
      if(fndct.le.0)then
         call out('   1) font-size: [not available]')
      else
         if(linelt.gt.0)call out('   1) font-size: '//line(1:linelt))
         if(linelt.le.0)call out('   1) font-size:')
      endif
      call getval(ssetag(sse)(1:ssetgl),'color',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
      if(fndct.lt.0)call out('** wizhlp: Program error; lost tag!')
      if(fndct.le.0)then
         call out('   2) color: [not available]')
      else
         if(linelt.gt.0)call out('   2) color: '//line(1:linelt))
         if(linelt.le.0)call out('   2) color:')
      endif
      call getval(ssetag(sse)(1:ssetgl),'background-color',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
      if(fndct.lt.0)call out('** wizhlp: Program error; lost tag!')
      if(fndct.le.0)then
         call out('   3) background-color: [not available]')
      else
         if(linelt.gt.0)call out('   3) background-color: '//
     &    line(1:linelt))
         if(linelt.le.0)call out('   3) background-color:')
      endif
      line='Select the attribute to change (0 to quit): 0'
      call inkey(5,'0123',line,ans)
      if(ans.eq.'0')goto 1510
      if(ans.eq.'1')then
         call getval(ssetag(sse)(1:ssetgl),'font-size',
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
         cnum(1:1)=' '
         cnuml=1
         if(linelt.gt.0)then
            i=max(index(line(1:linelt),'px'),index(line(1:linelt),'PX'))
            if(i.gt.1)then
               cnuml=i-1
               cnum(1:cnuml)=line(1:cnuml)
            endif
         endif
         if(fndct.le.0)then
            line='Create the tag?  Are you sure? [y/n]: n'
            call inkey(5,'yn',line,ans)
            if(ans.ne.'y')goto 1540
         endif
         call type('askfsiz')
         goto 1551
 1550    call out('** invalid number of pixels')
 1551    line='Enter the number of pixels: '//cnum(1:cnuml)
         call in2(line,linelt)
         if(line(1:linelt).eq.'q')goto 9980
         if(line(1:linelt).eq.cnum(1:cnuml))goto 1540
         call readiv(line(1:linelt),fntsiz,*1550)
         if(fntsiz.lt.2 .or. fntsiz.gt.99)goto 1550
         call encint(fntsiz,cnuml,cnum)
         call setval(ssetag(sse)(1:ssetgl),'font-size',
     &    cnum(1:cnuml)//'px',1,
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
         wrtidx=wrtidx+subct
         if(sse.eq.1 .and. subct.ne.0)webpag=delete
         goto 1540
      else if(ans.eq.'2')then
         call getval(ssetag(sse)(1:ssetgl),'color',
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
         dblack=' '
         if(linelt.eq.7)then
            if(line(1:1).ne.'#')goto 1562
            do 1561 i=2,7
               if(line(i:i).ge.'a' .and. line(i:i).le.'z')
     &          line(i:i)=char(ichar(line(i:i))-32)
               if(index(hex,line(i:i)).le.0)goto 1562
 1561       continue
            dblack=line(2:7)
 1562       continue
         endif
         if(fndct.le.0)then
            line='Create the tag?  Are you sure? [y/n]: n'
            call inkey(5,'yn',line,ans)
            if(ans.ne.'y')goto 1540
         endif
         call askcol('Enter the new color',dblack,rblack)
         if(rblack.eq.dblack)goto 1540
         call setval(ssetag(sse)(1:ssetgl),'color',
     &    '#'//rblack,1,
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
         wrtidx=wrtidx+subct
         goto 1540
      else if(ans.eq.'3')then
         if(sse.eq.1)call typep('ghost')
         call getval(ssetag(sse)(1:ssetgl),'background-color',
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,line,linelt)
         dblack=' '
         if(linelt.eq.7)then
            if(line(1:1).ne.'#')goto 1572
            do 1571 i=2,7
               if(line(i:i).ge.'a' .and. line(i:i).le.'z')
     &          line(i:i)=char(ichar(line(i:i))-32)
               if(index(hex,line(i:i)).le.0)goto 1572
 1571       continue
            dblack=line(2:7)
 1572       continue
         endif
         if(fndct.le.0)then
            line='Create the tag?  Are you sure? [y/n]: n'
            call inkey(5,'yn',line,ans)
            if(ans.ne.'y')goto 1540
         endif
         call askcol('Enter the new background color',dblack,rblack)
         if(rblack.eq.dblack)goto 1540
         call setval(ssetag(sse)(1:ssetgl),'background-color',
     &    '#'//rblack,1,
     &    bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
         wrtidx=wrtidx+subct
         if(sse.eq.1 .and. subct.ne.0)webpag=delete
         goto 1540
      else
         call out('** wizhlp: Program error in inkey')
      endif

c     convert lf into crlf
 1590 if(crlf.ne.0)call trnit('\\J','\\M\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)

c     write the new index.css file
      if(wrtidx.ne.0)then
         if(use2.eq.0)then
            call writfu('index@new.css',13,2,bufdim,bufmax,bufstr)
         else
            call writfu('index@new.css',13,2,bufdim,bufmx2,bufst2)
         endif
      endif

c     done
      goto 9900

 1600 if(argt(2)(1:argtl(2)).ne.'myicons')goto 1800

c *****************************************************************************
c     myicons: swap in my own icon set
c *****************************************************************************

c     change settings.pl

c     read the file to change into storage
 1700 call readfu('settings.pl',11,2,bufdim,bufmax,bufstr)
      use2=0
      if(bufstr(bufmax).eq.char(26))call warn('** ^Z')

c     convert crlf into lf
      call trnit('\\M\\J','\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,crlf)
      call trnit('\\M\\J','\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,itemp)

c     we must replace the buttons
      webpag=copbut

c     create the new iconsizes
      icons='%iconsizes = (\\J'
      iconsl=16
      do 1750 nav=1,navmax
         navfll=index(navfil(nav),' ')-1
         navnml=index(navnam(nav),' ')-1
         call gifsz(2,'buttons@new'//sep//navfil(nav)(1:navfll)//'.gif',
     &    xsiz,ysiz)
         if(xsiz.le.-3)call out('*** Required button '//
     &    navfil(nav)(1:navfll)//
     &    '.gif not found in folder buttons@new!')
         if(xsiz.le.-2)call out('*** Cannot access button '//
     &    navfil(nav)(1:navfll)//'.gif in folder buttons@new!')
         if(xsiz.le.0 .or. ysiz.le.0)call out(
     &    '*** Cannot find the size of button '//navfil(nav)(1:navfll)//
     &    '.gif in folder buttons@new!')
         icons(iconsl+1:iconsl+8+navnml+10)=char(9)//'      '''//
     &    navnam(nav)(1:navnml)//''',''WIDTH="'
         i=iconsl+8
         iconsl=iconsl+8+navnml+10
         call encint(xsiz,cnuml,cnum)
         icons(iconsl+1:iconsl+cnuml+10)=cnum(1:cnuml)//'" HEIGHT="'
         iconsl=iconsl+cnuml+10
         call encint(ysiz,cnuml,cnum)
         if(nav.le.6 .or. nav.eq.13 .or. nav.eq.14)then
            icons(iconsl+1:iconsl+cnuml+19)=cnum(1:cnuml)//
     &       '" ALIGN="TOP"'',\\J'
            iconsl=iconsl+cnuml+17
         else if(nav.eq.8 .or. nav.eq.10 .or. nav.eq.12)then
            icons(iconsl+1:iconsl+cnuml+19)=cnum(1:cnuml)//
     &       '" ALIGN="RIGHT"'',\\J'
            iconsl=iconsl+cnuml+19
         else
            icons(iconsl+1:iconsl+cnuml+5)=cnum(1:cnuml)//'"'',\\J'
            iconsl=iconsl+cnuml+5
         endif
         call out2(icons(i:iconsl-2))
 1750 continue
      iconsl=iconsl-3
      icons(iconsl+1:iconsl+10)='\\J'//char(9)//'      )'
      iconsl=iconsl+10

c     convert the string
      call trnit(
     & '%iconsizes\\?\\+=\\?\\+(\\?\\~\\&()\\*)',icons(1:iconsl),
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,wrtl2h)
      if(fndct.le.0)then
         call type('restl2hi')
         call out('*** Unable to change the navigation icon '//
     &    'sizes in settings.pl!')
      endif

c     remake is needed if the sizes have changed
      if(wrtl2h.ne.0)webpag=copbut+remake

c     set the alternative icons
      call trnit('$ALTERNATIVE_ICONS\\?\\+=\\-\\?\\+"\\?\\~""',
     & '$ALTERNATIVE_ICONS = "..${dd}buttons"',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)
      wrtl2h=wrtl2h+subct
      if(fndct.le.0)then
         call type('restl2hi')
         call out('*** Unable to change the icon folder '//
     &    'location in settings.pl!')
      endif

c     convert lf into crlf
      if(crlf.ne.0)call trnit('\\J','\\M\\J',
     & bufdim,bufmax,bufstr,bufmx2,bufst2,use2,fndct,subct)

c     write the new settings.pl file
      if(wrtl2h.ne.0)then
         if(use2.eq.0)then
            call writfu('settings@new.pl',15,2,bufdim,bufmax,bufstr)
         else
            call writfu('settings@new.pl',15,2,bufdim,bufmx2,bufst2)
         endif
      endif

c     done
      goto 9900


 1800 if(argt(2)(1:argtl(2)).ne.'chicons')goto 2000

c *****************************************************************************
c     chicons: change the icon color scheme
c *****************************************************************************

c     open the color list file and the substitution files
      inquire(file='buttons@new'//sep//'colors.lst',
     & exist=exists,err=9910)
      if(.not.exists)call out('*** file buttons@new'//sep//
     & 'colors.lst does not exist!')
      open(1,file='buttons@new'//sep//'colors.lst',
     & status='old',err=9920)
      open(2,file='buttons@new'//sep//'@colors.sub',err=9920)
      open(3,file='buttons@new'//sep//'@colors_g.sub',err=9920)

c     read the color list and ask for replacements
      iter=0
      goto 1920
 1910 call out('*** file colors.lst in folder buttons@new is corrupt!')
 1920 read(1,1930,end=1990,err=9930)line
 1930 format(a80)
      if(line.eq.' ')goto 1920
      if(line(1:1).eq.'#')goto 1920
      if(line(7:7).ne.' ' .or. line(14:14).ne.' ')goto 1910
      do 1940 i=1,13
         if(i.eq.7)goto 1940
         if(line(i:i).ge.'a' .and. line(i:i).le.'z')
     &    line(i:i)=char(ichar(line(i:i))-32)
         if(index(hex,line(i:i)).le.0)goto 1910
 1940 continue
      do 1950 i=80,15,-1
         linelt=i
         if(line(i:i).gt.' ')goto 1951
 1950 continue
      goto 1910
 1951 call askcol(line(15:linelt),line(8:13),rcolor)
      iter=iter+1
      if(index(line(14:linelt),' disabled').le.0)
     & write(2,10,err=9940)line(1:7)//rcolor
      if(index(line(14:linelt),' enabled').le.0)
     & write(3,10,err=9940)line(1:7)//rcolor
      goto 1920
 1990 close(1,err=9950)
      close(2,err=9950)
      close(3,err=9950)
      if(iter.le.0)call out('*** There are no colors!')

c     go change the current icons
      goto 1700

 2000 if(argt(2)(1:argtl(2)).ne.'askcol')goto 2200

c *****************************************************************************
c     askcol: ask for a color
c *****************************************************************************

c     check arguments
      if(argmx.ne.5)goto 9990
      if(argtl(3).le.0 .or. argtl(3).gt.66)goto 9990
      if(argtl(4).le.0)goto 9990
      do 2040 i=1,argtl(4)
         if(argt(4)(i:i).ge.'a' .and. argt(4)(i:i).le.'z')goto 2040
         goto 9990
 2040 continue
      if(argtl(5).ne.6)goto 9990
      do 2050 i=1,argtl(5)
         if(argt(5)(i:i).ge.'0' .and. argt(5)(i:i).le.'9')goto 2050
         if(argt(5)(i:i).ge.'A' .and. argt(5)(i:i).le.'F')goto 2050
         goto 9990
 2050 continue

c     ask for the color
      call askcol(argt(3)(1:argtl(3)),argt(5)(1:argtl(5)),rcolor)

c     write it to file
      open(1,file='@tmp.bat',err=9920)
      if(sep.eq.'\\')then
         write(1,11,err=9940)
     &    '@echo off',
     &    'set '//argt(4)(1:argtl(4))//'='//rcolor,
     &    'if not "%'//argt(4)(1:argtl(4))//'%" == "'//rcolor//
     &    '" echo exit | find "error"'
      else
         write(1,11,err=9940)
     &    'set '//argt(4)(1:argtl(4))//'='''//rcolor//''''
      endif
      close(1,err=9950)

c     done
      goto 9900

 2200 if(argt(2)(1:argtl(2)).ne.'askint')goto 2400

c *****************************************************************************
c     askint: ask for an integer
c *****************************************************************************

c     check arguments
      if(argmx.ne.7)goto 9990
      if(argtl(3).le.0 .or. argtl(3)+argtl(7).gt.77)goto 9990
      if(argtl(4).le.0)goto 9990
      do 2240 i=1,argtl(4)
         if(argt(4)(i:i).ge.'a' .and. argt(4)(i:i).le.'z')goto 2240
         goto 9990
 2240 continue
      if(argtl(5).le.0)goto 9990
      call readiv(argt(5)(1:argtl(5)),intmin,*9990)
      if(argtl(6).le.0)goto 9990
      call readiv(argt(6)(1:argtl(6)),intmax,*9990)
      if(intmax.lt.intmin)goto 9990
      if(argtl(7).le.0)goto 9990
      call readiv(argt(7)(1:argtl(7)),itemp,*9990)
      if(itemp.lt.intmin .or. itemp.gt.intmax)goto 9990

c     ask for the number
 2350 line=argt(3)(1:argtl(3))//': '//argt(7)(1:argtl(7))
      call in2(line,linelt)
      if(line(1:linelt).eq.'q')goto 9980
      call readiv(line(1:linelt),itemp,*2350)
      if(itemp.lt.intmin .or. itemp.gt.intmax)then
         call out('** number must be in the range from '//
     &    argt(5)(1:argtl(5))//' to '//argt(6)(1:argtl(6)))
         goto 2350
      endif

c     write it to file
      open(1,file='@tmp.bat',err=9920)
      if(sep.eq.'\\')then
         write(1,11,err=9940)
     &    '@echo off',
     &    'set '//argt(4)(1:argtl(4))//'='//line(1:linelt),
     &    'if not "%'//argt(4)(1:argtl(4))//'%" == "'//line(1:linelt)//
     &    '" echo exit | find "error"'
      else
         write(1,11,err=9940)
     &    'set '//argt(4)(1:argtl(4))//'='''//line(1:linelt)//''''
      endif
      close(1,err=9950)

c     done
      goto 9900

 2400 if(argt(2)(1:argtl(2)).ne.'asknum')goto 3000

c *****************************************************************************
c     asknum: ask for an number
c *****************************************************************************

c     check arguments
      if(argmx.ne.7)goto 9990
      if(argtl(3).le.0 .or. argtl(3)+argtl(7).gt.77)goto 9990
      if(argtl(4).le.0)goto 9990
      do 2440 i=1,argtl(4)
         if(argt(4)(i:i).ge.'a' .and. argt(4)(i:i).le.'z')goto 2440
         goto 9990
 2440 continue
      if(argtl(5).le.0)goto 9990
      call readvl(argt(5)(1:argtl(5)),valmin,*9990)
      if(argtl(6).le.0)goto 9990
      call readvl(argt(6)(1:argtl(6)),valmax,*9990)
      if(valmax.lt.valmin)goto 9990
      if(argtl(7).le.0)goto 9990
      call readvl(argt(7)(1:argtl(7)),temp,*9990)
      if(temp.lt.valmin .or. temp.gt.valmax)goto 9990

c     ask for the number
 2550 line=argt(3)(1:argtl(3))//': '//argt(7)(1:argtl(7))
      call in2(line,linelt)
      if(line(1:linelt).eq.'q')goto 9980
      call readvl(line(1:linelt),temp,*2550)
      if(temp.lt.valmin .or. temp.gt.valmax)then
         call out('** number must be in the range from '//
     &    argt(5)(1:argtl(5))//' to '//argt(6)(1:argtl(6)))
         goto 2550
      endif

c     write it to file
      open(1,file='@tmp.bat',err=9920)
      if(sep.eq.'\\')then
         write(1,11,err=9940)
     &    '@echo off',
     &    'set '//argt(4)(1:argtl(4))//'='//line(1:linelt),
     &    'if not "%'//argt(4)(1:argtl(4))//'%" == "'//line(1:linelt)//
     &    '" echo exit | find "error"'
      else
         write(1,11,err=9940)
     &    'set '//argt(4)(1:argtl(4))//'='''//line(1:linelt)//''''
      endif
      close(1,err=9950)

c     done
      goto 9900

c     keyword not found
 3000 call out(
     & '*** wizhlp: unrecognized task "'//argt(2)(1:argtl(2))//'"!')

c     exit
 9900 if(webpag.eq.0)call exit(0)
      if(mod(webpag,2*copbut).ge.copbut)
     & call out2('Any buttons in web-pages will have to be replaced.')
      if(mod(webpag,2*remake).ge.remake)
     & call out2('The web pages will have to be remade.')
      if(mod(webpag,2*delete).ge.delete)
     & call out2('The web pages will have to be remade from scratch.')
      call exit(0)

c     inquire errors
 9910 call out('*** wizhlp: Unable to look for a file!')
c     open errors
 9920 call out('*** wizhlp: Unable to open a file!')
c     file read errors
 9930 call out('*** wizhlp: Unable to read from a file!')
c     file write errors
 9940 call out('*** wizhlp: Unable to write to a file!')
c     close errors
 9950 call out('*** wizhlp: Unable to close a file!')
c     user abort
 9980 call out('*** wizhlp: User entered q  to quit!')
c     print usage
 9990 call out('Usage: wizhlp.exe L2HPATH TASK [PARAMETERS]')
      call errxit
      end
