# latex2html defaults

# See the latex2html manual for more.  Making changes to the settings
# below might break something.  So do not play around needlessly.  And
# be sure to check the results.

# Lines starting with "# " are comments that do not affect settings.



# Internationalization:

# The following option properly sets the UTF-8 encoding tag in the web pages.
# That makes UTF-8 characters in index.tex properly understood by browsers.
# I got this solution from:
# http://miguel.leugim.com.mx/index.php/2008/05/18/latex2html-and-utf8-encoding/
$HTML_VERSION = '4.0,latin1,unicode';

# Replace "english" with another language in order to tell LaTeX2HTML that you
# want some generated section titles (eg "Table of Contents" or "References")
# to appear in a different language.  See the files in the styles subdirectory
# of l2h for what is supported.
$TITLES_LANGUAGE = "english";
$default_language = 'english';	# Affects ONLY the way accents are processed



# Appearance:

$NUMBERED_FOOTNOTES = 1;	# ensure that footnotes are numbered
$TOC_STARS = 1;			# starred section in table of contents
$SHORT_INDEX = 1;		# needed unless you unfloat DT in index.css
$ADDRESS = '';			# No automatic address.  If I want one...
$COLOR_HTML = 1;		# can use colors
$WHITE_BACKGROUND = '';		# use a white background if 1
$INFO = 0;			# 1 = add "About this document..." section

# If you change the below option from 1 to 0, I believe it will break the web
# page naming scheme.  So in that case, disable the scheme inside index.tex.
$SHOW_SECTION_NUMBERS = 1;	# 0 = hide the chapter/section/... numbers



# Splitting and navigation:

$MAX_SPLIT_DEPTH = 3;		# overwritten in index.tex
$TOC_DEPTH=8;			# how deep to expand the table of contents
$MAX_LINK_DEPTH = 4;		# depth of subtable of contents (0 = none)
$CHILDLINE = "<BR><HR>\n";	# separates subtable of contents from text
$NO_NAVIGATION = 0;		# 1 = no navigation panel at the top
$AUTO_NAVIGATION = 0;		# 1 = guess if there is one at the bottom
$TOP_NAVIGATION = 1;
$BOTTOM_NAVIGATION = 1;
$WORDS_IN_PAGE = 300;		# bottom navigation point in AUTO_NAVIGATION
$INDEX_IN_NAVIGATION = 1;	# 0 = no link to the index page
$CONTENTS_IN_NAVIGATION = 1;	# 0 = no link to the table of contents
$NEXT_PAGE_IN_NAVIGATION = 1;	# 0 = no link to the next logical page
$PREVIOUS_PAGE_IN_NAVIGATION = 1;# 0 = no link to the previous logical page
$WORDS_IN_NAVIGATION_PANEL_TITLES = 12;# don't cut the section names
$LINKPOINT = '"$FILE$EXTN"';	# index.html target; 0 = no index.html
$NO_FOOTNODE = 1;		# put footnotes on the same web page



# Appearance

# This number will determine the size of the equations, special characters,
# and anything which will be converted into an inlined image
# *except* "image generating environments" such as "figure", "table"
# or "minipage".
# Effective values are those greater than 0.
# Sensible values are between 0.1 - 4.
#
# Leon: Use the body font size in px, from index.css, times 0.1185 here
$MATH_SCALE_FACTOR = 1.595;

# This number, when defined, determines extra scaling for displayed equations.
# It multiplies with the $MATH_SCALE_FACTOR to give the total scaling.
# It is especially useful when \scriptscriptstyle text is used frequently,
# which would otherwise be extremely difficult to read on-screen.
#
$DISP_SCALE_FACTOR = 1;

# -antialias_text
# 1 =  use anti-aliasing in the generation of images of typeset material;
#      e.g. mathematics and text, e.g. in tables and {makeimage} environments.
$ANTI_ALIAS_TEXT = 1;

# This number will determine the size of
# image generating environments such as "figure", "table" or "minipage".
# Effective values are those greater than 0.
# Sensible values are between 0.1 - 4.
$FIGURE_SCALE_FACTOR = 1.6;

# ANTI-ALIASING within generated images
# -antialias
# 1 =  use anti-aliasing in the generation of images of figures .
$ANTI_ALIAS = 1;

# variable values definitely needed for proper appearance
$ACCENT_IMAGES = 'large';	# makes images of accented letters if needed
$NO_SIMPLE_MATH = 1;		# inconsistent math symbols if set to zero



# external links

# web page's external parent link and its label, overwritten in index.tex
$EXTERNAL_UP_LINK = '../index.html';
$EXTERNAL_UP_TITLE = 'Return';
# web page's external previous link and its label
$EXTERNAL_PREV_LINK = '';
$EXTERNAL_PREV_TITLE = '';
# web page's external next link and its label
$EXTERNAL_DOWN_LINK = '';
$EXTERNAL_DOWN_TITLE = '';



# Operational:

$PREFIX = '';			# prefix added to generated file names
$NO_SUBDIR = 0;			# l2h needs this and the next line AS IS
if ($NO_SUBDIR == 0){$DESTDIR = 'web-pages';}
$REUSE = 2;			# leave as is; simply delete web-pages instead
$NOLATEX = 0;			# 1 = do not use latex even if needed
$EXTERNAL_IMAGES = 0;		# 1 = leave the images outside the document
$ASCII_MODE = 0;		# 1 = do not use any icons or images
$PS_IMAGES = 0;			# 1 = keep images outside the web page as ps
$DEBUG = 0;			# 1 = debug mode (watch disk space if set!)
$VERBOSITY = 1;			# verbosity level (see latex2html manual)
$LOCAL_ICONS = 1;		# you *will* regret changing this



# Junk:

$LINE_WIDTH = 500;		# no longer used
$NETSCAPE_HTML = 0;		# don't use
$PAPERSIZE = "a4";		# changes figure handling, apparently.



#  Web page headers and footers:

# The below values will be overwritten by the values in index.tex
$TITLE="LaTeX Document";	# we lose $TITLE (!)
$RUNNING_TITLE="$TITLE";
$TOP_LEFT_TEXT="";
$TOP_CENTER_TEXT="";
$TOP_RIGHT_TEXT="";
$TOP_CLASS="";
$TOP_BORDER='4';
$BOT_LEFT_TEXT="";
$BOT_CENTER_TEXT="";
$BOT_RIGHT_TEXT="";
$BOT_CLASS="";
$BOT_BORDER='4';

# Where to get the icons for navigation panels, footnotes, lists,
# etcetera, and their dimensions.
if ($ENV{L2HTOP}) {
$ALTERNATIVE_ICONS = "$ENV{L2HTOP}${dd}themes${dd}original";
%iconsizes = (
	      'up','WIDTH="26" HEIGHT="24" ALIGN="TOP"',
	      'next','WIDTH="37" HEIGHT="24" ALIGN="TOP"',
	      'previous','WIDTH="63" HEIGHT="24" ALIGN="TOP"',
	      'next_group','WIDTH="81" HEIGHT="24" ALIGN="TOP"',
	      'next_inactive','WIDTH="37" HEIGHT="24" ALIGN="TOP"',
	      'previous_group','WIDTH="107" HEIGHT="24" ALIGN="TOP"',
	      'change_begin','WIDTH="104" HEIGHT="24"',
	      'change_begin_right','WIDTH="104" HEIGHT="24" ALIGN="RIGHT"',
	      'change_end','WIDTH="104" HEIGHT="24"',
	      'change_end_right','WIDTH="104" HEIGHT="24" ALIGN="RIGHT"',
	      'change_delete','WIDTH="109" HEIGHT="24"',
	      'change_delete_right','WIDTH="109" HEIGHT="24" ALIGN="RIGHT"',
	      'contents','WIDTH="65" HEIGHT="24" ALIGN="TOP"',
	      'index','WIDTH="43" HEIGHT="24" ALIGN="TOP"',
	      'image','WIDTH="48" HEIGHT="24"'
	      );
}

# Navigation panels:
#
# The navigation panels are constructed out of buttons and section titles.
# These can be configured in any combination with arbitrary text and
# HTML tags interspersed between them.
# The buttons available are:
# $PREVIOUS - points to the previous section
# $UP  - points up to the "parent" section
# $NEXT - points to the next section
# $NEXT_GROUP - points to the next "group" section
# $PREVIOUS_GROUP - points to the previous "group" section
# $CONTENTS - points to the contents page if there is one
# $INDEX - points to the index page if there is one
#
# If the corresponding section exists the button will contain an
# active link to that section. If the corresponding section does
# not exist the button will be inactive.
#
# Also for each of the $PREVIOUS $UP $NEXT $NEXT_GROUP and $PREVIOUS_GROUP
# buttons there are equivalent $PREVIOUS_TITLE, $UP_TITLE, etc variables
# which contain the titles of their corresponding sections.
# Each title is empty if there is no corresponding section.
#
# The subroutine below constructs the navigation panels in each page.
# Feel free to mix and match buttons, titles, your own text, your logos,
# and arbitrary HTML (the "." is the Perl concatenation operator).
#
# Leon: you cannot make the buttons dynamic within latex2html.  It
# could surely be done through subbin.exe, but I never got excited
# about trying it.  For one, you would need additional "active" button
# images.  If it is a big deal, contact me and I give it some thought.

sub top_navigation_panel {
    "\n<TABLE WIDTH=\"100%\" BORDER=$TOP_BORDER CELLPADDING=0 CELLSPACING=0><TR>\n" .
    "<TD><TABLE WIDTH=\"100%\" BORDER=0 CELLPADDING=2 CELLSPACING=0><TR>\n" .
    "<TD $TOP_CLASS>$TOP_LEFT_TEXT</TD>\n" .
    "<TD $TOP_CLASS ALIGN=\"CENTER\">$TOP_CENTER_TEXT</TD>\n" .
    "<TD $TOP_CLASS ALIGN=\"RIGHT\">$TOP_RIGHT_TEXT</TD>\n" .
    "</TR></TABLE></TD>\n" .
    "</TR></TABLE>\n" .
    "\n" .
    "<BR CLASS=\"small\">\n" .
    "\n" .
    "<TABLE WIDTH=\"100%\" BORDER=0 CELLPADDING=0 CELLSPACING=0><TR>\n" .
    "<TD> $PREVIOUS $UP $NEXT \n" .
    ($NEXT_TITLE ? " To: $NEXT_TITLE\n" : "\n") .
    "</TD>\n" .
    "<TD ALIGN=\"RIGHT\">$INDEX $CONTENTS $CUSTOM_BUTTONS </TD>\n" .
    "</TR></TABLE>\n" .
    "\n" .
    "<HR>\n" .
    "\n"
}

sub bot_navigation_panel {
    "\n<BR CLEAR=ALL><HR>\n" .
    "\n" .
    "<TABLE WIDTH=\"100%\" BORDER=0 CELLPADDING=0 CELLSPACING=0><TR>\n" .
    "<TD> $PREVIOUS $UP $NEXT \n" .
    ($NEXT_TITLE ? " To: $NEXT_TITLE\n" : "\n") .
    "</TD>\n" .
    "<TD ALIGN=\"RIGHT\">$INDEX $CONTENTS $CUSTOM_BUTTONS </TD>\n" .
    "</TR></TABLE>\n" .
    "\n" .
    "<BR CLASS=\"small\">\n" .
    "\n" .
    "<TABLE WIDTH=\"100%\" BORDER=$BOT_BORDER CELLPADDING=0 CELLSPACING=0><TR>\n" .
    "<TD><TABLE WIDTH=\"100%\" BORDER=0 CELLPADDING=2 CELLSPACING=0><TR>\n" .
    "<TD $BOT_CLASS>$BOT_LEFT_TEXT</TD>\n" .
    "<TD $BOT_CLASS ALIGN=\"CENTER\">$BOT_CENTER_TEXT</TD>\n" .
    "<TD $BOT_CLASS ALIGN=\"RIGHT\">$BOT_RIGHT_TEXT</TD>\n" .
    "</TR></TABLE></TD>\n" .
    "</TR></TABLE>\n"
}
